@@ -13,7 +13,12 @@
  You should have received a copy of the GNU General Public License along with
  GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
  */
-import { canonicalizeBaseUrl } from "@gnu-taler/taler-util";
+import {
+  CacheEvictor,
+  TalerExchangeCacheEviction,
+  assertUnreachable,
+  canonicalizeBaseUrl,
+} from "@gnu-taler/taler-util";
 import {
   BrowserHashNavigationProvider,
   ExchangeApiProvider,
@@ -31,6 +36,8 @@ import { strings } from "./i18n/strings.js";
 import "./scss/main.css";
 import { UiSettings, fetchUiSettings } from "./context/ui-settings.js";
 import { UiFormsProvider, fetchUiForms } from "./context/ui-forms.js";
+import { revalidateAccountDecisions } from "./hooks/decisions.js";
+import { revalidateAccountInformation } from "./hooks/account.js";
 
 const WITH_LOCAL_STORAGE_CACHE = false;
 
@@ -56,6 +63,9 @@ export function App(): VNode {
         <ExchangeApiProvider
           baseUrl={new URL("/", baseUrl)}
           frameOnError={ExchangeAmlFrame}
+          evictors={{
+            exchange: evictExchangeSwrCache,
+          }}
         >
           <SWRConfig
             value={{
@@ -111,7 +121,7 @@ function getInitialBackendBaseURL(
 ): string {
   const overrideUrl =
     typeof localStorage !== "undefined"
-      ? localStorage.getItem("exchange-base-url")
+      ? localStorage.getItem("aml-base-url")
       : undefined;
   let result: string;
 
@@ -136,3 +146,22 @@ function getInitialBackendBaseURL(
     return canonicalizeBaseUrl(window.origin);
   }
 }
+
+const evictExchangeSwrCache: CacheEvictor<TalerExchangeCacheEviction> = {
+  async notifySuccess(op) {
+    switch (op) {
+      case TalerExchangeCacheEviction.MAKE_AML_DECISION: {
+        await revalidateAccountDecisions();
+        await revalidateAccountInformation();
+        return;
+      }
+      case TalerExchangeCacheEviction.CREATE_DESCISION:
+      case TalerExchangeCacheEviction.UPLOAD_KYC_FORM: {
+        return;
+      }
+      default: {
+        assertUnreachable(op);
+      }
+    }
+  },
+};
