@@ -21,21 +21,33 @@
 
 import * as tests from "@gnu-taler/web-util/testing";
 import { CasesUI as TestedComponent } from "./Cases.js";
-import { AmountString, TalerExchangeApi } from "@gnu-taler/taler-util";
 
 export default {
   title: "cases",
 };
 
 export const OneRow = tests.createExample(TestedComponent, {
-  filter: TalerExchangeApi.AmlState.normal,
-  onChangeFilter: () => null,
   records: [
     {
-      current_state: TalerExchangeApi.AmlState.normal,
+      // current_state: TalerExchangeApi.AmlState.normal,
       h_payto: "QWEQWEQWEQWE",
       rowid: 1,
-      threshold: "USD:1" as AmountString,
+      decision_time: {
+        t_s: "never"
+      },
+      is_active: false,
+      limits: {
+        custom_measures: {},
+        expiration_time: {
+          t_s: "never"
+        },
+        rules: [],
+        successor_measure: undefined,
+      },
+      to_investigate: false,
+      justification: undefined,
+      properties: undefined,
+      // threshold: "USD:1" as AmountString,
     },
   ],
 });
