@@ -137,12 +137,13 @@ export * from "./reducer-types.js";
 export * as validators from "./validators.js";
 export * from "./challenge-feedback-types.js";
 
-const httpLib = createPlatformHttpLib({
-  enableThrottling: false,
-});
-
 const logger = new Logger("anastasis-core:index.ts");
 
+const http = createPlatformHttpLib({
+  enableThrottling: true,
+  requireTls: false,
+});
+
 const ANASTASIS_HTTP_HEADER_POLICY_META_DATA = "Anastasis-Policy-Meta-Data";
 
 function getContinents(): ContinentInfo[] {
@@ -285,15 +286,20 @@ async function getProviderInfo(
   // FIXME: Use a reasonable timeout here.
   let resp: HttpResponse;
   try {
-    resp = await httpLib.fetch(new URL("config", providerBaseUrl).href);
+    resp = await http.fetch(new URL("config", providerBaseUrl).href);
   } catch (e) {
+    console.warn(
+      "Encountered an HTTP error whilst trying to get the provider's config: ",
+      e,
+    );
     return {
       status: "error",
       code: TalerErrorCode.ANASTASIS_REDUCER_NETWORK_FAILED,
-      hint: "request to provider failed",
+      hint: "request to anastasis provider failed",
     };
   }
-  if (resp.status !== 200) {
+  if (resp.status < 200 || resp.status > 299) {
+    console.warn("Got bad response code whilst getting provider config", resp);
     return {
       status: "error",
       code: TalerErrorCode.ANASTASIS_REDUCER_NETWORK_FAILED,
@@ -555,7 +561,7 @@ async function uploadSecret(
       // FIXME: Get this from the params
       reqUrl.searchParams.set("timeout_ms", "500");
     }
-    const resp = await httpLib.fetch(reqUrl.href, {
+    const resp = await http.fetch(reqUrl.href, {
       method: "POST",
       headers: {
         "content-type": "application/json",
@@ -645,11 +651,11 @@ async function uploadSecret(
       reqUrl.searchParams.set("timeout_ms", "500");
     }
     logger.info(`uploading policy to ${prov.provider_url}`);
-    const resp = await httpLib.fetch(reqUrl.href, {
+    const resp = await http.fetch(reqUrl.href, {
       method: "POST",
       headers: {
         "Anastasis-Policy-Signature": encodeCrock(sig),
-        "If-None-Match": encodeCrock(bodyHash),
+        "If-None-Match": JSON.stringify(encodeCrock(bodyHash)),
         [ANASTASIS_HTTP_HEADER_POLICY_META_DATA]: metadataEnc,
         ...(paySecret
           ? {
@@ -756,7 +762,7 @@ async function downloadPolicyFromProvider(
   const acctKeypair = accountKeypairDerive(userId);
   const reqUrl = new URL(`policy/${acctKeypair.pub}`, providerUrl);
   reqUrl.searchParams.set("version", `${version}`);
-  const resp = await httpLib.fetch(reqUrl.href);
+  const resp = await http.fetch(reqUrl.href);
   if (resp.status !== 200) {
     logger.info(
       `Could not download policy from provider ${providerUrl}, status ${resp.status}`,
@@ -983,7 +989,7 @@ async function requestTruth(
   let resp: HttpResponse;
 
   try {
-    resp = await httpLib.fetch(url.href, {
+    resp = await http.fetch(url.href, {
       method: "POST",
       headers: {
         Accept: "application/json",
@@ -1140,7 +1146,7 @@ async function selectChallenge(
   let resp: HttpResponse;
 
   try {
-    resp = await httpLib.fetch(url.href, {
+    resp = await http.fetch(url.href, {
       method: "POST",
       headers: {
         Accept: "application/json",
@@ -1858,7 +1864,7 @@ export async function discoverPolicies(
     );
     const acctKeypair = accountKeypairDerive(userId);
     const reqUrl = new URL(`policy/${acctKeypair.pub}/meta`, providerUrl);
-    const resp = await httpLib.fetch(reqUrl.href);
+    const resp = await http.fetch(reqUrl.href);
     if (resp.status !== 200) {
       logger.warn(`Could not fetch policy metadate from ${reqUrl.href}`);
       continue;
