@@ -22,6 +22,7 @@
 import { h, FunctionalComponent } from "preact";
 import { useState } from "preact/hooks";
 import { DurationPicker as TestedComponent } from "./DurationPicker.js";
+import * as tests from "@gnu-taler/web-util/testing";
 
 export default {
   component: TestedComponent,
@@ -31,16 +32,7 @@ export default {
   },
 };
 
-function createExample<Props>(
-  Component: FunctionalComponent<Props>,
-  props: Partial<Props>,
-) {
-  const r = (args: any) => <Component {...args} />;
-  r.args = props;
-  return r;
-}
-
-export const Example = createExample(TestedComponent, {
+export const Example = tests.createExample(TestedComponent, {
   days: true,
   minutes: true,
   hours: true,
