@@ -14,12 +14,11 @@
  GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
  */
 
+/**
+ * Imports.
+ */
 import { VNode } from "preact";
 
-export interface KeyValue {
-  [key: string]: string;
-}
-
 export interface Notification {
   message: string;
   description?: string | VNode;
