@@ -167,7 +167,7 @@ function PublicRounting({
         <WithdrawalOperationPage
           operationId={location.values.wopid}
           routeWithdrawalDetails={publicPages.operationDetails}
-          purpose="after-confirmation"
+          origin="from-wallet-ui"
           onOperationAborted={() => navigateTo(publicPages.login.url({}))}
           routeClose={publicPages.login}
           onAuthorizationRequired={() =>
@@ -284,7 +284,7 @@ function PrivateRouting({
         <WithdrawalOperationPage
           operationId={location.values.wopid}
           routeWithdrawalDetails={privatePages.operationDetails}
-          purpose="after-confirmation"
+          origin="from-wallet-ui"
           onOperationAborted={() => navigateTo(privatePages.home.url({}))}
           routeClose={privatePages.home}
           onAuthorizationRequired={() =>
@@ -298,7 +298,7 @@ function PrivateRouting({
         <WithdrawalOperationPage
           operationId={location.values.wopid}
           routeWithdrawalDetails={privatePages.operationDetails}
-          purpose="after-creation"
+          origin="from-bank-ui"
           onOperationAborted={() => navigateTo(privatePages.home.url({}))}
           routeClose={privatePages.home}
           onAuthorizationRequired={() =>
