@@ -725,11 +725,9 @@ export function InputAmount(
     currency,
     name,
     value,
-    error,
     left,
     onChange,
   }: {
-    error?: string;
     currency: string;
     name: string;
     left?: boolean | undefined;
@@ -777,11 +775,15 @@ export function InputAmount(
           }}
         />
       </div>
-      <ShowInputErrorLabel message={error} isDirty={value !== undefined} />
     </div>
   );
 }
 
+/**
+ * send to web-utils
+ * @param param0
+ * @returns
+ */
 export function RenderAmount({
   value,
   spec,
