@@ -17,6 +17,7 @@
 import {
   AbsoluteTime,
   AmountJson,
+  Amounts,
   HttpStatusCode,
   PaytoUri,
   PaytoUriIBAN,
@@ -51,7 +52,7 @@ interface Props {
     account: PaytoUri;
     reserve: string;
     username: string;
-    amount: AmountJson;
+    amount?: AmountJson;
   };
   onAuthorizationRequired: () => void;
 }
@@ -79,6 +80,11 @@ export function WithdrawalConfirmationQuestion({
     lib: { bank: api },
   } = useBankCoreApiContext();
 
+  const wireFee =
+    config.wire_transfer_fees === undefined
+      ? Amounts.zeroOfCurrency(config.currency)
+      : Amounts.parseOrThrow(config.wire_transfer_fees);
+
   async function doTransfer() {
     await handleError(async () => {
       if (!creds) return;
@@ -351,12 +357,35 @@ export function WithdrawalConfirmationQuestion({
                               <i18n.Translate>Amount</i18n.Translate>
                             </dt>
                             <dd class="mt-1 text-sm leading-6 text-gray-700 sm:col-span-2 sm:mt-0">
-                              <RenderAmount
-                                value={details.amount}
-                                spec={config.currency_specification}
-                              />
+                              {details.amount !== undefined ? (
+                                <RenderAmount
+                                  value={details.amount}
+                                  spec={config.currency_specification}
+                                />
+                              ) : (
+                                <i18n.Translate>
+                                  No amount specified yet.
+                                </i18n.Translate>
+                              )}
                             </dd>
                           </div>
+                          {Amounts.isZero(wireFee) ? undefined : (
+                            <Fragment>
+                              <div class="px-4 py-2 sm:grid sm:grid-cols-3 sm:gap-4 sm:px-0">
+                                <dt class="text-sm font-medium leading-6 text-gray-900">
+                                  <i18n.Translate>Cost</i18n.Translate>
+                                </dt>
+                                <dd class="mt-1 text-sm leading-6 text-gray-700 sm:col-span-2 sm:mt-0">
+                                  <RenderAmount
+                                    value={wireFee}
+                                    negative
+                                    withColor
+                                    spec={config.currency_specification}
+                                  />
+                                </dd>
+                              </div>
+                            </Fragment>
+                          )}
                         </dl>
                       </div>
                     </div>
