@@ -15,13 +15,14 @@
  */
 import {
   AbsoluteTime,
+  AccountLetter,
   HttpStatusCode,
   TalerCorebankApi,
   TalerError,
   TalerErrorCode,
   TranslatedString,
   assertUnreachable,
-  parsePaytoUri
+  parsePaytoUri,
 } from "@gnu-taler/taler-util";
 import {
   CopyButton,
@@ -42,6 +43,7 @@ import { useSessionState } from "../../hooks/session.js";
 import { LoginForm } from "../LoginForm.js";
 import { ProfileNavigation } from "../ProfileNavigation.js";
 import { AccountForm } from "../admin/AccountForm.js";
+import { usePreferences } from "../../hooks/preferences.js";
 
 export function ShowAccountDetails({
   account,
@@ -67,6 +69,7 @@ export function ShowAccountDetails({
   account: string;
 }): VNode {
   const { i18n } = useTranslationContext();
+  const [preferences] = usePreferences();
   const { state: credentials } = useSessionState();
   const creds = credentials.status !== "loggedIn" ? undefined : credentials;
   const {
@@ -88,7 +91,12 @@ export function ShowAccountDetails({
     return <Loading />;
   }
   if (result instanceof TalerError) {
-    return <ErrorLoadingWithDebug error={result} />;
+    return (
+      <Fragment>
+        <ErrorLoadingWithDebug error={result} />
+        <LoginForm currentUser={account} />
+      </Fragment>
+    );
   }
   if (result.type === "fail") {
     switch (result.case) {
@@ -183,6 +191,15 @@ export function ShowAccountDetails({
               when: AbsoluteTime.now(),
             });
           }
+          case TalerErrorCode.BANK_NON_ADMIN_SET_MIN_CASHOUT: {
+            return notify({
+              type: "error",
+              title: i18n.str`Only the administrator can change the minimum cashout limit.`,
+              description: resp.detail.hint as TranslatedString,
+              debug: resp.detail,
+              when: AbsoluteTime.now(),
+            });
+          }
           default:
             assertUnreachable(resp);
         }
@@ -191,28 +208,19 @@ export function ShowAccountDetails({
   }
 
   const url = bank.getRevenueAPI(account);
-  url.username = account;
   const baseURL = url.href;
-
+  const revenueURL = new URL(baseURL);
+  revenueURL.username = account;
+  revenueURL.password;
   const ac = parsePaytoUri(result.body.payto_uri);
   const payto = !ac?.isKnown ? undefined : ac;
-  let accountLetter: string | undefined = undefined;
-  if (payto) {
-    switch (payto.targetType) {
-      case "iban": {
-        accountLetter = `account-info-url=${url.href}\naccount-type=${payto.targetType}\niban=${payto.iban}\nreceiver-name=${result.body.name}\n`;
-        break;
-      }
-      case "x-taler-bank": {
-        accountLetter = `account-info-url=${url.href}\naccount-type=${payto.targetType}\naccount=${payto.account}\nhost=${payto.host}\nreceiver-name=${result.body.name}\n`;
-        break;
-      }
-      case "bitcoin": {
-        accountLetter = `account-info-url=${url.href}\naccount-type=${payto.targetType}\naddress=${payto.address}\nreceiver-name=${result.body.name}\n`;
-        break;
-      }
-    }
-  }
+  const accountLetter: AccountLetter | undefined = !payto
+    ? undefined
+    : {
+        accountURI: result.body.payto_uri,
+        infoURL: revenueURL.href,
+        accountToken: creds?.token,
+      };
 
   return (
     <Fragment>
@@ -318,7 +326,7 @@ export function ShowAccountDetails({
                       name="account-type"
                       id="account-type"
                       disabled={true}
-                      value={account}
+                      value={payto.targetType}
                       autocomplete="off"
                     />
                   </div>
@@ -340,15 +348,21 @@ export function ShowAccountDetails({
                             {i18n.str`IBAN`}
                           </label>
                           <div class="mt-2">
-                            <input
-                              type="text"
-                              class="block w-full disabled:bg-gray-100 rounded-md border-0 py-1.5 text-gray-900 shadow-sm ring-1 ring-inset ring-gray-300 data-[error=true]:ring-red-500 placeholder:text-gray-400 focus:ring-2 focus:ring-inset focus:ring-indigo-600 sm:text-sm sm:leading-6"
-                              name="iban"
-                              id="iban"
-                              disabled={true}
-                              value={payto.iban}
-                              autocomplete="off"
-                            />
+                            <div class="flex justify-between">
+                              <input
+                                type="text"
+                                class="block w-full disabled:bg-gray-100 rounded-md border-0 py-1.5 text-gray-900 shadow-sm ring-1 ring-inset ring-gray-300 data-[error=true]:ring-red-500 placeholder:text-gray-400 focus:ring-2 focus:ring-inset focus:ring-indigo-600 sm:text-sm sm:leading-6"
+                                name="iban"
+                                id="iban"
+                                disabled={true}
+                                value={payto.iban}
+                                autocomplete="off"
+                              />
+                              <CopyButton
+                                class="p-2 rounded-full  text-black shadow-sm  focus-visible:outline focus-visible:outline-2 focus-visible:outline-offset-2 "
+                                getContent={() => payto.iban}
+                              />
+                            </div>
                           </div>
                           <p class="mt-2 text-sm text-gray-500">
                             <i18n.Translate>
@@ -360,30 +374,70 @@ export function ShowAccountDetails({
                     }
                     case "x-taler-bank": {
                       return (
-                        <div class="sm:col-span-5">
-                          <label
-                            class="block text-sm font-medium leading-6 text-gray-900"
-                            for="iban"
-                          >
-                            {i18n.str`IBAN`}
-                          </label>
-                          <div class="mt-2">
-                            <input
-                              type="text"
-                              class="block w-full disabled:bg-gray-100 rounded-md border-0 py-1.5 text-gray-900 shadow-sm ring-1 ring-inset ring-gray-300 data-[error=true]:ring-red-500 placeholder:text-gray-400 focus:ring-2 focus:ring-inset focus:ring-indigo-600 sm:text-sm sm:leading-6"
-                              name="iban"
-                              id="iban"
-                              disabled={true}
-                              value={payto.account}
-                              autocomplete="off"
-                            />
+                        <Fragment>
+                          <div class="sm:col-span-5">
+                            <label
+                              class="block text-sm font-medium leading-6 text-gray-900"
+                              for="account-host"
+                            >
+                              {i18n.str`Account name`}
+                            </label>
+                            <div class="mt-2">
+                              <div class="flex justify-between">
+                                <input
+                                  type="text"
+                                  class="block w-full disabled:bg-gray-100 rounded-md border-0 py-1.5 text-gray-900 shadow-sm ring-1 ring-inset ring-gray-300 data-[error=true]:ring-red-500 placeholder:text-gray-400 focus:ring-2 focus:ring-inset focus:ring-indigo-600 sm:text-sm sm:leading-6"
+                                  name="account-host"
+                                  id="account-host"
+                                  disabled={true}
+                                  value={payto.host}
+                                  autocomplete="off"
+                                />
+                              </div>
+                              <CopyButton
+                                class="p-2 rounded-full  text-black shadow-sm  focus-visible:outline focus-visible:outline-2 focus-visible:outline-offset-2 "
+                                getContent={() => payto.host}
+                              />
+                            </div>
+
+                            <p class="mt-2 text-sm text-gray-500">
+                              <i18n.Translate>
+                                Bank host where the service is located.
+                              </i18n.Translate>
+                            </p>
                           </div>
-                          <p class="mt-2 text-sm text-gray-500">
-                            <i18n.Translate>
-                              International Bank Account Number.
-                            </i18n.Translate>
-                          </p>
-                        </div>
+                          <div class="sm:col-span-5">
+                            <label
+                              class="block text-sm font-medium leading-6 text-gray-900"
+                              for="account-name"
+                            >
+                              {i18n.str`Account name`}
+                            </label>
+                            <div class="mt-2">
+                              <div class="flex justify-between">
+                                <input
+                                  type="text"
+                                  class="block w-full disabled:bg-gray-100 rounded-md border-0 py-1.5 text-gray-900 shadow-sm ring-1 ring-inset ring-gray-300 data-[error=true]:ring-red-500 placeholder:text-gray-400 focus:ring-2 focus:ring-inset focus:ring-indigo-600 sm:text-sm sm:leading-6"
+                                  name="account-name"
+                                  id="account-name"
+                                  disabled={true}
+                                  value={payto.account}
+                                  autocomplete="off"
+                                />
+                              </div>
+                              <CopyButton
+                                class="p-2 rounded-full  text-black shadow-sm  focus-visible:outline focus-visible:outline-2 focus-visible:outline-offset-2 "
+                                getContent={() => payto.account}
+                              />
+                            </div>
+
+                            <p class="mt-2 text-sm text-gray-500">
+                              <i18n.Translate>
+                                Bank account identifier for wire transfers.
+                              </i18n.Translate>
+                            </p>
+                          </div>
+                        </Fragment>
                       );
                     }
                     case "bitcoin": {
@@ -402,9 +456,13 @@ export function ShowAccountDetails({
                               name="iban"
                               id="iban"
                               disabled={true}
-                              value={"DE1231231231"}
+                              value={"asd"}
                               autocomplete="off"
                             />
+                            <CopyButton
+                              class="p-2 rounded-full  text-black shadow-sm  focus-visible:outline focus-visible:outline-2 focus-visible:outline-offset-2 "
+                              getContent={() => "Asd"}
+                            />
                           </div>
                           <p class="mt-2 text-sm text-gray-500">
                             <i18n.Translate>
@@ -425,15 +483,21 @@ export function ShowAccountDetails({
                     {i18n.str`Owner's name`}
                   </label>
                   <div class="mt-2">
-                    <input
-                      type="text"
-                      class="block w-full disabled:bg-gray-100 rounded-md border-0 py-1.5 text-gray-900 shadow-sm ring-1 ring-inset ring-gray-300 data-[error=true]:ring-red-500 placeholder:text-gray-400 focus:ring-2 focus:ring-inset focus:ring-indigo-600 sm:text-sm sm:leading-6"
-                      name="iban"
-                      id="iban"
-                      disabled={true}
-                      value={result.body.name}
-                      autocomplete="off"
-                    />
+                    <div class="flex justify-between">
+                      <input
+                        type="text"
+                        class="block w-full disabled:bg-gray-100 rounded-md border-0 py-1.5 text-gray-900 shadow-sm ring-1 ring-inset ring-gray-300 data-[error=true]:ring-red-500 placeholder:text-gray-400 focus:ring-2 focus:ring-inset focus:ring-indigo-600 sm:text-sm sm:leading-6"
+                        name="iban"
+                        id="iban"
+                        disabled={true}
+                        value={result.body.name}
+                        autocomplete="off"
+                      />
+                      <CopyButton
+                        class="p-2 rounded-full  text-black shadow-sm  focus-visible:outline focus-visible:outline-2 focus-visible:outline-offset-2 "
+                        getContent={() => result.body.name}
+                      />
+                    </div>
                   </div>
                   <p class="mt-2 text-sm text-gray-500">
                     <i18n.Translate>
@@ -449,15 +513,21 @@ export function ShowAccountDetails({
                     {i18n.str`Account info URL`}
                   </label>
                   <div class="mt-2">
-                    <input
-                      type="text"
-                      class="block w-full disabled:bg-gray-100 rounded-md border-0 py-1.5 text-gray-900 shadow-sm ring-1 ring-inset ring-gray-300 data-[error=true]:ring-red-500 placeholder:text-gray-400 focus:ring-2 focus:ring-inset focus:ring-indigo-600 sm:text-sm sm:leading-6"
-                      name="iban"
-                      id="iban"
-                      disabled={true}
-                      value={baseURL}
-                      autocomplete="off"
-                    />
+                    <div class="flex justify-between">
+                      <input
+                        type="text"
+                        class="block w-full disabled:bg-gray-100 rounded-md border-0 py-1.5 text-gray-900 shadow-sm ring-1 ring-inset ring-gray-300 data-[error=true]:ring-red-500 placeholder:text-gray-400 focus:ring-2 focus:ring-inset focus:ring-indigo-600 sm:text-sm sm:leading-6"
+                        name="iban"
+                        id="iban"
+                        disabled={true}
+                        value={baseURL}
+                        autocomplete="off"
+                      />
+                      <CopyButton
+                        class="p-2 rounded-full  text-black shadow-sm  focus-visible:outline focus-visible:outline-2 focus-visible:outline-offset-2 "
+                        getContent={() => baseURL}
+                      />
+                    </div>
                   </div>
                   <p class="mt-2 text-sm text-gray-500">
                     <i18n.Translate>
@@ -476,12 +546,20 @@ export function ShowAccountDetails({
               >
                 <i18n.Translate>Cancel</i18n.Translate>
               </a>
-              <CopyButton
-                getContent={() => accountLetter ?? ""}
-                class="flex text-center disabled:opacity-50 disabled:cursor-default cursor-pointer rounded-md bg-indigo-600 px-3 py-2 text-sm font-semibold text-white shadow-sm hover:bg-indigo-500 focus-visible:outline focus-visible:outline-2 focus-visible:outline-offset-2 focus-visible:outline-indigo-600"
-              >
-                <i18n.Translate>Copy</i18n.Translate>
-              </CopyButton>
+              <span></span>
+
+              {!preferences.showCopyAccount ? (
+                <span />
+              ) : (
+                <CopyButton
+                  getContent={() =>
+                    !accountLetter ? "" : JSON.stringify(accountLetter)
+                  }
+                  class="flex text-center disabled:opacity-50 disabled:cursor-default cursor-pointer rounded-md bg-indigo-600 px-3 py-2 text-sm font-semibold text-white shadow-sm hover:bg-indigo-500 focus-visible:outline focus-visible:outline-2 focus-visible:outline-offset-2 focus-visible:outline-indigo-600"
+                >
+                  <i18n.Translate>Copy</i18n.Translate>
+                </CopyButton>
+              )}
             </div>
           </div>
         )}
