@@ -276,10 +276,9 @@ function Metrics({
           name="tabs"
           class="block w-full rounded-md border-gray-300 focus:border-indigo-500 focus:ring-indigo-500"
           onChange={(e) => {
-            // const op = e.currentTarget.value as typeof metricType
             setMetricType(
-              e.currentTarget
-                .value as unknown as TalerCorebankApi.MonitorTimeframeParam,
+              parseInt(e.currentTarget
+                .value, 10) as TalerCorebankApi.MonitorTimeframeParam,
             );
           }}
         >
