@@ -29,15 +29,16 @@ import {
   Attention,
   Loading,
   LocalNotificationBanner,
+  RouteDefinition,
   ShowInputErrorLabel,
   notifyInfo,
+  useBankCoreApiContext,
   useLocalNotification,
   useTranslationContext,
 } from "@gnu-taler/web-util/browser";
 import { Fragment, VNode, h } from "preact";
 import { useEffect, useState } from "preact/hooks";
 import { ErrorLoadingWithDebug } from "../../components/ErrorLoadingWithDebug.js";
-import { useBankCoreApiContext } from "@gnu-taler/web-util/browser";
 import { useAccountDetails } from "../../hooks/account.js";
 import { useBankState } from "../../hooks/bank-state.js";
 import {
@@ -46,7 +47,6 @@ import {
   useConversionInfo,
 } from "../../hooks/regional.js";
 import { useSessionState } from "../../hooks/session.js";
-import { RouteDefinition } from "@gnu-taler/web-util/browser";
 import { TanChannel, undefinedIfEmpty } from "../../utils.js";
 import { LoginForm } from "../LoginForm.js";
 import {
@@ -182,7 +182,10 @@ export function CreateCashout({
     balanceIsDebit:
       resultAccount.body.balance.credit_debit_indicator == "debit",
     debitThreshold: Amounts.parseOrThrow(resultAccount.body.debit_threshold),
-    minCashout: resultAccount.body.min_cashout === undefined ? regionalZero : Amounts.parseOrThrow(resultAccount.body.min_cashout)
+    minCashout:
+      resultAccount.body.min_cashout === undefined
+        ? regionalZero
+        : Amounts.parseOrThrow(resultAccount.body.min_cashout),
   };
 
   const limit = account.balanceIsDebit
@@ -245,25 +248,24 @@ export function CreateCashout({
         : Amounts.cmp(limit, calc.debit) === -1
           ? i18n.str`Balance is not enough`
           : calculationResult === "amount-is-too-small"
-              ? i18n.str`Amount needs to be higher`
-          : Amounts.cmp(calc.debit, conversionInfo.cashout_min_amount) < 0
-            ? i18n.str`No account can't cashout less than ${
-                Amounts.stringifyValueWithSpec(
-                  Amounts.parseOrThrow(conversionInfo.cashout_min_amount),
-                  regional_currency_specification,
-                ).normal
-              }`
-              : Amounts.cmp(calc.debit, account.minCashout) < 0
-              ? i18n.str`Your account can't cashout less than ${
+            ? i18n.str`Amount needs to be higher`
+            : Amounts.cmp(calc.debit, conversionInfo.cashout_min_amount) < 0
+              ? i18n.str`No account can't cashout less than ${
                   Amounts.stringifyValueWithSpec(
-                    Amounts.parseOrThrow(account.minCashout),
+                    Amounts.parseOrThrow(conversionInfo.cashout_min_amount),
                     regional_currency_specification,
                   ).normal
                 }`
-              
-              : Amounts.isZero(calc.credit)
-                ? i18n.str`The total transfer at destination will be zero`
-                : undefined,
+              : Amounts.cmp(calc.debit, account.minCashout) < 0
+                ? i18n.str`Your account can't cashout less than ${
+                    Amounts.stringifyValueWithSpec(
+                      Amounts.parseOrThrow(account.minCashout),
+                      regional_currency_specification,
+                    ).normal
+                  }`
+                : Amounts.isZero(calc.credit)
+                  ? i18n.str`The total transfer at destination will be zero`
+                  : undefined,
   });
   const trimmedAmountStr = form.amount?.trim();
 
@@ -364,7 +366,7 @@ export function CreateCashout({
     });
   }
   const cashoutDisabled =
-    config.supported_tan_channels.length < 1 ||
+    (config.supported_tan_channels ?? []).length < 1 ||
     !resultAccount.body.cashout_payto_uri;
 
   const cashoutAccount = !resultAccount.body.cashout_payto_uri
