@@ -992,6 +992,11 @@ export class SqliteBackend implements Backend {
       object_store_id: objectStoreId,
       name: indexName,
     });
+    if (!idxInfo) {
+      throw Error(
+        `index ${indexName} on object store ${objectStoreName} not found`,
+      );
+    }
     const indexUnique = expectDbNumber(idxInfo, "unique_index");
     const indexMultiEntry = expectDbNumber(idxInfo, "multientry");
     const indexKeyPath = deserializeKeyPath(
