@@ -52,6 +52,8 @@ function Head({ order_summary }: { order_summary?: string }): VNode {
   return <Fragment>
     <meta charSet="UTF-8" />
     <meta name="viewport" content="width=device-width, initial-scale=1.0" />
+    <meta name="taler-support" content="uri" />
+    <meta name="taler-uri" content="{{ taler_refund_uri }}"></meta>
     <noscript>
       <meta http-equiv="refresh" content="1" />
     </noscript>
@@ -61,6 +63,8 @@ function Head({ order_summary }: { order_summary?: string }): VNode {
 
 export function OfferRefund({ refundURI, qr_code, order_status_url }: Props): VNode {
   useEffect(() => {
+    const longpollDelayMs = 60 * 1000;
+    const delayMs = 500;
     let checkUrl: URL;
     try {
       checkUrl = new URL(order_status_url ? order_status_url : "{{& order_status_url }}");
@@ -68,7 +72,7 @@ export function OfferRefund({ refundURI, qr_code, order_status_url }: Props): VN
       return;
     }
     checkUrl.searchParams.set("await_refund_obtained", "yes");
-    const delayMs = 500;
+    checkUrl.searchParams.set("timeout_ms", longpollDelayMs.toString());
     function check() {
       let retried = false;
       function retryOnce() {
