@@ -55,6 +55,8 @@ function Head({ order_summary }: { order_summary?: string }): VNode {
     <Fragment>
       <meta charSet="UTF-8" />
       <meta name="viewport" content="width=device-width, initial-scale=1.0" />
+      <meta name="taler-support" content="uri" />
+      <meta name="taler-uri" content="{{ taler_pay_uri }}"></meta>
       <noscript>
         <meta http-equiv="refresh" content="1" />
       </noscript>
@@ -81,7 +83,8 @@ export function RequestPayment({
     } catch (e) {
       return;
     }
-    checkUrl.searchParams.set("timeout_s", longpollDelayMs.toString());
+    checkUrl.searchParams.set("timeout_ms", longpollDelayMs.toString());
+    const delayMs = 500;
     function check() {
       let retried = false;
       function retryOnce() {
@@ -98,6 +101,8 @@ export function RequestPayment({
               const resp = JSON.parse(req.responseText);
               if (resp.fulfillment_url) {
                 window.location.replace(resp.fulfillment_url);
+              } else {
+                window.location.reload()
               }
             } catch (e) {
               console.error("could not parse response:", e);
@@ -108,6 +113,8 @@ export function RequestPayment({
               const resp = JSON.parse(req.responseText);
               if (resp.fulfillment_url) {
                 window.location.replace(resp.fulfillment_url);
+              } else {
+                window.location.reload()
               }
             } catch (e) {
               console.error("could not parse response:", e);
@@ -123,20 +130,20 @@ export function RequestPayment({
               console.error("could not parse response:", e);
             }
           }
-          setTimeout(retryOnce, 500);
+          setTimeout(retryOnce, delayMs);
         }
       };
       req.onerror = function () {
-        setTimeout(retryOnce, 500);
+        setTimeout(retryOnce, delayMs);
       };
       req.ontimeout = function () {
-        setTimeout(retryOnce, 500);
+        setTimeout(retryOnce, delayMs);
       };
       req.timeout = longpollDelayMs;
       req.open("GET", checkUrl.href);
       req.send();
     }
-    setTimeout(check, 500);
+    setTimeout(check, delayMs);
   });
   return (
     <Page>
