@@ -83,6 +83,7 @@ import {
   buildDefaultBackendBaseURL,
   fetchSettings,
 } from "./settings.js";
+import { revalidateInstanceCategories } from "./hooks/category.js";
 const WITH_LOCAL_STORAGE_CACHE = false;
 
 export function Application(): VNode {
@@ -280,19 +281,38 @@ const swrCacheEvictor = new (class
         await Promise.all([revalidateInstanceBankAccounts()]);
         return;
       }
+      case TalerMerchantInstanceCacheEviction.CREATE_CATEGORY: {
+        await Promise.all([revalidateInstanceCategories()]);
+        return;
+      }
+      case TalerMerchantInstanceCacheEviction.UPDATE_CATEGORY: {
+        await Promise.all([revalidateInstanceCategories()]);
+        return;
+      }
+      case TalerMerchantInstanceCacheEviction.DELETE_CATEGORY: {
+        await Promise.all([revalidateInstanceCategories()]);
+        return;
+      }
       case TalerMerchantInstanceCacheEviction.CREATE_PRODUCT: {
-        await Promise.all([revalidateInstanceProducts()]);
+        await Promise.all([
+          revalidateInstanceProducts(),
+          revalidateInstanceCategories(),
+        ]);
         return;
       }
       case TalerMerchantInstanceCacheEviction.UPDATE_PRODUCT: {
         await Promise.all([
           revalidateProductDetails(),
           revalidateInstanceProducts(),
+          revalidateInstanceCategories(),
         ]);
         return;
       }
       case TalerMerchantInstanceCacheEviction.DELETE_PRODUCT: {
-        await Promise.all([revalidateInstanceProducts()]);
+        await Promise.all([
+          revalidateInstanceProducts(),
+          revalidateInstanceCategories(),
+        ]);
         return;
       }
       case TalerMerchantInstanceCacheEviction.CREATE_TRANSFER: {
