@@ -75,6 +75,9 @@ import WebhookUpdatePage from "./paths/instance/webhooks/update/index.js";
 import { LoginPage } from "./paths/login/index.js";
 import { Settings } from "./paths/settings/index.js";
 import { Notification } from "./utils/types.js";
+import ListCategories from "./paths/instance/categories/list/index.js";
+import CreateCategory from "./paths/instance/categories/create/index.js";
+import UpdateCategory from "./paths/instance/categories/update/index.js";
 
 export enum InstancePaths {
   error = "/error",
@@ -85,6 +88,10 @@ export enum InstancePaths {
   bank_update = "/bank/:bid/update",
   bank_new = "/bank/new",
 
+  category_list = "/category",
+  category_update = "/category/:cid/update",
+  category_new = "/category/new",
+
   inventory_list = "/inventory",
   inventory_update = "/inventory/:pid/update",
   inventory_new = "/inventory/new",
@@ -298,6 +305,39 @@ export function Routing(_p: Props): VNode {
             route(InstancePaths.order_list);
           }}
         />
+        {/**
+         * Category pages
+         */}
+        <Route
+          path={InstancePaths.category_list}
+          component={ListCategories}
+          onCreate={() => {
+            route(InstancePaths.category_new);
+          }}
+          onSelect={(id: string) => {
+            route(InstancePaths.category_update.replace(":cid", id));
+          }}
+        />
+        <Route
+          path={InstancePaths.category_update}
+          component={UpdateCategory}
+          onConfirm={() => {
+            route(InstancePaths.category_list);
+          }}
+          onBack={() => {
+            route(InstancePaths.category_list);
+          }}
+        />
+        <Route
+          path={InstancePaths.category_new}
+          component={CreateCategory}
+          onConfirm={() => {
+            route(InstancePaths.category_list);
+          }}
+          onBack={() => {
+            route(InstancePaths.category_list);
+          }}
+        />
         {/**
          * Inventory pages
          */}
@@ -593,13 +633,9 @@ function AdminInstanceUpdatePage({
   id,
   ...rest
 }: { id: string } & InstanceUpdatePageProps): VNode {
-
   return (
     <Fragment>
-      <InstanceAdminUpdatePage
-        {...rest}
-        instanceId={id}
-      />
+      <InstanceAdminUpdatePage {...rest} instanceId={id} />
     </Fragment>
   );
 }
