@@ -52,7 +52,10 @@ export function InputDuration<T>({
 
   const { error, required, value: anyValue, onChange } = useField<T>(name);
   let strValue = "";
-  const value: Duration = anyValue;
+  const value: Duration =
+    anyValue.d_us !== undefined
+      ? Duration.fromTalerProtocolDuration(anyValue)
+      : anyValue;
   if (!value) {
     strValue = "";
   } else if (value.d_ms === "forever") {
