@@ -96,7 +96,7 @@ export function InputStock<T>({
               {!alreadyExist ? (
                 <button
                   class="button"
-                  data-tooltip={i18n.str`click here to configure the stock of the product, leave it as is and the backend will not control stock`}
+                  data-tooltip={i18n.str`Click here to configure the stock of the product, leave it as is and the backend will not control stock.`}
                   onClick={(): void => {
                     valueHandler({
                       current: 0,
@@ -112,7 +112,7 @@ export function InputStock<T>({
               ) : (
                 <button
                   class="button"
-                  data-tooltip={i18n.str`this product has been configured without stock control`}
+                  data-tooltip={i18n.str`This product has been configured without stock control`}
                   disabled
                 >
                   <span>
@@ -133,17 +133,11 @@ export function InputStock<T>({
   const stockAddedErrors: FormErrors<typeof addedStock> = {
     lost:
       currentStock + addedStock.incoming < addedStock.lost
-        ? i18n.str`lost cannot be greater than current and incoming (max ${currentStock + addedStock.incoming
+        ? i18n.str`Lost can't be greater than current and incoming (max ${currentStock + addedStock.incoming
           })`
         : undefined,
   };
 
-  // const stockUpdateDescription = stockAddedErrors.lost ? '' : (
-  //   !!addedStock.incoming || !!addedStock.lost ?
-  //     i18n.str`current stock will change from ${currentStock} to ${currentStock + addedStock.incoming - addedStock.lost}` :
-  //     i18n.str`current stock will stay at ${currentStock}`
-  // )
-
   return (
     <Fragment>
       <div class="card">
@@ -192,7 +186,7 @@ export function InputStock<T>({
                 side={
                   <button
                     class="button is-danger"
-                    data-tooltip={i18n.str`remove stock control for this product`}
+                    data-tooltip={i18n.str`Remove stock control for this product`}
                     onClick={(): void => {
                       valueHandler(undefined as any);
                     }}
