@@ -19,19 +19,22 @@
  * @author Sebastian Javier Marchano (sebasjm)
  */
 
-import { useTranslationContext } from "@gnu-taler/web-util/browser";
-import { ComponentChildren, Fragment, h, VNode } from "preact";
-import { useState } from "preact/hooks";
-import { DEFAULT_REQUEST_TIMEOUT } from "../../utils/constants.js";
-import { Spinner } from "../exception/loading.js";
-import { FormErrors, FormProvider } from "../form/FormProvider.js";
-import { Input } from "../form/Input.js";
-import { useSessionContext } from "../../context/session.js";
 import {
   AccountLetter,
   codecForAccountLetter,
   PaytoString,
+  PaytoUri,
+  stringifyPaytoUri,
 } from "@gnu-taler/taler-util";
+import { useTranslationContext } from "@gnu-taler/web-util/browser";
+import { ComponentChildren, Fragment, h, VNode } from "preact";
+import { useState } from "preact/hooks";
+import { useSessionContext } from "../../context/session.js";
+import { DEFAULT_REQUEST_TIMEOUT } from "../../utils/constants.js";
+import { undefinedIfEmpty } from "../../utils/table.js";
+import { Spinner } from "../exception/loading.js";
+import { FormErrors, FormProvider } from "../form/FormProvider.js";
+import { Input } from "../form/Input.js";
 
 interface Props {
   active?: boolean;
@@ -140,7 +143,13 @@ export function ContinueModal({
   );
 }
 
-export function SimpleModal({ onCancel, children }: any): VNode {
+export function SimpleModal({
+  onCancel,
+  children,
+}: {
+  onCancel: () => void;
+  children: ComponentChildren;
+}): VNode {
   return (
     <div class="modal is-active">
       <div class="modal-background " onClick={onCancel} />
@@ -230,15 +239,13 @@ export function ImportingAccountModal({
       parsed !== undefined ? codecForAccountLetter().decode(parsed) : undefined;
   } catch (e) {
     account = undefined;
-    if (e instanceof Error) {
-      parsingError = e.message;
-    }
+    parsingError = e instanceof Error ? e.message : String(e);
   }
   const errors: FormErrors<{ letter: string }> = {
     letter: !letter
-      ? i18n.str`required`
+      ? i18n.str`Required`
       : parsed === undefined
-        ? i18n.str`letter should be a JSON string`
+        ? i18n.str`Letter must be a JSON string`
         : account === undefined
           ? i18n.str`JSON string is invalid`
           : undefined,
@@ -288,6 +295,131 @@ export function ImportingAccountModal({
   );
 }
 
+interface CompareAccountsModalProps {
+  onCancel: () => void;
+  onConfirm: (account: PaytoString) => void;
+  formPayto: PaytoUri | undefined;
+  testPayto: PaytoUri;
+}
+
+function getHostFromHostPath(s: string | undefined) {
+  if (!s) return undefined;
+  try {
+    const u = new URL(`https://${s}`);
+    const endpath = u.pathname.lastIndexOf("/");
+    return u.origin + u.pathname.substring(0, endpath);
+  } catch (e) {
+    return undefined;
+  }
+}
+
+function getAccountIdFromHostPath(s: string | undefined) {
+  if (!s) return undefined;
+  try {
+    const u = new URL(`https://${s}`);
+    const endpath = u.pathname.lastIndexOf("/");
+    return u.pathname.substring(endpath + 1);
+  } catch (e) {
+    return undefined;
+  }
+}
+
+export function CompareAccountsModal({
+  onCancel,
+  onConfirm,
+  formPayto,
+  testPayto,
+}: CompareAccountsModalProps): VNode {
+  const { i18n } = useTranslationContext();
+  return (
+    <ConfirmModal
+      label={i18n.str`Correct form`}
+      description={i18n.str`Comparing account details`}
+      active
+      onCancel={onCancel}
+      onConfirm={() => onConfirm(stringifyPaytoUri(testPayto))}
+    >
+      <p>
+        <i18n.Translate>
+          Testing against the account info URL succeeded but the account
+          information reported is different with the account details form.
+        </i18n.Translate>
+      </p>
+      <div class="table-container">
+        <table class="table is-fullwidth is-striped is-hoverable is-fullwidth">
+          <thead>
+            <tr>
+              <td>
+                <i18n.Translate>Field</i18n.Translate>
+              </td>
+              <td>
+                <i18n.Translate>In the form</i18n.Translate>
+              </td>
+              <td>
+                <i18n.Translate>Reported</i18n.Translate>
+              </td>
+            </tr>
+          </thead>
+          <tbody>
+            <tr>
+              <td>
+                <i18n.Translate>Type</i18n.Translate>
+              </td>
+              <td>{formPayto?.targetType ?? "--"}</td>
+              <td>{testPayto.targetType}</td>
+            </tr>
+            {testPayto.targetType === "iban" && (
+              <tr>
+                <td>
+                  <i18n.Translate>IBAN</i18n.Translate>
+                </td>
+                <td>{formPayto?.targetPath ?? "--"}</td>
+                <td>{testPayto.targetPath}</td>
+              </tr>
+            )}
+            {testPayto.targetType === "bitcoin" && (
+              <tr>
+                <td>
+                  <i18n.Translate>Address</i18n.Translate>
+                </td>
+                <td>{formPayto?.targetPath ?? "--"}</td>
+                <td>{testPayto.targetPath}</td>
+              </tr>
+            )}
+            {testPayto.targetType === "x-taler-bank" && (
+              <Fragment>
+                <tr>
+                  <td>
+                    <i18n.Translate>Host</i18n.Translate>
+                  </td>
+                  <td>{getHostFromHostPath(formPayto?.targetPath) ?? "--"}</td>
+                  <td>{getHostFromHostPath(testPayto.targetPath)}</td>
+                </tr>
+                <tr>
+                  <td>
+                    <i18n.Translate>Account id</i18n.Translate>
+                  </td>
+                  <td>
+                    {getAccountIdFromHostPath(formPayto?.targetPath) ?? "--"}
+                  </td>
+                  <td>{getAccountIdFromHostPath(testPayto.targetPath)}</td>
+                </tr>
+              </Fragment>
+            )}
+            <tr>
+              <td>
+                <i18n.Translate>Owner's name</i18n.Translate>
+              </td>
+              <td>{formPayto?.params["receiver-name"] ?? "--"}</td>
+              <td>{testPayto.params["receiver-name"]}</td>
+            </tr>
+          </tbody>
+        </table>
+      </div>
+    </ConfirmModal>
+  );
+}
+
 interface DeleteModalProps {
   element: { id: string; name: string };
   onCancel: () => void;
@@ -299,6 +431,7 @@ export function DeleteModal({
   onCancel,
   onConfirm,
 }: DeleteModalProps): VNode {
+  const { i18n } = useTranslationContext();
   return (
     <ConfirmModal
       label={`Delete instance`}
@@ -309,17 +442,27 @@ export function DeleteModal({
       onConfirm={() => onConfirm(element.id)}
     >
       <p>
-        If you delete the instance named <b>&quot;{element.name}&quot;</b> (ID:{" "}
-        <b>{element.id}</b>), the merchant will no longer be able to process
-        orders or refunds
+        <i18n.Translate>
+          If you delete the instance named <b>&quot;{element.name}&quot;</b>{" "}
+          (ID: <b>{element.id}</b>), the merchant will no longer be able to
+          process orders or refunds
+        </i18n.Translate>
       </p>
       <p>
-        This action deletes the instance private key, but preserves all
-        transaction data. You can still access that data after deleting the
-        instance.
+        <i18n.Translate>
+          This action deletes the instance private key, but preserves all
+          transaction data. You can still access that data after deleting the
+          instance.
+        </i18n.Translate>
       </p>
       <p class="warning">
-        Deleting an instance <b>cannot be undone</b>.
+        <i18n.Translate>
+          Deleting an instance{" "}
+          <b>
+            <i18n.Translate>Can't be undone</i18n.Translate>
+          </b>
+          .
+        </i18n.Translate>
       </p>
     </ConfirmModal>
   );
@@ -330,6 +473,7 @@ export function PurgeModal({
   onCancel,
   onConfirm,
 }: DeleteModalProps): VNode {
+  const { i18n } = useTranslationContext();
   return (
     <ConfirmModal
       label={`Purge the instance`}
@@ -340,16 +484,26 @@ export function PurgeModal({
       onConfirm={() => onConfirm(element.id)}
     >
       <p>
-        If you purge the instance named <b>&quot;{element.name}&quot;</b> (ID:{" "}
-        <b>{element.id}</b>), you will also delete all it&apos;s transaction
-        data.
+        <i18n.Translate>
+          If you purge the instance named <b>&quot;{element.name}&quot;</b> (ID:{" "}
+          <b>{element.id}</b>), you will also delete all it&apos;s transaction
+          data.
+        </i18n.Translate>
       </p>
       <p>
-        The instance will disappear from your list, and you will no longer be
-        able to access it&apos;s data.
+        <i18n.Translate>
+          The instance will disappear from your list, and you will no longer be
+          able to access it&apos;s data.
+        </i18n.Translate>
       </p>
       <p class="warning">
-        Purging an instance <b>cannot be undone</b>.
+        <i18n.Translate>
+          Purging an instance{" "}
+          <b>
+            <i18n.Translate>Can't be undone</i18n.Translate>
+          </b>
+          .
+        </i18n.Translate>
       </p>
     </ConfirmModal>
   );
@@ -378,24 +532,22 @@ export function UpdateTokenModal({
   const { i18n } = useTranslationContext();
 
   const hasInputTheCorrectOldToken = oldToken && oldToken !== form.old_token;
-  const errors = {
+  const errors = undefinedIfEmpty({
     old_token: hasInputTheCorrectOldToken
-      ? i18n.str`is not the same as the current access token`
+      ? i18n.str`Is not the same as the current access token`
       : undefined,
     new_token: !form.new_token
-      ? i18n.str`cannot be empty`
+      ? i18n.str`Required`
       : form.new_token === form.old_token
-        ? i18n.str`cannot be the same as the old token`
+        ? i18n.str`Can't be the same as the old token`
         : undefined,
     repeat_token:
       form.new_token !== form.repeat_token
-        ? i18n.str`is not the same`
+        ? i18n.str`Is not the same`
         : undefined,
-  };
+  });
 
-  const hasErrors = Object.keys(errors).some(
-    (k) => (errors as any)[k] !== undefined,
-  );
+  const hasErrors = errors !== undefined;
 
   const { state } = useSessionContext();
 
@@ -416,20 +568,20 @@ export function UpdateTokenModal({
               <Input<State>
                 name="old_token"
                 label={i18n.str`Old access token`}
-                tooltip={i18n.str`access token currently in use`}
+                tooltip={i18n.str`Access token currently in use`}
                 inputType="password"
               />
             )}
             <Input<State>
               name="new_token"
               label={i18n.str`New access token`}
-              tooltip={i18n.str`next access token to be used`}
+              tooltip={i18n.str`Next access token to be used`}
               inputType="password"
             />
             <Input<State>
               name="repeat_token"
               label={i18n.str`Repeat access token`}
-              tooltip={i18n.str`confirm the same access token`}
+              tooltip={i18n.str`Confirm the same access token`}
               inputType="password"
             />
           </FormProvider>
@@ -457,21 +609,19 @@ export function SetTokenNewInstanceModal({
   });
   const { i18n } = useTranslationContext();
 
-  const errors = {
+  const errors = undefinedIfEmpty({
     new_token: !form.new_token
-      ? i18n.str`cannot be empty`
+      ? i18n.str`Required`
       : form.new_token === form.old_token
-        ? i18n.str`cannot be the same as the old access token`
+        ? i18n.str`Can't be the same as the old access token`
         : undefined,
     repeat_token:
       form.new_token !== form.repeat_token
-        ? i18n.str`is not the same`
+        ? i18n.str`Is not the same`
         : undefined,
-  };
+  });
 
-  const hasErrors = Object.keys(errors).some(
-    (k) => (errors as any)[k] !== undefined,
-  );
+  const hasErrors = errors !== undefined;
 
   return (
     <div class="modal is-active">
@@ -493,13 +643,13 @@ export function SetTokenNewInstanceModal({
                 <Input<State>
                   name="new_token"
                   label={i18n.str`New access token`}
-                  tooltip={i18n.str`next access token to be used`}
+                  tooltip={i18n.str`Next access token to be used`}
                   inputType="password"
                 />
                 <Input<State>
                   name="repeat_token"
                   label={i18n.str`Repeat access token`}
-                  tooltip={i18n.str`confirm the same access token`}
+                  tooltip={i18n.str`Confirm the same access token`}
                   inputType="password"
                 />
               </FormProvider>
