@@ -1,6 +1,6 @@
 /*
  This file is part of GNU Taler
- (C) 2021-2023 Taler Systems S.A.
+ (C) 2021-2024 Taler Systems S.A.
 
  GNU Taler is free software; you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
@@ -42,7 +42,7 @@ export function DurationPicker({
   onChange,
   value,
 }: Props): VNode {
-  const ss = 1000 * 1000;
+  const ss = 1000;
   const ms = ss * 60;
   const hs = ms * 60;
   const ds = hs * 24;
