@@ -34,19 +34,19 @@ export function ProductList({ list, actions = [] }: Props): VNode {
         <thead>
           <tr>
             <th>
-              <i18n.Translate>image</i18n.Translate>
+              <i18n.Translate>Image</i18n.Translate>
             </th>
             <th>
-              <i18n.Translate>description</i18n.Translate>
+              <i18n.Translate>Description</i18n.Translate>
             </th>
             <th>
-              <i18n.Translate>quantity</i18n.Translate>
+              <i18n.Translate>Quantity</i18n.Translate>
             </th>
             <th>
-              <i18n.Translate>unit price</i18n.Translate>
+              <i18n.Translate>Unit price</i18n.Translate>
             </th>
             <th>
-              <i18n.Translate>total price</i18n.Translate>
+              <i18n.Translate>Total price</i18n.Translate>
             </th>
             <th />
           </tr>
