@@ -13,36 +13,36 @@
  You should have received a copy of the GNU General Public License along with
  GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
  */
-import {
-  useMerchantApiContext
-} from "@gnu-taler/web-util/browser";
 
 // FIX default import https://github.com/microsoft/TypeScript/issues/49189
-import { AccessToken, TalerHttpError, TalerMerchantManagementResultByMethod } from "@gnu-taler/taler-util";
+import {
+  AccessToken,
+  TalerHttpError,
+  TalerMerchantManagementResultByMethod,
+} from "@gnu-taler/taler-util";
 import _useSWR, { SWRHook, mutate } from "swr";
 import { useSessionContext } from "../context/session.js";
 const useSWR = _useSWR as unknown as SWRHook;
 
-
 export function revalidateInstanceDetails() {
   return mutate(
-    (key) => Array.isArray(key) && key[key.length - 1] === "getCurrentInstanceDetails",
+    (key) =>
+      Array.isArray(key) && key[key.length - 1] === "getCurrentInstanceDetails",
     undefined,
     { revalidate: true },
   );
 }
 export function useInstanceDetails() {
-  const { state: session } = useSessionContext();
-  const { lib: { instance } } = useMerchantApiContext();
+  const { state, lib } = useSessionContext();
 
   async function fetcher([token]: [AccessToken]) {
-    return await instance.getCurrentInstanceDetails(token);
+    return await lib.instance.getCurrentInstanceDetails(token);
   }
 
   const { data, error } = useSWR<
     TalerMerchantManagementResultByMethod<"getCurrentInstanceDetails">,
     TalerHttpError
-  >([session.token, "getCurrentInstanceDetails"], fetcher);
+  >([state.token, "getCurrentInstanceDetails"], fetcher);
 
   if (data) return data;
   if (error) return error;
@@ -51,29 +51,28 @@ export function useInstanceDetails() {
 
 export function revalidateInstanceKYCDetails() {
   return mutate(
-    (key) => Array.isArray(key) && key[key.length - 1] === "getCurrentIntanceKycStatus",
+    (key) =>
+      Array.isArray(key) &&
+      key[key.length - 1] === "getCurrentIntanceKycStatus",
     undefined,
     { revalidate: true },
   );
 }
 export function useInstanceKYCDetails() {
-  const { state: session } = useSessionContext();
-  const { lib: { instance } } = useMerchantApiContext();
+  const { state, lib } = useSessionContext();
 
   async function fetcher([token]: [AccessToken]) {
-    return await instance.getCurrentIntanceKycStatus(token, {});
+    return await lib.instance.getCurrentIntanceKycStatus(token, {});
   }
 
   const { data, error } = useSWR<
     TalerMerchantManagementResultByMethod<"getCurrentIntanceKycStatus">,
     TalerHttpError
-  >([session.token, "getCurrentIntanceKycStatus"], fetcher);
+  >([state.token, "getCurrentIntanceKycStatus"], fetcher);
 
   if (data) return data;
   if (error) return error;
   return undefined;
-
-
 }
 
 export function revalidateManagedInstanceDetails() {
@@ -84,17 +83,16 @@ export function revalidateManagedInstanceDetails() {
   );
 }
 export function useManagedInstanceDetails(instanceId: string) {
-  const { state: session } = useSessionContext();
-  const { lib: { instance } } = useMerchantApiContext();
+  const { state, lib } = useSessionContext();
 
   async function fetcher([token, instanceId]: [AccessToken, string]) {
-    return await instance.getInstanceDetails(token, instanceId);
+    return await lib.instance.getInstanceDetails(token, instanceId);
   }
 
   const { data, error } = useSWR<
     TalerMerchantManagementResultByMethod<"getInstanceDetails">,
     TalerHttpError
-  >([session.token, instanceId, "getInstanceDetails"], fetcher);
+  >([state.token, instanceId, "getInstanceDetails"], fetcher);
 
   if (data) return data;
   if (error) return error;
@@ -109,17 +107,16 @@ export function revalidateBackendInstances() {
   );
 }
 export function useBackendInstances() {
-  const { state: session } = useSessionContext();
-  const { lib: { instance } } = useMerchantApiContext();
+  const { state, lib } = useSessionContext();
 
   async function fetcher([token]: [AccessToken]) {
-    return await instance.listInstances(token);
+    return await lib.instance.listInstances(token);
   }
 
   const { data, error } = useSWR<
     TalerMerchantManagementResultByMethod<"listInstances">,
     TalerHttpError
-  >([session.token, "listInstances"], fetcher);
+  >([state.token, "listInstances"], fetcher);
 
   if (data) return data;
   if (error) return error;
