@@ -32,13 +32,12 @@ export function revalidateInstanceWebhooks() {
   );
 }
 export function useInstanceWebhooks() {
-  const { state: session } = useSessionContext();
-  const { lib: { instance } } = useSessionContext();
+  const { state, lib } = useSessionContext();
 
   // const [offset, setOffset] = useState<string | undefined>();
 
   async function fetcher([token, _bid]: [AccessToken, string]) {
-    return await instance.listWebhooks(token, {
+    return await lib.instance.listWebhooks(token, {
       // limit: PAGINATED_LIST_REQUEST,
       // offset: bid,
       // order: "dec",
@@ -48,7 +47,7 @@ export function useInstanceWebhooks() {
   const { data, error } = useSWR<
     TalerMerchantManagementResultByMethod<"listWebhooks">,
     TalerHttpError
-  >([session.token, "offset", "listWebhooks"], fetcher);
+  >([state.token, "offset", "listWebhooks"], fetcher);
 
   if (error) return error;
   if (data === undefined) return undefined;
@@ -100,17 +99,16 @@ export function revalidateWebhookDetails() {
   );
 }
 export function useWebhookDetails(webhookId: string) {
-  const { state: session } = useSessionContext();
-  const { lib: { instance } } = useSessionContext();
+  const { state, lib } = useSessionContext();
 
   async function fetcher([hookId, token]: [string, AccessToken]) {
-    return await instance.getWebhookDetails(token, hookId);
+    return await lib.instance.getWebhookDetails(token, hookId);
   }
 
   const { data, error } = useSWR<
     TalerMerchantManagementResultByMethod<"getWebhookDetails">,
     TalerHttpError
-  >([webhookId, session.token, "getWebhookDetails"], fetcher);
+  >([webhookId, state.token, "getWebhookDetails"], fetcher);
 
   if (data) return data;
   if (error) return error;
