@@ -21,7 +21,6 @@
 
 import { TalerMerchantApi } from "@gnu-taler/taler-util";
 import {
-  useMerchantApiContext,
   useTranslationContext
 } from "@gnu-taler/web-util/browser";
 import { VNode, h } from "preact";
@@ -98,7 +97,7 @@ export function CardTable({
         <div class="card-header-icon" aria-label="more options">
           <span
             class="has-tooltip-left"
-            data-tooltip={i18n.str`add new instance`}
+            data-tooltip={i18n.str`Add new instance`}
           >
             <button class="button is-info" type="button" onClick={onCreate}>
               <span class="icon is-small">
@@ -152,8 +151,7 @@ function Table({
   onPurge,
 }: TableProps): VNode {
   const { i18n } = useTranslationContext();
-  const { lib } = useMerchantApiContext();
-  const { impersonate } = useSessionContext();
+  const { lib, impersonate } = useSessionContext();
   return (
     <div class="table-container">
       <table class="table is-fullwidth is-striped is-hoverable is-fullwidth">
@@ -203,15 +201,11 @@ function Table({
                 <td>
                   <a
                     href={`#/orders`}
-                    onClick={async (e) => {
-                      e.preventDefault();
+                    onClick={async (_e) => {
+                      // e.preventDefault();
                       const newInstanceApi = lib.subInstanceApi(i.id);
                       //not checking /config since this comes from instance list
-                      impersonate({
-                        instance: i.id,
-                        baseUrl: new URL(newInstanceApi.instance.baseUrl),
-                        token: undefined,
-                      });
+                      impersonate(new URL(newInstanceApi.instance.baseUrl));
                     }}
                   >
                     {i.id}
