@@ -19,11 +19,13 @@
  * @author Sebastian Javier Marchano (sebasjm)
  */
 
-import { HttpStatusCode, TalerError, TalerMerchantApi, assertUnreachable } from "@gnu-taler/taler-util";
 import {
-  useMerchantApiContext,
-  useTranslationContext
-} from "@gnu-taler/web-util/browser";
+  HttpStatusCode,
+  TalerError,
+  TalerMerchantApi,
+  assertUnreachable,
+} from "@gnu-taler/taler-util";
+import { useTranslationContext } from "@gnu-taler/web-util/browser";
 import { Fragment, VNode, h } from "preact";
 import { useState } from "preact/hooks";
 import { ErrorLoadingMerchant } from "../../../components/ErrorLoadingMerchant.js";
@@ -33,8 +35,8 @@ import { DeleteModal, PurgeModal } from "../../../components/modal/index.js";
 import { useSessionContext } from "../../../context/session.js";
 import { useBackendInstances } from "../../../hooks/instance.js";
 import { Notification } from "../../../utils/types.js";
-import { View } from "./View.js";
 import { LoginPage } from "../../login/index.js";
+import { View } from "./View.js";
 
 interface Props {
   onCreate: () => void;
@@ -42,31 +44,29 @@ interface Props {
   instances: TalerMerchantApi.Instance[];
 }
 
-export default function Instances({
-  onCreate,
-  onUpdate,
-}: Props): VNode {
+export default function Instances({ onCreate, onUpdate }: Props): VNode {
   const result = useBackendInstances();
-  const [deleting, setDeleting] =
-    useState<TalerMerchantApi.Instance | null>(null);
-  const [purging, setPurging] =
-    useState<TalerMerchantApi.Instance | null>(null);
+  const [deleting, setDeleting] = useState<TalerMerchantApi.Instance | null>(
+    null,
+  );
+  const [purging, setPurging] = useState<TalerMerchantApi.Instance | null>(
+    null,
+  );
   const [notif, setNotif] = useState<Notification | undefined>(undefined);
   const { i18n } = useTranslationContext();
-  const { lib } = useMerchantApiContext();
-  const { state } = useSessionContext();
+  const { state, lib } = useSessionContext();
 
-  if (!result) return <Loading />
+  if (!result) return <Loading />;
   if (result instanceof TalerError) {
-    return <ErrorLoadingMerchant error={result} />
+    return <ErrorLoadingMerchant error={result} />;
   }
   if (result.type === "fail") {
-    switch(result.case) {
+    switch (result.case) {
       case HttpStatusCode.Unauthorized: {
-        return <LoginPage />
+        return <LoginPage />;
       }
       default: {
-        assertUnreachable(result.case)
+        assertUnreachable(result.case);
       }
     }
   }
@@ -91,12 +91,22 @@ export default function Instances({
               return;
             }
             try {
-              await lib.instance.deleteInstance(state.token, deleting.id);
-              // pushNotification({message: 'delete_success', type: 'SUCCESS' })
-              setNotif({
-                message: i18n.str`Instance "${deleting.name}" (ID: ${deleting.id}) has been deleted`,
-                type: "SUCCESS",
-              });
+              const resp = await lib.instance.deleteInstance(
+                state.token,
+                deleting.id,
+              );
+              if (resp.type === "ok") {
+                setNotif({
+                  message: i18n.str`Instance "${deleting.name}" (ID: ${deleting.id}) has been deleted`,
+                  type: "SUCCESS",
+                });
+              } else {
+                setNotif({
+                  message: i18n.str`Failed to delete instance`,
+                  type: "ERROR",
+                  description: resp.detail.hint,
+                });
+              }
             } catch (error) {
               setNotif({
                 message: i18n.str`Failed to delete instance`,
@@ -118,11 +128,25 @@ export default function Instances({
               return;
             }
             try {
-              await lib.instance.deleteInstance(state.token, purging.id, { purge: true });
-              setNotif({
-                message: i18n.str`Instance '${purging.name}' (ID: ${purging.id}) has been disabled`,
-                type: "SUCCESS",
-              });
+              const resp = await lib.instance.deleteInstance(
+                state.token,
+                purging.id,
+                {
+                  purge: true,
+                },
+              );
+              if (resp.type === "ok") {
+                setNotif({
+                  message: i18n.str`Instance '${purging.name}' (ID: ${purging.id}) has been purge`,
+                  type: "SUCCESS",
+                });
+              } else {
+                setNotif({
+                  message: i18n.str`Failed to purge instance`,
+                  type: "ERROR",
+                  description: resp.detail.hint,
+                });
+              }
             } catch (error) {
               setNotif({
                 message: i18n.str`Failed to purge instance`,
