@@ -19,9 +19,17 @@
  * @author Sebastian Javier Marchano (sebasjm)
  */
 
-import { TalerMerchantApi } from "@gnu-taler/taler-util";
+import {
+  HttpStatusCode,
+  PaytoUri,
+  TalerError,
+  TalerMerchantApi,
+  TranslatedString,
+  assertUnreachable,
+  parsePaytoUri
+} from "@gnu-taler/taler-util";
 import { useTranslationContext } from "@gnu-taler/web-util/browser";
-import { Fragment, h, VNode } from "preact";
+import { Fragment, VNode, h } from "preact";
 import { useState } from "preact/hooks";
 import { AsyncButton } from "../../../../components/exception/AsyncButton.js";
 import {
@@ -31,11 +39,16 @@ import {
 import { Input } from "../../../../components/form/Input.js";
 import { InputPaytoForm } from "../../../../components/form/InputPaytoForm.js";
 import { InputSelector } from "../../../../components/form/InputSelector.js";
-import { ImportingAccountModal } from "../../../../components/modal/index.js";
+import { InputToggle } from "../../../../components/form/InputToggle.js";
+import {
+  CompareAccountsModal,
+  ImportingAccountModal,
+} from "../../../../components/modal/index.js";
 import { undefinedIfEmpty } from "../../../../utils/table.js";
 import { safeConvertURL } from "../update/UpdatePage.js";
+import { testRevenueAPI } from "./index.js";
 
-type Entity = TalerMerchantApi.AccountAddDetails & { repeatPassword: string };
+type Entity = TalerMerchantApi.AccountAddDetails & { verified?: boolean };
 
 interface Props {
   onCreate: (d: TalerMerchantApi.AccountAddDetails) => Promise<void>;
@@ -50,8 +63,16 @@ export function CreatePage({ onCreate, onBack }: Props): VNode {
   const [importing, setImporting] = useState(false);
   const [state, setState] = useState<Partial<Entity>>({});
   const facadeURL = safeConvertURL(state.credit_facade_url);
+
+  const [revenuePayto, setRevenuePayto] = useState<PaytoUri | undefined>(
+    // parsePaytoUri("payto://x-taler-bank/asd.com:1010/asd/pepe"),
+    undefined,
+  );
+  const [testError, setTestError] = useState<TranslatedString | undefined>(
+    undefined,
+  );
   const errors: FormErrors<Entity> = {
-    payto_uri: !state.payto_uri ? i18n.str`required` : undefined,
+    payto_uri: !state.payto_uri ? i18n.str`Required` : undefined,
 
     credit_facade_credentials: !state.credit_facade_credentials
       ? undefined
@@ -59,12 +80,12 @@ export function CreatePage({ onCreate, onBack }: Props): VNode {
           username:
             state.credit_facade_credentials.type === "basic" &&
             !state.credit_facade_credentials.username
-              ? i18n.str`required`
+              ? i18n.str`Required`
               : undefined,
           password:
             state.credit_facade_credentials.type === "basic" &&
             !state.credit_facade_credentials.password
-              ? i18n.str`required`
+              ? i18n.str`Required`
               : undefined,
         }),
     credit_facade_url: !state.credit_facade_url
@@ -72,19 +93,12 @@ export function CreatePage({ onCreate, onBack }: Props): VNode {
       : !facadeURL
         ? i18n.str`Invalid url`
         : !facadeURL.href.endsWith("/")
-          ? i18n.str`URL should end with a '/'`
+          ? i18n.str`URL must end with a '/'`
           : facadeURL.searchParams.size > 0
-            ? i18n.str`URL should not contain params`
+            ? i18n.str`URL must not contain params`
             : facadeURL.hash
-              ? i18n.str`URL should not hash param`
+              ? i18n.str`URL must not hash param`
               : undefined,
-    repeatPassword: !state.credit_facade_credentials
-      ? undefined
-      : state.credit_facade_credentials.type === "basic" &&
-          (!state.credit_facade_credentials.password ||
-            state.credit_facade_credentials.password !== state.repeatPassword)
-        ? i18n.str`is not the same`
-        : undefined,
   };
 
   const hasErrors = Object.keys(errors).some(
@@ -117,25 +131,73 @@ export function CreatePage({ onCreate, onBack }: Props): VNode {
       credit_facade_url,
     });
   };
-  return (
-    <div>
-      {importing && <ImportingAccountModal onCancel={()=> {setImporting(false)}} onConfirm={(ac) => {
-        state.payto_uri = ac.accountURI
-        const u = new URL(ac.infoURL)
-        u.password = ""
-        if (u.username || u.password) {
-          state.credit_facade_credentials = {
-            type: "basic",
-            password: u.password,
-            username: u.username,
+
+  async function testAccountInfo() {
+    const revenueAPI = !state.credit_facade_url
+      ? undefined
+      : new URL("./", state.credit_facade_url);
+
+    if (revenueAPI) {
+      const resp = await testRevenueAPI(
+        revenueAPI,
+        state.credit_facade_credentials,
+      );
+      if (resp instanceof TalerError) {
+        setTestError(i18n.str`The request to check the revenue API failed.`);
+        setState({
+          ...state,
+          verified: undefined,
+        });
+        return;
+      } else if (resp.type === "fail") {
+        switch (resp.case) {
+          case HttpStatusCode.BadRequest: {
+            setTestError(i18n.str`Server replied with "bad request".`);
+            setState({
+              ...state,
+              verified: undefined,
+            });
+            return;
+          }
+          case HttpStatusCode.Unauthorized: {
+            setTestError(i18n.str`Unauthorized, check credentials.`);
+            setState({
+              ...state,
+              verified: undefined,
+            });
+            return;
+          }
+          case HttpStatusCode.NotFound: {
+            setTestError(
+              i18n.str`The endpoint doesn't seems to be a Taler Revenue API.`,
+            );
+            setState({
+              ...state,
+              verified: undefined,
+            });
+            return;
+          }
+          default: {
+            assertUnreachable(resp);
           }
-          state.repeatPassword = u.password
         }
-        u.password = ""
-        u.username = ""
-        state.credit_facade_url = u.href;
-        setImporting(false)
-      }}  />}
+      } else {
+        const found = resp.body;
+        const match = state.payto_uri === found;
+        setState({
+          ...state,
+          verified: match,
+        });
+        if (!match) {
+          setRevenuePayto(parsePaytoUri(resp.body));
+        }
+        setTestError(undefined);
+      }
+    }
+  }
+
+  return (
+    <Fragment>
       <section class="section is-main-section">
         <div class="columns">
           <div class="column" />
@@ -147,12 +209,20 @@ export function CreatePage({ onCreate, onBack }: Props): VNode {
             >
               <InputPaytoForm<Entity>
                 name="payto_uri"
-                label={i18n.str`Account`}
+                label={i18n.str`Account details`}
               />
+              <div class="message-body" style={{ marginBottom: 10 }}>
+                <p>
+                  <i18n.Translate>
+                    If the bank supports Taler Revenue API then you can add the
+                    endpoint URL below to keep the revenue information in sync.
+                  </i18n.Translate>
+                </p>
+              </div>
               <Input<Entity>
                 name="credit_facade_url"
-                label={i18n.str`Account info URL`}
-                help="https://bank.com"
+                label={i18n.str`Endpoint URL`}
+                help="https://bank.demo.taler.net/accounts/${USERNAME}/taler-revenue/"
                 expand
                 tooltip={i18n.str`From where the merchant can download information about incoming wire transfers to this account`}
               />
@@ -179,21 +249,43 @@ export function CreatePage({ onCreate, onBack }: Props): VNode {
                     label={i18n.str`Password`}
                     tooltip={i18n.str`Password to access the account information.`}
                   />
-                  <Input
-                    name="repeatPassword"
-                    inputType="password"
-                    label={i18n.str`Repeat password`}
-                  />
                 </Fragment>
               ) : undefined}
+              <InputToggle<Entity>
+                label={i18n.str`Match`}
+                tooltip={i18n.str`Check where the information match against the server info.`}
+                name="verified"
+                readonly
+                threeState
+                help={
+                  testError !== undefined
+                    ? testError
+                    : state.verified === undefined
+                      ? i18n.str`Not verified`
+                      : state.verified
+                        ? i18n.str`Last test was ok`
+                        : i18n.str`Last test failed`
+                }
+                side={
+                  <button
+                    class="button is-info"
+                    data-tooltip={i18n.str`Compare info from server with account form`}
+                    disabled={!state.credit_facade_url}
+                    onClick={async () => {
+                      await testAccountInfo();
+                    }}
+                  >
+                    <i18n.Translate>Test</i18n.Translate>
+                  </button>
+                }
+              />
             </FormProvider>
 
             <div class="buttons is-right mt-5">
               <button
                 class="button is-info"
-                data-tooltip={i18n.str`Need to complete marked fields`}
                 onClick={() => {
-                  setImporting(true)
+                  setImporting(true);
                 }}
               >
                 <i18n.Translate>Import from bank</i18n.Translate>
@@ -209,7 +301,7 @@ export function CreatePage({ onCreate, onBack }: Props): VNode {
                 data-tooltip={
                   hasErrors
                     ? i18n.str`Need to complete marked fields`
-                    : "confirm operation"
+                    : i18n.str`Confirm operation`
                 }
                 onClick={submitForm}
               >
@@ -220,6 +312,52 @@ export function CreatePage({ onCreate, onBack }: Props): VNode {
           <div class="column" />
         </div>
       </section>
-    </div>
+      {!importing ? undefined : (
+        <ImportingAccountModal
+          onCancel={() => {
+            setImporting(false);
+          }}
+          onConfirm={(ac) => {
+            const u = new URL(ac.infoURL);
+            const user = u.username;
+            const pwd = u.password;
+            u.password = "";
+            u.username = "";
+            const credit_facade_url = u.href;
+            setState({
+              payto_uri: ac.accountURI,
+              credit_facade_credentials:
+                user || pwd
+                  ? {
+                      type: "basic",
+                      password: pwd,
+                      username: user,
+                    }
+                  : undefined,
+              credit_facade_url,
+            });
+            setImporting(false);
+          }}
+        />
+      )}
+      {!revenuePayto ? undefined : (
+        <CompareAccountsModal
+          onCancel={() => {
+            setRevenuePayto(undefined);
+          }}
+          onConfirm={(d) => {
+            setState({
+              ...state,
+              payto_uri: d,
+            });
+            setRevenuePayto(undefined);
+          }}
+          formPayto={
+            !state.payto_uri ? undefined : parsePaytoUri(state.payto_uri)
+          }
+          testPayto={revenuePayto}
+        />
+      )}
+    </Fragment>
   );
 }
