@@ -51,6 +51,7 @@ export default function OrderCreate({ onConfirm, onBack }: Props): VNode {
   const { state, lib } = useSessionContext();
   const [notif, setNotif] = useState<Notification | undefined>(undefined);
   const detailsResult = useInstanceDetails();
+  // FIXME: if the product list is big the will bring a lot of info
   const inventoryResult = useInstanceProducts();
   const { i18n } = useTranslationContext();
 
@@ -116,7 +117,8 @@ export default function OrderCreate({ onConfirm, onBack }: Props): VNode {
               setNotif({
                 message: i18n.str`Could not create order`,
                 type: "ERROR",
-                description: error instanceof Error ? error.message : String(error),
+                description:
+                  error instanceof Error ? error.message : String(error),
               });
             });
         }}
