@@ -1,6 +1,6 @@
 /*
  This file is part of GNU Taler
- (C) 2021-2023 Taler Systems S.A.
+ (C) 2021-2024 Taler Systems S.A.
 
  GNU Taler is free software; you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
@@ -19,9 +19,9 @@
  * @author Sebastian Javier Marchano (sebasjm)
  */
 
+import { AmountString, TalerMerchantApi } from "@gnu-taler/taler-util";
 import { addDays } from "date-fns";
-import { h, VNode, FunctionalComponent } from "preact";
-import { MerchantBackend } from "../../../../declaration.js";
+import { FunctionalComponent, h } from "preact";
 import { DetailPage as TestedComponent } from "./DetailPage.js";
 
 export default {
@@ -42,14 +42,13 @@ function createExample<Props>(
   return r;
 }
 
-const defaultContractTerm = {
-  amount: "TESTKUDOS:10",
+const defaultContractTerm: TalerMerchantApi.ContractTerms = {
+  amount: "TESTKUDOS:10" as AmountString,
   timestamp: {
     t_s: new Date().getTime() / 1000,
   },
-  auditors: [],
   exchanges: [],
-  max_fee: "TESTKUDOS:1",
+  max_fee: "TESTKUDOS:1" as AmountString,
   merchant: {} as any,
   merchant_base_url: "http://merchant.url/",
   order_id: "2021.165-03GDFC26Y1NNG",
@@ -66,7 +65,7 @@ const defaultContractTerm = {
   },
   wire_method: "x-taler-bank",
   h_wire: "asd",
-} as MerchantBackend.ContractTerms;
+};
 
 // contract_terms: defaultContracTerm,
 export const Claimed = createExample(TestedComponent, {
@@ -83,15 +82,16 @@ export const PaidNotRefundable = createExample(TestedComponent, {
     order_status: "paid",
     contract_terms: defaultContractTerm,
     refunded: false,
-    deposit_total: "TESTKUDOS:10",
-    exchange_ec: 0,
+    deposit_total: "TESTKUDOS:10" as AmountString,
+    exchange_code: 0,
     order_status_url: "http://merchant.backend/status",
-    exchange_hc: 0,
-    refund_amount: "TESTKUDOS:0",
+    exchange_http_status: 0,
+    refund_amount: "TESTKUDOS:0" as AmountString,
     refund_details: [],
     refund_pending: false,
     wire_details: [],
     wired: false,
+    wire_reports: [],
   },
 });
 
@@ -106,12 +106,13 @@ export const PaidRefundable = createExample(TestedComponent, {
       },
     },
     refunded: false,
-    deposit_total: "TESTKUDOS:10",
-    exchange_ec: 0,
+    deposit_total: "TESTKUDOS:10" as AmountString,
+    exchange_code: 0,
     order_status_url: "http://merchant.backend/status",
-    exchange_hc: 0,
-    refund_amount: "TESTKUDOS:0",
+    exchange_http_status: 0,
+    refund_amount: "TESTKUDOS:0" as AmountString,
     refund_details: [],
+    wire_reports: [],
     refund_pending: false,
     wire_details: [],
     wired: false,
@@ -128,6 +129,6 @@ export const Unpaid = createExample(TestedComponent, {
     },
     summary: "text summary",
     taler_pay_uri: "pay uri",
-    total_amount: "TESTKUDOS:10",
+    total_amount: "TESTKUDOS:10" as AmountString,
   },
 });
