@@ -26,8 +26,7 @@ import {
   stringifyRefundUri,
 } from "@gnu-taler/taler-util";
 import {
-  useMerchantApiContext,
-  useTranslationContext,
+  useTranslationContext
 } from "@gnu-taler/web-util/browser";
 import { format, formatDistance } from "date-fns";
 import { Fragment, VNode, h } from "preact";
@@ -41,6 +40,7 @@ import { InputGroup } from "../../../../components/form/InputGroup.js";
 import { InputLocation } from "../../../../components/form/InputLocation.js";
 import { TextField } from "../../../../components/form/TextField.js";
 import { ProductList } from "../../../../components/product/ProductList.js";
+import { useSessionContext } from "../../../../context/session.js";
 import {
   datetimeFormatForSettings,
   usePreference,
@@ -75,13 +75,13 @@ function ContractTerms({ value }: { value: CT }) {
           readonly
           name="summary"
           label={i18n.str`Summary`}
-          tooltip={i18n.str`human-readable description of the whole purchase`}
+          tooltip={i18n.str`Human-readable description of the whole purchase`}
         />
         <InputCurrency<CT>
           readonly
           name="amount"
           label={i18n.str`Amount`}
-          tooltip={i18n.str`total price for the transaction`}
+          tooltip={i18n.str`Total price for the transaction`}
         />
         {value.fulfillment_url && (
           <Input<CT>
@@ -95,43 +95,43 @@ function ContractTerms({ value }: { value: CT }) {
           readonly
           name="max_fee"
           label={i18n.str`Max fee`}
-          tooltip={i18n.str`maximum total deposit fee accepted by the merchant for this contract`}
+          tooltip={i18n.str`Maximum total deposit fee accepted by the merchant for this contract`}
         />
         <InputDate<CT>
           readonly
           name="timestamp"
           label={i18n.str`Created at`}
-          tooltip={i18n.str`time when this contract was generated`}
+          tooltip={i18n.str`Time when this contract was generated`}
         />
         <InputDate<CT>
           readonly
           name="refund_deadline"
           label={i18n.str`Refund deadline`}
-          tooltip={i18n.str`after this deadline has passed no refunds will be accepted`}
+          tooltip={i18n.str`After this deadline has passed no refunds will be accepted`}
         />
         <InputDate<CT>
           readonly
           name="pay_deadline"
           label={i18n.str`Payment deadline`}
-          tooltip={i18n.str`after this deadline, the merchant won't accept payments for the contract`}
+          tooltip={i18n.str`After this deadline, the merchant won't accept payments for the contract`}
         />
         <InputDate<CT>
           readonly
           name="wire_transfer_deadline"
           label={i18n.str`Wire transfer deadline`}
-          tooltip={i18n.str`transfer deadline for the exchange`}
+          tooltip={i18n.str`Transfer deadline for the exchange`}
         />
         <InputDate<CT>
           readonly
           name="delivery_date"
           label={i18n.str`Delivery date`}
-          tooltip={i18n.str`time indicating when the order should be delivered`}
+          tooltip={i18n.str`Time indicating when the order should be delivered`}
         />
         {value.delivery_date && (
           <InputGroup
             name="delivery_location"
             label={i18n.str`Location`}
-            tooltip={i18n.str`where the order will be delivered`}
+            tooltip={i18n.str`Where the order will be delivered`}
           >
             <InputLocation name="payments.delivery_location" />
           </InputGroup>
@@ -140,13 +140,13 @@ function ContractTerms({ value }: { value: CT }) {
           readonly
           name="auto_refund"
           label={i18n.str`Auto-refund delay`}
-          tooltip={i18n.str`how long the wallet should try to get an automatic refund for the purchase`}
+          tooltip={i18n.str`How long the wallet should try to get an automatic refund for the purchase`}
         />
         <Input<CT>
           readonly
           name="extra"
           label={i18n.str`Extra info`}
-          tooltip={i18n.str`extra data that is only interpreted by the merchant frontend`}
+          tooltip={i18n.str`Extra data that is only interpreted by the merchant frontend`}
         />
       </FormProvider>
     </InputGroup>
@@ -221,7 +221,7 @@ function ClaimedPage({
                     <div class="level-item">
                       <i18n.Translate>Order</i18n.Translate> #{id}
                       <div class="tag is-info ml-4">
-                        <i18n.Translate>claimed</i18n.Translate>
+                        <i18n.Translate>Claimed</i18n.Translate>
                       </div>
                     </div>
                   </div>
@@ -248,7 +248,7 @@ function ClaimedPage({
                       >
                         <p>
                           <b>
-                            <i18n.Translate>claimed at</i18n.Translate>:
+                            <i18n.Translate>Claimed at</i18n.Translate>:
                           </b>{" "}
                           {order.contract_terms.timestamp.t_s === "never"
                             ? "never"
@@ -430,10 +430,10 @@ function PaidPage({
   });
 
   const [value, valueHandler] = useState<Partial<Paid>>(order);
-  const { url: backendUrl } = useMerchantApiContext();
+  const { state } = useSessionContext();
 
   const refundurl = stringifyRefundUri({
-    merchantBaseUrl: backendUrl.href,
+    merchantBaseUrl: state.backendUrl.href,
     orderId: order.contract_terms.order_id,
   });
   const { i18n } = useTranslationContext();
@@ -458,16 +458,16 @@ function PaidPage({
                     <div class="level-item">
                       <i18n.Translate>Order</i18n.Translate> #{id}
                       <div class="tag is-success ml-4">
-                        <i18n.Translate>paid</i18n.Translate>
+                        <i18n.Translate>Paid</i18n.Translate>
                       </div>
                       {order.wired ? (
                         <div class="tag is-success ml-4">
-                          <i18n.Translate>wired</i18n.Translate>
+                          <i18n.Translate>Wired</i18n.Translate>
                         </div>
                       ) : null}
                       {order.refunded ? (
                         <div class="tag is-danger ml-4">
-                          <i18n.Translate>refunded</i18n.Translate>
+                          <i18n.Translate>Refunded</i18n.Translate>
                         </div>
                       ) : null}
                     </div>
@@ -487,8 +487,8 @@ function PaidPage({
                             class="has-tooltip-left"
                             data-tooltip={
                               refundable
-                                ? i18n.str`refund order`
-                                : i18n.str`not refundable`
+                                ? i18n.str`Refund order`
+                                : i18n.str`Not refundable`
                             }
                           >
                             <button
@@ -496,7 +496,7 @@ function PaidPage({
                               disabled={!refundable}
                               onClick={() => onRefund(id)}
                             >
-                              <i18n.Translate>refund</i18n.Translate>
+                              <i18n.Translate>Refund</i18n.Translate>
                             </button>
                           </span>
                         </div>
@@ -638,7 +638,7 @@ function UnpaidPage({
                 </h1>
               </div>
               <div class="tag is-dark">
-                <i18n.Translate>unpaid</i18n.Translate>
+                <i18n.Translate>Unpaid</i18n.Translate>
               </div>
             </div>
           </div>
@@ -656,7 +656,7 @@ function UnpaidPage({
                 >
                   <p>
                     <b>
-                      <i18n.Translate>pay at</i18n.Translate>:
+                      <i18n.Translate>Pay at</i18n.Translate>:
                     </b>{" "}
                     <a
                       href={order.order_status_url}
@@ -668,7 +668,7 @@ function UnpaidPage({
                   </p>
                   <p>
                     <b>
-                      <i18n.Translate>created at</i18n.Translate>:
+                      <i18n.Translate>Created at</i18n.Translate>:
                     </b>{" "}
                     {order.creation_time.t_s === "never"
                       ? "never"
@@ -693,13 +693,13 @@ function UnpaidPage({
                 readonly
                 name="summary"
                 label={i18n.str`Summary`}
-                tooltip={i18n.str`human-readable description of the whole purchase`}
+                tooltip={i18n.str`Human-readable description of the whole purchase`}
               />
               <InputCurrency<Unpaid>
                 readonly
                 name="total_amount"
                 label={i18n.str`Amount`}
-                tooltip={i18n.str`total price for the transaction`}
+                tooltip={i18n.str`Total price for the transaction`}
               />
               <Input<Unpaid>
                 name="order_status"
