@@ -155,7 +155,7 @@ function Table({
                       data-tooltip={i18n.str`Delete selected devices from the database`}
                       onClick={() => onDelete(i)}
                     >
-                      Delete
+                      <i18n.Translate>Delete</i18n.Translate>
                     </button>
                   </div>
                 </td>
