@@ -23,11 +23,9 @@ import {
   HttpStatusCode,
   TalerError,
   TalerMerchantApi,
-  assertUnreachable
+  assertUnreachable,
 } from "@gnu-taler/taler-util";
-import {
-  useTranslationContext
-} from "@gnu-taler/web-util/browser";
+import { useTranslationContext } from "@gnu-taler/web-util/browser";
 import { Fragment, VNode, h } from "preact";
 import { useState } from "preact/hooks";
 import { ErrorLoadingMerchant } from "../../../../components/ErrorLoadingMerchant.js";
@@ -38,7 +36,7 @@ import { useInstanceOtpDevices } from "../../../../hooks/otp.js";
 import { Notification } from "../../../../utils/types.js";
 import { LoginPage } from "../../../login/index.js";
 import { NotFoundPageOrAdminCreate } from "../../../notfound/index.js";
-import { ListPage } from "./ListPage.js";
+import { CardTable } from "./Table.js";
 
 interface Props {
   onCreate: () => void;
@@ -49,8 +47,7 @@ export default function ListOtpDevices({ onCreate, onSelect }: Props): VNode {
   // const [position, setPosition] = useState<string | undefined>(undefined);
   const { i18n } = useTranslationContext();
   const [notif, setNotif] = useState<Notification | undefined>(undefined);
-  const { lib } = useSessionContext();
-  const { state } = useSessionContext();
+  const { state, lib } = useSessionContext();
   const result = useInstanceOtpDevices();
 
   if (!result) return <Loading />;
@@ -63,7 +60,7 @@ export default function ListOtpDevices({ onCreate, onSelect }: Props): VNode {
         return <NotFoundPageOrAdminCreate />;
       }
       case HttpStatusCode.Unauthorized: {
-        return <LoginPage />
+        return <LoginPage />;
       }
       default: {
         assertUnreachable(result);
@@ -75,32 +72,42 @@ export default function ListOtpDevices({ onCreate, onSelect }: Props): VNode {
     <Fragment>
       <NotificationCard notification={notif} />
 
-      <ListPage
-        devices={result.body.otp_devices}
-        onLoadMoreBefore={undefined} //result.isFirstPage ? undefined : result.loadFirst}
-        onLoadMoreAfter={undefined} //result.isLastPage ? undefined : result.loadNext}
-        onCreate={onCreate}
-        onSelect={(e) => {
-          onSelect(e.otp_device_id);
-        }}
-        onDelete={(e: TalerMerchantApi.OtpDeviceEntry) => {
-          return lib.instance
-            .deleteOtpDevice(state.token, e.otp_device_id)
-            .then(() =>
-              setNotif({
-                message: i18n.str`validator delete successfully`,
-                type: "SUCCESS",
-              }),
-            )
-            .catch((error) =>
-              setNotif({
-                message: i18n.str`could not delete the validator`,
-                type: "ERROR",
-                description: error.message,
-              }),
-            );
-        }}
-      />
+      <section class="section is-main-section">
+        <CardTable
+          devices={result.body.otp_devices}
+          onLoadMoreBefore={undefined} //result.isFirstPage ? undefined : result.loadFirst}
+          onLoadMoreAfter={undefined} //result.isLastPage ? undefined : result.loadNext}
+          onCreate={onCreate}
+          onSelect={(e) => {
+            onSelect(e.otp_device_id);
+          }}
+          onDelete={async (e: TalerMerchantApi.OtpDeviceEntry) => {
+            return lib.instance
+              .deleteOtpDevice(state.token, e.otp_device_id)
+              .then((resp) => {
+                if (resp.type === "ok") {
+                  setNotif({
+                    message: i18n.str`Device delete successfully`,
+                    type: "SUCCESS",
+                  });
+                } else {
+                  setNotif({
+                    message: i18n.str`Could not delete the device`,
+                    type: "ERROR",
+                    description: resp.detail.hint,
+                  });
+                }
+              })
+              .catch((error) =>
+                setNotif({
+                  message: i18n.str`Could not delete the device`,
+                  type: "ERROR",
+                  description: error instanceof Error ? error.message : String(error),
+                }),
+              );
+          }}
+        />
+      </section>
     </Fragment>
   );
 }
