@@ -19,10 +19,13 @@
  * @author Sebastian Javier Marchano (sebasjm)
  */
 
-import { HttpStatusCode, TalerError, TalerMerchantApi, assertUnreachable } from "@gnu-taler/taler-util";
 import {
-  useTranslationContext
-} from "@gnu-taler/web-util/browser";
+  HttpStatusCode,
+  TalerError,
+  TalerMerchantApi,
+  assertUnreachable,
+} from "@gnu-taler/taler-util";
+import { useTranslationContext } from "@gnu-taler/web-util/browser";
 import { VNode, h } from "preact";
 import { useState } from "preact/hooks";
 import { ErrorLoadingMerchant } from "../../../../components/ErrorLoadingMerchant.js";
@@ -31,27 +34,23 @@ import { JumpToElementById } from "../../../../components/form/JumpToElementById
 import { NotificationCard } from "../../../../components/menu/index.js";
 import { ConfirmModal } from "../../../../components/modal/index.js";
 import { useSessionContext } from "../../../../context/session.js";
-import {
-  useInstanceProducts
-} from "../../../../hooks/product.js";
+import { useInstanceProducts } from "../../../../hooks/product.js";
 import { Notification } from "../../../../utils/types.js";
 import { LoginPage } from "../../../login/index.js";
 import { NotFoundPageOrAdminCreate } from "../../../notfound/index.js";
 import { CardTable } from "./Table.js";
+import { WithId } from "../../../../declaration.js";
 
 interface Props {
   onCreate: () => void;
   onSelect: (id: string) => void;
 }
-export default function ProductList({
-  onCreate,
-  onSelect,
-}: Props): VNode {
+export default function ProductList({ onCreate, onSelect }: Props): VNode {
   const result = useInstanceProducts();
-  const { lib } = useSessionContext();
-  const { state } = useSessionContext();
-  const [deleting, setDeleting] =
-    useState<TalerMerchantApi.ProductDetail & WithId | null>(null);
+  const { state, lib } = useSessionContext();
+  const [deleting, setDeleting] = useState<
+    (TalerMerchantApi.ProductDetail & WithId) | null
+  >(null);
   const [notif, setNotif] = useState<Notification | undefined>(undefined);
 
   const { i18n } = useTranslationContext();
@@ -66,7 +65,7 @@ export default function ProductList({
         return <NotFoundPageOrAdminCreate />;
       }
       case HttpStatusCode.Unauthorized: {
-        return <LoginPage />
+        return <LoginPage />;
       }
       default: {
         assertUnreachable(result);
@@ -84,8 +83,8 @@ export default function ProductList({
           return resp.type === "ok";
         }}
         onSelect={onSelect}
-        description={i18n.str`jump to product with the given product ID`}
-        placeholder={i18n.str`product id`}
+        description={i18n.str`Jump to product with the given product ID`}
+        placeholder={i18n.str`Product id`}
       />
 
       <CardTable
@@ -95,19 +94,31 @@ export default function ProductList({
         onCreate={onCreate}
         onUpdate={async (id, prod) => {
           try {
-            await lib.instance.updateProduct(state.token, id, prod);
-            setNotif({
-              message: i18n.str`product updated successfully`,
-              type: "SUCCESS",
-            });
+            const resp = await lib.instance.updateProduct(
+              state.token,
+              id,
+              prod,
+            );
+            if (resp.type === "ok") {
+              setNotif({
+                message: i18n.str`Product updated successfully`,
+                type: "SUCCESS",
+              });
+            } else {
+              setNotif({
+                message: i18n.str`Could not update the product`,
+                type: "ERROR",
+                description: resp.detail.hint,
+              });
+            }
           } catch (error) {
             setNotif({
-              message: i18n.str`could not update the product`,
+              message: i18n.str`Could not update the product`,
               type: "ERROR",
               description: error instanceof Error ? error.message : undefined,
             });
           }
-          return
+          return;
         }}
         onSelect={(product) => onSelect(product.id)}
         onDelete={(prod: TalerMerchantApi.ProductDetail & WithId) =>
@@ -124,14 +135,25 @@ export default function ProductList({
           onCancel={() => setDeleting(null)}
           onConfirm={async (): Promise<void> => {
             try {
-              await lib.instance.deleteProduct(state.token, deleting.id);
-              setNotif({
-                message: i18n.str`Product "${deleting.description}" (ID: ${deleting.id}) has been deleted`,
-                type: "SUCCESS",
-              });
+              const resp = await lib.instance.deleteProduct(
+                state.token,
+                deleting.id,
+              );
+              if (resp.type === "ok") {
+                setNotif({
+                  message: i18n.str`Product "${deleting.description}" (ID: ${deleting.id}) has been deleted`,
+                  type: "SUCCESS",
+                });
+              } else {
+                setNotif({
+                  message: i18n.str`Could not delete the product`,
+                  type: "ERROR",
+                  description: resp.detail.hint,
+                });
+              }
             } catch (error) {
               setNotif({
-                message: i18n.str`Failed to delete product`,
+                message: i18n.str`Could not delete the product`,
                 type: "ERROR",
                 description: error instanceof Error ? error.message : undefined,
               });
@@ -140,11 +162,17 @@ export default function ProductList({
           }}
         >
           <p>
-            If you delete the product named <b>&quot;{deleting.description}&quot;</b> (ID:{" "}
-            <b>{deleting.id}</b>), the stock and related information will be lost
+            <i18n.Translate>
+              If you delete the product named{" "}
+              <b>&quot;{deleting.description}&quot;</b> (ID:{" "}
+              <b>{deleting.id}</b>
+              ), the stock and related information will be lost
+            </i18n.Translate>
           </p>
           <p class="warning">
-            Deleting an product <b>cannot be undone</b>.
+            <i18n.Translate>
+              Deleting an product can't be undone.
+            </i18n.Translate>
           </p>
         </ConfirmModal>
       )}
