@@ -27,10 +27,7 @@ import {
   TalerMerchantApi,
   TranslatedString,
 } from "@gnu-taler/taler-util";
-import {
-  useMerchantApiContext,
-  useTranslationContext,
-} from "@gnu-taler/web-util/browser";
+import { useTranslationContext } from "@gnu-taler/web-util/browser";
 import { Fragment, VNode, h } from "preact";
 import { useState } from "preact/hooks";
 import { AsyncButton } from "../../../../components/exception/AsyncButton.js";
@@ -46,6 +43,7 @@ import { InputSelector } from "../../../../components/form/InputSelector.js";
 import { InputToggle } from "../../../../components/form/InputToggle.js";
 import { InputWithAddon } from "../../../../components/form/InputWithAddon.js";
 import { TextField } from "../../../../components/form/TextField.js";
+import { useSessionContext } from "../../../../context/session.js";
 import { useInstanceOtpDevices } from "../../../../hooks/otp.js";
 
 // type Entity = TalerMerchantApi.TemplateAddDetails & { type: Steps };
@@ -69,7 +67,7 @@ interface Props {
 
 export function CreatePage({ onCreate, onBack }: Props): VNode {
   const { i18n } = useTranslationContext();
-  const { url: backendUrl, config } = useMerchantApiContext();
+  const { config, state: session } = useSessionContext();
   const devices = useInstanceOtpDevices();
 
   const [state, setState] = useState<Partial<Entity>>({
@@ -93,28 +91,32 @@ export function CreatePage({ onCreate, onBack }: Props): VNode {
 
   const errors: FormErrors<Entity> = {
     id: !state.id
-      ? i18n.str`should not be empty`
+      ? i18n.str`Required`
       : !/[a-zA-Z0-9]*/.test(state.id)
-        ? i18n.str`no valid. only characters and numbers`
+        ? i18n.str`Invalid. only characters and numbers`
         : undefined,
-    description: !state.description ? i18n.str`should not be empty` : undefined,
+    description: !state.description ? i18n.str`Required` : undefined,
     amount: !state.amount
-      ? undefined
+      ? state.amount_editable
+        ? undefined
+        : i18n.str`Required`
       : !parsedPrice
-        ? i18n.str`not valid`
+        ? i18n.str`Invalid`
         : Amounts.isZero(parsedPrice)
-          ? i18n.str`must be greater than 0`
+          ? state.amount_editable
+            ? undefined
+            : i18n.str`Must be greater than 0`
           : undefined,
     minimum_age:
       state.minimum_age && state.minimum_age < 0
-        ? i18n.str`should be greater that 0`
+        ? i18n.str`Must be greater that 0`
         : undefined,
     pay_duration: !state.pay_duration
-      ? i18n.str`can't be empty`
+      ? i18n.str`Required`
       : state.pay_duration.d_ms === "forever"
         ? undefined
         : state.pay_duration.d_ms < 1000 //less than one second
-          ? i18n.str`to short`
+          ? i18n.str`To short`
           : undefined,
   };
 
@@ -124,24 +126,31 @@ export function CreatePage({ onCreate, onBack }: Props): VNode {
     (k) => (errors as Record<string, unknown>)[k] !== undefined,
   );
 
+  const zero = Amounts.stringify(Amounts.zeroOfCurrency(config.currency));
+
   const submitForm = () => {
     if (hasErrors) return Promise.reject();
+    const contract_amount = state.amount_editable
+      ? undefined
+      : (state.amount as AmountString);
+    const contract_summary = state.summary_editable ? undefined : state.summary;
+    const template_contract: TalerMerchantApi.TemplateContractDetails = {
+      minimum_age: state.minimum_age!,
+      pay_duration: Duration.toTalerProtocolDuration(state.pay_duration!),
+      amount: contract_amount,
+      summary: contract_summary,
+      currency:
+        cList.length > 1 && state.currency_editable
+          ? undefined
+          : config.currency,
+    };
     return onCreate({
       template_id: state.id!,
       template_description: state.description!,
-      template_contract: {
-        minimum_age: state.minimum_age!,
-        pay_duration: Duration.toTalerProtocolDuration(state.pay_duration!),
-        amount: state.amount_editable ? undefined : state.amount,
-        summary: state.summary_editable ? undefined : state.summary,
-        currency:
-          cList.length > 1 && state.currency_editable
-            ? undefined
-            : config.currency,
-      },
+      template_contract,
       editable_defaults: {
-        amount: !state.amount_editable ? undefined : state.amount,
-        summary: !state.summary_editable ? undefined : state.summary,
+        amount: !state.amount_editable ? undefined : state.amount ?? zero,
+        summary: !state.summary_editable ? undefined : state.summary ?? "",
         currency:
           cList.length === 1 || !state.currency_editable
             ? undefined
@@ -175,7 +184,10 @@ export function CreatePage({ onCreate, onBack }: Props): VNode {
               <InputWithAddon<Entity>
                 name="id"
                 help={
-                  new URL(`templates/${state.id ?? ""}`, backendUrl.href).href
+                  new URL(
+                    `templates/${state.id ?? ""}`,
+                    session.backendUrl.href,
+                  ).href
                 }
                 label={i18n.str`Identifier`}
                 tooltip={i18n.str`Name of the template in URLs.`}
@@ -218,7 +230,9 @@ export function CreatePage({ onCreate, onBack }: Props): VNode {
                     tooltip={i18n.str`Allow the user to change currency.`}
                   />
                   <TextField name="sc" label={i18n.str`Supported currencies`}>
-                    <i18n.Translate>supported currencies: {cList.join(", ")}</i18n.Translate>
+                    <i18n.Translate>
+                      Supported currencies: {cList.join(", ")}
+                    </i18n.Translate>
                   </TextField>
                 </Fragment>
               )}
@@ -275,7 +289,7 @@ export function CreatePage({ onCreate, onBack }: Props): VNode {
                 data-tooltip={
                   hasErrors
                     ? i18n.str`Need to complete marked fields`
-                    : "confirm operation"
+                    : i18n.str`Confirm operation`
                 }
                 onClick={submitForm}
               >
