@@ -28,15 +28,13 @@ import { useSessionContext } from "../../../../context/session.js";
 import { Notification } from "../../../../utils/types.js";
 import { CreatePage } from "./CreatePage.js";
 
-export type Entity = TalerMerchantApi.TransferInformation;
 interface Props {
   onBack?: () => void;
   onConfirm: () => void;
 }
 
-export default function CreateTransfer({ onConfirm, onBack }: Props): VNode {
-  const { lib } = useSessionContext();
-  const { state } = useSessionContext();
+export default function CreateTemplate({ onConfirm, onBack }: Props): VNode {
+  const { state, lib } = useSessionContext();
   const [notif, setNotif] = useState<Notification | undefined>(undefined);
   const { i18n } = useTranslationContext();
 
@@ -45,14 +43,29 @@ export default function CreateTransfer({ onConfirm, onBack }: Props): VNode {
       <NotificationCard notification={notif} />
       <CreatePage
         onBack={onBack}
-        onCreate={(request: TalerMerchantApi.TemplateAddDetails) => {
-          return lib.instance.addTemplate(state.token, request)
-            .then(() => onConfirm())
+        onCreate={async (request: TalerMerchantApi.TemplateAddDetails) => {
+          return lib.instance
+            .addTemplate(state.token, request)
+            .then((resp) => {
+              if (resp.type === "ok") {
+                setNotif({
+                  message: i18n.str`Template has been created`,
+                  type: "SUCCESS",
+                });
+                onConfirm();
+              } else {
+                setNotif({
+                  message: i18n.str`Could not create template`,
+                  type: "ERROR",
+                  description: resp.detail.hint,
+                });
+              }
+            })
             .catch((error) => {
               setNotif({
-                message: i18n.str`could not inform template`,
+                message: i18n.str`Could not create template`,
                 type: "ERROR",
-                description: error.message,
+                description: error instanceof Error ? error.message : String(error),
               });
             });
         }}
