@@ -19,10 +19,13 @@
  * @author Sebastian Javier Marchano (sebasjm)
  */
 
-import { HttpStatusCode, TalerError, TalerMerchantApi, assertUnreachable } from "@gnu-taler/taler-util";
 import {
-  useTranslationContext
-} from "@gnu-taler/web-util/browser";
+  HttpStatusCode,
+  TalerError,
+  TalerMerchantApi,
+  assertUnreachable,
+} from "@gnu-taler/taler-util";
+import { useTranslationContext } from "@gnu-taler/web-util/browser";
 import { VNode, h } from "preact";
 import { useState } from "preact/hooks";
 import { ErrorLoadingMerchant } from "../../../../components/ErrorLoadingMerchant.js";
@@ -31,13 +34,11 @@ import { JumpToElementById } from "../../../../components/form/JumpToElementById
 import { NotificationCard } from "../../../../components/menu/index.js";
 import { ConfirmModal } from "../../../../components/modal/index.js";
 import { useSessionContext } from "../../../../context/session.js";
-import {
-  useInstanceTemplates
-} from "../../../../hooks/templates.js";
+import { useInstanceTemplates } from "../../../../hooks/templates.js";
 import { Notification } from "../../../../utils/types.js";
 import { LoginPage } from "../../../login/index.js";
 import { NotFoundPageOrAdminCreate } from "../../../notfound/index.js";
-import { ListPage } from "./ListPage.js";
+import { CardTable } from "./Table.js";
 
 interface Props {
   onCreate: () => void;
@@ -54,26 +55,25 @@ export default function ListTemplates({
 }: Props): VNode {
   const { i18n } = useTranslationContext();
   const [notif, setNotif] = useState<Notification | undefined>(undefined);
-  const { lib } = useSessionContext();
+  const { state, lib } = useSessionContext();
   const result = useInstanceTemplates();
   const [deleting, setDeleting] =
     useState<TalerMerchantApi.TemplateEntry | null>(null);
-  const { state } = useSessionContext();
 
-  if (!result) return <Loading />
+  if (!result) return <Loading />;
   if (result instanceof TalerError) {
-    return <ErrorLoadingMerchant error={result} />
+    return <ErrorLoadingMerchant error={result} />;
   }
   if (result.type === "fail") {
-    switch(result.case) {
+    switch (result.case) {
       case HttpStatusCode.NotFound: {
-        return <NotFoundPageOrAdminCreate />
+        return <NotFoundPageOrAdminCreate />;
       }
       case HttpStatusCode.Unauthorized: {
-        return <LoginPage />
+        return <LoginPage />;
       }
       default: {
-        assertUnreachable(result)
+        assertUnreachable(result);
       }
     }
   }
@@ -84,25 +84,21 @@ export default function ListTemplates({
 
       <JumpToElementById
         testIfExist={async (id) => {
-          const resp = await lib.instance.getTemplateDetails(state.token, id)
-          return resp.type === "ok"
+          const resp = await lib.instance.getTemplateDetails(state.token, id);
+          return resp.type === "ok";
         }}
         onSelect={onSelect}
-        description={i18n.str`jump to template with the given template ID`}
-        placeholder={i18n.str`template id`}
+        description={i18n.str`Jump to template with the given template ID`}
+        placeholder={i18n.str`Template identification`}
       />
 
-      <ListPage
-        // templates={result.body}
-        // onLoadMoreBefore={
-        //   result.isFirstPage ? undefined: result.loadFirst
-        // }
-        // onLoadMoreAfter={result.isLastPage ? undefined : result.loadNext}
-        
-        templates={result.body.templates}
+      <CardTable
+        templates={result.body.templates.map((o) => ({
+          ...o,
+          id: String(o.template_id),
+        }))}
         onLoadMoreBefore={undefined}
         onLoadMoreAfter={undefined}
-
         onCreate={onCreate}
         onSelect={(e) => {
           onSelect(e.template_id);
@@ -114,9 +110,8 @@ export default function ListTemplates({
           onQR(e.template_id);
         }}
         onDelete={(e: TalerMerchantApi.TemplateEntry) => {
-          setDeleting(e)
-        }
-        }
+          setDeleting(e);
+        }}
       />
 
       {deleting && (
@@ -128,11 +123,22 @@ export default function ListTemplates({
           onCancel={() => setDeleting(null)}
           onConfirm={async (): Promise<void> => {
             try {
-              await lib.instance.deleteTemplate(state.token, deleting.template_id);
-              setNotif({
-                message: i18n.str`Template "${deleting.template_description}" (ID: ${deleting.template_id}) has been deleted`,
-                type: "SUCCESS",
-              });
+              const resp = await lib.instance.deleteTemplate(
+                state.token,
+                deleting.template_id,
+              );
+              if (resp.type === "ok") {
+                setNotif({
+                  message: i18n.str`Template "${deleting.template_description}" (ID: ${deleting.template_id}) has been deleted`,
+                  type: "SUCCESS",
+                });
+              } else {
+                setNotif({
+                  message: i18n.str`Failed to delete template`,
+                  type: "ERROR",
+                  description: resp.detail.hint,
+                });
+              }
             } catch (error) {
               setNotif({
                 message: i18n.str`Failed to delete template`,
@@ -144,11 +150,18 @@ export default function ListTemplates({
           }}
         >
           <p>
-            If you delete the template <b>&quot;{deleting.template_description}&quot;</b> (ID:{" "}
-            <b>{deleting.template_id}</b>) you may loose information
+            <i18n.Translate>
+              If you delete the template{" "}
+              <b>&quot;{deleting.template_description}&quot;</b> (ID:{" "}
+              <b>{deleting.template_id}</b>) you may loose information
+            </i18n.Translate>
           </p>
           <p class="warning">
-            Deleting an template <b>cannot be undone</b>.
+            <i18n.Translate>Deleting an template </i18n.Translate>
+            <b>
+              <i18n.Translate>can't be undone</i18n.Translate>
+            </b>
+            .
           </p>
         </ConfirmModal>
       )}
