@@ -59,7 +59,7 @@ export function CardTable({
         <div class="card-header-icon" aria-label="more options">
           <span
             class="has-tooltip-left"
-            data-tooltip={i18n.str`add new webhooks`}
+            data-tooltip={i18n.str`Add new webhooks`}
           >
             <button class="button is-info" type="button" onClick={onCreate}>
               <span class="icon is-small">
@@ -114,10 +114,10 @@ function Table({
       {onLoadMoreBefore && (
         <button
           class="button is-fullwidth"
-          data-tooltip={i18n.str`load more webhooks before the first one`}
+          data-tooltip={i18n.str`Load more webhooks before the first one`}
           onClick={onLoadMoreBefore}
         >
-          <i18n.Translate>load first page</i18n.Translate>
+          <i18n.Translate>Load first page</i18n.Translate>
         </button>
       )}
       <table class="table is-fullwidth is-striped is-hoverable is-fullwidth">
@@ -152,18 +152,11 @@ function Table({
                   <div class="buttons is-right">
                     <button
                       class="button is-danger is-small has-tooltip-left"
-                      data-tooltip={i18n.str`delete selected webhook from the database`}
+                      data-tooltip={i18n.str`Delete selected webhook from the database`}
                       onClick={() => onDelete(i)}
                     >
-                      Delete
+                      <i18n.Translate>Delete</i18n.Translate>
                     </button>
-                    {/* <button
-                      class="button is-info is-small has-tooltip-left"
-                      data-tooltip={i18n.str`test webhook`}
-                      onClick={() => onNewOrder(i)}
-                    >
-                      Test
-                    </button> */}
                   </div>
                 </td>
               </tr>
@@ -174,10 +167,10 @@ function Table({
       {onLoadMoreAfter && (
         <button
           class="button is-fullwidth"
-          data-tooltip={i18n.str`load more webhooks after the last one`}
+          data-tooltip={i18n.str`Load more webhooks after the last one`}
           onClick={onLoadMoreAfter}
         >
-          <i18n.Translate>load next page</i18n.Translate>
+          <i18n.Translate>Load next page</i18n.Translate>
         </button>
       )}
     </div>
