@@ -29,6 +29,7 @@ import { ErrorLoadingMerchant } from "../../../../components/ErrorLoadingMerchan
 import { Loading } from "../../../../components/exception/loading.js";
 import { NotificationCard } from "../../../../components/menu/index.js";
 import { useSessionContext } from "../../../../context/session.js";
+import { WithId } from "../../../../declaration.js";
 import {
   useWebhookDetails,
 } from "../../../../hooks/webhooks.js";
@@ -49,8 +50,7 @@ export default function UpdateWebhook({
   onConfirm,
   onBack,
 }: Props): VNode {
-  const { lib } = useSessionContext();
-  const { state } = useSessionContext();
+  const { state, lib } = useSessionContext();
   const result = useWebhookDetails(tid);
   const [notif, setNotif] = useState<Notification | undefined>(undefined);
 
@@ -80,14 +80,29 @@ export default function UpdateWebhook({
       <UpdatePage
         webhook={{ ...result.body, id: tid }}
         onBack={onBack}
-        onUpdate={(data) => {
+        onUpdate={async (data) => {
           return lib.instance.updateWebhook(state.token, tid, data)
-            .then(onConfirm)
+            .then((resp) => {
+              if (resp.type === "ok") {
+                setNotif({
+                  message: i18n.str`Webhook updated`,
+                  type: "SUCCESS",
+                });
+                onConfirm()  
+              } else {
+                setNotif({
+                  message: i18n.str`Could not update webhook`,
+                  type: "ERROR",
+                  description: resp.detail.hint,
+                });
+  
+              }
+            })
             .catch((error) => {
               setNotif({
-                message: i18n.str`could not update template`,
+                message: i18n.str`Could not update webhook`,
                 type: "ERROR",
-                description: error.message,
+                description: error instanceof Error ? error.message : String(error),
               });
             });
         }}
