@@ -14,6 +14,8 @@
  GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
  */
 
+import { WithId } from "../declaration.js";
+
 
 /**
  *
@@ -49,7 +51,7 @@ export function buildActions<T extends WithId>(
  * @returns
  */
 export function undefinedIfEmpty<
-  T extends Record<string, unknown> | Array<unknown>,
+  T extends object | Record<string, unknown> | Array<unknown>,
 >(obj: T | undefined): T | undefined {
   if (obj === undefined) return undefined;
   return Object.values(obj).some((v) => v !== undefined) ? obj : undefined;
