@@ -171,12 +171,12 @@ function processFile(
   }
 
   function formatMsgLine(head: string, msg: string) {
+    const m = msg.match(/(.*\n|.+$)/g)
+    if (!m) return;
     // Do escaping, wrap break at newlines
     console.log("head", JSON.stringify(head));
     console.log("msg", JSON.stringify(msg));
-    let parts = msg
-      .match(/(.*\n|.+$)/g)
-      .map((x) => x.replace(/\n/g, "\\n").replace(/"/g, '\\"'))
+    let parts = m.map((x) => x.replace(/\n/g, "\\n").replace(/"/g, '\\"'))
       .map((p) => wordwrap(p))
       .reduce((a, b) => a.concat(b));
     if (parts.length == 1) {
