@@ -29,7 +29,6 @@ import {
 } from "@gnu-taler/taler-util";
 import {
   AccessStats,
-  applyRunConfigDefaults,
   createNativeWalletHost2,
   Wallet,
   WalletApiOperation,
@@ -75,7 +74,7 @@ export async function runBench1(configJson: any): Promise<void> {
     // my assumption is that the in-memory db file gets too large
     if (i % restartWallet == 0) {
       if (Object.keys(wallet).length !== 0) {
-        wallet.stop();
+        await wallet.client.call(WalletApiOperation.Shutdown, {});
         console.log("wallet DB stats", j2s(getDbStats!()));
       }
 
@@ -105,9 +104,7 @@ export async function runBench1(configJson: any): Promise<void> {
       exchangeBaseUrl: b1conf.exchange,
     });
 
-    await wallet.runTaskLoop({
-      stopWhenDone: true,
-    });
+    await wallet.client.call(WalletApiOperation.TestingWaitTasksDone, {});
 
     logger.info(
       `Finished withdrawal amount=${withdrawAmount} time=${Date.now() - start}`,
@@ -123,16 +120,14 @@ export async function runBench1(configJson: any): Promise<void> {
           depositPaytoUri: b1conf.payto,
         });
 
-        await wallet.runTaskLoop({
-          stopWhenDone: true,
-        });
+        await wallet.client.call(WalletApiOperation.TestingWaitTasksDone, {});
 
         logger.info(`Finished deposit amount=10 time=${Date.now() - start}`);
       }
     }
   }
 
-  wallet.stop();
+  await wallet.client.call(WalletApiOperation.Shutdown, {});
   console.log("wallet DB stats", j2s(getDbStats!()));
 }
 
