@@ -1618,6 +1618,26 @@ export class ExchangeService implements ExchangeServiceInterface {
     }
   }
 
+  async enableAmlAccount(
+    amlStaffPub: string,
+    legalName: string,
+  ): Promise<void> {
+    await runCommand(
+      this.globalState,
+      "exchange-offline",
+      "taler-exchange-offline",
+      [
+        "-c",
+        this.configFilename,
+        "aml-enable",
+        amlStaffPub,
+        legalName,
+        "rw",
+        "upload",
+      ],
+    );
+  }
+
   async pingUntilAvailable(): Promise<void> {
     // We request /management/keys, since /keys can block
     // when we didn't do the key setup yet.
