@@ -24,16 +24,24 @@
  * Imports
  */
 import {
+  AmlDecisionRequest,
+  AmlDecisionRequestWithoutSignature,
   AmountString,
   ConfirmPayResultType,
+  decodeCrock,
   Duration,
+  encodeCrock,
+  HttpStatusCode,
+  LegitimizationRuleSet,
   Logger,
   MerchantApiClient,
   NotificationType,
   PartialWalletRunConfig,
   PreparePayResultType,
+  signAmlDecision,
   TalerCorebankApiClient,
   TalerMerchantApi,
+  TalerProtocolTimestamp,
   TransactionMajorState,
   WalletNotification,
 } from "@gnu-taler/taler-util";
@@ -49,19 +57,20 @@ import {
   ExchangeService,
   ExchangeServiceInterface,
   FakebankService,
+  generateRandomPayto,
   GlobalTestState,
   HarnessExchangeBankAccount,
+  harnessHttpLib,
   LibeufinBankService,
   MerchantService,
   MerchantServiceInterface,
+  setupDb,
+  setupSharedDb,
+  useLibeufinBank,
   WalletCli,
   WalletClient,
   WalletService,
   WithAuthorization,
-  generateRandomPayto,
-  setupDb,
-  setupSharedDb,
-  useLibeufinBank,
 } from "./harness.js";
 
 import * as fs from "fs";
@@ -558,7 +567,8 @@ export async function createSimpleTestkudosEnvironmentV3(
     ),
   });
 
-  const { walletClient, walletService } = await createWalletDaemonWithClient(t,
+  const { walletClient, walletService } = await createWalletDaemonWithClient(
+    t,
     {
       name: "wallet",
       persistent: true,
@@ -623,7 +633,9 @@ export async function createWalletDaemonWithClient(
   const defaultRunConfig = {
     testing: {
       skipDefaults: true,
-      emitObservabilityEvents: !!process.env["TALER_TEST_OBSERVABILITY"] || !!args.emitObservabilityEvents,
+      emitObservabilityEvents:
+        !!process.env["TALER_TEST_OBSERVABILITY"] ||
+        !!args.emitObservabilityEvents,
     },
   } satisfies PartialWalletRunConfig;
   await walletClient.client.call(WalletApiOperation.InitWallet, {
@@ -748,7 +760,21 @@ export async function createFaultInjectedMerchantTestkudosEnvironment(
 }
 
 export interface WithdrawViaBankResult {
+  /**
+   * Payto URI of the account that the withdrawal
+   * originated from. Typically a new account used for testing.
+   */
+  accountPaytoUri: string;
+
+  /**
+   * Helper promise that resolves when withdrawal has finished successfully.
+   */
   withdrawalFinishedCond: Promise<true>;
+
+  /**
+   * The wallet-core withdrawal transaction ID.
+   */
+  transactionId: string;
 }
 
 /**
@@ -805,7 +831,9 @@ export async function withdrawViaBankV2(
   });
 
   return {
+    accountPaytoUri: user.accountPaytoUri,
     withdrawalFinishedCond,
+    transactionId: acceptRes.transactionId,
   };
 }
 
@@ -869,7 +897,9 @@ export async function withdrawViaBankV3(
   });
 
   return {
+    accountPaytoUri: user.accountPaytoUri,
     withdrawalFinishedCond,
+    transactionId: acceptRes.transactionId,
   };
 }
 
@@ -961,3 +991,99 @@ export async function makeTestPaymentV2(
 
   t.assertTrue(orderStatus.order_status === "paid");
 }
+
+/**
+ * Post an AML decision that no rules shall apply for the given account.
+ */
+export async function postAmlDecisionNoRules(
+  t: GlobalTestState,
+  req: {
+    exchangeBaseUrl: string;
+    paytoHash: string;
+    amlPriv: string;
+    amlPub: string;
+  },
+) {
+  const { exchangeBaseUrl, paytoHash, amlPriv, amlPub } = req;
+
+  const sigData: AmlDecisionRequestWithoutSignature = {
+    decision_time: TalerProtocolTimestamp.now(),
+    h_payto: paytoHash,
+    justification: "Bla",
+    keep_investigating: false,
+    new_rules: {
+      custom_measures: {},
+      expiration_time: TalerProtocolTimestamp.never(),
+      rules: [],
+      successor_measure: undefined,
+    },
+    properties: {
+      foo: "42",
+    },
+  };
+
+  const sig = signAmlDecision(decodeCrock(amlPriv), sigData);
+
+  const reqBody: AmlDecisionRequest = {
+    ...sigData,
+    officer_sig: encodeCrock(sig),
+  };
+
+  const reqUrl = new URL(`aml/${amlPub}/decision`, exchangeBaseUrl);
+
+  const resp = await harnessHttpLib.fetch(reqUrl.href, {
+    method: "POST",
+    body: reqBody,
+  });
+
+  console.log(`aml decision status: ${resp.status}`);
+
+  t.assertDeepEqual(resp.status, HttpStatusCode.NoContent);
+}
+
+/**
+ * Post an AML decision that no rules shall apply for the given account.
+ */
+export async function postAmlDecision(
+  t: GlobalTestState,
+  req: {
+    exchangeBaseUrl: string;
+    paytoHash: string;
+    amlPriv: string;
+    amlPub: string;
+    newRules: LegitimizationRuleSet;
+    newMeasure?: string | undefined;
+  },
+) {
+  const { exchangeBaseUrl, paytoHash, amlPriv, amlPub } = req;
+
+  const sigData: AmlDecisionRequestWithoutSignature = {
+    decision_time: TalerProtocolTimestamp.now(),
+    h_payto: paytoHash,
+    justification: "Bla",
+    keep_investigating: false,
+    new_rules: req.newRules,
+    new_measure: req.newMeasure,
+    properties: {
+      foo: "42",
+    },
+  };
+
+  const sig = signAmlDecision(decodeCrock(amlPriv), sigData);
+
+  const reqBody: AmlDecisionRequest = {
+    ...sigData,
+    officer_sig: encodeCrock(sig),
+  };
+
+  const reqUrl = new URL(`aml/${amlPub}/decision`, exchangeBaseUrl);
+
+  const resp = await harnessHttpLib.fetch(reqUrl.href, {
+    method: "POST",
+    body: reqBody,
+  });
+
+  console.log(`aml decision status: ${resp.status}`);
+
+  t.assertDeepEqual(resp.status, HttpStatusCode.NoContent);
+}
