@@ -34,7 +34,7 @@ import {
   depositCoin,
   downloadExchangeInfo,
   findDenomOrThrow,
-  topupReserveWithDemobank,
+  topupReserveWithBank,
   withdrawCoin,
 } from "@gnu-taler/taler-wallet-core/dbless";
 import { GlobalTestState } from "../harness/harness.js";
@@ -63,7 +63,7 @@ export async function runExchangeDepositTest(t: GlobalTestState) {
 
     const reserveKeyPair = await cryptoApi.createEddsaKeypair({});
 
-    await topupReserveWithDemobank({
+    await topupReserveWithBank({
       http,
       amount: "TESTKUDOS:10" as AmountString,
       corebankApiBaseUrl: bank.corebankApiBaseUrl,
