@@ -72,11 +72,9 @@ export async function runLibeufinBankTest(t: GlobalTestState) {
     database: db.connStr,
   });
 
-  const exchangeIban = generateRandomTestIban();
   const exchangeBankUsername = "exchange";
   const exchangeBankPw = "mypw";
-  const exchangePlainPayto = `payto://iban/${exchangeIban}`;
-  const exchangeExtendedPayto = `payto://iban/${exchangeIban}?receiver-name=Exchange`;
+  const exchangePayto = generateRandomPayto(exchangeBankUsername);
   const wireGatewayApiBaseUrl = new URL(
     "accounts/exchange/taler-wire-gateway/",
     bank.baseUrl,
@@ -88,7 +86,7 @@ export async function runLibeufinBankTest(t: GlobalTestState) {
     wireGatewayApiBaseUrl,
     accountName: exchangeBankUsername,
     accountPassword: exchangeBankPw,
-    accountPaytoUri: exchangeExtendedPayto,
+    accountPaytoUri: exchangePayto,
   });
 
   bank.setSuggestedExchange(exchange);
@@ -138,7 +136,7 @@ export async function runLibeufinBankTest(t: GlobalTestState) {
     password: exchangeBankPw,
     username: exchangeBankUsername,
     is_taler_exchange: true,
-    payto_uri: exchangePlainPayto,
+    payto_uri: exchangePayto,
   });
 
   const bankUser = await bankClient.registerAccount("user1", "pw1");
