@@ -25,7 +25,6 @@ import {
 } from "@gnu-taler/taler-util";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
 import {
-  BankServiceHandle,
   ExchangeServiceInterface,
   GlobalTestState,
   MerchantServiceInterface,
@@ -33,15 +32,14 @@ import {
   harnessHttpLib,
 } from "../harness/harness.js";
 import {
-  createSimpleTestkudosEnvironmentV2,
-  withdrawViaBankV2,
+  createSimpleTestkudosEnvironmentV3,
+  withdrawViaBankV3,
 } from "../harness/helpers.js";
 
 async function testRefundApiWithFulfillmentUrl(
   t: GlobalTestState,
   env: {
     merchant: MerchantServiceInterface;
-    bank: BankServiceHandle;
     walletClient: WalletClient;
     exchange: ExchangeServiceInterface;
   },
@@ -157,7 +155,6 @@ async function testRefundApiWithFulfillmentMessage(
   t: GlobalTestState,
   env: {
     merchant: MerchantServiceInterface;
-    bank: BankServiceHandle;
     walletClient: WalletClient;
     exchange: ExchangeServiceInterface;
   },
@@ -276,14 +273,14 @@ async function testRefundApiWithFulfillmentMessage(
 export async function runMerchantRefundApiTest(t: GlobalTestState) {
   // Set up test environment
 
-  const { walletClient, bank, exchange, merchant } =
-    await createSimpleTestkudosEnvironmentV2(t);
+  const { walletClient, bankClient, exchange, merchant } =
+    await createSimpleTestkudosEnvironmentV3(t);
 
   // Withdraw digital cash into the wallet.
 
-  const wres = await withdrawViaBankV2(t, {
+  const wres = await withdrawViaBankV3(t, {
     walletClient,
-    bank,
+    bankClient,
     exchange,
     amount: "TESTKUDOS:20",
   });
@@ -291,14 +288,12 @@ export async function runMerchantRefundApiTest(t: GlobalTestState) {
 
   await testRefundApiWithFulfillmentUrl(t, {
     walletClient,
-    bank,
     exchange,
     merchant,
   });
 
   await testRefundApiWithFulfillmentMessage(t, {
     walletClient,
-    bank,
     exchange,
     merchant,
   });
