@@ -21,28 +21,28 @@ import {
   ConfirmPayResultType,
   MerchantApiClient,
   PreparePayResultType,
+  TalerCorebankApiClient,
   URL,
   encodeCrock,
   getRandomBytes,
 } from "@gnu-taler/taler-util";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
 import {
-  BankService,
   ExchangeService,
   GlobalTestState,
   MerchantService,
   harnessHttpLib,
 } from "../harness/harness.js";
 import {
-  createSimpleTestkudosEnvironmentV2,
+  createSimpleTestkudosEnvironmentV3,
   createWalletDaemonWithClient,
-  withdrawViaBankV2,
+  withdrawViaBankV3,
 } from "../harness/helpers.js";
 
 interface Context {
   merchant: MerchantService;
   merchantBaseUrl: string;
-  bank: BankService;
+  bankClient: TalerCorebankApiClient;
   exchange: ExchangeService;
 }
 
@@ -55,11 +55,11 @@ async function testWithClaimToken(
   const { walletClient } = await createWalletDaemonWithClient(t, {
     name: "wct",
   });
-  const { bank, exchange } = c;
+  const { bankClient, exchange } = c;
   const { merchant, merchantBaseUrl } = c;
-  const wres = await withdrawViaBankV2(t, {
+  const wres = await withdrawViaBankV3(t, {
     walletClient,
-    bank,
+    bankClient,
     exchange,
     amount: "TESTKUDOS:20",
   });
@@ -317,12 +317,12 @@ async function testWithoutClaimToken(
     name: "wnoct",
   });
   const sessionId = "mysession2";
-  const { bank, exchange } = c;
+  const { bankClient, exchange } = c;
   const { merchant, merchantBaseUrl } = c;
   const merchantClient = new MerchantApiClient(merchant.makeInstanceBaseUrl());
-  const wres = await withdrawViaBankV2(t, {
+  const wres = await withdrawViaBankV3(t, {
     walletClient,
-    bank,
+    bankClient,
     exchange,
     amount: "TESTKUDOS:20",
   });
@@ -573,8 +573,8 @@ async function testWithoutClaimToken(
  * specification of the endpoint.
  */
 export async function runMerchantSpecPublicOrdersTest(t: GlobalTestState) {
-  const { bank, exchange, merchant } =
-    await createSimpleTestkudosEnvironmentV2(t);
+  const { bankClient, exchange, merchant } =
+    await createSimpleTestkudosEnvironmentV3(t);
 
   // Base URL for the default instance.
   const merchantBaseUrl = merchant.makeInstanceBaseUrl();
@@ -617,14 +617,14 @@ export async function runMerchantSpecPublicOrdersTest(t: GlobalTestState) {
     merchant,
     merchantBaseUrl,
     exchange,
-    bank,
+    bankClient,
   });
 
   await testWithoutClaimToken(t, {
     merchant,
     merchantBaseUrl,
     exchange,
-    bank,
+    bankClient,
   });
 }
 
