@@ -30,8 +30,8 @@ import {
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
 import { GlobalTestState } from "../harness/harness.js";
 import {
-  createSimpleTestkudosEnvironmentV2,
-  withdrawViaBankV2,
+  createSimpleTestkudosEnvironmentV3,
+  withdrawViaBankV3,
 } from "../harness/helpers.js";
 
 /**
@@ -40,8 +40,8 @@ import {
 export async function runOtpTest(t: GlobalTestState) {
   // Set up test environment
 
-  const { walletClient, bank, exchange, merchant } =
-    await createSimpleTestkudosEnvironmentV2(t);
+  const { walletClient, bankClient, exchange, merchant } =
+    await createSimpleTestkudosEnvironmentV3(t);
 
   const merchantClient = new MerchantApiClient(merchant.makeInstanceBaseUrl());
   const createOtpRes = await merchantClient.createOtpDevice({
@@ -69,12 +69,12 @@ export async function runOtpTest(t: GlobalTestState) {
 
   const getTemplateResp = await merchantClient.getTemplate("tpl1");
   narrowOpSuccessOrThrow("getTemplate", getTemplateResp);
-  
+
   console.log(`template: ${j2s(getTemplateResp.body)}`);
 
-  const wres = await withdrawViaBankV2(t, {
+  const wres = await withdrawViaBankV3(t, {
     walletClient,
-    bank,
+    bankClient,
     exchange,
     amount: "TESTKUDOS:20",
   });
