@@ -25,9 +25,9 @@ import {
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
 import { GlobalTestState } from "../harness/harness.js";
 import {
-  createSimpleTestkudosEnvironmentV2,
+  createSimpleTestkudosEnvironmentV3,
   createWalletDaemonWithClient,
-  withdrawViaBankV2,
+  withdrawViaBankV3,
 } from "../harness/helpers.js";
 
 /**
@@ -36,8 +36,8 @@ import {
 export async function runPaymentClaimTest(t: GlobalTestState) {
   // Set up test environment
 
-  const { walletClient, bank, exchange, merchant } =
-    await createSimpleTestkudosEnvironmentV2(t);
+  const { walletClient, bankClient, exchange, merchant } =
+    await createSimpleTestkudosEnvironmentV3(t);
 
   const merchantClient = new MerchantApiClient(merchant.makeInstanceBaseUrl());
 
@@ -45,9 +45,9 @@ export async function runPaymentClaimTest(t: GlobalTestState) {
 
   // Withdraw digital cash into the wallet.
 
-  const wres = await withdrawViaBankV2(t, {
+  const wres = await withdrawViaBankV3(t, {
     walletClient,
-    bank,
+    bankClient,
     exchange,
     amount: "TESTKUDOS:20",
   });
