@@ -18,6 +18,7 @@
  * Imports.
  */
 import {
+  AmountString,
   ConfirmPayResultType,
   MerchantApiClient,
   NotificationType,
@@ -27,9 +28,9 @@ import {
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
 import { GlobalTestState } from "../harness/harness.js";
 import {
-  createSimpleTestkudosEnvironmentV2,
+  createSimpleTestkudosEnvironmentV3,
   createWalletDaemonWithClient,
-  withdrawViaBankV2,
+  withdrawViaBankV3,
 } from "../harness/helpers.js";
 
 /**
@@ -39,17 +40,17 @@ export async function runPaymentShareTest(t: GlobalTestState) {
   // Set up test environment
   const {
     walletClient: firstWallet,
-    bank,
+    bankClient,
     exchange,
     merchant,
-  } = await createSimpleTestkudosEnvironmentV2(t);
+  } = await createSimpleTestkudosEnvironmentV3(t);
 
   const merchantClient = new MerchantApiClient(merchant.makeInstanceBaseUrl());
 
   // Withdraw digital cash into the wallet.
-  await withdrawViaBankV2(t, {
+  await withdrawViaBankV3(t, {
     walletClient: firstWallet,
-    bank,
+    bankClient,
     exchange,
     amount: "TESTKUDOS:20",
   });
@@ -59,9 +60,9 @@ export async function runPaymentShareTest(t: GlobalTestState) {
     name: "wallet2",
   });
 
-  await withdrawViaBankV2(t, {
+  await withdrawViaBankV3(t, {
     walletClient: secondWallet,
-    bank,
+    bankClient,
     exchange,
     amount: "TESTKUDOS:20",
   });
@@ -80,7 +81,7 @@ export async function runPaymentShareTest(t: GlobalTestState) {
   async function createOrder(amount: string) {
     const order = {
       summary: "Buy me!",
-      amount,
+      amount: amount as AmountString,
       fulfillment_url: "taler://fulfillment-success/thx",
     };
 
