@@ -20,8 +20,9 @@
 import {
   Duration,
   MerchantApiClient,
-  MerchantContractTerms,
   PreparePayResultType,
+  TalerCorebankApiClient,
+  TalerMerchantApi,
   TransactionMajorState,
 } from "@gnu-taler/taler-util";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
@@ -36,7 +37,7 @@ import {
 } from "../harness/harness.js";
 import {
   createWalletDaemonWithClient,
-  withdrawViaBankV2,
+  withdrawViaBankV3,
 } from "../harness/helpers.js";
 
 /**
@@ -70,17 +71,42 @@ export async function runWalletWirefeesTest(t: GlobalTestState) {
     database: db.connStr,
   });
 
-  const exchangeBankAccount = await bank.createExchangeAccount(
-    "myexchange",
-    "x",
-  );
-  await exchange.addBankAccount("1", exchangeBankAccount);
+  let receiverName = "Exchange";
+  let exchangeBankUsername = "exchange";
+  let exchangeBankPassword = "mypw";
+  let exchangePaytoUri = generateRandomPayto(exchangeBankUsername);
+
+  await exchange.addBankAccount("1", {
+    accountName: exchangeBankUsername,
+    accountPassword: exchangeBankPassword,
+    wireGatewayApiBaseUrl: new URL(
+      "accounts/exchange/taler-wire-gateway/",
+      bank.baseUrl,
+    ).href,
+    accountPaytoUri: exchangePaytoUri,
+  });
+
+  bank.setSuggestedExchange(exchange, exchangePaytoUri);
 
-  bank.setSuggestedExchange(exchange, exchangeBankAccount.accountPaytoUri);
   await bank.start();
 
   await bank.pingUntilAvailable();
 
+  const bankClient = new TalerCorebankApiClient(bank.corebankApiBaseUrl, {
+    auth: {
+      username: "admin",
+      password: "adminpw",
+    },
+  });
+
+  await bankClient.registerAccountExtended({
+    name: receiverName,
+    password: exchangeBankPassword,
+    username: exchangeBankUsername,
+    is_taler_exchange: true,
+    payto_uri: exchangePaytoUri,
+  });
+
   const coinConfig: CoinConfig[] = defaultCoinConfig.map((x) => x("TESTKUDOS"));
   exchange.addCoinConfigList(coinConfig);
 
@@ -119,22 +145,22 @@ export async function runWalletWirefeesTest(t: GlobalTestState) {
 
   // Withdraw digital cash into the wallet.
 
-  await withdrawViaBankV2(t, {
+  await withdrawViaBankV3(t, {
     walletClient,
-    bank,
+    bankClient,
     exchange,
     amount: "TESTKUDOS:20",
   });
 
   await walletClient.call(WalletApiOperation.TestingWaitTransactionsFinal, {});
 
-  const order = {
+  const order: TalerMerchantApi.Order = {
     summary: "Buy me!",
     amount: "TESTKUDOS:1",
     fulfillment_url: "taler://fulfillment-success/thx",
     //max_wire_fee: "TESTKUDOS:0.1",
     max_fee: "TESTKUDOS:0.1",
-  } satisfies Partial<MerchantContractTerms>;
+  };
 
   const merchantClient = new MerchantApiClient(merchant.makeInstanceBaseUrl());
 
