@@ -21,15 +21,16 @@ import {
   GlobalTestState,
   setupDb,
   ExchangeService,
-  FakebankService,
   WalletService,
   WalletClient,
+  BankService,
 } from "../harness/harness.js";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
 import { CoinConfig, defaultCoinConfig } from "../harness/denomStructures.js";
 import {
   AmountString,
   NotificationType,
+  TalerCorebankApiClient,
   TransactionMajorState,
   URL,
 } from "@gnu-taler/taler-util";
@@ -45,7 +46,7 @@ export async function runWithdrawalHugeTest(t: GlobalTestState) {
 
   const db = await setupDb(t);
 
-  const bank = await FakebankService.create(t, {
+  const bank = await BankService.create(t, {
     currency: "TESTKUDOS",
     httpPort: 8082,
     allowRegistrations: true,
@@ -60,17 +61,36 @@ export async function runWithdrawalHugeTest(t: GlobalTestState) {
     database: db.connStr,
   });
 
-  exchange.addBankAccount("1", {
+  let paytoUri = "payto://x-taler-bank/localhost/exchange";
+
+  await exchange.addBankAccount("1", {
     accountName: "exchange",
     accountPassword: "x",
-    wireGatewayApiBaseUrl: new URL("/exchange/", bank.baseUrl).href,
-    accountPaytoUri: "payto://x-taler-bank/localhost/exchange",
+    wireGatewayApiBaseUrl: new URL("accounts/exchange/taler-wire-gateway/", bank.baseUrl).href,
+    accountPaytoUri: paytoUri,
   });
 
+  bank.setSuggestedExchange(exchange, paytoUri);
+
   await bank.start();
 
   await bank.pingUntilAvailable();
 
+  const bankClient = new TalerCorebankApiClient(bank.corebankApiBaseUrl, {
+    auth: {
+      username: "admin",
+      password: "adminpw",
+    }
+  });
+
+  await bankClient.registerAccountExtended({
+    name: "Exchange",
+    password: "x",
+    username: "exchange",
+    is_taler_exchange: true,
+    payto_uri: paytoUri,
+  });
+
   const coinConfig: CoinConfig[] = defaultCoinConfig.map((x) => x("TESTKUDOS"));
   exchange.addCoinConfigList(coinConfig);
 
