@@ -47,6 +47,14 @@ import { runExchangePurseTest } from "./test-exchange-purse.js";
 import { runExchangeTimetravelTest } from "./test-exchange-timetravel.js";
 import { runFeeRegressionTest } from "./test-fee-regression.js";
 import { runForcedSelectionTest } from "./test-forced-selection.js";
+import { runKycBalanceWithdrawalTest } from "./test-kyc-balance-withdrawal.js";
+import { runKycDepositAggregateTest } from "./test-kyc-deposit-aggregate.js";
+import { runKycExchangeWalletTest } from "./test-kyc-exchange-wallet.js";
+import { runKycFormWithdrawalTest } from "./test-kyc-form-withdrawal.js";
+import { runKycNewMeasureTest } from "./test-kyc-new-measure.js";
+import { runKycPeerPullTest } from "./test-kyc-peer-pull.js";
+import { runKycPeerPushTest } from "./test-kyc-peer-push.js";
+import { runKycThresholdWithdrawalTest } from "./test-kyc-threshold-withdrawal.js";
 import { runKycTest } from "./test-kyc.js";
 import { runLibeufinBankTest } from "./test-libeufin-bank.js";
 import { runMerchantCategoriesTest } from "./test-merchant-categories.js";
@@ -244,6 +252,14 @@ const allTests: TestMainFunction[] = [
   runMerchantCategoriesTest,
   runWithdrawalExternalTest,
   runWithdrawalIdempotentTest,
+  runKycThresholdWithdrawalTest,
+  runKycExchangeWalletTest,
+  runKycPeerPushTest,
+  runKycPeerPullTest,
+  runKycDepositAggregateTest,
+  runKycFormWithdrawalTest,
+  runKycBalanceWithdrawalTest,
+  runKycNewMeasureTest,
 ];
 
 export interface TestRunSpec {
