@@ -38,7 +38,7 @@ import {
   EddsaSignatureString,
   EddsaPublicKeyString,
   CoinPublicKeyString,
-} from "./taler-types.js";
+} from "./types-taler-common.js";
 import {
   AbsoluteTime,
   codecForTimestamp,
