@@ -22,6 +22,7 @@
  * Imports.
  */
 import {
+  AccountData,
   AmountString,
   base64FromArrayBuffer,
   buildCodecForObject,
@@ -36,6 +37,7 @@ import {
   opEmptySuccess,
   opKnownHttpFailure,
   opUnknownFailure,
+  RegisterAccountRequest,
   stringToBytes,
   TalerError,
   TalerErrorCode,
@@ -46,7 +48,6 @@ import {
   expectSuccessResponseOrThrow,
   HttpRequestLibrary,
   readSuccessResponseJsonOrThrow,
-  readSuccessResponseTextOrThrow,
   readTalerErrorResponse,
 } from "@gnu-taler/taler-util/http";
 
@@ -152,83 +153,11 @@ export class WireGatewayApiClient {
   }
 }
 
-export interface ChallengeContactData {
-  // E-Mail address
-  email?: string;
-
-  // Phone number.
-  phone?: string;
-}
-
 export interface AccountBalance {
   amount: AmountString;
   credit_debit_indicator: "credit" | "debit";
 }
 
-export interface RegisterAccountRequest {
-  // Username
-  username: string;
-
-  // Password.
-  password: string;
-
-  // Legal name of the account owner
-  name: string;
-
-  // Defaults to false.
-  is_public?: boolean;
-
-  // Is this a taler exchange account?
-  // If true:
-  // - incoming transactions to the account that do not
-  //   have a valid reserve public key are automatically
-  // - the account provides the taler-wire-gateway-api endpoints
-  // Defaults to false.
-  is_taler_exchange?: boolean;
-
-  // Addresses where to send the TAN for transactions.
-  // Currently only used for cashouts.
-  // If missing, cashouts will fail.
-  // In the future, might be used for other transactions
-  // as well.
-  challenge_contact_data?: ChallengeContactData;
-
-  // 'payto' address pointing a bank account
-  // external to the libeufin-bank.
-  // Payments will be sent to this bank account
-  // when the user wants to convert the local currency
-  // back to fiat currency outside libeufin-bank.
-  cashout_payto_uri?: string;
-
-  // Internal payto URI of this bank account.
-  // Used mostly for testing.
-  payto_uri?: string;
-}
-
-export interface AccountData {
-  // Legal name of the account owner.
-  name: string;
-
-  // Available balance on the account.
-  balance: AccountBalance;
-
-  // payto://-URI of the account.
-  payto_uri: string;
-
-  // Number indicating the max debit allowed for the requesting user.
-  debit_threshold: AmountString;
-
-  contact_data?: ChallengeContactData;
-
-  // 'payto' address pointing the bank account
-  // where to send cashouts.  This field is optional
-  // because not all the accounts are required to participate
-  // in the merchants' circuit.  One example is the exchange:
-  // that never cashouts.  Registering these accounts can
-  // be done via the access API.
-  cashout_payto_uri?: string;
-}
-
 export interface ConfirmWithdrawalArgs {
   withdrawalOperationId: string;
 }
