@@ -32,6 +32,8 @@ import {
   codecForAccountKycRedirects,
   codecForAccountsSummaryResponse,
   codecForBankAccountDetail,
+  codecForCategoryListResponse,
+  codecForCategoryProductList,
   codecForClaimResponse,
   codecForInstancesResponse,
   codecForInventorySummaryResponse,
@@ -99,6 +101,9 @@ export enum TalerMerchantInstanceCacheEviction {
   CREATE_PRODUCT,
   UPDATE_PRODUCT,
   DELETE_PRODUCT,
+  CREATE_CATEGORY,
+  UPDATE_CATEGORY,
+  DELETE_CATEGORY,
   CREATE_TRANSFER,
   DELETE_TRANSFER,
   CREATE_DEVICE,
@@ -132,7 +137,7 @@ export enum TalerMerchantManagementCacheEviction {
  * Uses libtool's current:revision:age versioning.
  */
 export class TalerMerchantInstanceHttpClient {
-  public readonly PROTOCOL_VERSION = "15:0:0";
+  public readonly PROTOCOL_VERSION = "16:0:0";
 
   readonly httpLib: HttpRequestLibrary;
   readonly cacheEvictor: CacheEvictor<TalerMerchantInstanceCacheEviction>;
@@ -506,7 +511,7 @@ export class TalerMerchantInstanceHttpClient {
    * https://docs.taler.net/core/api-merchant.html#get-[-instances-$INSTANCE]-private
    *
    */
-  async getCurrentInstanceDetails(token: AccessToken) {
+  async getCurrentInstanceDetails(token: AccessToken | undefined) {
     const url = new URL(`private`, this.baseUrl);
 
     const headers: Record<string, string> = {};
@@ -701,7 +706,10 @@ export class TalerMerchantInstanceHttpClient {
   /**
    * https://docs.taler.net/core/api-merchant.html#get-[-instances-$INSTANCE]-private-accounts
    */
-  async listBankAccounts(token: AccessToken, params?: PaginationParams) {
+  async listBankAccounts(
+    token: AccessToken | undefined,
+    params?: PaginationParams,
+  ) {
     const url = new URL(`private/accounts`, this.baseUrl);
 
     // addMerchantPaginationParams(url, params);
@@ -792,6 +800,173 @@ export class TalerMerchantInstanceHttpClient {
   // Inventory Management
   //
 
+  /**
+   * https://docs.taler.net/core/api-merchant.html#get-[-instances-$INSTANCE]-private-categories
+   */
+  async listCategories(
+    token: AccessToken | undefined,
+    params?: PaginationParams,
+  ) {
+    const url = new URL(`private/categories`, this.baseUrl);
+
+    // addMerchantPaginationParams(url, params);
+
+    const headers: Record<string, string> = {};
+    if (token) {
+      headers.Authorization = makeBearerTokenAuthHeader(token);
+    }
+    const resp = await this.httpLib.fetch(url.href, {
+      method: "GET",
+      headers,
+    });
+
+    switch (resp.status) {
+      case HttpStatusCode.Ok:
+        return opSuccessFromHttp(resp, codecForCategoryListResponse());
+      case HttpStatusCode.NotFound:
+        return opKnownHttpFailure(resp.status, resp);
+      case HttpStatusCode.Unauthorized: // FIXME: missing in docs
+        return opKnownHttpFailure(resp.status, resp);
+      default:
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
+    }
+  }
+
+  /**
+   * https://docs.taler.net/core/api-merchant.html#get-[-instances-$INSTANCE]-private-categories-$CATEGORY_ID
+   */
+  async getCategoryDetails(token: AccessToken | undefined, cId: string) {
+    const url = new URL(`private/categories/${cId}`, this.baseUrl);
+
+    const headers: Record<string, string> = {};
+    if (token) {
+      headers.Authorization = makeBearerTokenAuthHeader(token);
+    }
+    const resp = await this.httpLib.fetch(url.href, {
+      method: "GET",
+      headers,
+    });
+
+    switch (resp.status) {
+      case HttpStatusCode.Ok:
+        return opSuccessFromHttp(resp, codecForCategoryProductList());
+      case HttpStatusCode.Unauthorized: // FIXME: missing in docs
+        return opKnownHttpFailure(resp.status, resp);
+      case HttpStatusCode.NotFound:
+        return opKnownHttpFailure(resp.status, resp);
+      default:
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
+    }
+  }
+
+  /**
+   * https://docs.taler.net/core/api-merchant.html#post-[-instances-$INSTANCE]-private-categories
+   */
+  async addCategory(
+    token: AccessToken | undefined,
+    body: TalerMerchantApi.CategoryCreateRequest,
+  ) {
+    const url = new URL(`private/categories`, this.baseUrl);
+
+    const headers: Record<string, string> = {};
+    if (token) {
+      headers.Authorization = makeBearerTokenAuthHeader(token);
+    }
+    const resp = await this.httpLib.fetch(url.href, {
+      method: "POST",
+      body,
+      headers,
+    });
+
+    switch (resp.status) {
+      case HttpStatusCode.Ok: {
+        this.cacheEvictor.notifySuccess(
+          TalerMerchantInstanceCacheEviction.CREATE_CATEGORY,
+        );
+        return opEmptySuccess(resp);
+      }
+      case HttpStatusCode.Unauthorized: // FIXME: missing in docs
+        return opKnownHttpFailure(resp.status, resp);
+      case HttpStatusCode.NotFound: // FIXME: missing in docs
+        return opKnownHttpFailure(resp.status, resp);
+      // case HttpStatusCode.Conflict:
+      //   return opKnownHttpFailure(resp.status, resp);
+      default:
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
+    }
+  }
+
+  /**
+   * https://docs.taler.net/core/api-merchant.html#post-[-instances-$INSTANCE]-private-categories
+   */
+  async updateCategory(
+    token: AccessToken | undefined,
+    cid: string,
+    body: TalerMerchantApi.CategoryCreateRequest,
+  ) {
+    const url = new URL(`private/categories/${cid}`, this.baseUrl);
+
+    const headers: Record<string, string> = {};
+    if (token) {
+      headers.Authorization = makeBearerTokenAuthHeader(token);
+    }
+    const resp = await this.httpLib.fetch(url.href, {
+      method: "PATCH",
+      body,
+      headers,
+    });
+
+    switch (resp.status) {
+      case HttpStatusCode.NoContent: {
+        this.cacheEvictor.notifySuccess(
+          TalerMerchantInstanceCacheEviction.UPDATE_CATEGORY,
+        );
+        return opEmptySuccess(resp);
+      }
+      case HttpStatusCode.Unauthorized: // FIXME: missing in docs
+        return opKnownHttpFailure(resp.status, resp);
+      case HttpStatusCode.NotFound: // FIXME: missing in docs
+        return opKnownHttpFailure(resp.status, resp);
+      // case HttpStatusCode.Conflict:
+      //   return opKnownHttpFailure(resp.status, resp);
+      default:
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
+    }
+  }
+
+  /**
+   * https://docs.taler.net/core/api-merchant.html#delete-[-instances-$INSTANCE]-private-categories-$CATEGORY_ID
+   */
+  async deleteCategory(token: AccessToken | undefined, cId: string) {
+    const url = new URL(`private/categories/${cId}`, this.baseUrl);
+
+    const headers: Record<string, string> = {};
+    if (token) {
+      headers.Authorization = makeBearerTokenAuthHeader(token);
+    }
+    const resp = await this.httpLib.fetch(url.href, {
+      method: "DELETE",
+      headers,
+    });
+
+    switch (resp.status) {
+      case HttpStatusCode.NoContent: {
+        this.cacheEvictor.notifySuccess(
+          TalerMerchantInstanceCacheEviction.DELETE_CATEGORY,
+        );
+        return opEmptySuccess(resp);
+      }
+      case HttpStatusCode.Unauthorized: // FIXME: missing in docs
+        return opKnownHttpFailure(resp.status, resp);
+      case HttpStatusCode.NotFound:
+        return opKnownHttpFailure(resp.status, resp);
+      // case HttpStatusCode.Conflict:
+      //   return opKnownHttpFailure(resp.status, resp);
+      default:
+        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
+    }
+  }
+
   /**
    * https://docs.taler.net/core/api-merchant.html#post-[-instances-$INSTANCE]-private-products
    */
@@ -952,7 +1127,7 @@ export class TalerMerchantInstanceHttpClient {
   }
 
   /**
-   * https://docs.taler.net/core/api-merchant.html#reserving-inventory
+   * https://docs.taler.net/core/api-merchant.html#post-[-instances-$INSTANCE]-private-products-$PRODUCT_ID-lock
    */
   async lockProduct(
     token: AccessToken | undefined,
@@ -990,7 +1165,7 @@ export class TalerMerchantInstanceHttpClient {
   }
 
   /**
-   * https://docs.taler.net/core/api-merchant.html#removing-products-from-inventory
+   * https://docs.taler.net/core/api-merchant.html#delete-[-instances-$INSTANCE]-private-products-$PRODUCT_ID
    */
   async deleteProduct(token: AccessToken | undefined, productId: string) {
     const url = new URL(`private/products/${productId}`, this.baseUrl);
