@@ -17,8 +17,11 @@
 import {
   EncryptionNonce,
   LockedAccount,
+  LockedReserve,
   OfficerAccount,
   OfficerId,
+  ReserveAccount,
+  ReserveId,
   SigningKey,
   createEddsaKeyPair,
   decodeCrock,
@@ -96,6 +99,71 @@ export async function createNewOfficerAccount(
   return { id: accountId, signingKey, safe };
 }
 
+/**
+ * Restore previous session and unlock account with password
+ *
+ * @param salt string from which crypto params will be derived
+ * @param key secured private key
+ * @param password password for the private key
+ * @returns
+ */
+export async function unlockWalletKycAccount(
+  account: LockedReserve,
+  password: string,
+): Promise<ReserveAccount> {
+  const rawKey = decodeCrock(account);
+  const rawPassword = stringToBytes(password);
+
+  const signingKey = (await decryptWithDerivedKey(
+    rawKey,
+    rawPassword,
+    password,
+  ).catch((e) => {
+    throw new UnwrapKeyError(e instanceof Error ? e.message : String(e));
+  })) as SigningKey;
+
+  const publicKey = eddsaGetPublic(signingKey);
+
+  const accountId = encodeCrock(publicKey) as ReserveId;
+
+  return { id: accountId, signingKey };
+}
+
+/**
+ * Create new account (secured private key)
+ * secured with the given password
+ *
+ * @param extraNonce
+ * @param password
+ * @returns
+ */
+export async function createNewWalletKycAccount(
+  extraNonce: EncryptionNonce,
+  password: string,
+): Promise<OfficerAccount & { safe: LockedAccount }> {
+  const { eddsaPriv, eddsaPub } = createEddsaKeyPair();
+
+  const key = stringToBytes(password);
+
+  const localRnd = getRandomBytesF(24);
+  const mergedRnd: EncryptionNonce = extraNonce
+    ? kdf(24, stringToBytes("aml-officer"), extraNonce, localRnd)
+    : localRnd;
+
+  const protectedPrivKey = await encryptWithDerivedKey(
+    mergedRnd,
+    key,
+    eddsaPriv,
+    password,
+  );
+
+  const signingKey = eddsaPriv as SigningKey;
+  const accountId = encodeCrock(eddsaPub) as OfficerId;
+  const safe = encodeCrock(protectedPrivKey) as LockedAccount;
+
+  return { id: accountId, signingKey, safe };
+}
+
 export class UnwrapKeyError extends Error {
   public cause: string;
   constructor(cause: string) {
