@@ -19,7 +19,11 @@
  */
 import { base64FromArrayBuffer } from "../base64.js";
 import { encodeCrock, getRandomBytes, stringToBytes } from "../taler-crypto.js";
-import { AccessToken, LongPollParams, PaginationParams } from "./types.js";
+import {
+  AccessToken,
+  LongPollParams,
+  PaginationParams,
+} from "../types-taler-common.js";
 
 /**
  * Helper function to generate the "Authorization" HTTP header.
@@ -39,7 +43,7 @@ export function makeBasicAuthHeader(
  * @returns
  */
 export function makeBearerTokenAuthHeader(token: AccessToken): string {
-  return `Bearer secret-token:${token}`;
+  return `Bearer ${token}`;
 }
 
 /**
@@ -99,17 +103,17 @@ export class IdempotencyRetry {
   private constructor(timesLeft: number, maxTimesLeft: number) {
     this.timesLeft = timesLeft;
     this.maxTries = maxTimesLeft;
-    this.uid = encodeCrock(getRandomBytes(32))
+    this.uid = encodeCrock(getRandomBytes(32));
   }
-  
+
   static tryFiveTimes() {
-    return new IdempotencyRetry(5, 5)
+    return new IdempotencyRetry(5, 5);
   }
 
   next(): IdempotencyRetry | undefined {
-    const left = this.timesLeft -1
+    const left = this.timesLeft - 1;
     if (left <= 0) {
-      return undefined
+      return undefined;
     }
     return new IdempotencyRetry(left, this.maxTries);
   }
