@@ -274,10 +274,10 @@ export async function readSuccessResponseJsonOrErrorCode<T>(
   };
 }
 
-export async function readResponseJsonOrErrorCode<T>(
+export async function readResponseJsonOrThrow<T>(
   httpResponse: HttpResponse,
   codec: Codec<T>,
-): Promise<{ isError: boolean; response: T }> {
+): Promise<T> {
   let respJson;
   try {
     respJson = await httpResponse.json();
@@ -310,13 +310,9 @@ export async function readResponseJsonOrErrorCode<T>(
       "Response invalid",
     );
   }
-  return {
-    isError: !(httpResponse.status >= 200 && httpResponse.status < 300),
-    response: parsedResponse,
-  };
+  return parsedResponse;
 }
 
-
 type HttpErrorDetails = {
   requestUrl: string;
   requestMethod: string;
@@ -506,6 +502,10 @@ export function encodeBody(body: unknown): ArrayBuffer {
     return body.buffer;
   } else if (body instanceof ArrayBuffer) {
     return body;
+  } else if (body instanceof URLSearchParams) {
+    return body as unknown as ArrayBuffer;
+  } else if (typeof body === "object" && body.constructor.name === "FormData") {
+    return body as ArrayBuffer;
   } else if (typeof body === "object") {
     return textEncoder.encode(JSON.stringify(body)).buffer;
   }
