@@ -167,7 +167,7 @@ export class HttpLibImpl implements HttpRequestLibrary {
             requestMethod: method,
             httpStatusCode: 0,
           },
-          `Request timed out`,
+          `Request timed out (timeout ${opt?.timeout?.d_ms ?? "none"})`,
         );
       }
       throw e;
