@@ -31,11 +31,12 @@ import { Logger } from "./logging.js";
 import * as nacl from "./nacl-fast.js";
 import { secretbox } from "./nacl-fast.js";
 import { TalerProtocolDuration, TalerProtocolTimestamp } from "./time.js";
+import { CoinPublicKeyString, HashCodeString } from "./types-taler-common.js";
 import {
-  CoinPublicKeyString,
-  HashCodeString,
-} from "./types-taler-common.js";
-import { CoinEnvelope, DenomKeyType, DenominationPubKey } from "./types-taler-exchange.js";
+  CoinEnvelope,
+  DenomKeyType,
+  DenominationPubKey,
+} from "./types-taler-exchange.js";
 
 export type Flavor<T, FlavorT extends string> = T & {
   _flavor?: `taler.${FlavorT}`;
@@ -985,6 +986,7 @@ export enum TalerSignaturePurpose {
   MERCHANT_REFUND = 1102,
   WALLET_COIN_RECOUP = 1203,
   WALLET_COIN_LINK = 1204,
+  WALLET_ACCOUNT_SETUP = 1205,
   WALLET_COIN_RECOUP_REFRESH = 1206,
   WALLET_AGE_ATTESTATION = 1207,
   WALLET_PURSE_CREATE = 1210,
@@ -996,9 +998,10 @@ export enum TalerSignaturePurpose {
   WALLET_COIN_HISTORY = 1209,
   EXCHANGE_CONFIRM_RECOUP = 1039,
   EXCHANGE_CONFIRM_RECOUP_REFRESH = 1041,
-  TALER_SIGNATURE_AML_DECISION = 1350,
-  TALER_SIGNATURE_AML_QUERY = 1351,
-  TALER_SIGNATURE_MASTER_AML_KEY = 1017,
+  AML_DECISION = 1350,
+  AML_QUERY = 1351,
+  MASTER_AML_KEY = 1017,
+  KYC_AUTH = 1360,
   ANASTASIS_POLICY_UPLOAD = 1400,
   ANASTASIS_POLICY_DOWNLOAD = 1401,
   SYNC_BACKUP_UPLOAD = 1450,
