@@ -15,7 +15,7 @@
  */
 
 import test from "ava";
-import { AmountString } from "./taler-types.js";
+import { AmountString } from "./types-taler-common.js";
 import {
   parseAddExchangeUri,
   parseDevExperimentUri,
@@ -54,6 +54,18 @@ test("taler withdraw uri parsing", (t) => {
   t.is(r1.bankIntegrationApiBaseUrl, "https://bank.example.com/");
 });
 
+test("taler withdraw uri parsing with external confirmation", (t) => {
+  const url1 = "taler://withdraw/bank.example.com/12345?external-confirmation=1";
+  const r1 = parseWithdrawUri(url1);
+  if (!r1) {
+    t.fail();
+    return;
+  }
+  t.is(r1.externalConfirmation, true);
+  t.is(r1.withdrawalOperationId, "12345");
+  t.is(r1.bankIntegrationApiBaseUrl, "https://bank.example.com/");
+});
+
 test("taler withdraw uri parsing (http)", (t) => {
   const url1 = "taler+http://withdraw/bank.example.com/12345";
   const r1 = parseWithdrawUri(url1);
