@@ -27,8 +27,9 @@ import { Codec, Context, DecodingError, renderContext } from "./codec.js";
 import { canonicalizeBaseUrl } from "./helpers.js";
 import { opFixedSuccess, opKnownTalerFailure } from "./operation.js";
 import { TalerErrorCode } from "./taler-error-codes.js";
-import { AmountString } from "./taler-types.js";
+import { AmountString } from "./types-taler-common.js";
 import { URL, URLSearchParams } from "./url.js";
+
 /**
  * A parsed taler URI.
  */
@@ -83,13 +84,13 @@ export interface PayTemplateUriResult {
   type: TalerUriAction.PayTemplate;
   merchantBaseUrl: string;
   templateId: string;
-  templateParams: TemplateParams;
 }
 
 export interface WithdrawUriResult {
   type: TalerUriAction.Withdraw;
   bankIntegrationApiBaseUrl: string;
   withdrawalOperationId: string;
+  externalConfirmation?: boolean;
 }
 
 export interface RefundUriResult {
@@ -124,7 +125,6 @@ export interface BackupRestoreUri {
 export interface WithdrawExchangeUri {
   type: TalerUriAction.WithdrawExchange;
   exchangeBaseUrl: string;
-  exchangePub?: string;
   amount?: AmountString;
 }
 
@@ -142,7 +142,12 @@ export function parseWithdrawUriWithError(s: string) {
   if (pi.type === "fail") {
     return pi;
   }
-  const parts = pi.body.rest.split("/");
+
+  const c = pi.body.rest.split("?", 2);
+  const path = c[0];
+  const q = new URLSearchParams(c[1] ?? "");
+
+  const parts = path.split("/");
 
   if (parts.length < 2) {
     return opKnownTalerFailure(TalerErrorCode.WALLET_TALER_URI_MALFORMED, {
@@ -168,6 +173,7 @@ export function parseWithdrawUriWithError(s: string) {
       `${pi.body.innerProto}://${p}/`,
     ),
     withdrawalOperationId: withdrawId,
+    externalConfirmation: q.get("external-confirmation") == "1",
   };
   return opFixedSuccess(result);
 }
@@ -212,9 +218,7 @@ export function parseAddExchangeUriWithError(s: string) {
 
   const result: AddExchangeUri = {
     type: TalerUriAction.AddExchange,
-    exchangeBaseUrl: canonicalizeBaseUrl(
-      `${pi.body.innerProto}://${p}/`,
-    ),
+    exchangeBaseUrl: canonicalizeBaseUrl(`${pi.body.innerProto}://${p}/`),
   };
   return opFixedSuccess(result);
 }
@@ -440,7 +444,6 @@ export function parsePayTemplateUri(
     type: TalerUriAction.PayTemplate,
     merchantBaseUrl,
     templateId,
-    templateParams: params,
   };
 }
 
@@ -507,7 +510,14 @@ export function parseWithdrawExchangeUri(
     return undefined;
   }
   const host = parts[0].toLowerCase();
-  const exchangePub = parts.length > 1 ? parts[parts.length - 1] : undefined;
+  // Used to be the reserve public key, now it's empty!
+  const lastPathComponent =
+    parts.length > 1 ? parts[parts.length - 1] : undefined;
+
+  if (lastPathComponent) {
+    // invalid taler://withdraw-exchange URI, must end with a slash
+    return undefined;
+  }
   const pathSegments = parts.slice(1, parts.length - 1);
   const hostAndSegments = [host, ...pathSegments].join("/");
   const exchangeBaseUrl = canonicalizeBaseUrl(
@@ -519,7 +529,6 @@ export function parseWithdrawExchangeUri(
   return {
     type: TalerUriAction.WithdrawExchange,
     exchangeBaseUrl,
-    exchangePub: exchangePub != "" ? exchangePub : undefined,
     amount,
   };
 }
@@ -641,13 +650,12 @@ export function stringifyRestoreUri({
 
 export function stringifyWithdrawExchange({
   exchangeBaseUrl,
-  exchangePub,
   amount,
 }: Omit<WithdrawExchangeUri, "type">): string {
   const { proto, path, query } = getUrlInfo(exchangeBaseUrl, {
     a: amount,
   });
-  return `${proto}://withdraw-exchange/${path}${exchangePub ?? ""}${query}`;
+  return `${proto}://withdraw-exchange/${path}${query}`;
 }
 
 export function stringifyAddExchange({
@@ -666,9 +674,8 @@ export function stringifyDevExperimentUri({
 export function stringifyPayTemplateUri({
   merchantBaseUrl,
   templateId,
-  templateParams,
 }: Omit<PayTemplateUriResult, "type">): string {
-  const { proto, path, query } = getUrlInfo(merchantBaseUrl, templateParams);
+  const { proto, path, query } = getUrlInfo(merchantBaseUrl);
   return `${proto}://pay-template/${path}${templateId}${query}`;
 }
 
