@@ -37,7 +37,11 @@ import {
   codecForString,
   codecOptional,
 } from "./codec.js";
-import { TalerPreciseTimestamp, TalerProtocolTimestamp } from "./time.js";
+import {
+  TalerPreciseTimestamp,
+  TalerProtocolDuration,
+  TalerProtocolTimestamp,
+} from "./time.js";
 import {
   AmountString,
   InternationalizedString,
@@ -123,8 +127,9 @@ export enum TransactionMinorState {
   Unknown = "unknown",
   Deposit = "deposit",
   KycRequired = "kyc",
-  AmlRequired = "aml",
   MergeKycRequired = "merge-kyc",
+  BalanceKycRequired = "balance-kyc",
+  BalanceKycInit = "balance-kyc-init",
   Track = "track",
   SubmitPayment = "submit-payment",
   RebindSession = "rebind-session",
@@ -186,6 +191,11 @@ export interface TransactionCommon {
   // main timestamp of the transaction
   timestamp: TalerPreciseTimestamp;
 
+  /**
+   * Scope of this tx
+   */
+  scopes: ScopeInfo[];
+
   /**
    * Transaction state, as per DD37.
    */
@@ -213,6 +223,16 @@ export interface TransactionCommon {
    * have the location where the user need to go to complete KYC information.
    */
   kycUrl?: string;
+
+  /**
+   * KYC payto hash. Useful for testing, not so useful for UIs.
+   */
+  kycPaytoHash?: string;
+
+  /**
+   * KYC access token. Useful for testing, not so useful for UIs.
+   */
+  kycAccessToken?: string;
 }
 
 export type Transaction =
@@ -274,6 +294,12 @@ interface WithdrawalDetailsForManualTransfer {
    * Is the reserve ready for withdrawal?
    */
   reserveIsReady: boolean;
+
+  /**
+   * How long does the exchange wait to transfer back funds from a
+   * reserve?
+   */
+  reserveClosingDelay: TalerProtocolDuration;
 }
 
 interface WithdrawalDetailsForTalerBankIntegrationApi {
@@ -740,16 +766,6 @@ export const codecForTransactionByIdRequest =
       .property("transactionId", codecForString())
       .build("TransactionByIdRequest");
 
-export interface WithdrawalTransactionByURIRequest {
-  talerWithdrawUri: string;
-}
-
-export const codecForWithdrawalTransactionByURIRequest =
-  (): Codec<WithdrawalTransactionByURIRequest> =>
-    buildCodecForObject<WithdrawalTransactionByURIRequest>()
-      .property("talerWithdrawUri", codecForString())
-      .build("WithdrawalTransactionByURIRequest");
-
 export const codecForTransactionsRequest = (): Codec<TransactionsRequest> =>
   buildCodecForObject<TransactionsRequest>()
     .property("currency", codecOptional(codecForString()))
