@@ -21,6 +21,7 @@ import {
   AbsoluteTime,
   addPaytoQueryParams,
   AgeRestriction,
+  Amounts,
   AmountString,
   codecForList,
   codecForString,
@@ -722,6 +723,16 @@ walletCli
             },
           );
           console.log("withdrawInfo", withdrawInfo);
+          let amount: AmountString | undefined = undefined;
+          if (withdrawInfo.editableAmount) {
+            if (withdrawInfo.amount) {
+              console.log(`Default amount: ${withdrawInfo.amount}`);
+            }
+            const res = await readlinePrompt(
+              `Amount (in ${withdrawInfo.currency}): `,
+            );
+            amount = Amounts.stringify(Amounts.parseOrThrow(res));
+          }
           const selectedExchange =
             args.handleUri.withdrawalExchange ??
             withdrawInfo.defaultExchangeBaseUrl;
@@ -732,6 +743,10 @@ walletCli
             processExit(1);
             return;
           }
+          // FIXME: Maybe prompt for this?
+          await wallet.client.call(WalletApiOperation.SetExchangeTosAccepted, {
+            exchangeBaseUrl: selectedExchange,
+          });
           const res = await wallet.client.call(
             WalletApiOperation.AcceptBankIntegratedWithdrawal,
             {
@@ -1365,13 +1380,8 @@ advancedCli
 advancedCli
   .subcommand("pending", "pending", { help: "Show pending operations." })
   .action(async (args) => {
-    await withWallet(args, { lazyTaskLoop: true }, async (wallet) => {
-      const pending = await wallet.client.call(
-        WalletApiOperation.GetPendingOperations,
-        {},
-      );
-      console.log(JSON.stringify(pending, undefined, 2));
-    });
+    console.error("Subcommand removed due to deprecation.");
+    process.exit(1);
   });
 
 advancedCli
@@ -1695,10 +1705,25 @@ advancedCli
     await withWallet(args, { lazyTaskLoop: true }, async (wallet) => {
       const coins = await wallet.client.call(WalletApiOperation.DumpCoins, {});
       for (const coin of coins.coins) {
-        console.log(`coin ${coin.coin_pub}`);
-        console.log(` exchange ${coin.exchange_base_url}`);
-        console.log(` denomPubHash ${coin.denom_pub_hash}`);
-        console.log(` status ${coin.coin_status}`);
+        console.log(`coin ${coin.coinPub}`);
+        console.log(` exchange ${coin.exchangeBaseUrl}`);
+        console.log(` denomPubHash ${coin.denomPubHash}`);
+        console.log(` status ${coin.coinStatus}`);
+        if (coin.history.length > 0) {
+          console.log(` history`);
+          for (const hi of coin.history) {
+            switch (hi.type) {
+              case "spend":
+                console.log(`   spend ${hi.transactionId} ${hi.amount}`);
+                break;
+              case "refresh":
+                console.log(`   refresh ${hi.transactionId} ${hi.amount}`);
+                break;
+              default:
+                console.log(`   unknown (${hi.type})`);
+            }
+          }
+        }
       }
     });
   });
