@@ -94,7 +94,6 @@ interface WalletBalance {
   pendingIncoming: AmountJson;
   pendingOutgoing: AmountJson;
   flagIncomingKyc: boolean;
-  flagIncomingAml: boolean;
   flagIncomingConfirmation: boolean;
   flagOutgoingKyc: boolean;
 }
@@ -170,7 +169,6 @@ class BalancesStore {
         available: zero,
         pendingIncoming: zero,
         pendingOutgoing: zero,
-        flagIncomingAml: false,
         flagIncomingConfirmation: false,
         flagIncomingKyc: false,
         flagOutgoingKyc: false,
@@ -210,14 +208,6 @@ class BalancesStore {
     b.pendingOutgoing = Amounts.add(b.pendingOutgoing, amount).amount;
   }
 
-  async setFlagIncomingAml(
-    currency: string,
-    exchangeBaseUrl: string,
-  ): Promise<void> {
-    const b = await this.initBalance(currency, exchangeBaseUrl);
-    b.flagIncomingAml = true;
-  }
-
   async setFlagIncomingKyc(
     currency: string,
     exchangeBaseUrl: string,
@@ -254,9 +244,6 @@ class BalancesStore {
       .forEach((c) => {
         const v = balanceStore[c];
         const flags: BalanceFlag[] = [];
-        if (v.flagIncomingAml) {
-          flags.push(BalanceFlag.IncomingAml);
-        }
         if (v.flagIncomingKyc) {
           flags.push(BalanceFlag.IncomingKyc);
         }
@@ -373,7 +360,11 @@ export async function getBalancesInsideTransaction(
         case WithdrawalGroupStatus.SuspendedQueryingStatus:
           // Pending, but no special flag.
           break;
+        case WithdrawalGroupStatus.PendingBalanceKyc:
+        case WithdrawalGroupStatus.SuspendedBalanceKyc:
         case WithdrawalGroupStatus.SuspendedKyc:
+        case WithdrawalGroupStatus.PendingBalanceKycInit:
+        case WithdrawalGroupStatus.SuspendedBalanceKycInit:
         case WithdrawalGroupStatus.PendingKyc: {
           checkDbInvariant(
             wg.denomsSel !== undefined,
@@ -387,20 +378,6 @@ export async function getBalancesInsideTransaction(
           await balanceStore.setFlagIncomingKyc(currency, wg.exchangeBaseUrl);
           break;
         }
-        case WithdrawalGroupStatus.PendingAml:
-        case WithdrawalGroupStatus.SuspendedAml: {
-          checkDbInvariant(
-            wg.denomsSel !== undefined,
-            "wg in aml state should have been initialized",
-          );
-          checkDbInvariant(
-            wg.exchangeBaseUrl !== undefined,
-            "wg in kyc state should have been initialized",
-          );
-          const currency = Amounts.currencyOf(wg.denomsSel.totalCoinValue);
-          await balanceStore.setFlagIncomingAml(currency, wg.exchangeBaseUrl);
-          break;
-        }
         case WithdrawalGroupStatus.PendingRegisteringBank: {
           if (wg.denomsSel && wg.exchangeBaseUrl) {
             const currency = Amounts.currencyOf(wg.denomsSel.totalCoinValue);
