@@ -167,6 +167,7 @@ export async function spendCoins(
       "refreshGroups",
       "refreshSessions",
       "denominations",
+      "transactionsMeta",
     ]
   >,
   csi: CoinsSpendInfo,
@@ -599,6 +600,7 @@ export function getAutoRefreshExecuteThreshold(d: {
 
 export enum PendingTaskType {
   ExchangeUpdate = "exchange-update",
+  ExchangeWalletKyc = "exchange-wallet-kyc",
   Purchase = "purchase",
   Refresh = "refresh",
   Recoup = "recoup",
@@ -621,6 +623,7 @@ export type ParsedTaskIdentifier =
       withdrawalGroupId: string;
     }
   | { tag: PendingTaskType.ExchangeUpdate; exchangeBaseUrl: string }
+  | { tag: PendingTaskType.ExchangeWalletKyc; exchangeBaseUrl: string }
   | { tag: PendingTaskType.Backup; backupProviderBaseUrl: string }
   | { tag: PendingTaskType.Deposit; depositGroupId: string }
   | { tag: PendingTaskType.PeerPullDebit; peerPullDebitId: string }
@@ -647,6 +650,8 @@ export function parseTaskIdentifier(x: string): ParsedTaskIdentifier {
       return { tag: type, depositGroupId: rest[0] };
     case PendingTaskType.ExchangeUpdate:
       return { tag: type, exchangeBaseUrl: decodeURIComponent(rest[0]) };
+    case PendingTaskType.ExchangeWalletKyc:
+      return { tag: type, exchangeBaseUrl: decodeURIComponent(rest[0]) };
     case PendingTaskType.PeerPullCredit:
       return { tag: type, pursePub: rest[0] };
     case PendingTaskType.PeerPullDebit:
@@ -678,6 +683,8 @@ export function constructTaskIdentifier(p: ParsedTaskIdentifier): TaskIdStr {
       return `${p.tag}:${p.depositGroupId}` as TaskIdStr;
     case PendingTaskType.ExchangeUpdate:
       return `${p.tag}:${encodeURIComponent(p.exchangeBaseUrl)}` as TaskIdStr;
+    case PendingTaskType.ExchangeWalletKyc:
+      return `${p.tag}:${encodeURIComponent(p.exchangeBaseUrl)}` as TaskIdStr;
     case PendingTaskType.PeerPullDebit:
       return `${p.tag}:${p.peerPullDebitId}` as TaskIdStr;
     case PendingTaskType.PeerPushCredit:
@@ -856,6 +863,58 @@ export async function genericWaitForState(
   }
 }
 
+/**
+ * Wait until the wallet is in a particular state.
+ *
+ * Two functions must be provided:
+ * 1. checkState, which checks if the wallet is in the
+ *    desired state.
+ * 2. filterNotification, which checks whether a notification
+ *    might have lead to a state change.
+ */
+export async function genericWaitForStateVal<T>(
+  wex: WalletExecutionContext,
+  args: {
+    checkState: () => Promise<T | undefined>;
+    filterNotification: (notif: WalletNotification) => boolean;
+  },
+): Promise<T> {
+  await wex.taskScheduler.ensureRunning();
+
+  // FIXME: Clean up using the new JS "using" / Symbol.dispose syntax.
+  const flag = new AsyncFlag();
+  // Raise purchaseNotifFlag whenever we get a notification
+  // about our refresh.
+  const cancelNotif = wex.ws.addNotificationListener((notif) => {
+    if (args.filterNotification(notif)) {
+      flag.raise();
+    }
+  });
+  const unregisterOnCancelled = wex.cancellationToken.onCancelled((reason) => {
+    cancelNotif();
+    flag.raise();
+  });
+
+  try {
+    while (true) {
+      if (wex.cancellationToken.isCancelled) {
+        throw Error("cancelled");
+      }
+      const val = await args.checkState();
+      if (val != null) {
+        return val;
+      }
+      // Wait for the next transition
+      await flag.wait();
+      flag.reset();
+    }
+  } catch (e) {
+    unregisterOnCancelled();
+    cancelNotif();
+    throw e;
+  }
+}
+
 export function requireExchangeTosAcceptedOrThrow(
   exchange: ReadyExchangeSummary,
 ): void {
