@@ -114,6 +114,10 @@ import {
   SignReservePurseCreateRequest,
   SignReservePurseCreateResponse,
   SignTrackTransactionRequest,
+  SignWalletAccountSetupRequest,
+  SignWalletAccountSetupResponse,
+  SignWalletKycAuthRequest,
+  SignWalletKycAuthResponse,
 } from "./cryptoTypes.js";
 
 const logger = new Logger("cryptoImplementation.ts");
@@ -240,6 +244,10 @@ export interface TalerCryptoInterface {
 
   signPurseMerge(req: SignPurseMergeRequest): Promise<SignPurseMergeResponse>;
 
+  signWalletAccountSetup(
+    req: SignWalletAccountSetupRequest,
+  ): Promise<SignWalletAccountSetupResponse>;
+
   signReservePurseCreate(
     req: SignReservePurseCreateRequest,
   ): Promise<SignReservePurseCreateResponse>;
@@ -253,6 +261,10 @@ export interface TalerCryptoInterface {
   signCoinHistoryRequest(
     req: SignCoinHistoryRequest,
   ): Promise<SignCoinHistoryResponse>;
+
+  signWalletKycAuth(
+    req: SignWalletKycAuthRequest,
+  ): Promise<SignWalletKycAuthResponse>;
 }
 
 /**
@@ -447,6 +459,16 @@ export const nullCrypto: TalerCryptoInterface = {
   ): Promise<SignReserveHistoryReqResponse> {
     throw new Error("Function not implemented.");
   },
+  signWalletAccountSetup: function (
+    req: SignWalletAccountSetupRequest,
+  ): Promise<SignWalletAccountSetupResponse> {
+    throw new Error("Function not implemented.");
+  },
+  signWalletKycAuth: function (
+    req: SignWalletKycAuthRequest,
+  ): Promise<SignWalletKycAuthResponse> {
+    throw new Error("Function not implemented.");
+  },
 };
 
 export type WithArg<X> = X extends (req: infer T) => infer R
@@ -1765,6 +1787,34 @@ export const nativeCryptoR: TalerCryptoInterfaceR = {
       sig: sigResp.sig,
     };
   },
+  async signWalletAccountSetup(
+    tci: TalerCryptoInterfaceR,
+    req: SignWalletAccountSetupRequest,
+  ): Promise<SignWalletAccountSetupResponse> {
+    const sigData = buildSigPS(TalerSignaturePurpose.WALLET_ACCOUNT_SETUP)
+      .put(amountToBuffer(req.threshold))
+      .build();
+    const sigResp = await tci.eddsaSign(tci, {
+      msg: encodeCrock(sigData),
+      priv: req.reservePriv,
+    });
+    return {
+      sig: sigResp.sig,
+    };
+  },
+  async signWalletKycAuth(
+    tci: TalerCryptoInterfaceR,
+    req: SignWalletKycAuthRequest,
+  ): Promise<SignWalletKycAuthResponse> {
+    const sigData = buildSigPS(TalerSignaturePurpose.KYC_AUTH).build();
+    const sigResp = await tci.eddsaSign(tci, {
+      msg: encodeCrock(sigData),
+      priv: req.accountPriv,
+    });
+    return {
+      sig: sigResp.sig,
+    };
+  },
 };
 
 export interface EddsaSignRequest {
