@@ -303,6 +303,27 @@ export enum WithdrawalGroupStatus {
   PendingReady = 0x0100_0004,
   SuspendedReady = 0x0110_0004,
 
+  /**
+   * Exchange wants KYC info from the user.
+   */
+  PendingKyc = 0x0100_0005,
+  SuspendedKyc = 0x0110_005,
+
+  /**
+   * Exchange wants KYC info from the user.
+   * KYC link is ready.
+   */
+  PendingBalanceKyc = 0x0100_0006,
+  SuspendedBalanceKyc = 0x0110_006,
+
+  /**
+   * Exchange wants KYC info from the user.
+   *
+   * KYC link is not ready yet, the KYC process is still initializing.
+   */
+  PendingBalanceKycInit = 0x0100_0007,
+  SuspendedBalanceKycInit = 0x0110_007,
+
   /**
    * Proposed to the user, has can choose to accept/refuse.
    */
@@ -315,18 +336,6 @@ export enum WithdrawalGroupStatus {
   AbortingBank = 0x0103_0001,
   SuspendedAbortingBank = 0x0113_0001,
 
-  /**
-   * Exchange wants KYC info from the user.
-   */
-  PendingKyc = 0x0100_0005,
-  SuspendedKyc = 0x0110_005,
-
-  /**
-   * Exchange is doing AML checks.
-   */
-  PendingAml = 0x0100_0006,
-  SuspendedAml = 0x0110_0006,
-
   /**
    * The corresponding withdraw record has been created.
    * No further processing is done, unless explicitly requested
@@ -613,6 +622,8 @@ export interface ExchangeDetailsRecord {
    * Age restrictions supported by the exchange (bitmask).
    */
   ageMask?: number;
+
+  walletBalanceLimits?: AmountString[];
 }
 
 export interface ExchangeDetailsPointer {
@@ -716,7 +727,7 @@ export interface ExchangeEntryRecord {
    * existing exchange entry.
    *
    * We keep the retry counter here instead of using the task retries,
-   * as the task succeeded, the exchange is just not useable.
+   * as the task succeeded, the exchange is just not usable.
    */
   updateRetryCounter?: number;
 
@@ -1353,6 +1364,7 @@ export enum ConfigRecordKey {
   // Only for testing, do not use!
   TestLoopTx = "testTxLoop",
   LastInitInfo = "lastInitInfo",
+  MaterializedTransactionsVersion = "materializedTransactionsVersion",
 }
 
 /**
@@ -1366,7 +1378,8 @@ export type ConfigRecord =
     }
   | { key: ConfigRecordKey.CurrencyDefaultsApplied; value: boolean }
   | { key: ConfigRecordKey.TestLoopTx; value: number }
-  | { key: ConfigRecordKey.LastInitInfo; value: DbProtocolTimestamp };
+  | { key: ConfigRecordKey.LastInitInfo; value: DbProtocolTimestamp }
+  | { key: ConfigRecordKey.MaterializedTransactionsVersion; value: number };
 
 export interface WalletBackupConfState {
   deviceId: string;
@@ -1457,6 +1470,7 @@ export interface KycPendingInfo {
   paytoHash: string;
   requirementRow: number;
 }
+
 /**
  * Group of withdrawal operations that need to be executed.
  * (Either for a normal withdrawal or from a reward.)
@@ -1476,6 +1490,8 @@ export interface WithdrawalGroupRecord {
 
   kycUrl?: string;
 
+  kycAccessToken?: string;
+
   /**
    * Secret seed used to derive planchets.
    * Stored since planchets are created lazily.
@@ -1930,22 +1946,33 @@ export interface PeerPushDebitRecord {
 }
 
 export enum PeerPullPaymentCreditStatus {
+  /**
+   * Typically the initial state of the peer-pull-credit transaction,
+   * purse will be created.
+   */
   PendingCreatePurse = 0x0100_0000,
+  SuspendedCreatePurse = 0x0110_0000,
+
   /**
    * Purse created, waiting for the other party to accept the
    * invoice and deposit money into it.
    */
   PendingReady = 0x0100_0001,
-  PendingMergeKycRequired = 0x0100_0002,
-  PendingWithdrawing = 0x0100_0003,
-
-  AbortingDeletePurse = 0x0103_0000,
-
-  SuspendedCreatePurse = 0x0110_0000,
   SuspendedReady = 0x0110_0001,
+
+  PendingMergeKycRequired = 0x0100_0002,
   SuspendedMergeKycRequired = 0x0110_0002,
+
+  PendingWithdrawing = 0x0100_0003,
   SuspendedWithdrawing = 0x0110_0003,
 
+  PendingBalanceKycRequired = 0x0100_0004,
+  SuspendedBalanceKycRequired = 0x0110_0004,
+
+  PendingBalanceKycInit = 0x0100_0005,
+  SuspendedBalanceKycInit = 0x0110_0005,
+
+  AbortingDeletePurse = 0x0103_0000,
   SuspendedAbortingDeletePurse = 0x0113_0000,
 
   Done = 0x0500_0000,
@@ -2006,22 +2033,31 @@ export interface PeerPullCreditRecord {
 
   kycUrl?: string;
 
+  kycAccessToken?: string;
+
   withdrawalGroupId: string | undefined;
 }
 
 export enum PeerPushCreditStatus {
   PendingMerge = 0x0100_0000,
+  SuspendedMerge = 0x0110_0000,
+
   PendingMergeKycRequired = 0x0100_0001,
+  SuspendedMergeKycRequired = 0x0110_0001,
+
   /**
    * Merge was successful and withdrawal group has been created, now
    * everything is in the hand of the withdrawal group.
    */
   PendingWithdrawing = 0x0100_0002,
-
-  SuspendedMerge = 0x0110_0000,
-  SuspendedMergeKycRequired = 0x0110_0001,
   SuspendedWithdrawing = 0x0110_0002,
 
+  PendingBalanceKycRequired = 0x0100_0003,
+  SuspendedBalanceKycRequired = 0x0110_0003,
+
+  PendingBalanceKycInit = 0x0100_0004,
+  SuspendedBalanceKycInit = 0x0110_0004,
+
   DialogProposed = 0x0101_0000,
 
   Done = 0x0500_0000,
@@ -2076,6 +2112,8 @@ export interface PeerPushPaymentIncomingRecord {
   kycInfo?: KycPendingInfo;
 
   kycUrl?: string;
+
+  kycAccessToken?: string;
 }
 
 export enum PeerPullDebitRecordStatus {
@@ -2139,6 +2177,20 @@ export interface PeerPullPaymentIncomingRecord {
   coinSel?: PeerPullPaymentCoinSelection;
 }
 
+export enum ReserveRecordStatus {
+  // Need to call the "/kyc-wallet" endpoint
+  PendingLegiInit = 0x0100_0001,
+  SuspendedLegiInit = 0x0110_0001,
+  // Need to wait for user to pass legitimization
+  PendingLegi = 0x0100_0002,
+  SuspendedLegi = 0x0110_0002,
+
+  /**
+   * Done with KYC.
+   */
+  Done = 0x0500_0000,
+}
+
 /**
  * Store for extra information about a reserve.
  *
@@ -2151,6 +2203,29 @@ export interface ReserveRecord {
   rowId?: number;
   reservePub: string;
   reservePriv: string;
+
+  status?: ReserveRecordStatus;
+
+  requirementRow?: number;
+
+  /**
+   * Balance threshold that we're currently requesting KYC for.
+   */
+  thresholdRequested?: AmountString;
+
+  /**
+   * Balance threshold that we already have passed KYC for.
+   */
+  thresholdGranted?: AmountString;
+
+  /**
+   * Threshold that will trigger the next KYC.
+   */
+  thresholdNext?: AmountString;
+
+  kycAccessToken?: string;
+
+  amlReview?: boolean;
 }
 
 export interface OperationRetryRecord {
