@@ -56,9 +56,9 @@ import {
 } from "@gnu-taler/taler-util/http";
 import { TalerCryptoInterface } from "./crypto/cryptoImplementation.js";
 import { DenominationRecord } from "./db.js";
+import { isWithdrawableDenom } from "./denominations.js";
 import { ExchangeInfo, downloadExchangeInfo } from "./exchanges.js";
 import { assembleRefreshRevealRequest } from "./refresh.js";
-import { isWithdrawableDenom } from "./denominations.js";
 import { getBankStatusUrl, getBankWithdrawalInfo } from "./withdraw.js";
 
 export { downloadExchangeInfo };
@@ -85,26 +85,6 @@ export interface CoinInfo {
   maxAge: number;
 }
 
-/**
- * Check the status of a reserve, use long-polling to wait
- * until the reserve actually has been created.
- */
-export async function checkReserve(
-  http: HttpRequestLibrary,
-  exchangeBaseUrl: string,
-  reservePub: string,
-  longpollTimeoutMs: number = 500,
-): Promise<void> {
-  const reqUrl = new URL(`reserves/${reservePub}`, exchangeBaseUrl);
-  if (longpollTimeoutMs) {
-    reqUrl.searchParams.set("timeout_ms", `${longpollTimeoutMs}`);
-  }
-  const resp = await http.fetch(reqUrl.href, { method: "GET" });
-  if (resp.status !== 200) {
-    throw new Error("reserve not okay");
-  }
-}
-
 export interface TopupReserveWithBankArgs {
   http: HttpRequestLibrary;
   reservePub: string;
@@ -113,9 +93,7 @@ export interface TopupReserveWithBankArgs {
   amount: AmountString;
 }
 
-export async function topupReserveWithBank(
-  args: TopupReserveWithBankArgs,
-) {
+export async function topupReserveWithBank(args: TopupReserveWithBankArgs) {
   const { http, corebankApiBaseUrl, amount, exchangeInfo, reservePub } = args;
   const bankClient = new TalerCorebankApiClient(corebankApiBaseUrl);
   const bankUser = await bankClient.createRandomBankUser();
@@ -125,7 +103,7 @@ export async function topupReserveWithBank(
   );
   const bankInfo = await getBankWithdrawalInfo(http, wopi.taler_withdraw_uri);
   const bankStatusUrl = getBankStatusUrl(wopi.taler_withdraw_uri);
-  if (!bankInfo.suggestedExchange) {
+  if (!bankInfo.exchange) {
     throw Error("no suggested exchange");
   }
   const plainPaytoUris =
@@ -417,3 +395,25 @@ export async function createTestingReserve(args: {
   );
   await readSuccessResponseJsonOrThrow(fbReq, codecForAny());
 }
+
+/**
+ * Check the status of a reserve, use long-polling to wait
+ * until the reserve actually has been created.
+ */
+export async function checkReserve(
+  http: HttpRequestLibrary,
+  exchangeBaseUrl: string,
+  reservePub: string,
+  longpollTimeoutMs: number = 500,
+): Promise<void> {
+  const reqUrl = new URL(`reserves/${reservePub}`, exchangeBaseUrl);
+  if (longpollTimeoutMs) {
+    reqUrl.searchParams.set("timeout_ms", `${longpollTimeoutMs}`);
+  }
+  const resp = await http.fetch(reqUrl.href, {
+    method: "GET",
+  });
+  if (resp.status !== 200) {
+    throw new Error("reserve not okay");
+  }
+}
