@@ -57,7 +57,6 @@ import {
   TransactionState,
   TransactionType,
   URL,
-  WireFee,
   assertUnreachable,
   canonicalJson,
   checkDbInvariant,
@@ -94,12 +93,17 @@ import {
   KycPendingInfo,
   RefreshOperationStatus,
   WalletDbAllStoresReadOnlyTransaction,
+  WalletDbReadWriteTransaction,
   timestampPreciseFromDb,
   timestampPreciseToDb,
   timestampProtocolFromDb,
   timestampProtocolToDb,
 } from "./db.js";
-import { getExchangeWireDetailsInTx } from "./exchanges.js";
+import {
+  getExchangeWireDetailsInTx,
+  getExchangeWireFee,
+  getScopeForAllExchanges,
+} from "./exchanges.js";
 import {
   extractContractData,
   generateDepositPermissions,
@@ -193,6 +197,10 @@ export class DepositTransactionContext implements TransactionContext {
     return {
       type: TransactionType.Deposit,
       txState,
+      scopes: await getScopeForAllExchanges(
+        tx,
+        !dg.infoPerExchange ? [] : Object.keys(dg.infoPerExchange),
+      ),
       txActions: computeDepositTransactionActions(dg),
       amountRaw: Amounts.stringify(dg.counterpartyEffectiveDepositAmount),
       amountEffective: isUnsuccessfulTransaction(txState)
@@ -213,13 +221,33 @@ export class DepositTransactionContext implements TransactionContext {
     };
   }
 
+  /**
+   * Update the metadata of the transaction in the database.
+   */
+  async updateTransactionMeta(
+    tx: WalletDbReadWriteTransaction<["depositGroups", "transactionsMeta"]>,
+  ): Promise<void> {
+    const depositRec = await tx.depositGroups.get(this.depositGroupId);
+    if (!depositRec) {
+      await tx.transactionsMeta.delete(this.transactionId);
+      return;
+    }
+    await tx.transactionsMeta.put({
+      transactionId: this.transactionId,
+      status: depositRec.operationStatus,
+      timestamp: depositRec.timestampCreated,
+      currency: depositRec.currency,
+      exchanges: Object.keys(depositRec.infoPerExchange ?? {}),
+    });
+  }
+
   async deleteTransaction(): Promise<void> {
     const depositGroupId = this.depositGroupId;
     const ws = this.wex;
     // FIXME: We should check first if we are in a final state
     // where deletion is allowed.
     await ws.db.runReadWriteTx(
-      { storeNames: ["depositGroups", "tombstones"] },
+      { storeNames: ["depositGroups", "tombstones", "transactionsMeta"] },
       async (tx) => {
         const tipRecord = await tx.depositGroups.get(depositGroupId);
         if (tipRecord) {
@@ -227,6 +255,7 @@ export class DepositTransactionContext implements TransactionContext {
           await tx.tombstones.put({
             id: TombstoneTag.DeleteDepositGroup + ":" + depositGroupId,
           });
+          await this.updateTransactionMeta(tx);
         }
       },
     );
@@ -236,7 +265,7 @@ export class DepositTransactionContext implements TransactionContext {
   async suspendTransaction(): Promise<void> {
     const { wex, depositGroupId, transactionId, taskId: retryTag } = this;
     const transitionInfo = await wex.db.runReadWriteTx(
-      { storeNames: ["depositGroups"] },
+      { storeNames: ["depositGroups", "transactionsMeta"] },
       async (tx) => {
         const dg = await tx.depositGroups.get(depositGroupId);
         if (!dg) {
@@ -266,6 +295,7 @@ export class DepositTransactionContext implements TransactionContext {
         }
         dg.operationStatus = newOpStatus;
         await tx.depositGroups.put(dg);
+        await this.updateTransactionMeta(tx);
         return {
           oldTxState: oldState,
           newTxState: computeDepositTransactionStatus(dg),
@@ -279,7 +309,7 @@ export class DepositTransactionContext implements TransactionContext {
   async abortTransaction(): Promise<void> {
     const { wex, depositGroupId, transactionId, taskId: retryTag } = this;
     const transitionInfo = await wex.db.runReadWriteTx(
-      { storeNames: ["depositGroups"] },
+      { storeNames: ["depositGroups", "transactionsMeta"] },
       async (tx) => {
         const dg = await tx.depositGroups.get(depositGroupId);
         if (!dg) {
@@ -296,6 +326,7 @@ export class DepositTransactionContext implements TransactionContext {
           case DepositOperationStatus.SuspendedDeposit: {
             dg.operationStatus = DepositOperationStatus.Aborting;
             await tx.depositGroups.put(dg);
+            await this.updateTransactionMeta(tx);
             return {
               oldTxState: oldState,
               newTxState: computeDepositTransactionStatus(dg),
@@ -317,7 +348,7 @@ export class DepositTransactionContext implements TransactionContext {
   async resumeTransaction(): Promise<void> {
     const { wex, depositGroupId, transactionId, taskId: retryTag } = this;
     const transitionInfo = await wex.db.runReadWriteTx(
-      { storeNames: ["depositGroups"] },
+      { storeNames: ["depositGroups", "transactionsMeta"] },
       async (tx) => {
         const dg = await tx.depositGroups.get(depositGroupId);
         if (!dg) {
@@ -347,6 +378,7 @@ export class DepositTransactionContext implements TransactionContext {
         }
         dg.operationStatus = newOpStatus;
         await tx.depositGroups.put(dg);
+        await this.updateTransactionMeta(tx);
         return {
           oldTxState: oldState,
           newTxState: computeDepositTransactionStatus(dg),
@@ -360,7 +392,7 @@ export class DepositTransactionContext implements TransactionContext {
   async failTransaction(): Promise<void> {
     const { wex, depositGroupId, transactionId, taskId } = this;
     const transitionInfo = await wex.db.runReadWriteTx(
-      { storeNames: ["depositGroups"] },
+      { storeNames: ["depositGroups", "transactionsMeta"] },
       async (tx) => {
         const dg = await tx.depositGroups.get(depositGroupId);
         if (!dg) {
@@ -375,6 +407,7 @@ export class DepositTransactionContext implements TransactionContext {
           case DepositOperationStatus.Aborting: {
             dg.operationStatus = DepositOperationStatus.Failed;
             await tx.depositGroups.put(dg);
+            await this.updateTransactionMeta(tx);
             return {
               oldTxState: oldState,
               newTxState: computeDepositTransactionStatus(dg),
@@ -594,6 +627,8 @@ async function refundDepositGroup(
 
   const currency = Amounts.currencyOf(depositGroup.totalPayCost);
 
+  const ctx = new DepositTransactionContext(wex, depositGroup.depositGroupId);
+
   const res = await wex.db.runReadWriteTx(
     {
       storeNames: [
@@ -604,6 +639,7 @@ async function refundDepositGroup(
         "depositGroups",
         "refreshGroups",
         "refreshSessions",
+        "transactionsMeta",
       ],
     },
     async (tx) => {
@@ -635,6 +671,7 @@ async function refundDepositGroup(
         newDg.abortRefreshGroupId = refreshRes.refreshGroupId;
       }
       await tx.depositGroups.put(newDg);
+      await ctx.updateTransactionMeta(tx);
       return { refreshRes };
     },
   );
@@ -667,12 +704,9 @@ async function waitForRefreshOnDepositGroup(
 ): Promise<TaskRunResult> {
   const abortRefreshGroupId = depositGroup.abortRefreshGroupId;
   checkLogicInvariant(!!abortRefreshGroupId);
-  const transactionId = constructTransactionIdentifier({
-    tag: TransactionType.Deposit,
-    depositGroupId: depositGroup.depositGroupId,
-  });
+  const ctx = new DepositTransactionContext(wex, depositGroup.depositGroupId);
   const transitionInfo = await wex.db.runReadWriteTx(
-    { storeNames: ["depositGroups", "refreshGroups"] },
+    { storeNames: ["depositGroups", "refreshGroups", "transactionsMeta"] },
     async (tx) => {
       const refreshGroup = await tx.refreshGroups.get(abortRefreshGroupId);
       let newOpState: DepositOperationStatus | undefined;
@@ -699,16 +733,17 @@ async function waitForRefreshOnDepositGroup(
         newDg.operationStatus = newOpState;
         const newTxState = computeDepositTransactionStatus(newDg);
         await tx.depositGroups.put(newDg);
+        await ctx.updateTransactionMeta(tx);
         return { oldTxState, newTxState };
       }
       return undefined;
     },
   );
 
-  notifyTransition(wex, transactionId, transitionInfo);
+  notifyTransition(wex, ctx.transactionId, transitionInfo);
   wex.ws.notify({
     type: NotificationType.BalanceChange,
-    hintTransactionId: transactionId,
+    hintTransactionId: ctx.transactionId,
   });
   return TaskRunResult.backoff();
 }
@@ -738,14 +773,13 @@ async function processDepositGroupPendingKyc(
   });
 
   const kycInfo = depositGroup.kycInfo;
-  const userType = "individual";
 
   if (!kycInfo) {
     throw Error("invalid DB state, in pending(kyc), but no kycInfo present");
   }
 
   const url = new URL(
-    `kyc-check/${kycInfo.requirementRow}/${kycInfo.paytoHash}/${userType}`,
+    `kyc-check/${kycInfo.requirementRow}`,
     kycInfo.exchangeBaseUrl,
   );
 
@@ -762,6 +796,8 @@ async function processDepositGroupPendingKyc(
     },
   );
 
+  const ctx = new DepositTransactionContext(wex, depositGroupId);
+
   if (
     kycStatusRes.status === HttpStatusCode.Ok ||
     //FIXME: NoContent is not expected https://docs.taler.net/core/api-exchange.html#post--purses-$PURSE_PUB-merge
@@ -769,7 +805,7 @@ async function processDepositGroupPendingKyc(
     kycStatusRes.status === HttpStatusCode.NoContent
   ) {
     const transitionInfo = await wex.db.runReadWriteTx(
-      { storeNames: ["depositGroups"] },
+      { storeNames: ["depositGroups", "transactionsMeta"] },
       async (tx) => {
         const newDg = await tx.depositGroups.get(depositGroupId);
         if (!newDg) {
@@ -782,6 +818,7 @@ async function processDepositGroupPendingKyc(
         newDg.operationStatus = DepositOperationStatus.PendingTrack;
         const newTxState = computeDepositTransactionStatus(newDg);
         await tx.depositGroups.put(newDg);
+        await ctx.updateTransactionMeta(tx);
         return { oldTxState, newTxState };
       },
     );
@@ -806,17 +843,10 @@ async function transitionToKycRequired(
   exchangeUrl: string,
 ): Promise<TaskRunResult> {
   const { depositGroupId } = depositGroup;
-  const userType = "individual";
 
-  const transactionId = constructTransactionIdentifier({
-    tag: TransactionType.Deposit,
-    depositGroupId,
-  });
+  const ctx = new DepositTransactionContext(wex, depositGroupId);
 
-  const url = new URL(
-    `kyc-check/${kycInfo.requirementRow}/${kycInfo.paytoHash}/${userType}`,
-    exchangeUrl,
-  );
+  const url = new URL(`kyc-check/${kycInfo.requirementRow}`, exchangeUrl);
   logger.info(`kyc url ${url.href}`);
   const kycStatusReq = await wex.http.fetch(url.href, {
     method: "GET",
@@ -828,7 +858,7 @@ async function transitionToKycRequired(
     const kycStatus = await kycStatusReq.json();
     logger.info(`kyc status: ${j2s(kycStatus)}`);
     const transitionInfo = await wex.db.runReadWriteTx(
-      { storeNames: ["depositGroups"] },
+      { storeNames: ["depositGroups", "transactionsMeta"] },
       async (tx) => {
         const dg = await tx.depositGroups.get(depositGroupId);
         if (!dg) {
@@ -845,11 +875,12 @@ async function transitionToKycRequired(
           requirementRow: kycInfo.requirementRow,
         };
         await tx.depositGroups.put(dg);
+        await ctx.updateTransactionMeta(tx);
         const newTxState = computeDepositTransactionStatus(dg);
         return { oldTxState, newTxState };
       },
     );
-    notifyTransition(wex, transactionId, transitionInfo);
+    notifyTransition(wex, ctx.transactionId, transitionInfo);
     return TaskRunResult.finished();
   } else {
     throw Error(`unexpected response from kyc-check (${kycStatusReq.status})`);
@@ -873,6 +904,7 @@ async function processDepositGroupPendingTrack(
     );
   }
   const { depositGroupId } = depositGroup;
+  const ctx = new DepositTransactionContext(wex, depositGroupId);
   for (let i = 0; i < statusPerCoin.length; i++) {
     const coinPub = payCoinSelection.coinPubs[i];
     // FIXME: Make the URL part of the coin selection?
@@ -954,7 +986,7 @@ async function processDepositGroupPendingTrack(
 
     if (updatedTxStatus !== undefined) {
       await wex.db.runReadWriteTx(
-        { storeNames: ["depositGroups"] },
+        { storeNames: ["depositGroups", "transactionsMeta"] },
         async (tx) => {
           const dg = await tx.depositGroups.get(depositGroupId);
           if (!dg) {
@@ -982,6 +1014,7 @@ async function processDepositGroupPendingTrack(
             dg.trackingState[newWiredCoin.id] = newWiredCoin.value;
           }
           await tx.depositGroups.put(dg);
+          await ctx.updateTransactionMeta(tx);
         },
       );
     }
@@ -990,7 +1023,7 @@ async function processDepositGroupPendingTrack(
   let allWired = true;
 
   const transitionInfo = await wex.db.runReadWriteTx(
-    { storeNames: ["depositGroups"] },
+    { storeNames: ["depositGroups", "transactionsMeta"] },
     async (tx) => {
       const dg = await tx.depositGroups.get(depositGroupId);
       if (!dg) {
@@ -1012,20 +1045,17 @@ async function processDepositGroupPendingTrack(
         );
         dg.operationStatus = DepositOperationStatus.Finished;
         await tx.depositGroups.put(dg);
+        await ctx.updateTransactionMeta(tx);
       }
       const newTxState = computeDepositTransactionStatus(dg);
       return { oldTxState, newTxState };
     },
   );
-  const transactionId = constructTransactionIdentifier({
-    tag: TransactionType.Deposit,
-    depositGroupId,
-  });
-  notifyTransition(wex, transactionId, transitionInfo);
+  notifyTransition(wex, ctx.transactionId, transitionInfo);
   if (allWired) {
     wex.ws.notify({
       type: NotificationType.BalanceChange,
-      hintTransactionId: transactionId,
+      hintTransactionId: ctx.transactionId,
     });
     return TaskRunResult.finished();
   } else {
@@ -1057,10 +1087,7 @@ async function processDepositGroupPendingDeposit(
     "",
   );
 
-  const transactionId = constructTransactionIdentifier({
-    tag: TransactionType.Deposit,
-    depositGroupId,
-  });
+  const ctx = new DepositTransactionContext(wex, depositGroupId);
 
   // Check for cancellation before expensive operations.
   cancellationToken?.throwIfCancelled();
@@ -1081,6 +1108,7 @@ async function processDepositGroupPendingDeposit(
           "refreshGroups",
           "refreshSessions",
           "denominations",
+          "transactionsMeta",
         ],
       },
       async (tx) => {
@@ -1141,8 +1169,9 @@ async function processDepositGroupPendingDeposit(
           () => DepositElementStatus.DepositPending,
         );
         await tx.depositGroups.put(dg);
+        await ctx.updateTransactionMeta(tx);
         await spendCoins(wex, tx, {
-          transactionId,
+          transactionId: ctx.transactionId,
           coinPubs: dg.payCoinSelection.coinPubs,
           contributions: dg.payCoinSelection.coinContributions.map((x) =>
             Amounts.parseOrThrow(x),
@@ -1222,7 +1251,7 @@ async function processDepositGroupPendingDeposit(
     );
 
     await wex.db.runReadWriteTx(
-      { storeNames: ["depositGroups"] },
+      { storeNames: ["depositGroups", "transactionsMeta"] },
       async (tx) => {
         const dg = await tx.depositGroups.get(depositGroupId);
         if (!dg) {
@@ -1239,12 +1268,13 @@ async function processDepositGroupPendingDeposit(
               await tx.depositGroups.put(dg);
           }
         }
+        await ctx.updateTransactionMeta(tx);
       },
     );
   }
 
   const transitionInfo = await wex.db.runReadWriteTx(
-    { storeNames: ["depositGroups"] },
+    { storeNames: ["depositGroups", "transactionsMeta"] },
     async (tx) => {
       const dg = await tx.depositGroups.get(depositGroupId);
       if (!dg) {
@@ -1253,12 +1283,13 @@ async function processDepositGroupPendingDeposit(
       const oldTxState = computeDepositTransactionStatus(dg);
       dg.operationStatus = DepositOperationStatus.PendingTrack;
       await tx.depositGroups.put(dg);
+      await ctx.updateTransactionMeta(tx);
       const newTxState = computeDepositTransactionStatus(dg);
       return { oldTxState, newTxState };
     },
   );
 
-  notifyTransition(wex, transactionId, transitionInfo);
+  notifyTransition(wex, ctx.transactionId, transitionInfo);
   return TaskRunResult.progress();
 }
 
@@ -1298,54 +1329,6 @@ export async function processDepositGroup(
   return TaskRunResult.finished();
 }
 
-/**
- * FIXME: Consider moving this to exchanges.ts.
- */
-async function getExchangeWireFee(
-  wex: WalletExecutionContext,
-  wireType: string,
-  baseUrl: string,
-  time: TalerProtocolTimestamp,
-): Promise<WireFee> {
-  const exchangeDetails = await wex.db.runReadOnlyTx(
-    { storeNames: ["exchangeDetails", "exchanges"] },
-    async (tx) => {
-      const ex = await tx.exchanges.get(baseUrl);
-      if (!ex || !ex.detailsPointer) return undefined;
-      return await tx.exchangeDetails.indexes.byPointer.get([
-        baseUrl,
-        ex.detailsPointer.currency,
-        ex.detailsPointer.masterPublicKey,
-      ]);
-    },
-  );
-
-  if (!exchangeDetails) {
-    throw Error(`exchange missing: ${baseUrl}`);
-  }
-
-  const fees = exchangeDetails.wireInfo.feesForType[wireType];
-  if (!fees || fees.length === 0) {
-    throw Error(
-      `exchange ${baseUrl} doesn't have fees for wire type ${wireType}`,
-    );
-  }
-  const fee = fees.find((x) => {
-    return AbsoluteTime.isBetween(
-      AbsoluteTime.fromProtocolTimestamp(time),
-      AbsoluteTime.fromProtocolTimestamp(x.startStamp),
-      AbsoluteTime.fromProtocolTimestamp(x.endStamp),
-    );
-  });
-  if (!fee) {
-    throw Error(
-      `exchange ${exchangeDetails.exchangeBaseUrl} doesn't have fees for wire type ${wireType} at ${time.t_s}`,
-    );
-  }
-
-  return fee;
-}
-
 async function trackDeposit(
   wex: WalletExecutionContext,
   depositGroup: DepositGroupRecord,
@@ -1742,6 +1725,7 @@ export async function createDepositGroup(
         "recoupGroups",
         "refreshGroups",
         "refreshSessions",
+        "transactionsMeta",
       ],
     },
     async (tx) => {
@@ -1760,6 +1744,7 @@ export async function createDepositGroup(
         contractTermsRaw: contractTerms,
         h: contractTermsHash,
       });
+      await ctx.updateTransactionMeta(tx);
       return computeDepositTransactionStatus(depositGroup);
     },
   );
@@ -1790,7 +1775,7 @@ export async function createDepositGroup(
  * Get the amount that will be deposited on the users bank
  * account after depositing, not considering aggregation.
  */
-export async function getCounterpartyEffectiveDepositAmount(
+async function getCounterpartyEffectiveDepositAmount(
   wex: WalletExecutionContext,
   wireType: string,
   pcs: SelectedProspectiveCoin[],
