@@ -47,6 +47,8 @@ import {
   RefreshOperationStatus,
   timestampPreciseToDb,
 } from "./db.js";
+import { DenomLossTransactionContext } from "./exchanges.js";
+import { RefreshTransactionContext } from "./refresh.js";
 import { WalletExecutionContext } from "./wallet.js";
 
 const logger = new Logger("dev-experiments.ts");
@@ -80,7 +82,7 @@ export async function applyDevExperiment(
     case "insert-pending-refresh": {
       const refreshGroupId = encodeCrock(getRandomBytes(32));
       await wex.db.runReadWriteTx(
-        { storeNames: ["refreshGroups"] },
+        { storeNames: ["refreshGroups", "transactionsMeta"] },
         async (tx) => {
           const newRg: RefreshGroupRecord = {
             currency: "TESTKUDOS",
@@ -97,6 +99,8 @@ export async function applyDevExperiment(
             infoPerExchange: {},
           };
           await tx.refreshGroups.put(newRg);
+          const ctx = new RefreshTransactionContext(wex, refreshGroupId);
+          await ctx.updateTransactionMeta(tx);
         },
       );
       wex.taskScheduler.startShepherdTask(
@@ -109,7 +113,7 @@ export async function applyDevExperiment(
     }
     case "insert-denom-loss": {
       await wex.db.runReadWriteTx(
-        { storeNames: ["denomLossEvents"] },
+        { storeNames: ["denomLossEvents", "transactionsMeta"] },
         async (tx) => {
           const eventId = encodeCrock(getRandomBytes(32));
           const newRg: DenomLossEventRecord = {
@@ -126,6 +130,11 @@ export async function applyDevExperiment(
             timestampCreated: timestampPreciseToDb(TalerPreciseTimestamp.now()),
           };
           await tx.denomLossEvents.put(newRg);
+          const ctx = new DenomLossTransactionContext(
+            wex,
+            newRg.denomLossEventId,
+          );
+          await ctx.updateTransactionMeta(tx);
         },
       );
       return;
