@@ -150,7 +150,14 @@ async function getAvailableDenoms(
   const operationType = getOperationType(TransactionType.Deposit);
 
   return await wex.db.runReadOnlyTx(
-    ["exchanges", "exchangeDetails", "denominations", "coinAvailability"],
+    {
+      storeNames: [
+        "exchanges",
+        "exchangeDetails",
+        "denominations",
+        "coinAvailability",
+      ],
+    },
     async (tx) => {
       const list: CoinInfo[] = [];
       const exchanges: Record<string, ExchangeInfo> = {};
@@ -276,7 +283,7 @@ async function getAvailableDenoms(
               coinAvail.exchangeBaseUrl,
               coinAvail.denomPubHash,
             ]);
-            checkDbInvariant(!!denom);
+            checkDbInvariant(!!denom, `denomination of a coin is missing hash: ${coinAvail.denomPubHash}`);
             if (denom.isRevoked || !denom.isOffered) {
               continue;
             }
@@ -465,7 +472,7 @@ export async function getMaxDepositAmount(
 export function getMaxDepositAmountForAvailableCoins(
   denoms: AvailableCoins,
   currency: string,
-) {
+): AmountWithFee {
   const zero = Amounts.zeroOfCurrency(currency);
   if (!denoms.list.length) {
     // no coins in the database
@@ -656,8 +663,13 @@ function rankDenominationForWithdrawals(
     //different exchanges may have different wireFee
     //ranking should take the relative contribution in the exchange
     //which is (value - denomFee / fixedFee)
-    const rate1 = Amounts.divmod(d1.value, d1.denomWithdraw).quotient;
-    const rate2 = Amounts.divmod(d2.value, d2.denomWithdraw).quotient;
+
+    const rate1 = Amounts.isZero(d1.denomWithdraw)
+      ? Number.MIN_SAFE_INTEGER
+      : Amounts.divmod(d1.value, d1.denomWithdraw).quotient;
+    const rate2 = Amounts.isZero(d2.denomWithdraw)
+      ? Number.MIN_SAFE_INTEGER
+      : Amounts.divmod(d2.value, d2.denomWithdraw).quotient;
     const contribCmp = rate1 === rate2 ? 0 : rate1 < rate2 ? 1 : -1;
     return (
       contribCmp ||
@@ -712,8 +724,13 @@ function rankDenominationForDeposit(
     //different exchanges may have different wireFee
     //ranking should take the relative contribution in the exchange
     //which is (value - denomFee / fixedFee)
-    const rate1 = Amounts.divmod(d1.value, d1.denomDeposit).quotient;
-    const rate2 = Amounts.divmod(d2.value, d2.denomDeposit).quotient;
+    const rate1 = Amounts.isZero(d1.denomDeposit)
+      ? Number.MIN_SAFE_INTEGER
+      : Amounts.divmod(d1.value, d1.denomDeposit).quotient;
+    const rate2 = Amounts.isZero(d2.denomDeposit)
+      ? Number.MIN_SAFE_INTEGER
+      : Amounts.divmod(d2.value, d2.denomDeposit).quotient;
+
     const contribCmp = rate1 === rate2 ? 0 : rate1 < rate2 ? 1 : -1;
     return (
       contribCmp ||
