@@ -14,6 +14,9 @@
  GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
  */
 
+/**
+ * Imports.
+ */
 import {
   AbsoluteTime,
   Amounts,
@@ -21,6 +24,7 @@ import {
   CheckPeerPullCreditResponse,
   ContractTermsUtil,
   ExchangeReservePurseRequest,
+  ExchangeWalletKycStatus,
   HttpStatusCode,
   InitiatePeerPullCreditRequest,
   InitiatePeerPullCreditResponse,
@@ -47,7 +51,6 @@ import {
   encodeCrock,
   getRandomBytes,
   j2s,
-  makeErrorDetail,
   stringifyPayPullUri,
   stringifyTalerUri,
   talerPaytoFromExchangeReserve,
@@ -58,20 +61,23 @@ import {
   TaskIdStr,
   TaskIdentifiers,
   TaskRunResult,
-  TaskRunResultType,
   TombstoneTag,
   TransactionContext,
+  TransitionResult,
+  TransitionResultType,
   constructTaskIdentifier,
+  genericWaitForStateVal,
   requireExchangeTosAcceptedOrThrow,
   runWithClientCancellation,
 } from "./common.js";
 import {
   KycPendingInfo,
-  KycUserType,
   OperationRetryRecord,
   PeerPullCreditRecord,
   PeerPullPaymentCreditStatus,
   WalletDbAllStoresReadOnlyTransaction,
+  WalletDbReadWriteTransaction,
+  WalletDbStoresArr,
   WithdrawalGroupRecord,
   WithdrawalGroupStatus,
   WithdrawalRecordType,
@@ -79,12 +85,19 @@ import {
   timestampPreciseFromDb,
   timestampPreciseToDb,
 } from "./db.js";
-import { fetchFreshExchange } from "./exchanges.js";
+import {
+  BalanceThresholdCheckResult,
+  checkIncomingAmountLegalUnderKycBalanceThreshold,
+  fetchFreshExchange,
+  getScopeForAllExchanges,
+  handleStartExchangeWalletKyc,
+} from "./exchanges.js";
 import {
   codecForExchangePurseStatus,
   getMergeReserveInfo,
 } from "./pay-peer-common.js";
 import {
+  TransitionInfo,
   constructTransactionIdentifier,
   isUnsuccessfulTransaction,
   notifyTransition,
@@ -116,6 +129,108 @@ export class PeerPullCreditTransactionContext implements TransactionContext {
     });
   }
 
+  /**
+   * Transition a peer-pull-credit transaction.
+   * Extra object stores may be accessed during the transition.
+   */
+  async transition<StoreNameArray extends WalletDbStoresArr = []>(
+    opts: { extraStores?: StoreNameArray; transactionLabel?: string },
+    f: (
+      rec: PeerPullCreditRecord | undefined,
+      tx: WalletDbReadWriteTransaction<
+        [
+          "peerPullCredit",
+          "transactionsMeta",
+          "operationRetries",
+          "exchanges",
+          "exchangeDetails",
+          ...StoreNameArray,
+        ]
+      >,
+    ) => Promise<TransitionResult<PeerPullCreditRecord>>,
+  ): Promise<TransitionInfo | undefined> {
+    const baseStores = [
+      "peerPullCredit" as const,
+      "transactionsMeta" as const,
+      "operationRetries" as const,
+      "exchanges" as const,
+      "exchangeDetails" as const,
+    ];
+    const stores = opts.extraStores
+      ? [...baseStores, ...opts.extraStores]
+      : baseStores;
+
+    let errorThrown: Error | undefined;
+    const transitionInfo = await this.wex.db.runReadWriteTx(
+      { storeNames: stores },
+      async (tx) => {
+        const rec = await tx.peerPullCredit.get(this.pursePub);
+        let oldTxState: TransactionState;
+        if (rec) {
+          oldTxState = computePeerPullCreditTransactionState(rec);
+        } else {
+          oldTxState = {
+            major: TransactionMajorState.None,
+          };
+        }
+        let res: TransitionResult<PeerPullCreditRecord> | undefined;
+        try {
+          res = await f(rec, tx);
+        } catch (error) {
+          if (error instanceof Error) {
+            errorThrown = error;
+          }
+          return undefined;
+        }
+
+        switch (res.type) {
+          case TransitionResultType.Transition: {
+            await tx.peerPullCredit.put(res.rec);
+            await this.updateTransactionMeta(tx);
+            const newTxState = computePeerPullCreditTransactionState(res.rec);
+            return {
+              oldTxState,
+              newTxState,
+            };
+          }
+          case TransitionResultType.Delete:
+            await tx.peerPullCredit.delete(this.pursePub);
+            await this.updateTransactionMeta(tx);
+            return {
+              oldTxState,
+              newTxState: {
+                major: TransactionMajorState.None,
+              },
+            };
+          default:
+            return undefined;
+        }
+      },
+    );
+    if (errorThrown) {
+      throw errorThrown;
+    }
+    notifyTransition(this.wex, this.transactionId, transitionInfo);
+    return transitionInfo;
+  }
+
+  async updateTransactionMeta(
+    tx: WalletDbReadWriteTransaction<["peerPullCredit", "transactionsMeta"]>,
+  ): Promise<void> {
+    const ppcRec = await tx.peerPullCredit.get(this.pursePub);
+    if (!ppcRec) {
+      await tx.transactionsMeta.delete(this.transactionId);
+      return;
+    }
+    await tx.transactionsMeta.put({
+      transactionId: this.transactionId,
+      status: ppcRec.status,
+      timestamp: ppcRec.mergeTimestamp,
+      currency: Amounts.currencyOf(ppcRec.amount),
+      exchanges: [ppcRec.exchangeBaseUrl],
+    });
+  }
+
   /**
    * Get the full transaction details for the transaction.
    *
@@ -173,6 +288,7 @@ export class PeerPullCreditTransactionContext implements TransactionContext {
       return {
         type: TransactionType.PeerPullCredit,
         txState,
+        scopes: await getScopeForAllExchanges(tx, [pullCredit.exchangeBaseUrl]),
         txActions: computePeerPullCreditTransactionActions(pullCredit),
         amountEffective: isUnsuccessfulTransaction(txState)
           ? Amounts.stringify(Amounts.zeroOfAmount(wsr.instructedAmount))
@@ -207,6 +323,7 @@ export class PeerPullCreditTransactionContext implements TransactionContext {
     return {
       type: TransactionType.PeerPullCredit,
       txState,
+      scopes: await getScopeForAllExchanges(tx, [pullCredit.exchangeBaseUrl]),
       txActions: computePeerPullCreditTransactionActions(pullCredit),
       amountEffective: isUnsuccessfulTransaction(txState)
         ? Amounts.stringify(Amounts.zeroOfAmount(peerContractTerms.amount))
@@ -234,7 +351,14 @@ export class PeerPullCreditTransactionContext implements TransactionContext {
   async deleteTransaction(): Promise<void> {
     const { wex: ws, pursePub } = this;
     await ws.db.runReadWriteTx(
-      { storeNames: ["withdrawalGroups", "peerPullCredit", "tombstones"] },
+      {
+        storeNames: [
+          "withdrawalGroups",
+          "peerPullCredit",
+          "tombstones",
+          "transactionsMeta",
+        ],
+      },
       async (tx) => {
         const pullIni = await tx.peerPullCredit.get(pursePub);
         if (!pullIni) {
@@ -252,6 +376,7 @@ export class PeerPullCreditTransactionContext implements TransactionContext {
           }
         }
         await tx.peerPullCredit.delete(pursePub);
+        await this.updateTransactionMeta(tx);
         await tx.tombstones.put({
           id: TombstoneTag.DeletePeerPullCredit + ":" + pursePub,
         });
@@ -264,7 +389,7 @@ export class PeerPullCreditTransactionContext implements TransactionContext {
   async suspendTransaction(): Promise<void> {
     const { wex, pursePub, taskId: retryTag, transactionId } = this;
     const transitionInfo = await wex.db.runReadWriteTx(
-      { storeNames: ["peerPullCredit"] },
+      { storeNames: ["peerPullCredit", "transactionsMeta"] },
       async (tx) => {
         const pullCreditRec = await tx.peerPullCredit.get(pursePub);
         if (!pullCreditRec) {
@@ -289,15 +414,23 @@ export class PeerPullCreditTransactionContext implements TransactionContext {
             newStatus =
               PeerPullPaymentCreditStatus.SuspendedAbortingDeletePurse;
             break;
-          case PeerPullPaymentCreditStatus.Done:
+          case PeerPullPaymentCreditStatus.PendingBalanceKycRequired:
+            newStatus = PeerPullPaymentCreditStatus.SuspendedBalanceKycRequired;
+            break;
+          case PeerPullPaymentCreditStatus.PendingBalanceKycInit:
+            newStatus = PeerPullPaymentCreditStatus.SuspendedBalanceKycInit;
+            break;
+          case PeerPullPaymentCreditStatus.SuspendedBalanceKycRequired:
           case PeerPullPaymentCreditStatus.SuspendedCreatePurse:
           case PeerPullPaymentCreditStatus.SuspendedMergeKycRequired:
           case PeerPullPaymentCreditStatus.SuspendedReady:
           case PeerPullPaymentCreditStatus.SuspendedWithdrawing:
+          case PeerPullPaymentCreditStatus.SuspendedBalanceKycInit:
+          case PeerPullPaymentCreditStatus.SuspendedAbortingDeletePurse:
+          case PeerPullPaymentCreditStatus.Done:
           case PeerPullPaymentCreditStatus.Aborted:
           case PeerPullPaymentCreditStatus.Failed:
           case PeerPullPaymentCreditStatus.Expired:
-          case PeerPullPaymentCreditStatus.SuspendedAbortingDeletePurse:
             break;
           default:
             assertUnreachable(pullCreditRec.status);
@@ -309,6 +442,7 @@ export class PeerPullCreditTransactionContext implements TransactionContext {
           const newTxState =
             computePeerPullCreditTransactionState(pullCreditRec);
           await tx.peerPullCredit.put(pullCreditRec);
+          await this.updateTransactionMeta(tx);
           return {
             oldTxState,
             newTxState,
@@ -324,7 +458,7 @@ export class PeerPullCreditTransactionContext implements TransactionContext {
   async failTransaction(): Promise<void> {
     const { wex, pursePub, taskId: retryTag, transactionId } = this;
     const transitionInfo = await wex.db.runReadWriteTx(
-      { storeNames: ["peerPullCredit"] },
+      { storeNames: ["peerPullCredit", "transactionsMeta"] },
       async (tx) => {
         const pullCreditRec = await tx.peerPullCredit.get(pursePub);
         if (!pullCreditRec) {
@@ -345,6 +479,10 @@ export class PeerPullCreditTransactionContext implements TransactionContext {
           case PeerPullPaymentCreditStatus.Aborted:
           case PeerPullPaymentCreditStatus.Failed:
           case PeerPullPaymentCreditStatus.Expired:
+          case PeerPullPaymentCreditStatus.PendingBalanceKycRequired:
+          case PeerPullPaymentCreditStatus.SuspendedBalanceKycRequired:
+          case PeerPullPaymentCreditStatus.PendingBalanceKycInit:
+          case PeerPullPaymentCreditStatus.SuspendedBalanceKycInit:
             break;
           case PeerPullPaymentCreditStatus.AbortingDeletePurse:
           case PeerPullPaymentCreditStatus.SuspendedAbortingDeletePurse:
@@ -360,6 +498,7 @@ export class PeerPullCreditTransactionContext implements TransactionContext {
           const newTxState =
             computePeerPullCreditTransactionState(pullCreditRec);
           await tx.peerPullCredit.put(pullCreditRec);
+          await this.updateTransactionMeta(tx);
           return {
             oldTxState,
             newTxState,
@@ -375,7 +514,7 @@ export class PeerPullCreditTransactionContext implements TransactionContext {
   async resumeTransaction(): Promise<void> {
     const { wex, pursePub, taskId: retryTag, transactionId } = this;
     const transitionInfo = await wex.db.runReadWriteTx(
-      { storeNames: ["peerPullCredit"] },
+      { storeNames: ["peerPullCredit", "transactionsMeta"] },
       async (tx) => {
         const pullCreditRec = await tx.peerPullCredit.get(pursePub);
         if (!pullCreditRec) {
@@ -388,12 +527,17 @@ export class PeerPullCreditTransactionContext implements TransactionContext {
           case PeerPullPaymentCreditStatus.PendingMergeKycRequired:
           case PeerPullPaymentCreditStatus.PendingWithdrawing:
           case PeerPullPaymentCreditStatus.PendingReady:
+          case PeerPullPaymentCreditStatus.PendingBalanceKycRequired:
+          case PeerPullPaymentCreditStatus.PendingBalanceKycInit:
           case PeerPullPaymentCreditStatus.AbortingDeletePurse:
           case PeerPullPaymentCreditStatus.Done:
           case PeerPullPaymentCreditStatus.Failed:
           case PeerPullPaymentCreditStatus.Expired:
           case PeerPullPaymentCreditStatus.Aborted:
             break;
+          case PeerPullPaymentCreditStatus.SuspendedBalanceKycInit:
+            newStatus = PeerPullPaymentCreditStatus.PendingBalanceKycInit;
+            break;
           case PeerPullPaymentCreditStatus.SuspendedCreatePurse:
             newStatus = PeerPullPaymentCreditStatus.PendingCreatePurse;
             break;
@@ -409,6 +553,9 @@ export class PeerPullCreditTransactionContext implements TransactionContext {
           case PeerPullPaymentCreditStatus.SuspendedAbortingDeletePurse:
             newStatus = PeerPullPaymentCreditStatus.AbortingDeletePurse;
             break;
+          case PeerPullPaymentCreditStatus.SuspendedBalanceKycRequired:
+            newStatus = PeerPullPaymentCreditStatus.PendingBalanceKycRequired;
+            break;
           default:
             assertUnreachable(pullCreditRec.status);
         }
@@ -419,6 +566,7 @@ export class PeerPullCreditTransactionContext implements TransactionContext {
           const newTxState =
             computePeerPullCreditTransactionState(pullCreditRec);
           await tx.peerPullCredit.put(pullCreditRec);
+          await this.updateTransactionMeta(tx);
           return {
             oldTxState,
             newTxState,
@@ -434,7 +582,7 @@ export class PeerPullCreditTransactionContext implements TransactionContext {
   async abortTransaction(): Promise<void> {
     const { wex, pursePub, taskId: retryTag, transactionId } = this;
     const transitionInfo = await wex.db.runReadWriteTx(
-      { storeNames: ["peerPullCredit"] },
+      { storeNames: ["peerPullCredit", "transactionsMeta"] },
       async (tx) => {
         const pullCreditRec = await tx.peerPullCredit.get(pursePub);
         if (!pullCreditRec) {
@@ -443,6 +591,10 @@ export class PeerPullCreditTransactionContext implements TransactionContext {
         }
         let newStatus: PeerPullPaymentCreditStatus | undefined = undefined;
         switch (pullCreditRec.status) {
+          case PeerPullPaymentCreditStatus.PendingBalanceKycRequired:
+          case PeerPullPaymentCreditStatus.SuspendedBalanceKycRequired:
+          case PeerPullPaymentCreditStatus.PendingBalanceKycInit:
+          case PeerPullPaymentCreditStatus.SuspendedBalanceKycInit:
           case PeerPullPaymentCreditStatus.PendingCreatePurse:
           case PeerPullPaymentCreditStatus.PendingMergeKycRequired:
             newStatus = PeerPullPaymentCreditStatus.AbortingDeletePurse;
@@ -473,6 +625,7 @@ export class PeerPullCreditTransactionContext implements TransactionContext {
           const newTxState =
             computePeerPullCreditTransactionState(pullCreditRec);
           await tx.peerPullCredit.put(pullCreditRec);
+          await this.updateTransactionMeta(tx);
           return {
             oldTxState,
             newTxState,
@@ -507,10 +660,7 @@ async function queryPurseForPeerPullCredit(
       });
     },
   );
-  const transactionId = constructTransactionIdentifier({
-    tag: TransactionType.PeerPullCredit,
-    pursePub: pullIni.pursePub,
-  });
+  const ctx = new PeerPullCreditTransactionContext(wex, pullIni.pursePub);
 
   logger.info(`purse status code: HTTP ${resp.status}`);
 
@@ -518,7 +668,7 @@ async function queryPurseForPeerPullCredit(
     case HttpStatusCode.Gone: {
       // Exchange says that purse doesn't exist anymore => expired!
       const transitionInfo = await wex.db.runReadWriteTx(
-        { storeNames: ["peerPullCredit"] },
+        { storeNames: ["peerPullCredit", "transactionsMeta"] },
         async (tx) => {
           const finPi = await tx.peerPullCredit.get(pullIni.pursePub);
           if (!finPi) {
@@ -530,11 +680,12 @@ async function queryPurseForPeerPullCredit(
             finPi.status = PeerPullPaymentCreditStatus.Expired;
           }
           await tx.peerPullCredit.put(finPi);
+          await ctx.updateTransactionMeta(tx);
           const newTxState = computePeerPullCreditTransactionState(finPi);
           return { oldTxState, newTxState };
         },
       );
-      notifyTransition(wex, transactionId, transitionInfo);
+      notifyTransition(wex, ctx.transactionId, transitionInfo);
       return TaskRunResult.backoff();
     }
     case HttpStatusCode.NotFound:
@@ -582,7 +733,7 @@ async function queryPurseForPeerPullCredit(
     },
   });
   const transitionInfo = await wex.db.runReadWriteTx(
-    { storeNames: ["peerPullCredit"] },
+    { storeNames: ["peerPullCredit", "transactionsMeta"] },
     async (tx) => {
       const finPi = await tx.peerPullCredit.get(pullIni.pursePub);
       if (!finPi) {
@@ -594,11 +745,12 @@ async function queryPurseForPeerPullCredit(
         finPi.status = PeerPullPaymentCreditStatus.PendingWithdrawing;
       }
       await tx.peerPullCredit.put(finPi);
+      await ctx.updateTransactionMeta(tx);
       const newTxState = computePeerPullCreditTransactionState(finPi);
       return { oldTxState, newTxState };
     },
   );
-  notifyTransition(wex, transactionId, transitionInfo);
+  notifyTransition(wex, ctx.transactionId, transitionInfo);
   return TaskRunResult.backoff();
 }
 
@@ -607,16 +759,19 @@ async function longpollKycStatus(
   pursePub: string,
   exchangeUrl: string,
   kycInfo: KycPendingInfo,
-  userType: KycUserType,
 ): Promise<TaskRunResult> {
-  const transactionId = constructTransactionIdentifier({
-    tag: TransactionType.PeerPullCredit,
-    pursePub,
+  // FIXME: What if this changes? Should be part of the p2p record
+  const mergeReserveInfo = await getMergeReserveInfo(wex, {
+    exchangeBaseUrl: exchangeUrl,
   });
-  const url = new URL(
-    `kyc-check/${kycInfo.requirementRow}/${kycInfo.paytoHash}/${userType}`,
-    exchangeUrl,
-  );
+
+  const sigResp = await wex.cryptoApi.signWalletKycAuth({
+    accountPriv: mergeReserveInfo.reservePriv,
+    accountPub: mergeReserveInfo.reservePub,
+  });
+
+  const ctx = new PeerPullCreditTransactionContext(wex, pursePub);
+  const url = new URL(`kyc-check/${kycInfo.requirementRow}`, exchangeUrl);
   const kycStatusRes = await wex.ws.runLongpollQueueing(
     wex,
     url.hostname,
@@ -625,6 +780,9 @@ async function longpollKycStatus(
       logger.info(`kyc url ${url.href}`);
       return await wex.http.fetch(url.href, {
         method: "GET",
+        headers: {
+          ["Account-Owner-Signature"]: sigResp.sig,
+        },
         cancellationToken: wex.cancellationToken,
       });
     },
@@ -636,7 +794,7 @@ async function longpollKycStatus(
     kycStatusRes.status === HttpStatusCode.NoContent
   ) {
     const transitionInfo = await wex.db.runReadWriteTx(
-      { storeNames: ["peerPullCredit"] },
+      { storeNames: ["peerPullCredit", "transactionsMeta"] },
       async (tx) => {
         const peerIni = await tx.peerPullCredit.get(pursePub);
         if (!peerIni) {
@@ -651,10 +809,11 @@ async function longpollKycStatus(
         peerIni.status = PeerPullPaymentCreditStatus.PendingCreatePurse;
         const newTxState = computePeerPullCreditTransactionState(peerIni);
         await tx.peerPullCredit.put(peerIni);
+        await ctx.updateTransactionMeta(tx);
         return { oldTxState, newTxState };
       },
     );
-    notifyTransition(wex, transactionId, transitionInfo);
+    notifyTransition(wex, ctx.transactionId, transitionInfo);
     return TaskRunResult.progress();
   } else if (kycStatusRes.status === HttpStatusCode.Accepted) {
     return TaskRunResult.longpollReturnedPending();
@@ -668,10 +827,7 @@ async function processPeerPullCreditAbortingDeletePurse(
   peerPullIni: PeerPullCreditRecord,
 ): Promise<TaskRunResult> {
   const { pursePub, pursePriv } = peerPullIni;
-  const transactionId = constructTransactionIdentifier({
-    tag: TransactionType.PeerPullCredit,
-    pursePub,
-  });
+  const ctx = new PeerPullCreditTransactionContext(wex, peerPullIni.pursePub);
 
   const sigResp = await wex.cryptoApi.signDeletePurse({
     pursePriv,
@@ -694,6 +850,7 @@ async function processPeerPullCreditAbortingDeletePurse(
         "denominations",
         "coinAvailability",
         "coins",
+        "transactionsMeta",
       ],
     },
     async (tx) => {
@@ -707,6 +864,7 @@ async function processPeerPullCreditAbortingDeletePurse(
       const oldTxState = computePeerPullCreditTransactionState(ppiRec);
       ppiRec.status = PeerPullPaymentCreditStatus.Aborted;
       await tx.peerPullCredit.put(ppiRec);
+      await ctx.updateTransactionMeta(tx);
       const newTxState = computePeerPullCreditTransactionState(ppiRec);
       return {
         oldTxState,
@@ -714,7 +872,7 @@ async function processPeerPullCreditAbortingDeletePurse(
       };
     },
   );
-  notifyTransition(wex, transactionId, transitionInfo);
+  notifyTransition(wex, ctx.transactionId, transitionInfo);
 
   return TaskRunResult.backoff();
 }
@@ -727,14 +885,11 @@ async function handlePeerPullCreditWithdrawing(
     throw Error("invalid db state (withdrawing, but no withdrawal group ID");
   }
   await waitWithdrawalFinal(wex, pullIni.withdrawalGroupId);
-  const transactionId = constructTransactionIdentifier({
-    tag: TransactionType.PeerPullCredit,
-    pursePub: pullIni.pursePub,
-  });
+  const ctx = new PeerPullCreditTransactionContext(wex, pullIni.pursePub);
   const wgId = pullIni.withdrawalGroupId;
   let finished: boolean = false;
   const transitionInfo = await wex.db.runReadWriteTx(
-    { storeNames: ["peerPullCredit", "withdrawalGroups"] },
+    { storeNames: ["peerPullCredit", "withdrawalGroups", "transactionsMeta"] },
     async (tx) => {
       const ppi = await tx.peerPullCredit.get(pullIni.pursePub);
       if (!ppi) {
@@ -759,6 +914,7 @@ async function handlePeerPullCreditWithdrawing(
         // FIXME: Also handle other final states!
       }
       await tx.peerPullCredit.put(ppi);
+      await ctx.updateTransactionMeta(tx);
       const newTxState = computePeerPullCreditTransactionState(ppi);
       return {
         oldTxState,
@@ -766,7 +922,7 @@ async function handlePeerPullCreditWithdrawing(
       };
     },
   );
-  notifyTransition(wex, transactionId, transitionInfo);
+  notifyTransition(wex, ctx.transactionId, transitionInfo);
   if (finished) {
     return TaskRunResult.finished();
   } else {
@@ -779,6 +935,33 @@ async function handlePeerPullCreditCreatePurse(
   wex: WalletExecutionContext,
   pullIni: PeerPullCreditRecord,
 ): Promise<TaskRunResult> {
+  const ctx = new PeerPullCreditTransactionContext(wex, pullIni.pursePub);
+
+  const kycCheckRes = await checkIncomingAmountLegalUnderKycBalanceThreshold(
+    wex,
+    pullIni.exchangeBaseUrl,
+    pullIni.estimatedAmountEffective,
+  );
+
+  if (kycCheckRes.result === "violation") {
+    // Do this before we transition so that the exchange is already in the right state.
+    await handleStartExchangeWalletKyc(wex, {
+      amount: kycCheckRes.nextThreshold,
+      exchangeBaseUrl: pullIni.exchangeBaseUrl,
+    });
+    await ctx.transition({}, async (rec) => {
+      if (!rec) {
+        return TransitionResult.stay();
+      }
+      if (rec.status !== PeerPullPaymentCreditStatus.PendingCreatePurse) {
+        return TransitionResult.stay();
+      }
+      rec.status = PeerPullPaymentCreditStatus.PendingBalanceKycInit;
+      return TransitionResult.transition(rec);
+    });
+    return TaskRunResult.progress();
+  }
+
   const purseFee = Amounts.stringify(Amounts.zeroOfAmount(pullIni.amount));
   const pursePub = pullIni.pursePub;
   const mergeReserve = await wex.db.runReadOnlyTx(
@@ -875,13 +1058,8 @@ async function handlePeerPullCreditCreatePurse(
 
   logger.info(`reserve merge response: ${j2s(resp)}`);
 
-  const transactionId = constructTransactionIdentifier({
-    tag: TransactionType.PeerPullCredit,
-    pursePub: pullIni.pursePub,
-  });
-
   const transitionInfo = await wex.db.runReadWriteTx(
-    { storeNames: ["peerPullCredit"] },
+    { storeNames: ["peerPullCredit", "transactionsMeta"] },
     async (tx) => {
       const pi2 = await tx.peerPullCredit.get(pursePub);
       if (!pi2) {
@@ -890,11 +1068,12 @@ async function handlePeerPullCreditCreatePurse(
       const oldTxState = computePeerPullCreditTransactionState(pi2);
       pi2.status = PeerPullPaymentCreditStatus.PendingReady;
       await tx.peerPullCredit.put(pi2);
+      await ctx.updateTransactionMeta(tx);
       const newTxState = computePeerPullCreditTransactionState(pi2);
       return { oldTxState, newTxState };
     },
   );
-  notifyTransition(wex, transactionId, transitionInfo);
+  notifyTransition(wex, ctx.transactionId, transitionInfo);
   return TaskRunResult.backoff();
 }
 
@@ -923,6 +1102,8 @@ export async function processPeerPullCredit(
 
   logger.trace(`processing ${retryTag}, status=${pullIni.status}`);
 
+  const ctx = new PeerPullCreditTransactionContext(wex, pullIni.pursePub);
+
   switch (pullIni.status) {
     case PeerPullPaymentCreditStatus.Done: {
       return TaskRunResult.finished();
@@ -938,7 +1119,6 @@ export async function processPeerPullCredit(
         pursePub,
         pullIni.exchangeBaseUrl,
         pullIni.kycInfo,
-        "individual",
       );
     }
     case PeerPullPaymentCreditStatus.PendingCreatePurse:
@@ -947,14 +1127,19 @@ export async function processPeerPullCredit(
       return await processPeerPullCreditAbortingDeletePurse(wex, pullIni);
     case PeerPullPaymentCreditStatus.PendingWithdrawing:
       return handlePeerPullCreditWithdrawing(wex, pullIni);
+    case PeerPullPaymentCreditStatus.PendingBalanceKycRequired:
+    case PeerPullPaymentCreditStatus.PendingBalanceKycInit:
+      return processPeerPullCreditBalanceKyc(ctx, pullIni);
     case PeerPullPaymentCreditStatus.Aborted:
     case PeerPullPaymentCreditStatus.Failed:
     case PeerPullPaymentCreditStatus.Expired:
+    case PeerPullPaymentCreditStatus.SuspendedBalanceKycRequired:
     case PeerPullPaymentCreditStatus.SuspendedAbortingDeletePurse:
     case PeerPullPaymentCreditStatus.SuspendedCreatePurse:
     case PeerPullPaymentCreditStatus.SuspendedMergeKycRequired:
     case PeerPullPaymentCreditStatus.SuspendedReady:
     case PeerPullPaymentCreditStatus.SuspendedWithdrawing:
+    case PeerPullPaymentCreditStatus.SuspendedBalanceKycInit:
       break;
     default:
       assertUnreachable(pullIni.status);
@@ -963,33 +1148,124 @@ export async function processPeerPullCredit(
   return TaskRunResult.finished();
 }
 
+async function processPeerPullCreditBalanceKyc(
+  ctx: PeerPullCreditTransactionContext,
+  peerInc: PeerPullCreditRecord,
+): Promise<TaskRunResult> {
+  const exchangeBaseUrl = peerInc.exchangeBaseUrl;
+  const amount = peerInc.estimatedAmountEffective;
+
+  const ret = await genericWaitForStateVal(ctx.wex, {
+    async checkState(): Promise<BalanceThresholdCheckResult | undefined> {
+      const checkRes = await checkIncomingAmountLegalUnderKycBalanceThreshold(
+        ctx.wex,
+        exchangeBaseUrl,
+        amount,
+      );
+      logger.info(
+        `balance check result for ${exchangeBaseUrl} +${amount}: ${j2s(
+          checkRes,
+        )}`,
+      );
+      if (checkRes.result === "ok") {
+        return checkRes;
+      }
+      if (
+        peerInc.status === PeerPullPaymentCreditStatus.PendingBalanceKycInit &&
+        checkRes.walletKycStatus === ExchangeWalletKycStatus.Legi
+      ) {
+        return checkRes;
+      }
+      await handleStartExchangeWalletKyc(ctx.wex, {
+        amount: checkRes.nextThreshold,
+        exchangeBaseUrl,
+      });
+      return undefined;
+    },
+    filterNotification(notif) {
+      return (
+        (notif.type === NotificationType.ExchangeStateTransition &&
+          notif.exchangeBaseUrl === exchangeBaseUrl) ||
+        notif.type === NotificationType.BalanceChange
+      );
+    },
+  });
+
+  if (ret.result === "ok") {
+    await ctx.transition({}, async (rec) => {
+      if (!rec) {
+        return TransitionResult.stay();
+      }
+      if (
+        rec.status !== PeerPullPaymentCreditStatus.PendingBalanceKycRequired
+      ) {
+        return TransitionResult.stay();
+      }
+      rec.status = PeerPullPaymentCreditStatus.PendingCreatePurse;
+      return TransitionResult.transition(rec);
+    });
+    return TaskRunResult.progress();
+  } else if (
+    peerInc.status === PeerPullPaymentCreditStatus.PendingBalanceKycInit &&
+    ret.walletKycStatus === ExchangeWalletKycStatus.Legi
+  ) {
+    await ctx.transition({}, async (rec) => {
+      if (!rec) {
+        return TransitionResult.stay();
+      }
+      if (rec.status !== PeerPullPaymentCreditStatus.PendingBalanceKycInit) {
+        return TransitionResult.stay();
+      }
+      rec.status = PeerPullPaymentCreditStatus.PendingBalanceKycRequired;
+      delete rec.kycInfo;
+      rec.kycAccessToken = ret.walletKycAccessToken;
+      // FIXME: #9109 this should not be constructed here, it should be an opaque URL from exchange response
+      rec.kycUrl = new URL(
+        `kyc-spa/${ret.walletKycAccessToken}`,
+        exchangeBaseUrl,
+      ).href;
+      return TransitionResult.transition(rec);
+    });
+    return TaskRunResult.progress();
+  } else {
+    throw Error("not reached");
+  }
+}
+
 async function processPeerPullCreditKycRequired(
   wex: WalletExecutionContext,
   peerIni: PeerPullCreditRecord,
   kycPending: WalletKycUuid,
 ): Promise<TaskRunResult> {
-  const transactionId = constructTransactionIdentifier({
-    tag: TransactionType.PeerPullCredit,
-    pursePub: peerIni.pursePub,
-  });
+  const ctx = new PeerPullCreditTransactionContext(wex, peerIni.pursePub);
   const { pursePub } = peerIni;
 
-  const userType = "individual";
+  // FIXME: What if this changes? Should be part of the p2p record
+  const mergeReserveInfo = await getMergeReserveInfo(wex, {
+    exchangeBaseUrl: peerIni.exchangeBaseUrl,
+  });
+
+  const sigResp = await wex.cryptoApi.signWalletKycAuth({
+    accountPriv: mergeReserveInfo.reservePriv,
+    accountPub: mergeReserveInfo.reservePub,
+  });
+
   const url = new URL(
-    `kyc-check/${kycPending.requirement_row}/${kycPending.h_payto}/${userType}`,
+    `kyc-check/${kycPending.requirement_row}`,
     peerIni.exchangeBaseUrl,
   );
 
   logger.info(`kyc url ${url.href}`);
   const kycStatusRes = await wex.http.fetch(url.href, {
     method: "GET",
+    headers: {
+      ["Account-Owner-Signature"]: sigResp.sig,
+    },
     cancellationToken: wex.cancellationToken,
   });
 
   if (
     kycStatusRes.status === HttpStatusCode.Ok ||
-    //FIXME: NoContent is not expected https://docs.taler.net/core/api-exchange.html#post--purses-$PURSE_PUB-merge
-    // remove after the exchange is fixed or clarified
     kycStatusRes.status === HttpStatusCode.NoContent
   ) {
     logger.warn("kyc requested, but already fulfilled");
@@ -998,7 +1274,7 @@ async function processPeerPullCreditKycRequired(
     const kycStatus = await kycStatusRes.json();
     logger.info(`kyc status: ${j2s(kycStatus)}`);
     const { transitionInfo, result } = await wex.db.runReadWriteTx(
-      { storeNames: ["peerPullCredit"] },
+      { storeNames: ["peerPullCredit", "transactionsMeta"] },
       async (tx) => {
         const peerInc = await tx.peerPullCredit.get(pursePub);
         if (!peerInc) {
@@ -1016,24 +1292,14 @@ async function processPeerPullCreditKycRequired(
         peerInc.status = PeerPullPaymentCreditStatus.PendingMergeKycRequired;
         const newTxState = computePeerPullCreditTransactionState(peerInc);
         await tx.peerPullCredit.put(peerInc);
-        // We'll remove this eventually!  New clients should rely on the
-        // kycUrl field of the transaction, not the error code.
-        const res: TaskRunResult = {
-          type: TaskRunResultType.Error,
-          errorDetail: makeErrorDetail(
-            TalerErrorCode.WALLET_WITHDRAWAL_KYC_REQUIRED,
-            {
-              kycUrl: kycStatus.kyc_url,
-            },
-          ),
-        };
+        await ctx.updateTransactionMeta(tx);
         return {
           transitionInfo: { oldTxState, newTxState },
-          result: res,
+          result: TaskRunResult.progress(),
         };
       },
     );
-    notifyTransition(wex, transactionId, transitionInfo);
+    notifyTransition(wex, ctx.transactionId, transitionInfo);
     return TaskRunResult.backoff();
   } else {
     throw Error(`unexpected response from kyc-check (${kycStatusRes.status})`);
@@ -1221,8 +1487,10 @@ export async function initiatePeerPullPayment(
 
   const mergeTimestamp = TalerPreciseTimestamp.now();
 
+  const ctx = new PeerPullCreditTransactionContext(wex, pursePair.pub);
+
   const transitionInfo = await wex.db.runReadWriteTx(
-    { storeNames: ["peerPullCredit", "contractTerms"] },
+    { storeNames: ["peerPullCredit", "contractTerms", "transactionsMeta"] },
     async (tx) => {
       const ppi: PeerPullCreditRecord = {
         amount: req.partialContractTerms.amount,
@@ -1242,6 +1510,7 @@ export async function initiatePeerPullPayment(
         estimatedAmountEffective: wi.withdrawalAmountEffective,
       };
       await tx.peerPullCredit.put(ppi);
+      await ctx.updateTransactionMeta(tx);
       const oldTxState: TransactionState = {
         major: TransactionMajorState.None,
       };
@@ -1254,8 +1523,6 @@ export async function initiatePeerPullPayment(
     },
   );
 
-  const ctx = new PeerPullCreditTransactionContext(wex, pursePair.pub);
-
   notifyTransition(wex, ctx.transactionId, transitionInfo);
   wex.taskScheduler.startShepherdTask(ctx.taskId);
 
@@ -1345,6 +1612,26 @@ export function computePeerPullCreditTransactionState(
         major: TransactionMajorState.Aborting,
         minor: TransactionMinorState.DeletePurse,
       };
+    case PeerPullPaymentCreditStatus.PendingBalanceKycRequired:
+      return {
+        major: TransactionMajorState.Pending,
+        minor: TransactionMinorState.BalanceKycRequired,
+      };
+    case PeerPullPaymentCreditStatus.SuspendedBalanceKycRequired:
+      return {
+        major: TransactionMajorState.Suspended,
+        minor: TransactionMinorState.BalanceKycRequired,
+      };
+    case PeerPullPaymentCreditStatus.PendingBalanceKycInit:
+      return {
+        major: TransactionMajorState.Pending,
+        minor: TransactionMinorState.BalanceKycInit,
+      };
+    case PeerPullPaymentCreditStatus.SuspendedBalanceKycInit:
+      return {
+        major: TransactionMajorState.Suspended,
+        minor: TransactionMinorState.BalanceKycInit,
+      };
   }
 }
 
@@ -1400,5 +1687,13 @@ export function computePeerPullCreditTransactionActions(
       return [TransactionAction.Delete];
     case PeerPullPaymentCreditStatus.SuspendedAbortingDeletePurse:
       return [TransactionAction.Resume, TransactionAction.Fail];
+    case PeerPullPaymentCreditStatus.PendingBalanceKycRequired:
+      return [TransactionAction.Suspend, TransactionAction.Abort];
+    case PeerPullPaymentCreditStatus.SuspendedBalanceKycRequired:
+      return [TransactionAction.Resume, TransactionAction.Abort];
+    case PeerPullPaymentCreditStatus.PendingBalanceKycInit:
+      return [TransactionAction.Suspend, TransactionAction.Abort];
+    case PeerPullPaymentCreditStatus.SuspendedBalanceKycInit:
+      return [TransactionAction.Resume, TransactionAction.Abort];
   }
 }
