@@ -84,6 +84,7 @@ import {
   PeerPullPaymentIncomingRecord,
   RefreshOperationStatus,
   WalletDbAllStoresReadOnlyTransaction,
+  WalletDbReadWriteTransaction,
   WalletStoresV1,
   timestampPreciseFromDb,
   timestampPreciseToDb,
@@ -102,6 +103,7 @@ import {
   parseTransactionIdentifier,
 } from "./transactions.js";
 import { WalletExecutionContext } from "./wallet.js";
+import { getScopeForAllExchanges } from "./exchanges.js";
 
 const logger = new Logger("pay-peer-pull-debit.ts");
 
@@ -127,6 +129,23 @@ export class PeerPullDebitTransactionContext implements TransactionContext {
     this.peerPullDebitId = peerPullDebitId;
   }
 
+  async updateTransactionMeta(
+    tx: WalletDbReadWriteTransaction<["peerPullDebit", "transactionsMeta"]>,
+  ): Promise<void> {
+    const ppdRec = await tx.peerPullDebit.get(this.peerPullDebitId);
+    if (!ppdRec) {
+      await tx.transactionsMeta.delete(this.transactionId);
+      return;
+    }
+    await tx.transactionsMeta.put({
+      transactionId: this.transactionId,
+      status: ppdRec.status,
+      timestamp: ppdRec.timestampCreated,
+      currency: Amounts.currencyOf(ppdRec.amount),
+      exchanges: [ppdRec.exchangeBaseUrl],
+    });
+  }
+
   /**
    * Get the full transaction details for the transaction.
    *
@@ -148,6 +167,7 @@ export class PeerPullDebitTransactionContext implements TransactionContext {
     return {
       type: TransactionType.PeerPullDebit,
       txState,
+      scopes: await getScopeForAllExchanges(tx, [pi.exchangeBaseUrl]),
       txActions: computePeerPullDebitTransactionActions(pi),
       amountEffective: isUnsuccessfulTransaction(txState)
         ? Amounts.stringify(Amounts.zeroOfAmount(pi.amount))
@@ -174,11 +194,12 @@ export class PeerPullDebitTransactionContext implements TransactionContext {
     const ws = this.wex;
     const peerPullDebitId = this.peerPullDebitId;
     await ws.db.runReadWriteTx(
-      { storeNames: ["peerPullDebit", "tombstones"] },
+      { storeNames: ["peerPullDebit", "tombstones", "transactionsMeta"] },
       async (tx) => {
         const debit = await tx.peerPullDebit.get(peerPullDebitId);
         if (debit) {
           await tx.peerPullDebit.delete(peerPullDebitId);
+          await this.updateTransactionMeta(tx);
           await tx.tombstones.put({ id: transactionId });
         }
       },
@@ -191,7 +212,7 @@ export class PeerPullDebitTransactionContext implements TransactionContext {
     const wex = this.wex;
     const peerPullDebitId = this.peerPullDebitId;
     const transitionInfo = await wex.db.runReadWriteTx(
-      { storeNames: ["peerPullDebit"] },
+      { storeNames: ["peerPullDebit", "transactionsMeta"] },
       async (tx) => {
         const pullDebitRec = await tx.peerPullDebit.get(peerPullDebitId);
         if (!pullDebitRec) {
@@ -226,6 +247,7 @@ export class PeerPullDebitTransactionContext implements TransactionContext {
           pullDebitRec.status = newStatus;
           const newTxState = computePeerPullDebitTransactionState(pullDebitRec);
           await tx.peerPullDebit.put(pullDebitRec);
+          await this.updateTransactionMeta(tx);
           return {
             oldTxState,
             newTxState,
@@ -349,14 +371,14 @@ export class PeerPullDebitTransactionContext implements TransactionContext {
       rec: PeerPullPaymentIncomingRecord,
       tx: DbReadWriteTransaction<
         typeof WalletStoresV1,
-        ["peerPullDebit", ...StoreNameArray]
+        ["peerPullDebit", "transactionsMeta", ...StoreNameArray]
       >,
     ) => Promise<TransitionResultType>,
   ): Promise<void> {
     const wex = this.wex;
     const extraStores = opts.extraStores ?? [];
     const transitionInfo = await wex.db.runReadWriteTx(
-      { storeNames: ["peerPullDebit", ...extraStores] },
+      { storeNames: ["peerPullDebit", "transactionsMeta", ...extraStores] },
       async (tx) => {
         const pi = await tx.peerPullDebit.get(this.peerPullDebitId);
         if (!pi) {
@@ -367,12 +389,23 @@ export class PeerPullDebitTransactionContext implements TransactionContext {
         switch (res) {
           case TransitionResultType.Transition: {
             await tx.peerPullDebit.put(pi);
+            await this.updateTransactionMeta(tx);
             const newTxState = computePeerPullDebitTransactionState(pi);
             return {
               oldTxState,
               newTxState,
             };
           }
+          case TransitionResultType.Delete: {
+            await tx.peerPullDebit.delete(this.peerPullDebitId);
+            await this.updateTransactionMeta(tx);
+            return {
+              oldTxState,
+              newTxState: {
+                major: TransactionMajorState.None,
+              },
+            };
+          }
           default:
             return undefined;
         }
@@ -449,27 +482,31 @@ async function handlePurseCreationConflict(
     coinSelRes.result.coins,
   );
 
-  await ws.db.runReadWriteTx({ storeNames: ["peerPullDebit"] }, async (tx) => {
-    const myPpi = await tx.peerPullDebit.get(peerPullInc.peerPullDebitId);
-    if (!myPpi) {
-      return;
-    }
-    switch (myPpi.status) {
-      case PeerPullDebitRecordStatus.PendingDeposit:
-      case PeerPullDebitRecordStatus.SuspendedDeposit: {
-        const sel = coinSelRes.result;
-        myPpi.coinSel = {
-          coinPubs: sel.coins.map((x) => x.coinPub),
-          contributions: sel.coins.map((x) => x.contribution),
-          totalCost: Amounts.stringify(totalAmount),
-        };
-        break;
-      }
-      default:
+  await ws.db.runReadWriteTx(
+    { storeNames: ["peerPullDebit", "transactionsMeta"] },
+    async (tx) => {
+      const myPpi = await tx.peerPullDebit.get(peerPullInc.peerPullDebitId);
+      if (!myPpi) {
         return;
-    }
-    await tx.peerPullDebit.put(myPpi);
-  });
+      }
+      switch (myPpi.status) {
+        case PeerPullDebitRecordStatus.PendingDeposit:
+        case PeerPullDebitRecordStatus.SuspendedDeposit: {
+          const sel = coinSelRes.result;
+          myPpi.coinSel = {
+            coinPubs: sel.coins.map((x) => x.coinPub),
+            contributions: sel.coins.map((x) => x.contribution),
+            totalCost: Amounts.stringify(totalAmount),
+          };
+          break;
+        }
+        default:
+          return;
+      }
+      await tx.peerPullDebit.put(myPpi);
+      await ctx.updateTransactionMeta(tx);
+    },
+  );
   return TaskRunResult.backoff();
 }
 
@@ -531,6 +568,7 @@ async function processPeerPullDebitPendingDeposit(
           "peerPullDebit",
           "refreshGroups",
           "refreshSessions",
+          "transactionsMeta",
         ],
       },
       async (tx) => {
@@ -558,6 +596,7 @@ async function processPeerPullDebitPendingDeposit(
           totalCost: Amounts.stringify(totalAmount),
         };
         await tx.peerPullDebit.put(pi);
+        await ctx.updateTransactionMeta(tx);
         return true;
       },
     );
@@ -653,12 +692,9 @@ async function processPeerPullDebitAbortingRefresh(
   const peerPullDebitId = peerPullInc.peerPullDebitId;
   const abortRefreshGroupId = peerPullInc.abortRefreshGroupId;
   checkLogicInvariant(!!abortRefreshGroupId);
-  const transactionId = constructTransactionIdentifier({
-    tag: TransactionType.PeerPullDebit,
-    peerPullDebitId,
-  });
+  const ctx = new PeerPullDebitTransactionContext(wex, peerPullDebitId);
   const transitionInfo = await wex.db.runReadWriteTx(
-    { storeNames: ["peerPullDebit", "refreshGroups"] },
+    { storeNames: ["peerPullDebit", "refreshGroups", "transactionsMeta"] },
     async (tx) => {
       const refreshGroup = await tx.refreshGroups.get(abortRefreshGroupId);
       let newOpState: PeerPullDebitRecordStatus | undefined;
@@ -685,12 +721,13 @@ async function processPeerPullDebitAbortingRefresh(
         newDg.status = newOpState;
         const newTxState = computePeerPullDebitTransactionState(newDg);
         await tx.peerPullDebit.put(newDg);
+        await ctx.updateTransactionMeta(tx);
         return { oldTxState, newTxState };
       }
       return undefined;
     },
   );
-  notifyTransition(wex, transactionId, transitionInfo);
+  notifyTransition(wex, ctx.transactionId, transitionInfo);
   // FIXME: Shouldn't this be finished in some cases?!
   return TaskRunResult.backoff();
 }
@@ -793,6 +830,7 @@ export async function confirmPeerPullDebit(
         "peerPullDebit",
         "refreshGroups",
         "refreshSessions",
+        "transactionsMeta",
       ],
     },
     async (tx) => {
@@ -819,6 +857,7 @@ export async function confirmPeerPullDebit(
         };
       }
       pi.status = PeerPullDebitRecordStatus.PendingDeposit;
+      await ctx.updateTransactionMeta(tx);
       await tx.peerPullDebit.put(pi);
     },
   );
@@ -962,24 +1001,27 @@ export async function preparePeerPullDebit(
 
   const totalAmount = await getTotalPeerPaymentCost(wex, coins);
 
+  const ctx = new PeerPullDebitTransactionContext(wex, peerPullDebitId);
+
   await wex.db.runReadWriteTx(
-    { storeNames: ["peerPullDebit", "contractTerms"] },
+    { storeNames: ["peerPullDebit", "contractTerms", "transactionsMeta"] },
     async (tx) => {
       await tx.contractTerms.put({
         h: contractTermsHash,
         contractTermsRaw: contractTerms,
-      }),
-        await tx.peerPullDebit.add({
-          peerPullDebitId,
-          contractPriv: contractPriv,
-          exchangeBaseUrl: exchangeBaseUrl,
-          pursePub: pursePub,
-          timestampCreated: timestampPreciseToDb(TalerPreciseTimestamp.now()),
-          contractTermsHash,
-          amount: contractTerms.amount,
-          status: PeerPullDebitRecordStatus.DialogProposed,
-          totalCostEstimated: Amounts.stringify(totalAmount),
-        });
+      });
+      await tx.peerPullDebit.add({
+        peerPullDebitId,
+        contractPriv: contractPriv,
+        exchangeBaseUrl: exchangeBaseUrl,
+        pursePub: pursePub,
+        timestampCreated: timestampPreciseToDb(TalerPreciseTimestamp.now()),
+        contractTermsHash,
+        amount: contractTerms.amount,
+        status: PeerPullDebitRecordStatus.DialogProposed,
+        totalCostEstimated: Amounts.stringify(totalAmount),
+      });
+      await ctx.updateTransactionMeta(tx);
     },
   );
 
