@@ -21,13 +21,13 @@ import {
   ConfirmPeerPushCreditRequest,
   ContractTermsUtil,
   ExchangePurseMergeRequest,
+  ExchangeWalletKycStatus,
   HttpStatusCode,
   Logger,
   NotificationType,
   PeerContractTerms,
   PreparePeerPushCreditRequest,
   PreparePeerPushCreditResponse,
-  TalerErrorCode,
   TalerPreciseTimestamp,
   Transaction,
   TransactionAction,
@@ -49,7 +49,6 @@ import {
   encodeCrock,
   getRandomBytes,
   j2s,
-  makeErrorDetail,
   parsePayPushUri,
   talerPaytoFromExchangeReserve,
 } from "@gnu-taler/taler-util";
@@ -59,26 +58,35 @@ import {
   TaskIdStr,
   TaskIdentifiers,
   TaskRunResult,
-  TaskRunResultType,
   TombstoneTag,
   TransactionContext,
+  TransitionResult,
+  TransitionResultType,
   constructTaskIdentifier,
+  genericWaitForStateVal,
   requireExchangeTosAcceptedOrThrow,
 } from "./common.js";
 import {
   KycPendingInfo,
-  KycUserType,
   OperationRetryRecord,
   PeerPushCreditStatus,
   PeerPushPaymentIncomingRecord,
   WalletDbAllStoresReadOnlyTransaction,
+  WalletDbReadWriteTransaction,
+  WalletDbStoresArr,
   WithdrawalGroupRecord,
   WithdrawalGroupStatus,
   WithdrawalRecordType,
   timestampPreciseFromDb,
   timestampPreciseToDb,
 } from "./db.js";
-import { fetchFreshExchange } from "./exchanges.js";
+import {
+  BalanceThresholdCheckResult,
+  checkIncomingAmountLegalUnderKycBalanceThreshold,
+  fetchFreshExchange,
+  getScopeForAllExchanges,
+  handleStartExchangeWalletKyc,
+} from "./exchanges.js";
 import {
   codecForExchangePurseStatus,
   getMergeReserveInfo,
@@ -119,6 +127,108 @@ export class PeerPushCreditTransactionContext implements TransactionContext {
     });
   }
 
+  /**
+   * Transition a peer-push-credit transaction.
+   * Extra object stores may be accessed during the transition.
+   */
+  async transition<StoreNameArray extends WalletDbStoresArr = []>(
+    opts: { extraStores?: StoreNameArray; transactionLabel?: string },
+    f: (
+      rec: PeerPushPaymentIncomingRecord | undefined,
+      tx: WalletDbReadWriteTransaction<
+        [
+          "peerPushCredit",
+          "transactionsMeta",
+          "operationRetries",
+          "exchanges",
+          "exchangeDetails",
+          ...StoreNameArray,
+        ]
+      >,
+    ) => Promise<TransitionResult<PeerPushPaymentIncomingRecord>>,
+  ): Promise<TransitionInfo | undefined> {
+    const baseStores = [
+      "peerPushCredit" as const,
+      "transactionsMeta" as const,
+      "operationRetries" as const,
+      "exchanges" as const,
+      "exchangeDetails" as const,
+    ];
+    const stores = opts.extraStores
+      ? [...baseStores, ...opts.extraStores]
+      : baseStores;
+
+    let errorThrown: Error | undefined;
+    const transitionInfo = await this.wex.db.runReadWriteTx(
+      { storeNames: stores },
+      async (tx) => {
+        const rec = await tx.peerPushCredit.get(this.peerPushCreditId);
+        let oldTxState: TransactionState;
+        if (rec) {
+          oldTxState = computePeerPushCreditTransactionState(rec);
+        } else {
+          oldTxState = {
+            major: TransactionMajorState.None,
+          };
+        }
+        let res: TransitionResult<PeerPushPaymentIncomingRecord> | undefined;
+        try {
+          res = await f(rec, tx);
+        } catch (error) {
+          if (error instanceof Error) {
+            errorThrown = error;
+          }
+          return undefined;
+        }
+
+        switch (res.type) {
+          case TransitionResultType.Transition: {
+            await tx.peerPushCredit.put(res.rec);
+            await this.updateTransactionMeta(tx);
+            const newTxState = computePeerPushCreditTransactionState(res.rec);
+            return {
+              oldTxState,
+              newTxState,
+            };
+          }
+          case TransitionResultType.Delete:
+            await tx.peerPushCredit.delete(this.peerPushCreditId);
+            await this.updateTransactionMeta(tx);
+            return {
+              oldTxState,
+              newTxState: {
+                major: TransactionMajorState.None,
+              },
+            };
+          default:
+            return undefined;
+        }
+      },
+    );
+    if (errorThrown) {
+      throw errorThrown;
+    }
+    notifyTransition(this.wex, this.transactionId, transitionInfo);
+    return transitionInfo;
+  }
+
+  async updateTransactionMeta(
+    tx: WalletDbReadWriteTransaction<["peerPushCredit", "transactionsMeta"]>,
+  ): Promise<void> {
+    const ppdRec = await tx.peerPushCredit.get(this.peerPushCreditId);
+    if (!ppdRec) {
+      await tx.transactionsMeta.delete(this.transactionId);
+      return;
+    }
+    await tx.transactionsMeta.put({
+      transactionId: this.transactionId,
+      status: ppdRec.status,
+      timestamp: ppdRec.timestamp,
+      currency: Amounts.currencyOf(ppdRec.estimatedAmountEffective),
+      exchanges: [ppdRec.exchangeBaseUrl],
+    });
+  }
+
   /**
    * Get the full transaction details for the transaction.
    *
@@ -165,6 +275,7 @@ export class PeerPushCreditTransactionContext implements TransactionContext {
       return {
         type: TransactionType.PeerPushCredit,
         txState,
+        scopes: await getScopeForAllExchanges(tx, [pushInc.exchangeBaseUrl]),
         txActions: computePeerPushCreditTransactionActions(pushInc),
         amountEffective: isUnsuccessfulTransaction(txState)
           ? Amounts.stringify(Amounts.zeroOfAmount(wg.instructedAmount))
@@ -180,7 +291,8 @@ export class PeerPushCreditTransactionContext implements TransactionContext {
           tag: TransactionType.PeerPushCredit,
           peerPushCreditId: pushInc.peerPushCreditId,
         }),
-        kycUrl: pushInc.kycUrl,
+        kycUrl: wg.kycUrl,
+        kycPaytoHash: wg.kycPending?.paytoHash,
         ...(wgRetryRecord?.lastError ? { error: wgRetryRecord.lastError } : {}),
       };
     }
@@ -189,6 +301,7 @@ export class PeerPushCreditTransactionContext implements TransactionContext {
     return {
       type: TransactionType.PeerPushCredit,
       txState,
+      scopes: await getScopeForAllExchanges(tx, [pushInc.exchangeBaseUrl]),
       txActions: computePeerPushCreditTransactionActions(pushInc),
       amountEffective: isUnsuccessfulTransaction(txState)
         ? Amounts.stringify(Amounts.zeroOfAmount(peerContractTerms.amount))
@@ -201,6 +314,7 @@ export class PeerPushCreditTransactionContext implements TransactionContext {
         summary: peerContractTerms.summary,
       },
       kycUrl: pushInc.kycUrl,
+      kycPaytoHash: pushInc.kycInfo?.paytoHash,
       timestamp: timestampPreciseFromDb(pushInc.timestamp),
       transactionId: constructTransactionIdentifier({
         tag: TransactionType.PeerPushCredit,
@@ -215,7 +329,14 @@ export class PeerPushCreditTransactionContext implements TransactionContext {
   async deleteTransaction(): Promise<void> {
     const { wex, peerPushCreditId } = this;
     await wex.db.runReadWriteTx(
-      { storeNames: ["withdrawalGroups", "peerPushCredit", "tombstones"] },
+      {
+        storeNames: [
+          "withdrawalGroups",
+          "peerPushCredit",
+          "tombstones",
+          "transactionsMeta",
+        ],
+      },
       async (tx) => {
         const pushInc = await tx.peerPushCredit.get(peerPushCreditId);
         if (!pushInc) {
@@ -233,6 +354,7 @@ export class PeerPushCreditTransactionContext implements TransactionContext {
           }
         }
         await tx.peerPushCredit.delete(peerPushCreditId);
+        await this.updateTransactionMeta(tx);
         await tx.tombstones.put({
           id: TombstoneTag.DeletePeerPushCredit + ":" + peerPushCreditId,
         });
@@ -242,63 +364,54 @@ export class PeerPushCreditTransactionContext implements TransactionContext {
   }
 
   async suspendTransaction(): Promise<void> {
-    const { wex, peerPushCreditId, taskId: retryTag, transactionId } = this;
-    const transitionInfo = await wex.db.runReadWriteTx(
-      { storeNames: ["peerPushCredit"] },
-      async (tx) => {
-        const pushCreditRec = await tx.peerPushCredit.get(peerPushCreditId);
-        if (!pushCreditRec) {
-          logger.warn(`peer push credit ${peerPushCreditId} not found`);
-          return;
-        }
-        let newStatus: PeerPushCreditStatus | undefined = undefined;
-        switch (pushCreditRec.status) {
-          case PeerPushCreditStatus.DialogProposed:
-          case PeerPushCreditStatus.Done:
-          case PeerPushCreditStatus.SuspendedMerge:
-          case PeerPushCreditStatus.SuspendedMergeKycRequired:
-          case PeerPushCreditStatus.SuspendedWithdrawing:
-            break;
-          case PeerPushCreditStatus.PendingMergeKycRequired:
-            newStatus = PeerPushCreditStatus.SuspendedMergeKycRequired;
-            break;
-          case PeerPushCreditStatus.PendingMerge:
-            newStatus = PeerPushCreditStatus.SuspendedMerge;
-            break;
-          case PeerPushCreditStatus.PendingWithdrawing:
-            // FIXME: Suspend internal withdrawal transaction!
-            newStatus = PeerPushCreditStatus.SuspendedWithdrawing;
-            break;
-          case PeerPushCreditStatus.Aborted:
-            break;
-          case PeerPushCreditStatus.Failed:
-            break;
-          default:
-            assertUnreachable(pushCreditRec.status);
-        }
-        if (newStatus != null) {
-          const oldTxState =
-            computePeerPushCreditTransactionState(pushCreditRec);
-          pushCreditRec.status = newStatus;
-          const newTxState =
-            computePeerPushCreditTransactionState(pushCreditRec);
-          await tx.peerPushCredit.put(pushCreditRec);
-          return {
-            oldTxState,
-            newTxState,
-          };
-        }
-        return undefined;
-      },
-    );
-    notifyTransition(wex, transactionId, transitionInfo);
-    wex.taskScheduler.stopShepherdTask(retryTag);
+    await this.transition({}, async (pushCreditRec) => {
+      if (!pushCreditRec) {
+        return TransitionResult.stay();
+      }
+      let newStatus: PeerPushCreditStatus | undefined = undefined;
+      switch (pushCreditRec.status) {
+        case PeerPushCreditStatus.DialogProposed:
+        case PeerPushCreditStatus.Done:
+        case PeerPushCreditStatus.SuspendedMerge:
+        case PeerPushCreditStatus.SuspendedMergeKycRequired:
+        case PeerPushCreditStatus.SuspendedWithdrawing:
+        case PeerPushCreditStatus.SuspendedBalanceKycRequired:
+        case PeerPushCreditStatus.SuspendedBalanceKycInit:
+        case PeerPushCreditStatus.Failed:
+        case PeerPushCreditStatus.Aborted:
+          break;
+        case PeerPushCreditStatus.PendingBalanceKycRequired:
+          newStatus = PeerPushCreditStatus.SuspendedBalanceKycRequired;
+          break;
+        case PeerPushCreditStatus.PendingBalanceKycInit:
+          newStatus = PeerPushCreditStatus.SuspendedBalanceKycInit;
+          break;
+        case PeerPushCreditStatus.PendingMergeKycRequired:
+          newStatus = PeerPushCreditStatus.SuspendedMergeKycRequired;
+          break;
+        case PeerPushCreditStatus.PendingMerge:
+          newStatus = PeerPushCreditStatus.SuspendedMerge;
+          break;
+        case PeerPushCreditStatus.PendingWithdrawing:
+          // FIXME: Suspend internal withdrawal transaction!
+          newStatus = PeerPushCreditStatus.SuspendedWithdrawing;
+          break;
+        default:
+          assertUnreachable(pushCreditRec.status);
+      }
+      if (newStatus != null) {
+        pushCreditRec.status = newStatus;
+        return TransitionResult.transition(pushCreditRec);
+      } else {
+        return TransitionResult.stay();
+      }
+    });
   }
 
   async abortTransaction(): Promise<void> {
     const { wex, peerPushCreditId, taskId: retryTag, transactionId } = this;
     const transitionInfo = await wex.db.runReadWriteTx(
-      { storeNames: ["peerPushCredit"] },
+      { storeNames: ["peerPushCredit", "transactionsMeta"] },
       async (tx) => {
         const pushCreditRec = await tx.peerPushCredit.get(peerPushCreditId);
         if (!pushCreditRec) {
@@ -307,29 +420,23 @@ export class PeerPushCreditTransactionContext implements TransactionContext {
         }
         let newStatus: PeerPushCreditStatus | undefined = undefined;
         switch (pushCreditRec.status) {
-          case PeerPushCreditStatus.DialogProposed:
-            newStatus = PeerPushCreditStatus.Aborted;
-            break;
+          case PeerPushCreditStatus.Failed:
+          case PeerPushCreditStatus.Aborted:
           case PeerPushCreditStatus.Done:
             break;
           case PeerPushCreditStatus.SuspendedMerge:
+          case PeerPushCreditStatus.DialogProposed:
           case PeerPushCreditStatus.SuspendedMergeKycRequired:
           case PeerPushCreditStatus.SuspendedWithdrawing:
-            newStatus = PeerPushCreditStatus.Aborted;
-            break;
-          case PeerPushCreditStatus.PendingMergeKycRequired:
-            newStatus = PeerPushCreditStatus.Aborted;
-            break;
-          case PeerPushCreditStatus.PendingMerge:
-            newStatus = PeerPushCreditStatus.Aborted;
-            break;
+          case PeerPushCreditStatus.PendingBalanceKycRequired:
+          case PeerPushCreditStatus.SuspendedBalanceKycRequired:
           case PeerPushCreditStatus.PendingWithdrawing:
+          case PeerPushCreditStatus.PendingMergeKycRequired:
+          case PeerPushCreditStatus.PendingMerge:
+          case PeerPushCreditStatus.PendingBalanceKycInit:
+          case PeerPushCreditStatus.SuspendedBalanceKycInit:
             newStatus = PeerPushCreditStatus.Aborted;
             break;
-          case PeerPushCreditStatus.Aborted:
-            break;
-          case PeerPushCreditStatus.Failed:
-            break;
           default:
             assertUnreachable(pushCreditRec.status);
         }
@@ -340,6 +447,7 @@ export class PeerPushCreditTransactionContext implements TransactionContext {
           const newTxState =
             computePeerPushCreditTransactionState(pushCreditRec);
           await tx.peerPushCredit.put(pushCreditRec);
+          await this.updateTransactionMeta(tx);
           return {
             oldTxState,
             newTxState,
@@ -355,7 +463,7 @@ export class PeerPushCreditTransactionContext implements TransactionContext {
   async resumeTransaction(): Promise<void> {
     const { wex, peerPushCreditId, taskId: retryTag, transactionId } = this;
     const transitionInfo = await wex.db.runReadWriteTx(
-      { storeNames: ["peerPushCredit"] },
+      { storeNames: ["peerPushCredit", "transactionsMeta"] },
       async (tx) => {
         const pushCreditRec = await tx.peerPushCredit.get(peerPushCreditId);
         if (!pushCreditRec) {
@@ -365,10 +473,15 @@ export class PeerPushCreditTransactionContext implements TransactionContext {
         let newStatus: PeerPushCreditStatus | undefined = undefined;
         switch (pushCreditRec.status) {
           case PeerPushCreditStatus.DialogProposed:
-          case PeerPushCreditStatus.Done:
           case PeerPushCreditStatus.PendingMergeKycRequired:
           case PeerPushCreditStatus.PendingMerge:
           case PeerPushCreditStatus.PendingWithdrawing:
+          case PeerPushCreditStatus.PendingBalanceKycRequired:
+          case PeerPushCreditStatus.PendingBalanceKycInit:
+          case PeerPushCreditStatus.Done:
+          case PeerPushCreditStatus.Aborted:
+          case PeerPushCreditStatus.Failed:
+            break;
           case PeerPushCreditStatus.SuspendedMerge:
             newStatus = PeerPushCreditStatus.PendingMerge;
             break;
@@ -379,9 +492,11 @@ export class PeerPushCreditTransactionContext implements TransactionContext {
             // FIXME: resume underlying "internal-withdrawal" transaction.
             newStatus = PeerPushCreditStatus.PendingWithdrawing;
             break;
-          case PeerPushCreditStatus.Aborted:
+          case PeerPushCreditStatus.SuspendedBalanceKycRequired:
+            newStatus = PeerPushCreditStatus.PendingBalanceKycRequired;
             break;
-          case PeerPushCreditStatus.Failed:
+          case PeerPushCreditStatus.SuspendedBalanceKycInit:
+            newStatus = PeerPushCreditStatus.PendingBalanceKycInit;
             break;
           default:
             assertUnreachable(pushCreditRec.status);
@@ -393,6 +508,7 @@ export class PeerPushCreditTransactionContext implements TransactionContext {
           const newTxState =
             computePeerPushCreditTransactionState(pushCreditRec);
           await tx.peerPushCredit.put(pushCreditRec);
+          await this.updateTransactionMeta(tx);
           return {
             oldTxState,
             newTxState,
@@ -408,7 +524,7 @@ export class PeerPushCreditTransactionContext implements TransactionContext {
   async failTransaction(): Promise<void> {
     const { wex, peerPushCreditId, taskId: retryTag, transactionId } = this;
     const transitionInfo = await wex.db.runReadWriteTx(
-      { storeNames: ["peerPushCredit"] },
+      { storeNames: ["peerPushCredit", "transactionsMeta"] },
       async (tx) => {
         const pushCreditRec = await tx.peerPushCredit.get(peerPushCreditId);
         if (!pushCreditRec) {
@@ -429,6 +545,10 @@ export class PeerPushCreditTransactionContext implements TransactionContext {
           case PeerPushCreditStatus.SuspendedMerge:
           case PeerPushCreditStatus.SuspendedMergeKycRequired:
           case PeerPushCreditStatus.SuspendedWithdrawing:
+          case PeerPushCreditStatus.PendingBalanceKycRequired:
+          case PeerPushCreditStatus.SuspendedBalanceKycRequired:
+          case PeerPushCreditStatus.PendingBalanceKycInit:
+          case PeerPushCreditStatus.SuspendedBalanceKycInit:
             newStatus = PeerPushCreditStatus.Failed;
             break;
           default:
@@ -441,6 +561,7 @@ export class PeerPushCreditTransactionContext implements TransactionContext {
           const newTxState =
             computePeerPushCreditTransactionState(pushCreditRec);
           await tx.peerPushCredit.put(pushCreditRec);
+          await this.updateTransactionMeta(tx);
           return {
             oldTxState,
             newTxState,
@@ -564,8 +685,10 @@ export async function preparePeerPushCredit(
     );
   }
 
+  const ctx = new PeerPushCreditTransactionContext(wex, withdrawalGroupId);
+
   const transitionInfo = await wex.db.runReadWriteTx(
-    { storeNames: ["contractTerms", "peerPushCredit"] },
+    { storeNames: ["contractTerms", "peerPushCredit", "transactionsMeta"] },
     async (tx) => {
       const rec: PeerPushPaymentIncomingRecord = {
         peerPushCreditId,
@@ -583,6 +706,7 @@ export async function preparePeerPushCredit(
         ),
       };
       await tx.peerPushCredit.add(rec);
+      await ctx.updateTransactionMeta(tx);
       await tx.contractTerms.put({
         h: contractTermsHash,
         contractTermsRaw: dec.contractTerms,
@@ -627,16 +751,20 @@ async function longpollKycStatus(
   peerPushCreditId: string,
   exchangeUrl: string,
   kycInfo: KycPendingInfo,
-  userType: KycUserType,
 ): Promise<TaskRunResult> {
-  const transactionId = constructTransactionIdentifier({
-    tag: TransactionType.PeerPushCredit,
-    peerPushCreditId,
+  const ctx = new PeerPushCreditTransactionContext(wex, peerPushCreditId);
+
+  // FIXME: What if this changes? Should be part of the p2p record
+  const mergeReserveInfo = await getMergeReserveInfo(wex, {
+    exchangeBaseUrl: exchangeUrl,
   });
-  const url = new URL(
-    `kyc-check/${kycInfo.requirementRow}/${kycInfo.paytoHash}/${userType}`,
-    exchangeUrl,
-  );
+
+  const sigResp = await wex.cryptoApi.signWalletKycAuth({
+    accountPriv: mergeReserveInfo.reservePriv,
+    accountPub: mergeReserveInfo.reservePub,
+  });
+
+  const url = new URL(`kyc-check/${kycInfo.requirementRow}`, exchangeUrl);
   logger.info(`kyc url ${url.href}`);
   const kycStatusRes = await wex.ws.runLongpollQueueing(
     wex,
@@ -645,6 +773,9 @@ async function longpollKycStatus(
       url.searchParams.set("timeout_ms", `${timeoutMs}`);
       return await wex.http.fetch(url.href, {
         method: "GET",
+        headers: {
+          ["Account-Owner-Signature"]: sigResp.sig,
+        },
         cancellationToken: wex.cancellationToken,
       });
     },
@@ -652,12 +783,10 @@ async function longpollKycStatus(
 
   if (
     kycStatusRes.status === HttpStatusCode.Ok ||
-    //FIXME: NoContent is not expected https://docs.taler.net/core/api-exchange.html#post--purses-$PURSE_PUB-merge
-    // remove after the exchange is fixed or clarified
     kycStatusRes.status === HttpStatusCode.NoContent
   ) {
     const transitionInfo = await wex.db.runReadWriteTx(
-      { storeNames: ["peerPushCredit"] },
+      { storeNames: ["peerPushCredit", "transactionsMeta"] },
       async (tx) => {
         const peerInc = await tx.peerPushCredit.get(peerPushCreditId);
         if (!peerInc) {
@@ -670,13 +799,14 @@ async function longpollKycStatus(
         peerInc.status = PeerPushCreditStatus.PendingMerge;
         const newTxState = computePeerPushCreditTransactionState(peerInc);
         await tx.peerPushCredit.put(peerInc);
+        await ctx.updateTransactionMeta(tx);
         return { oldTxState, newTxState };
       },
     );
-    notifyTransition(wex, transactionId, transitionInfo);
+    notifyTransition(wex, ctx.transactionId, transitionInfo);
     return TaskRunResult.progress();
   } else if (kycStatusRes.status === HttpStatusCode.Accepted) {
-    // FIXME: Do we have to update the URL here?
+    // Access token / URL stays the same, just long-poll again.
     return TaskRunResult.longpollReturnedPending();
   } else {
     throw Error(`unexpected response from kyc-check (${kycStatusRes.status})`);
@@ -692,24 +822,40 @@ async function processPeerPushCreditKycRequired(
     tag: TransactionType.PeerPushCredit,
     peerPushCreditId: peerInc.peerPushCreditId,
   });
+  const ctx = new PeerPushCreditTransactionContext(
+    wex,
+    peerInc.peerPushCreditId,
+  );
   const { peerPushCreditId } = peerInc;
 
-  const userType = "individual";
+  // FIXME: What if this changes? Should be part of the p2p record
+  const mergeReserveInfo = await getMergeReserveInfo(wex, {
+    exchangeBaseUrl: peerInc.exchangeBaseUrl,
+  });
+
+  const sigResp = await wex.cryptoApi.signWalletKycAuth({
+    accountPriv: mergeReserveInfo.reservePriv,
+    accountPub: mergeReserveInfo.reservePub,
+  });
+
   const url = new URL(
-    `kyc-check/${kycPending.requirement_row}/${kycPending.h_payto}/${userType}`,
+    `kyc-check/${kycPending.requirement_row}`,
     peerInc.exchangeBaseUrl,
   );
 
   logger.info(`kyc url ${url.href}`);
   const kycStatusRes = await wex.http.fetch(url.href, {
     method: "GET",
+    headers: {
+      ["Account-Owner-Signature"]: sigResp.sig,
+    },
     cancellationToken: wex.cancellationToken,
   });
 
+  logger.info(`kyc result status ${kycStatusRes.status}`);
+
   if (
     kycStatusRes.status === HttpStatusCode.Ok ||
-    //FIXME: NoContent is not expected https://docs.taler.net/core/api-exchange.html#post--purses-$PURSE_PUB-merge
-    // remove after the exchange is fixed or clarified
     kycStatusRes.status === HttpStatusCode.NoContent
   ) {
     logger.warn("kyc requested, but already fulfilled");
@@ -718,7 +864,7 @@ async function processPeerPushCreditKycRequired(
     const kycStatus = await kycStatusRes.json();
     logger.info(`kyc status: ${j2s(kycStatus)}`);
     const { transitionInfo, result } = await wex.db.runReadWriteTx(
-      { storeNames: ["peerPushCredit"] },
+      { storeNames: ["peerPushCredit", "transactionsMeta"] },
       async (tx) => {
         const peerInc = await tx.peerPushCredit.get(peerPushCreditId);
         if (!peerInc) {
@@ -736,20 +882,10 @@ async function processPeerPushCreditKycRequired(
         peerInc.status = PeerPushCreditStatus.PendingMergeKycRequired;
         const newTxState = computePeerPushCreditTransactionState(peerInc);
         await tx.peerPushCredit.put(peerInc);
-        // We'll remove this eventually!  New clients should rely on the
-        // kycUrl field of the transaction, not the error code.
-        const res: TaskRunResult = {
-          type: TaskRunResultType.Error,
-          errorDetail: makeErrorDetail(
-            TalerErrorCode.WALLET_WITHDRAWAL_KYC_REQUIRED,
-            {
-              kycUrl: kycStatus.kyc_url,
-            },
-          ),
-        };
+        await ctx.updateTransactionMeta(tx);
         return {
           transitionInfo: { oldTxState, newTxState },
-          result: res,
+          result: TaskRunResult.progress(),
         };
       },
     );
@@ -766,13 +902,36 @@ async function handlePendingMerge(
   contractTerms: PeerContractTerms,
 ): Promise<TaskRunResult> {
   const { peerPushCreditId } = peerInc;
-  const transactionId = constructTransactionIdentifier({
-    tag: TransactionType.PeerPushCredit,
-    peerPushCreditId,
-  });
+  const ctx = new PeerPushCreditTransactionContext(wex, peerPushCreditId);
+
+  const kycCheckRes = await checkIncomingAmountLegalUnderKycBalanceThreshold(
+    wex,
+    peerInc.exchangeBaseUrl,
+    peerInc.estimatedAmountEffective,
+  );
+
+  if (kycCheckRes.result === "violation") {
+    // Do this before we transition so that the exchange is already in the right state.
+    await handleStartExchangeWalletKyc(wex, {
+      amount: kycCheckRes.nextThreshold,
+      exchangeBaseUrl: peerInc.exchangeBaseUrl,
+    });
+    await ctx.transition({}, async (rec) => {
+      if (!rec) {
+        return TransitionResult.stay();
+      }
+      if (rec.status !== PeerPushCreditStatus.PendingMerge) {
+        return TransitionResult.stay();
+      }
+      rec.status = PeerPushCreditStatus.PendingBalanceKycInit;
+      return TransitionResult.transition(rec);
+    });
+    return TaskRunResult.progress();
+  }
 
   const amount = Amounts.parseOrThrow(contractTerms.amount);
 
+  // FIXME: What if this changes? Should be part of the p2p record
   const mergeReserveInfo = await getMergeReserveInfo(wex, {
     exchangeBaseUrl: peerInc.exchangeBaseUrl,
   });
@@ -855,6 +1014,7 @@ async function handlePendingMerge(
         "reserves",
         "exchanges",
         "exchangeDetails",
+        "transactionsMeta",
       ],
     },
     async (tx) => {
@@ -880,6 +1040,7 @@ async function handlePendingMerge(
         }
       }
       await tx.peerPushCredit.put(peerInc);
+      await ctx.updateTransactionMeta(tx);
       const newTxState = computePeerPushCreditTransactionState(peerInc);
       return {
         peerPushCreditTransition: { oldTxState, newTxState },
@@ -896,7 +1057,7 @@ async function handlePendingMerge(
     withdrawalGroupPrep.transactionId,
     txRes?.wgCreateRes?.transitionInfo,
   );
-  notifyTransition(wex, transactionId, txRes?.peerPushCreditTransition);
+  notifyTransition(wex, ctx.transactionId, txRes?.peerPushCreditTransition);
 
   return TaskRunResult.backoff();
 }
@@ -909,14 +1070,14 @@ async function handlePendingWithdrawing(
     throw Error("invalid db state (withdrawing, but no withdrawal group ID");
   }
   await waitWithdrawalFinal(wex, peerInc.withdrawalGroupId);
-  const transactionId = constructTransactionIdentifier({
-    tag: TransactionType.PeerPushCredit,
-    peerPushCreditId: peerInc.peerPushCreditId,
-  });
+  const ctx = new PeerPushCreditTransactionContext(
+    wex,
+    peerInc.peerPushCreditId,
+  );
   const wgId = peerInc.withdrawalGroupId;
   let finished: boolean = false;
   const transitionInfo = await wex.db.runReadWriteTx(
-    { storeNames: ["peerPushCredit", "withdrawalGroups"] },
+    { storeNames: ["peerPushCredit", "withdrawalGroups", "transactionsMeta"] },
     async (tx) => {
       const ppi = await tx.peerPushCredit.get(peerInc.peerPushCreditId);
       if (!ppi) {
@@ -941,6 +1102,7 @@ async function handlePendingWithdrawing(
         // FIXME: Also handle other final states!
       }
       await tx.peerPushCredit.put(ppi);
+      await ctx.updateTransactionMeta(tx);
       const newTxState = computePeerPushCreditTransactionState(ppi);
       return {
         oldTxState,
@@ -948,7 +1110,7 @@ async function handlePendingWithdrawing(
       };
     },
   );
-  notifyTransition(wex, transactionId, transitionInfo);
+  notifyTransition(wex, ctx.transactionId, transitionInfo);
   if (finished) {
     return TaskRunResult.finished();
   } else {
@@ -964,11 +1126,12 @@ export async function processPeerPushCredit(
   if (!wex.ws.networkAvailable) {
     return TaskRunResult.networkRequired();
   }
+  const ctx = new PeerPushCreditTransactionContext(wex, peerPushCreditId);
 
   let peerInc: PeerPushPaymentIncomingRecord | undefined;
   let contractTerms: PeerContractTerms | undefined;
   await wex.db.runReadWriteTx(
-    { storeNames: ["contractTerms", "peerPushCredit"] },
+    { storeNames: ["contractTerms", "peerPushCredit", "transactionsMeta"] },
     async (tx) => {
       peerInc = await tx.peerPushCredit.get(peerPushCreditId);
       if (!peerInc) {
@@ -979,6 +1142,7 @@ export async function processPeerPushCredit(
         contractTerms = ctRec.contractTermsRaw;
       }
       await tx.peerPushCredit.put(peerInc);
+      await ctx.updateTransactionMeta(tx);
     },
   );
 
@@ -1007,27 +1171,109 @@ export async function processPeerPushCredit(
         peerPushCreditId,
         peerInc.exchangeBaseUrl,
         peerInc.kycInfo,
-        "individual",
       );
     }
-
-    case PeerPushCreditStatus.PendingMerge:
+    case PeerPushCreditStatus.PendingMerge: {
       return handlePendingMerge(wex, peerInc, contractTerms);
-
-    case PeerPushCreditStatus.PendingWithdrawing:
+    }
+    case PeerPushCreditStatus.PendingWithdrawing: {
       return handlePendingWithdrawing(wex, peerInc);
-
+    }
+    case PeerPushCreditStatus.PendingBalanceKycInit:
+    case PeerPushCreditStatus.PendingBalanceKycRequired: {
+      return await processPeerPushCreditBalanceKyc(ctx, peerInc);
+    }
     default:
       return TaskRunResult.finished();
   }
 }
 
+async function processPeerPushCreditBalanceKyc(
+  ctx: PeerPushCreditTransactionContext,
+  peerInc: PeerPushPaymentIncomingRecord,
+): Promise<TaskRunResult> {
+  const exchangeBaseUrl = peerInc.exchangeBaseUrl;
+  const amount = peerInc.estimatedAmountEffective;
+
+  const ret = await genericWaitForStateVal(ctx.wex, {
+    async checkState(): Promise<BalanceThresholdCheckResult | undefined> {
+      const checkRes = await checkIncomingAmountLegalUnderKycBalanceThreshold(
+        ctx.wex,
+        exchangeBaseUrl,
+        amount,
+      );
+      logger.info(
+        `balance check result for ${exchangeBaseUrl} +${amount}: ${j2s(
+          checkRes,
+        )}`,
+      );
+      if (checkRes.result === "ok") {
+        return checkRes;
+      }
+      if (
+        peerInc.status === PeerPushCreditStatus.PendingBalanceKycInit &&
+        checkRes.walletKycStatus === ExchangeWalletKycStatus.Legi
+      ) {
+        return checkRes;
+      }
+      await handleStartExchangeWalletKyc(ctx.wex, {
+        amount: checkRes.nextThreshold,
+        exchangeBaseUrl,
+      });
+      return undefined;
+    },
+    filterNotification(notif) {
+      return (
+        (notif.type === NotificationType.ExchangeStateTransition &&
+          notif.exchangeBaseUrl === exchangeBaseUrl) ||
+        notif.type === NotificationType.BalanceChange
+      );
+    },
+  });
+
+  if (ret.result === "ok") {
+    await ctx.transition({}, async (rec) => {
+      if (!rec) {
+        return TransitionResult.stay();
+      }
+      if (rec.status !== PeerPushCreditStatus.PendingBalanceKycRequired) {
+        return TransitionResult.stay();
+      }
+      rec.status = PeerPushCreditStatus.PendingMerge;
+      return TransitionResult.transition(rec);
+    });
+    return TaskRunResult.progress();
+  } else if (
+    peerInc.status === PeerPushCreditStatus.PendingBalanceKycInit &&
+    ret.walletKycStatus === ExchangeWalletKycStatus.Legi
+  ) {
+    await ctx.transition({}, async (rec) => {
+      if (!rec) {
+        return TransitionResult.stay();
+      }
+      if (rec.status !== PeerPushCreditStatus.PendingBalanceKycInit) {
+        return TransitionResult.stay();
+      }
+      rec.status = PeerPushCreditStatus.PendingBalanceKycRequired;
+      delete rec.kycInfo;
+      rec.kycAccessToken = ret.walletKycAccessToken;
+      // FIXME: #9109 this should not be constructed here, it should be an opaque URL from exchange response
+      rec.kycUrl = new URL(
+        `kyc-spa/${ret.walletKycAccessToken}`,
+        exchangeBaseUrl,
+      ).href;
+      return TransitionResult.transition(rec);
+    });
+    return TaskRunResult.progress();
+  } else {
+    throw Error("not reached");
+  }
+}
+
 export async function confirmPeerPushCredit(
   wex: WalletExecutionContext,
   req: ConfirmPeerPushCreditRequest,
 ): Promise<AcceptPeerPushPaymentResponse> {
-  // PeerPushPaymentIncomingRecord | undefined;
-  let peerPushCreditId: string;
   const parsedTx = parseTransactionIdentifier(req.transactionId);
   if (!parsedTx) {
     throw Error("invalid transaction ID");
@@ -1035,14 +1281,17 @@ export async function confirmPeerPushCredit(
   if (parsedTx.tag !== TransactionType.PeerPushCredit) {
     throw Error("invalid transaction ID type");
   }
-  peerPushCreditId = parsedTx.peerPushCreditId;
+  const ctx = new PeerPushCreditTransactionContext(
+    wex,
+    parsedTx.peerPushCreditId,
+  );
 
-  logger.trace(`confirming peer-push-credit ${peerPushCreditId}`);
+  logger.trace(`confirming peer-push-credit ${ctx.peerPushCreditId}`);
 
   const peerInc = await wex.db.runReadWriteTx(
-    { storeNames: ["contractTerms", "peerPushCredit"] },
+    { storeNames: ["contractTerms", "peerPushCredit", "transactionsMeta"] },
     async (tx) => {
-      const rec = await tx.peerPushCredit.get(peerPushCreditId);
+      const rec = await tx.peerPushCredit.get(ctx.peerPushCreditId);
       if (!rec) {
         return;
       }
@@ -1050,6 +1299,7 @@ export async function confirmPeerPushCredit(
         rec.status = PeerPushCreditStatus.PendingMerge;
       }
       await tx.peerPushCredit.put(rec);
+      await ctx.updateTransactionMeta(tx);
       return rec;
     },
   );
@@ -1063,17 +1313,10 @@ export async function confirmPeerPushCredit(
   const exchange = await fetchFreshExchange(wex, peerInc.exchangeBaseUrl);
   requireExchangeTosAcceptedOrThrow(exchange);
 
-  const ctx = new PeerPushCreditTransactionContext(wex, peerPushCreditId);
-
   wex.taskScheduler.startShepherdTask(ctx.taskId);
 
-  const transactionId = constructTransactionIdentifier({
-    tag: TransactionType.PeerPushCredit,
-    peerPushCreditId,
-  });
-
   return {
-    transactionId,
+    transactionId: ctx.transactionId,
   };
 }
 
@@ -1098,7 +1341,7 @@ export function computePeerPushCreditTransactionState(
     case PeerPushCreditStatus.PendingMergeKycRequired:
       return {
         major: TransactionMajorState.Pending,
-        minor: TransactionMinorState.KycRequired,
+        minor: TransactionMinorState.MergeKycRequired,
       };
     case PeerPushCreditStatus.PendingWithdrawing:
       return {
@@ -1128,6 +1371,26 @@ export function computePeerPushCreditTransactionState(
       return {
         major: TransactionMajorState.Failed,
       };
+    case PeerPushCreditStatus.PendingBalanceKycRequired:
+      return {
+        major: TransactionMajorState.Pending,
+        minor: TransactionMinorState.BalanceKycRequired,
+      };
+    case PeerPushCreditStatus.SuspendedBalanceKycRequired:
+      return {
+        major: TransactionMajorState.Suspended,
+        minor: TransactionMinorState.BalanceKycRequired,
+      };
+    case PeerPushCreditStatus.PendingBalanceKycInit:
+      return {
+        major: TransactionMajorState.Pending,
+        minor: TransactionMinorState.BalanceKycInit,
+      };
+    case PeerPushCreditStatus.SuspendedBalanceKycInit:
+      return {
+        major: TransactionMajorState.Suspended,
+        minor: TransactionMinorState.BalanceKycInit,
+      };
     default:
       assertUnreachable(pushCreditRecord.status);
   }
@@ -1165,6 +1428,14 @@ export function computePeerPushCreditTransactionActions(
       return [TransactionAction.Resume, TransactionAction.Abort];
     case PeerPushCreditStatus.SuspendedWithdrawing:
       return [TransactionAction.Resume, TransactionAction.Fail];
+    case PeerPushCreditStatus.PendingBalanceKycRequired:
+      return [TransactionAction.Suspend, TransactionAction.Abort];
+    case PeerPushCreditStatus.SuspendedBalanceKycRequired:
+      return [TransactionAction.Resume, TransactionAction.Abort];
+    case PeerPushCreditStatus.PendingBalanceKycInit:
+      return [TransactionAction.Suspend, TransactionAction.Abort];
+    case PeerPushCreditStatus.SuspendedBalanceKycInit:
+      return [TransactionAction.Resume, TransactionAction.Abort];
     case PeerPushCreditStatus.Aborted:
       return [TransactionAction.Delete];
     case PeerPushCreditStatus.Failed:
