@@ -74,6 +74,7 @@ import {
   PeerPushDebitStatus,
   RefreshOperationStatus,
   WalletDbAllStoresReadOnlyTransaction,
+  WalletDbReadWriteTransaction,
   timestampPreciseFromDb,
   timestampPreciseToDb,
   timestampProtocolFromDb,
@@ -92,6 +93,7 @@ import {
   notifyTransition,
 } from "./transactions.js";
 import { WalletExecutionContext } from "./wallet.js";
+import { getScopeForAllExchanges } from "./exchanges.js";
 
 const logger = new Logger("pay-peer-push-debit.ts");
 
@@ -113,6 +115,23 @@ export class PeerPushDebitTransactionContext implements TransactionContext {
     });
   }
 
+  async updateTransactionMeta(
+    tx: WalletDbReadWriteTransaction<["peerPushDebit", "transactionsMeta"]>,
+  ): Promise<void> {
+    const ppdRec = await tx.peerPushDebit.get(this.pursePub);
+    if (!ppdRec) {
+      await tx.transactionsMeta.delete(this.transactionId);
+      return;
+    }
+    await tx.transactionsMeta.put({
+      transactionId: this.transactionId,
+      status: ppdRec.status,
+      timestamp: ppdRec.timestampCreated,
+      currency: Amounts.currencyOf(ppdRec.amount),
+      exchanges: [ppdRec.exchangeBaseUrl],
+    });
+  }
+
   /**
    * Get the full transaction details for the transaction.
    *
@@ -129,7 +148,10 @@ export class PeerPushDebitTransactionContext implements TransactionContext {
     const retryRec = await tx.operationRetries.get(this.taskId);
 
     const ctRec = await tx.contractTerms.get(pushDebitRec.contractTermsHash);
-    checkDbInvariant(!!ctRec, `no contract terms for p2p push ${this.pursePub}`);
+    checkDbInvariant(
+      !!ctRec,
+      `no contract terms for p2p push ${this.pursePub}`,
+    );
 
     const contractTerms = ctRec.contractTermsRaw;
 
@@ -146,6 +168,7 @@ export class PeerPushDebitTransactionContext implements TransactionContext {
     return {
       type: TransactionType.PeerPushDebit,
       txState,
+      scopes: await getScopeForAllExchanges(tx, [pushDebitRec.exchangeBaseUrl]),
       txActions: computePeerPushDebitTransactionActions(pushDebitRec),
       amountEffective: isUnsuccessfulTransaction(txState)
         ? Amounts.stringify(Amounts.zeroOfAmount(pushDebitRec.totalCost))
@@ -169,11 +192,12 @@ export class PeerPushDebitTransactionContext implements TransactionContext {
   async deleteTransaction(): Promise<void> {
     const { wex, pursePub, transactionId } = this;
     await wex.db.runReadWriteTx(
-      { storeNames: ["peerPushDebit", "tombstones"] },
+      { storeNames: ["peerPushDebit", "tombstones", "transactionsMeta"] },
       async (tx) => {
         const debit = await tx.peerPushDebit.get(pursePub);
         if (debit) {
           await tx.peerPushDebit.delete(pursePub);
+          await this.updateTransactionMeta(tx);
           await tx.tombstones.put({ id: transactionId });
         }
       },
@@ -183,7 +207,7 @@ export class PeerPushDebitTransactionContext implements TransactionContext {
   async suspendTransaction(): Promise<void> {
     const { wex, pursePub, transactionId, taskId: retryTag } = this;
     const transitionInfo = await wex.db.runReadWriteTx(
-      { storeNames: ["peerPushDebit"] },
+      { storeNames: ["peerPushDebit", "transactionsMeta"] },
       async (tx) => {
         const pushDebitRec = await tx.peerPushDebit.get(pursePub);
         if (!pushDebitRec) {
@@ -226,6 +250,7 @@ export class PeerPushDebitTransactionContext implements TransactionContext {
           pushDebitRec.status = newStatus;
           const newTxState = computePeerPushDebitTransactionState(pushDebitRec);
           await tx.peerPushDebit.put(pushDebitRec);
+          await this.updateTransactionMeta(tx);
           return {
             oldTxState,
             newTxState,
@@ -241,7 +266,7 @@ export class PeerPushDebitTransactionContext implements TransactionContext {
   async abortTransaction(): Promise<void> {
     const { wex, pursePub, transactionId, taskId: retryTag } = this;
     const transitionInfo = await wex.db.runReadWriteTx(
-      { storeNames: ["peerPushDebit"] },
+      { storeNames: ["peerPushDebit", "transactionsMeta"] },
       async (tx) => {
         const pushDebitRec = await tx.peerPushDebit.get(pursePub);
         if (!pushDebitRec) {
@@ -279,6 +304,7 @@ export class PeerPushDebitTransactionContext implements TransactionContext {
           pushDebitRec.status = newStatus;
           const newTxState = computePeerPushDebitTransactionState(pushDebitRec);
           await tx.peerPushDebit.put(pushDebitRec);
+          await this.updateTransactionMeta(tx);
           return {
             oldTxState,
             newTxState,
@@ -295,7 +321,7 @@ export class PeerPushDebitTransactionContext implements TransactionContext {
   async resumeTransaction(): Promise<void> {
     const { wex, pursePub, transactionId, taskId: retryTag } = this;
     const transitionInfo = await wex.db.runReadWriteTx(
-      { storeNames: ["peerPushDebit"] },
+      { storeNames: ["peerPushDebit", "transactionsMeta"] },
       async (tx) => {
         const pushDebitRec = await tx.peerPushDebit.get(pursePub);
         if (!pushDebitRec) {
@@ -338,6 +364,7 @@ export class PeerPushDebitTransactionContext implements TransactionContext {
           pushDebitRec.status = newStatus;
           const newTxState = computePeerPushDebitTransactionState(pushDebitRec);
           await tx.peerPushDebit.put(pushDebitRec);
+          await this.updateTransactionMeta(tx);
           return {
             oldTxState,
             newTxState,
@@ -353,7 +380,7 @@ export class PeerPushDebitTransactionContext implements TransactionContext {
   async failTransaction(): Promise<void> {
     const { wex, pursePub, transactionId, taskId: retryTag } = this;
     const transitionInfo = await wex.db.runReadWriteTx(
-      { storeNames: ["peerPushDebit"] },
+      { storeNames: ["peerPushDebit", "transactionsMeta"] },
       async (tx) => {
         const pushDebitRec = await tx.peerPushDebit.get(pursePub);
         if (!pushDebitRec) {
@@ -391,6 +418,7 @@ export class PeerPushDebitTransactionContext implements TransactionContext {
           pushDebitRec.status = newStatus;
           const newTxState = computePeerPushDebitTransactionState(pushDebitRec);
           await tx.peerPushDebit.put(pushDebitRec);
+          await this.updateTransactionMeta(tx);
           return {
             oldTxState,
             newTxState,
@@ -516,26 +544,30 @@ async function handlePurseCreationConflict(
       assertUnreachable(coinSelRes);
   }
 
-  await wex.db.runReadWriteTx({ storeNames: ["peerPushDebit"] }, async (tx) => {
-    const myPpi = await tx.peerPushDebit.get(peerPushInitiation.pursePub);
-    if (!myPpi) {
-      return;
-    }
-    switch (myPpi.status) {
-      case PeerPushDebitStatus.PendingCreatePurse:
-      case PeerPushDebitStatus.SuspendedCreatePurse: {
-        const sel = coinSelRes.result;
-        myPpi.coinSel = {
-          coinPubs: sel.coins.map((x) => x.coinPub),
-          contributions: sel.coins.map((x) => x.contribution),
-        };
-        break;
-      }
-      default:
+  await wex.db.runReadWriteTx(
+    { storeNames: ["peerPushDebit", "transactionsMeta"] },
+    async (tx) => {
+      const myPpi = await tx.peerPushDebit.get(peerPushInitiation.pursePub);
+      if (!myPpi) {
         return;
-    }
-    await tx.peerPushDebit.put(myPpi);
-  });
+      }
+      switch (myPpi.status) {
+        case PeerPushDebitStatus.PendingCreatePurse:
+        case PeerPushDebitStatus.SuspendedCreatePurse: {
+          const sel = coinSelRes.result;
+          myPpi.coinSel = {
+            coinPubs: sel.coins.map((x) => x.coinPub),
+            contributions: sel.coins.map((x) => x.contribution),
+          };
+          break;
+        }
+        default:
+          return;
+      }
+      await tx.peerPushDebit.put(myPpi);
+      await ctx.updateTransactionMeta(tx);
+    },
+  );
   return TaskRunResult.progress();
 }
 
@@ -596,6 +628,7 @@ async function processPeerPushDebitCreateReserve(
           "peerPushDebit",
           "refreshGroups",
           "refreshSessions",
+          "transactionsMeta",
         ],
       },
       async (tx) => {
@@ -624,6 +657,7 @@ async function processPeerPushDebitCreateReserve(
         });
 
         await tx.peerPushDebit.put(ppi);
+        await ctx.updateTransactionMeta(tx);
         return true;
       },
     );
@@ -780,10 +814,7 @@ async function processPeerPushDebitAbortingDeletePurse(
   peerPushInitiation: PeerPushDebitRecord,
 ): Promise<TaskRunResult> {
   const { pursePub, pursePriv } = peerPushInitiation;
-  const transactionId = constructTransactionIdentifier({
-    tag: TransactionType.PeerPushDebit,
-    pursePub,
-  });
+  const ctx = new PeerPushDebitTransactionContext(wex, pursePub);
 
   const sigResp = await wex.cryptoApi.signDeletePurse({
     pursePriv,
@@ -811,6 +842,7 @@ async function processPeerPushDebitAbortingDeletePurse(
         "peerPushDebit",
         "refreshGroups",
         "refreshSessions",
+        "transactionsMeta",
       ],
     },
     async (tx) => {
@@ -842,11 +874,12 @@ async function processPeerPushDebitAbortingDeletePurse(
         currency,
         coinPubs,
         RefreshReason.AbortPeerPushDebit,
-        transactionId,
+        ctx.transactionId,
       );
       ppiRec.status = PeerPushDebitStatus.AbortingRefreshDeleted;
       ppiRec.abortRefreshGroupId = refresh.refreshGroupId;
       await tx.peerPushDebit.put(ppiRec);
+      await ctx.updateTransactionMeta(tx);
       const newTxState = computePeerPushDebitTransactionState(ppiRec);
       return {
         oldTxState,
@@ -854,7 +887,7 @@ async function processPeerPushDebitAbortingDeletePurse(
       };
     },
   );
-  notifyTransition(wex, transactionId, transitionInfo);
+  notifyTransition(wex, ctx.transactionId, transitionInfo);
 
   return TaskRunResult.backoff();
 }
@@ -870,12 +903,9 @@ async function transitionPeerPushDebitTransaction(
   pursePub: string,
   transitionSpec: SimpleTransition,
 ): Promise<void> {
-  const transactionId = constructTransactionIdentifier({
-    tag: TransactionType.PeerPushDebit,
-    pursePub,
-  });
+  const ctx = new PeerPushDebitTransactionContext(wex, pursePub);
   const transitionInfo = await wex.db.runReadWriteTx(
-    { storeNames: ["peerPushDebit"] },
+    { storeNames: ["peerPushDebit", "transactionsMeta"] },
     async (tx) => {
       const ppiRec = await tx.peerPushDebit.get(pursePub);
       if (!ppiRec) {
@@ -887,6 +917,7 @@ async function transitionPeerPushDebitTransaction(
       const oldTxState = computePeerPushDebitTransactionState(ppiRec);
       ppiRec.status = transitionSpec.stTo;
       await tx.peerPushDebit.put(ppiRec);
+      await ctx.updateTransactionMeta(tx);
       const newTxState = computePeerPushDebitTransactionState(ppiRec);
       return {
         oldTxState,
@@ -894,7 +925,7 @@ async function transitionPeerPushDebitTransaction(
       };
     },
   );
-  notifyTransition(wex, transactionId, transitionInfo);
+  notifyTransition(wex, ctx.transactionId, transitionInfo);
 }
 
 async function processPeerPushDebitAbortingRefreshDeleted(
@@ -904,15 +935,12 @@ async function processPeerPushDebitAbortingRefreshDeleted(
   const pursePub = peerPushInitiation.pursePub;
   const abortRefreshGroupId = peerPushInitiation.abortRefreshGroupId;
   checkLogicInvariant(!!abortRefreshGroupId);
-  const transactionId = constructTransactionIdentifier({
-    tag: TransactionType.PeerPushDebit,
-    pursePub: peerPushInitiation.pursePub,
-  });
+  const ctx = new PeerPushDebitTransactionContext(wex, pursePub);
   if (peerPushInitiation.abortRefreshGroupId) {
     await waitRefreshFinal(wex, peerPushInitiation.abortRefreshGroupId);
   }
   const transitionInfo = await wex.db.runReadWriteTx(
-    { storeNames: ["refreshGroups", "peerPushDebit"] },
+    { storeNames: ["refreshGroups", "peerPushDebit", "transactionsMeta"] },
     async (tx) => {
       const refreshGroup = await tx.refreshGroups.get(abortRefreshGroupId);
       let newOpState: PeerPushDebitStatus | undefined;
@@ -939,12 +967,13 @@ async function processPeerPushDebitAbortingRefreshDeleted(
         newDg.status = newOpState;
         const newTxState = computePeerPushDebitTransactionState(newDg);
         await tx.peerPushDebit.put(newDg);
+        await ctx.updateTransactionMeta(tx);
         return { oldTxState, newTxState };
       }
       return undefined;
     },
   );
-  notifyTransition(wex, transactionId, transitionInfo);
+  notifyTransition(wex, ctx.transactionId, transitionInfo);
   // FIXME: Shouldn't this be finished in some cases?!
   return TaskRunResult.backoff();
 }
@@ -956,12 +985,9 @@ async function processPeerPushDebitAbortingRefreshExpired(
   const pursePub = peerPushInitiation.pursePub;
   const abortRefreshGroupId = peerPushInitiation.abortRefreshGroupId;
   checkLogicInvariant(!!abortRefreshGroupId);
-  const transactionId = constructTransactionIdentifier({
-    tag: TransactionType.PeerPushDebit,
-    pursePub: peerPushInitiation.pursePub,
-  });
+  const ctx = new PeerPushDebitTransactionContext(wex, pursePub);
   const transitionInfo = await wex.db.runReadWriteTx(
-    { storeNames: ["peerPushDebit", "refreshGroups"] },
+    { storeNames: ["peerPushDebit", "refreshGroups", "transactionsMeta"] },
     async (tx) => {
       const refreshGroup = await tx.refreshGroups.get(abortRefreshGroupId);
       let newOpState: PeerPushDebitStatus | undefined;
@@ -988,12 +1014,13 @@ async function processPeerPushDebitAbortingRefreshExpired(
         newDg.status = newOpState;
         const newTxState = computePeerPushDebitTransactionState(newDg);
         await tx.peerPushDebit.put(newDg);
+        await ctx.updateTransactionMeta(tx);
         return { oldTxState, newTxState };
       }
       return undefined;
     },
   );
-  notifyTransition(wex, transactionId, transitionInfo);
+  notifyTransition(wex, ctx.transactionId, transitionInfo);
   // FIXME: Shouldn't this be finished in some cases?!
   return TaskRunResult.backoff();
 }
@@ -1007,10 +1034,7 @@ async function processPeerPushDebitReady(
 ): Promise<TaskRunResult> {
   logger.trace("processing peer-push-debit pending(ready)");
   const pursePub = peerPushInitiation.pursePub;
-  const transactionId = constructTaskIdentifier({
-    tag: PendingTaskType.PeerPushDebit,
-    pursePub,
-  });
+  const ctx = new PeerPushDebitTransactionContext(wex, pursePub);
   const mergeUrl = new URL(
     `purses/${pursePub}/merge`,
     peerPushInitiation.exchangeBaseUrl,
@@ -1059,6 +1083,7 @@ async function processPeerPushDebitReady(
           "peerPushDebit",
           "refreshGroups",
           "refreshSessions",
+          "transactionsMeta",
         ],
       },
       async (tx) => {
@@ -1087,13 +1112,14 @@ async function processPeerPushDebitReady(
             currency,
             coinPubs,
             RefreshReason.AbortPeerPushDebit,
-            transactionId,
+            ctx.transactionId,
           );
 
           ppiRec.abortRefreshGroupId = refresh.refreshGroupId;
         }
         ppiRec.status = PeerPushDebitStatus.AbortingRefreshExpired;
         await tx.peerPushDebit.put(ppiRec);
+        await ctx.updateTransactionMeta(tx);
         const newTxState = computePeerPushDebitTransactionState(ppiRec);
         return {
           oldTxState,
@@ -1101,7 +1127,7 @@ async function processPeerPushDebitReady(
         };
       },
     );
-    notifyTransition(wex, transactionId, transitionInfo);
+    notifyTransition(wex, ctx.transactionId, transitionInfo);
     return TaskRunResult.backoff();
   } else {
     logger.warn(`unexpected HTTP status for purse: ${resp.status}`);
@@ -1196,6 +1222,7 @@ export async function initiatePeerPushDebit(
         "peerPushDebit",
         "refreshGroups",
         "refreshSessions",
+        "transactionsMeta",
       ],
     },
     async (tx) => {
@@ -1262,6 +1289,7 @@ export async function initiatePeerPushDebit(
       }
 
       await tx.peerPushDebit.add(ppi);
+      await ctx.updateTransactionMeta(tx);
 
       await tx.contractTerms.put({
         h: hContractTerms,
