@@ -15,8 +15,9 @@
  */
 
 /**
- * Database query abstractions.
- * @module Query
+ * @fileoverview
+ * Query helpers for IndexedDB databases.
+ *
  * @author Florian Dold
  */
 
@@ -35,7 +36,13 @@ import {
   IDBValidKey,
   IDBVersionChangeEvent,
 } from "@gnu-taler/idb-bridge";
-import { Codec, Logger, openPromise } from "@gnu-taler/taler-util";
+import {
+  CancellationToken,
+  Codec,
+  Logger,
+  openPromise,
+  safeStringifyException,
+} from "@gnu-taler/taler-util";
 
 const logger = new Logger("query.ts");
 
@@ -70,16 +77,42 @@ export interface IndexOptions {
   unique?: boolean;
 }
 
+/**
+ * Log extra stuff that would be too verbose even
+ * on loglevel TRACE.
+ */
+const logExtra = false;
+
+let idbRequestPromId = 1;
+
 function requestToPromise(req: IDBRequest): Promise<any> {
+  const myId = idbRequestPromId++;
+  if (logExtra) {
+    logger.trace(`started db request ${myId}`);
+  }
   const stack = Error("Failed request was started here.");
   return new Promise((resolve, reject) => {
     req.onsuccess = () => {
+      if (logExtra) {
+        logger.trace(`finished db request ${myId} with success`);
+      }
       resolve(req.result);
     };
     req.onerror = () => {
-      console.error("error in DB request", req.error);
+      if (logExtra) {
+        logger.trace(`finished db request ${myId} with error`);
+      }
+      if (
+        req.error != null &&
+        "name" in req.error &&
+        req.error.name === "AbortError"
+      ) {
+        logger.warn("DB request failed, transaction aborted");
+      } else {
+        logger.error(`error in DB request: ${req.error}`);
+        logger.error(`Request failed: ${stack}`);
+      }
       reject(req.error);
-      console.error("Request failed:", stack);
     };
   });
 }
@@ -558,33 +591,67 @@ function runTx<Arg, Res>(
   tx: IDBTransaction,
   arg: Arg,
   f: (t: Arg, t2: IDBTransaction) => Promise<Res>,
+  triggerContext: InternalTriggerContext,
+  cancellationToken: CancellationToken,
 ): Promise<Res> {
+  // Create stack trace in case we need to to print later where
+  // the transaction was started.
   const stack = Error("Failed transaction was started here.");
+
+  const unregisterOnCancelled = cancellationToken.onCancelled(() => {
+    tx.abort();
+  });
+
   return new Promise((resolve, reject) => {
     let funResult: any = undefined;
     let gotFunResult = false;
     let transactionException: any = undefined;
+    let aborted = false;
     tx.oncomplete = () => {
+      logger.trace("transaction completed");
       // This is a fatal error: The transaction completed *before*
       // the transaction function returned.  Likely, the transaction
       // function waited on a promise that is *not* resolved in the
       // microtask queue, thus triggering the auto-commit behavior.
       // Unfortunately, the auto-commit behavior of IDB can't be switched
-      // of.  There are some proposals to add this functionality in the future.
+      // off.  There are some proposals to add this functionality in the future.
       if (!gotFunResult) {
         const msg =
           "BUG: transaction closed before transaction function returned";
         logger.error(msg);
         logger.error(`${stack.stack}`);
         reject(Error(msg));
+      } else {
+        resolve(funResult);
       }
-      resolve(funResult);
+      triggerContext.handleAfterCommit();
+      unregisterOnCancelled();
     };
     tx.onerror = () => {
+      logger.trace("transaction had error");
+      if (cancellationToken.isCancelled) {
+        reject(
+          new CancellationToken.CancellationError(cancellationToken.reason),
+        );
+        return;
+      }
       logger.error("error in transaction");
       logger.error(`${stack.stack}`);
+      const txError = tx.error;
+      if (txError) {
+        reject(txError);
+      } else {
+        reject(new Error("unknown transaction error"));
+      }
     };
     tx.onabort = () => {
+      logger.trace("transaction was aborted");
+      if (cancellationToken.isCancelled) {
+        reject(
+          new CancellationToken.CancellationError(cancellationToken.reason),
+        );
+        return;
+      }
       let msg: string;
       if (tx.error) {
         msg = `Transaction aborted (transaction error): ${tx.error}`;
@@ -593,6 +660,8 @@ function runTx<Arg, Res>(
       } else {
         msg = "Transaction aborted (no DB error)";
       }
+      aborted = true;
+      unregisterOnCancelled();
       logger.error(msg);
       logger.error(`${stack.stack}`);
       reject(new TransactionAbortedError(msg));
@@ -600,21 +669,25 @@ function runTx<Arg, Res>(
     const resP = Promise.resolve().then(() => f(arg, tx));
     resP
       .then((result) => {
+        logger.trace("transaction function returned");
         gotFunResult = true;
         funResult = result;
       })
       .catch((e) => {
         if (e == TransactionAbort) {
           logger.trace("aborting transaction");
+          tx.abort();
+        } else if ("name" in e && e.name === "AbortError") {
+          logger.warn("got AbortError, transaction was aborted");
         } else {
           transactionException = e;
-          console.error("Transaction failed:", e);
-          console.error(stack);
+          logger.error(`Transaction failed: ${safeStringifyException(e)}`);
+          logger.error(`${stack}`);
           tx.abort();
         }
       })
       .catch((e) => {
-        console.error("fatal: aborting transaction failed", e);
+        logger.error(`aborting failed: ${safeStringifyException(e)}`);
       });
   });
 }
@@ -622,6 +695,7 @@ function runTx<Arg, Res>(
 function makeReadContext(
   tx: IDBTransaction,
   storePick: { [n: string]: StoreWithIndexes<any, any, any> },
+  triggerContext: InternalTriggerContext,
 ): any {
   const ctx: { [s: string]: StoreReadOnlyAccessor<any, any> } = {};
   for (const storeAlias in storePick) {
@@ -634,10 +708,12 @@ function makeReadContext(
       const indexName = indexDescriptor.name;
       indexes[indexAlias] = {
         get(key) {
+          triggerContext.storesAccessed.add(storeName);
           const req = tx.objectStore(storeName).index(indexName).get(key);
           return requestToPromise(req);
         },
         iter(query) {
+          triggerContext.storesAccessed.add(storeName);
           const req = tx
             .objectStore(storeName)
             .index(indexName)
@@ -645,6 +721,7 @@ function makeReadContext(
           return new ResultStream<any>(req);
         },
         getAll(query, count) {
+          triggerContext.storesAccessed.add(storeName);
           const req = tx
             .objectStore(storeName)
             .index(indexName)
@@ -652,6 +729,7 @@ function makeReadContext(
           return requestToPromise(req);
         },
         getAllKeys(query, count) {
+          triggerContext.storesAccessed.add(storeName);
           const req = tx
             .objectStore(storeName)
             .index(indexName)
@@ -659,6 +737,7 @@ function makeReadContext(
           return requestToPromise(req);
         },
         count(query) {
+          triggerContext.storesAccessed.add(storeName);
           const req = tx.objectStore(storeName).index(indexName).count(query);
           return requestToPromise(req);
         },
@@ -667,14 +746,17 @@ function makeReadContext(
     ctx[storeAlias] = {
       indexes,
       get(key) {
+        triggerContext.storesAccessed.add(storeName);
         const req = tx.objectStore(storeName).get(key);
         return requestToPromise(req);
       },
       getAll(query, count) {
+        triggerContext.storesAccessed.add(storeName);
         const req = tx.objectStore(storeName).getAll(query, count);
         return requestToPromise(req);
       },
       iter(query) {
+        triggerContext.storesAccessed.add(storeName);
         const req = tx.objectStore(storeName).openCursor(query);
         return new ResultStream<any>(req);
       },
@@ -686,6 +768,7 @@ function makeReadContext(
 function makeWriteContext(
   tx: IDBTransaction,
   storePick: { [n: string]: StoreWithIndexes<any, any, any> },
+  triggerContext: InternalTriggerContext,
 ): any {
   const ctx: { [s: string]: StoreReadWriteAccessor<any, any> } = {};
   for (const storeAlias in storePick) {
@@ -698,10 +781,12 @@ function makeWriteContext(
       const indexName = indexDescriptor.name;
       indexes[indexAlias] = {
         get(key) {
+          triggerContext.storesAccessed.add(storeName);
           const req = tx.objectStore(storeName).index(indexName).get(key);
           return requestToPromise(req);
         },
         iter(query) {
+          triggerContext.storesAccessed.add(storeName);
           const req = tx
             .objectStore(storeName)
             .index(indexName)
@@ -709,6 +794,7 @@ function makeWriteContext(
           return new ResultStream<any>(req);
         },
         getAll(query, count) {
+          triggerContext.storesAccessed.add(storeName);
           const req = tx
             .objectStore(storeName)
             .index(indexName)
@@ -716,6 +802,7 @@ function makeWriteContext(
           return requestToPromise(req);
         },
         getAllKeys(query, count) {
+          triggerContext.storesAccessed.add(storeName);
           const req = tx
             .objectStore(storeName)
             .index(indexName)
@@ -723,6 +810,7 @@ function makeWriteContext(
           return requestToPromise(req);
         },
         count(query) {
+          triggerContext.storesAccessed.add(storeName);
           const req = tx.objectStore(storeName).index(indexName).count(query);
           return requestToPromise(req);
         },
@@ -731,18 +819,23 @@ function makeWriteContext(
     ctx[storeAlias] = {
       indexes,
       get(key) {
+        triggerContext.storesAccessed.add(storeName);
         const req = tx.objectStore(storeName).get(key);
         return requestToPromise(req);
       },
       getAll(query, count) {
+        triggerContext.storesAccessed.add(storeName);
         const req = tx.objectStore(storeName).getAll(query, count);
         return requestToPromise(req);
       },
       iter(query) {
+        triggerContext.storesAccessed.add(storeName);
         const req = tx.objectStore(storeName).openCursor(query);
         return new ResultStream<any>(req);
       },
       async add(r, k) {
+        triggerContext.storesAccessed.add(storeName);
+        triggerContext.storesModified.add(storeName);
         const req = tx.objectStore(storeName).add(r, k);
         const key = await requestToPromise(req);
         return {
@@ -750,6 +843,8 @@ function makeWriteContext(
         };
       },
       async put(r, k) {
+        triggerContext.storesAccessed.add(storeName);
+        triggerContext.storesModified.add(storeName);
         const req = tx.objectStore(storeName).put(r, k);
         const key = await requestToPromise(req);
         return {
@@ -757,6 +852,8 @@ function makeWriteContext(
         };
       },
       delete(k) {
+        triggerContext.storesAccessed.add(storeName);
+        triggerContext.storesModified.add(storeName);
         const req = tx.objectStore(storeName).delete(k);
         return requestToPromise(req);
       },
@@ -765,9 +862,26 @@ function makeWriteContext(
   return ctx;
 }
 
+/**
+ * Handle for typed access to a database.
+ */
 export interface DbAccess<StoreMap> {
+  /**
+   * The underlying IndexedDB database handle.
+   *
+   * Use with caution, as using the handle directly will not
+   * properly run DB triggers.
+   */
   idbHandle(): IDBDatabase;
 
+  /**
+   * Run an async function in a "readwrite" transaction on the database, using
+   * all object store.
+   *
+   * The transaction function must run within the microtask queue.
+   * Waiting for macrotasks results in an autocommit and
+   * a subsequent exception thrown by this function.
+   */
   runAllStoresReadWriteTx<T>(
     options: {
       label?: string;
@@ -777,6 +891,14 @@ export interface DbAccess<StoreMap> {
     ) => Promise<T>,
   ): Promise<T>;
 
+  /**
+   * Run an async function in a "readonly" transaction on the database, using
+   * all object store.
+   *
+   * The transaction function must run within the microtask queue.
+   * Waiting for macrotasks results in an autocommit and
+   * a subsequent exception thrown by this function.
+   */
   runAllStoresReadOnlyTx<T>(
     options: {
       label?: string;
@@ -786,22 +908,80 @@ export interface DbAccess<StoreMap> {
     ) => Promise<T>,
   ): Promise<T>;
 
+  /**
+   * Run an async function in a "readwrite" transaction on the database, using
+   * the selected object store.
+   *
+   * The transaction function must run within the microtask queue.
+   * Waiting for macrotasks results in an autocommit and
+   * a subsequent exception thrown by this function.
+   */
   runReadWriteTx<T, StoreNameArray extends Array<StoreNames<StoreMap>>>(
-    storeNames: StoreNameArray,
+    opts: {
+      storeNames: StoreNameArray;
+      label?: string;
+    },
     txf: (tx: DbReadWriteTransaction<StoreMap, StoreNameArray>) => Promise<T>,
   ): Promise<T>;
 
+  /**
+   * Run an async function in a "readonly" transaction on the database, using
+   * the selected object store.
+   *
+   * The transaction function must run within the microtask queue.
+   * Waiting for macrotasks results in an autocommit and
+   * a subsequent exception thrown by this function.
+   */
   runReadOnlyTx<T, StoreNameArray extends Array<StoreNames<StoreMap>>>(
-    storeNames: StoreNameArray,
+    opts: {
+      storeNames: StoreNameArray;
+      label?: string;
+    },
     txf: (tx: DbReadOnlyTransaction<StoreMap, StoreNameArray>) => Promise<T>,
   ): Promise<T>;
 }
 
+export interface AfterCommitInfo {
+  mode: IDBTransactionMode;
+  scope: Set<string>;
+  accessedStores: Set<string>;
+  modifiedStores: Set<string>;
+}
+
 export interface TriggerSpec {
   /**
    * Trigger run after every successful commit, run outside of the transaction.
    */
-  afterCommit?: (mode: IDBTransactionMode, stores: string[]) => void;
+  afterCommit?: (info: AfterCommitInfo) => void;
+
+  // onRead(store, value)
+  // initState<State> () => State
+  // beforeCommit<State>? (tx: Transaction, s: State | undefined) => Promise<void>;
+}
+
+class InternalTriggerContext {
+  storesScope: Set<string>;
+  storesAccessed: Set<string> = new Set();
+  storesModified: Set<string> = new Set();
+
+  constructor(
+    private triggerSpec: TriggerSpec,
+    private mode: IDBTransactionMode,
+    scope: string[],
+  ) {
+    this.storesScope = new Set(scope);
+  }
+
+  handleAfterCommit() {
+    if (this.triggerSpec.afterCommit) {
+      this.triggerSpec.afterCommit({
+        mode: this.mode,
+        accessedStores: this.storesAccessed,
+        modifiedStores: this.storesModified,
+        scope: this.storesScope,
+      });
+    }
+  }
 }
 
 /**
@@ -814,13 +994,14 @@ export class DbAccessImpl<StoreMap> implements DbAccess<StoreMap> {
     private db: IDBDatabase,
     private stores: StoreMap,
     private triggers: TriggerSpec = {},
+    private cancellationToken: CancellationToken,
   ) {}
 
   idbHandle(): IDBDatabase {
     return this.db;
   }
 
-  runAllStoresReadWriteTx<T>(
+  async runAllStoresReadWriteTx<T>(
     options: {
       label?: string;
     },
@@ -828,6 +1009,7 @@ export class DbAccessImpl<StoreMap> implements DbAccess<StoreMap> {
       tx: DbReadWriteTransaction<StoreMap, Array<StoreNames<StoreMap>>>,
     ) => Promise<T>,
   ): Promise<T> {
+    this.cancellationToken.throwIfCancelled();
     const accessibleStores: { [x: string]: StoreWithIndexes<any, any, any> } =
       {};
     const strStoreNames: string[] = [];
@@ -836,12 +1018,24 @@ export class DbAccessImpl<StoreMap> implements DbAccess<StoreMap> {
       strStoreNames.push(swi.storeName);
       accessibleStores[swi.storeName] = swi;
     }
-    const tx = this.db.transaction(strStoreNames, "readwrite");
-    const writeContext = makeWriteContext(tx, accessibleStores);
-    return runTx(tx, writeContext, txf);
+    const mode = "readwrite";
+    const triggerContext = new InternalTriggerContext(
+      this.triggers,
+      mode,
+      strStoreNames,
+    );
+    const tx = this.db.transaction(strStoreNames, mode);
+    const writeContext = makeWriteContext(tx, accessibleStores, triggerContext);
+    return await runTx(
+      tx,
+      writeContext,
+      txf,
+      triggerContext,
+      this.cancellationToken,
+    );
   }
 
-  runAllStoresReadOnlyTx<T>(
+  async runAllStoresReadOnlyTx<T>(
     options: {
       label?: string;
     },
@@ -849,6 +1043,7 @@ export class DbAccessImpl<StoreMap> implements DbAccess<StoreMap> {
       tx: DbReadOnlyTransaction<StoreMap, Array<StoreNames<StoreMap>>>,
     ) => Promise<T>,
   ): Promise<T> {
+    this.cancellationToken.throwIfCancelled();
     const accessibleStores: { [x: string]: StoreWithIndexes<any, any, any> } =
       {};
     const strStoreNames: string[] = [];
@@ -857,56 +1052,87 @@ export class DbAccessImpl<StoreMap> implements DbAccess<StoreMap> {
       strStoreNames.push(swi.storeName);
       accessibleStores[swi.storeName] = swi;
     }
-    const tx = this.db.transaction(strStoreNames, "readonly");
-    const writeContext = makeReadContext(tx, accessibleStores);
-    return runTx(tx, writeContext, txf);
+    const mode = "readonly";
+    const triggerContext = new InternalTriggerContext(
+      this.triggers,
+      mode,
+      strStoreNames,
+    );
+    const tx = this.db.transaction(strStoreNames, mode);
+    const writeContext = makeReadContext(tx, accessibleStores, triggerContext);
+    const res = await runTx(
+      tx,
+      writeContext,
+      txf,
+      triggerContext,
+      this.cancellationToken,
+    );
+    return res;
   }
 
-  runReadWriteTx<T, StoreNameArray extends Array<StoreNames<StoreMap>>>(
-    storeNames: StoreNameArray,
+  async runReadWriteTx<T, StoreNameArray extends Array<StoreNames<StoreMap>>>(
+    opts: {
+      storeNames: StoreNameArray;
+    },
     txf: (tx: DbReadWriteTransaction<StoreMap, StoreNameArray>) => Promise<T>,
   ): Promise<T> {
+    this.cancellationToken.throwIfCancelled();
     const accessibleStores: { [x: string]: StoreWithIndexes<any, any, any> } =
       {};
     const strStoreNames: string[] = [];
-    for (const sn of storeNames) {
+    for (const sn of opts.storeNames) {
       const swi = (this.stores as any)[sn] as StoreWithIndexes<any, any, any>;
       strStoreNames.push(swi.storeName);
       accessibleStores[swi.storeName] = swi;
     }
     const mode = "readwrite";
+    const triggerContext = new InternalTriggerContext(
+      this.triggers,
+      mode,
+      strStoreNames,
+    );
     const tx = this.db.transaction(strStoreNames, mode);
-    const writeContext = makeWriteContext(tx, accessibleStores);
-    const res = runTx(tx, writeContext, txf);
-    if (this.triggers.afterCommit) {
-      this.triggers.afterCommit(mode, strStoreNames);
-    }
+    const writeContext = makeWriteContext(tx, accessibleStores, triggerContext);
+    const res = await runTx(
+      tx,
+      writeContext,
+      txf,
+      triggerContext,
+      this.cancellationToken,
+    );
     return res;
   }
 
-  runReadOnlyTx<T, StoreNameArray extends Array<StoreNames<StoreMap>>>(
-    storeNames: StoreNameArray,
+  async runReadOnlyTx<T, StoreNameArray extends Array<StoreNames<StoreMap>>>(
+    opts: {
+      storeNames: StoreNameArray;
+    },
     txf: (tx: DbReadOnlyTransaction<StoreMap, StoreNameArray>) => Promise<T>,
   ): Promise<T> {
+    this.cancellationToken.throwIfCancelled();
     const accessibleStores: { [x: string]: StoreWithIndexes<any, any, any> } =
       {};
     const strStoreNames: string[] = [];
-    for (const sn of storeNames) {
+    for (const sn of opts.storeNames) {
       const swi = (this.stores as any)[sn] as StoreWithIndexes<any, any, any>;
       strStoreNames.push(swi.storeName);
       accessibleStores[swi.storeName] = swi;
     }
     const mode = "readonly";
+    const triggerContext = new InternalTriggerContext(
+      this.triggers,
+      mode,
+      strStoreNames,
+    );
     const tx = this.db.transaction(strStoreNames, mode);
-    const readContext = makeReadContext(tx, accessibleStores);
-    const res = runTx(tx, readContext, txf);
-    if (this.triggers.afterCommit) {
-      this.triggers.afterCommit(mode, strStoreNames);
-    }
+    const readContext = makeReadContext(tx, accessibleStores, triggerContext);
+    const res = await runTx(
+      tx,
+      readContext,
+      txf,
+      triggerContext,
+      this.cancellationToken,
+    );
     return res;
   }
-
-  registerPostCommitTrigger(args: {
-    handler: (storeNames: string[]) => void;
-  }): void {}
 }
