@@ -122,26 +122,10 @@ import {
   WalletExecutionContext,
 } from "./wallet.js";
 import { getCandidateWithdrawalDenomsTx } from "./withdraw.js";
+import { getScopeForAllExchanges } from "./exchanges.js";
 
 const logger = new Logger("refresh.ts");
 
-/**
- * Update the materialized refresh transaction based
- * on the refresh group record.
- */
-async function updateRefreshTransaction(
-  ctx: RefreshTransactionContext,
-  tx: WalletDbReadWriteTransaction<
-    [
-      "refreshGroups",
-      "transactionsMeta",
-      "operationRetries",
-      "exchanges",
-      "exchangeDetails",
-    ]
-  >,
-): Promise<void> {}
-
 export class RefreshTransactionContext implements TransactionContext {
   readonly transactionId: TransactionIdStr;
   readonly taskId: TaskIdStr;
@@ -160,6 +144,23 @@ export class RefreshTransactionContext implements TransactionContext {
     });
   }
 
+  async updateTransactionMeta(
+    tx: WalletDbReadWriteTransaction<["refreshGroups", "transactionsMeta"]>,
+  ): Promise<void> {
+    const rgRec = await tx.refreshGroups.get(this.refreshGroupId);
+    if (!rgRec) {
+      await tx.transactionsMeta.delete(this.transactionId);
+      return;
+    }
+    await tx.transactionsMeta.put({
+      transactionId: this.transactionId,
+      status: rgRec.operationStatus,
+      timestamp: rgRec.timestampCreated,
+      currency: rgRec.currency,
+      exchanges: Object.keys(rgRec.infoPerExchange ?? {}),
+    });
+  }
+
   /**
    * Get the full transaction details for the transaction.
    *
@@ -186,6 +187,7 @@ export class RefreshTransactionContext implements TransactionContext {
     return {
       type: TransactionType.Refresh,
       txState,
+      scopes: await getScopeForAllExchanges(tx, !refreshGroupRecord.infoPerExchange? []: Object.keys(refreshGroupRecord.infoPerExchange)),
       txActions: computeRefreshTransactionActions(refreshGroupRecord),
       refreshReason: refreshGroupRecord.reason,
       amountEffective: isUnsuccessfulTransaction(txState)
@@ -252,7 +254,7 @@ export class RefreshTransactionContext implements TransactionContext {
         switch (res.type) {
           case TransitionResultType.Transition: {
             await tx.refreshGroups.put(res.rec);
-            await updateRefreshTransaction(this, tx);
+            await this.updateTransactionMeta(tx);
             const newTxState = computeRefreshTransactionState(res.rec);
             return {
               oldTxState,
@@ -261,7 +263,7 @@ export class RefreshTransactionContext implements TransactionContext {
           }
           case TransitionResultType.Delete:
             await tx.refreshGroups.delete(this.refreshGroupId);
-            await updateRefreshTransaction(this, tx);
+            await this.updateTransactionMeta(tx);
             return {
               oldTxState,
               newTxState: {
@@ -822,6 +824,7 @@ async function handleRefreshMeltGone(
         "coins",
         "denominations",
         "coinAvailability",
+        "transactionsMeta",
       ],
     },
     async (tx) => {
@@ -846,6 +849,7 @@ async function handleRefreshMeltGone(
       refreshSession.lastError = errDetails;
       await destroyRefreshSession(ctx.wex, tx, rg, refreshSession);
       await tx.refreshGroups.put(rg);
+      await ctx.updateTransactionMeta(tx);
       await tx.refreshSessions.put(refreshSession);
     },
   );
@@ -901,6 +905,7 @@ async function handleRefreshMeltConflict(
         "denominations",
         "coins",
         "coinAvailability",
+        "transactionsMeta",
       ],
     },
     async (tx) => {
@@ -927,6 +932,7 @@ async function handleRefreshMeltConflict(
         }
         refreshSession.lastError = errDetails;
         await tx.refreshGroups.put(rg);
+        await ctx.updateTransactionMeta(tx);
         await tx.refreshSessions.put(refreshSession);
       } else {
         // Try again with new denoms!
@@ -971,6 +977,7 @@ async function handleRefreshMeltNotFound(
         "coins",
         "denominations",
         "coinAvailability",
+        "transactionsMeta",
       ],
     },
     async (tx) => {
@@ -995,6 +1002,7 @@ async function handleRefreshMeltNotFound(
       await destroyRefreshSession(ctx.wex, tx, rg, refreshSession);
       refreshSession.lastError = errDetails;
       await tx.refreshGroups.put(rg);
+      await ctx.updateTransactionMeta(tx);
       await tx.refreshSessions.put(refreshSession);
     },
   );
@@ -1275,6 +1283,7 @@ async function refreshReveal(
         "coinAvailability",
         "refreshGroups",
         "refreshSessions",
+        "transactionsMeta",
       ],
     },
     async (tx) => {
@@ -1323,6 +1332,7 @@ async function refreshReveal(
         await tx.coinAvailability.put(car);
       }
       await tx.refreshGroups.put(rg);
+      await ctx.updateTransactionMeta(tx);
     },
   );
   logger.trace("refresh finished (end of reveal)");
@@ -1341,6 +1351,7 @@ async function handleRefreshRevealError(
         "coins",
         "denominations",
         "coinAvailability",
+        "transactionsMeta",
       ],
     },
     async (tx) => {
@@ -1366,6 +1377,7 @@ async function handleRefreshRevealError(
       await destroyRefreshSession(ctx.wex, tx, rg, refreshSession);
       await tx.refreshGroups.put(rg);
       await tx.refreshSessions.put(refreshSession);
+      await ctx.updateTransactionMeta(tx);
     },
   );
 }
@@ -1438,7 +1450,14 @@ export async function processRefreshGroup(
   // status of the whole refresh group.
 
   const transitionInfo = await wex.db.runReadWriteTx(
-    { storeNames: ["coins", "coinAvailability", "refreshGroups"] },
+    {
+      storeNames: [
+        "coins",
+        "coinAvailability",
+        "refreshGroups",
+        "transactionsMeta",
+      ],
+    },
     async (tx) => {
       const rg = await tx.refreshGroups.get(refreshGroupId);
       if (!rg) {
@@ -1474,6 +1493,7 @@ export async function processRefreshGroup(
         }
         await makeCoinsVisible(wex, tx, ctx.transactionId);
         await tx.refreshGroups.put(rg);
+        await ctx.updateTransactionMeta(tx);
         const newTxState = computeRefreshTransactionState(rg);
         return {
           oldTxState,
@@ -1711,6 +1731,7 @@ export async function createRefreshGroup(
       "refreshGroups",
       "refreshSessions",
       "coinAvailability",
+      "transactionsMeta",
     ]
   >,
   currency: string,
@@ -1758,14 +1779,15 @@ export async function createRefreshGroup(
     await initRefreshSession(wex, tx, refreshGroup, i);
   }
 
+  const ctx = new RefreshTransactionContext(wex, refreshGroupId);
+
   await tx.refreshGroups.put(refreshGroup);
+  await ctx.updateTransactionMeta(tx);
 
   const newTxState = computeRefreshTransactionState(refreshGroup);
 
   logger.trace(`created refresh group ${refreshGroupId}`);
 
-  const ctx = new RefreshTransactionContext(wex, refreshGroupId);
-
   // Shepherd the task.
   // If the current transaction fails to commit the refresh
   // group to the DB, the shepherd will give up.
@@ -1866,6 +1888,7 @@ export async function forceRefresh(
         "denominations",
         "coins",
         "coinHistory",
+        "transactionsMeta",
       ],
     },
     async (tx) => {
