@@ -53,6 +53,7 @@ import {
   OPERATION_STATUS_NONFINAL_FIRST,
   OPERATION_STATUS_NONFINAL_LAST,
   OperationRetryRecord,
+  ReserveRecordStatus,
   WalletDbAllStoresReadOnlyTransaction,
   WalletDbReadOnlyTransaction,
   timestampAbsoluteFromDb,
@@ -64,6 +65,7 @@ import {
 } from "./deposits.js";
 import {
   computeDenomLossTransactionStatus,
+  processExchangeKyc,
   updateExchangeFromUrlHandler,
 } from "./exchanges.js";
 import {
@@ -127,6 +129,7 @@ function taskGivesLiveness(taskId: string): boolean {
   switch (parsedTaskId.tag) {
     case PendingTaskType.Backup:
     case PendingTaskType.ExchangeUpdate:
+    case PendingTaskType.ExchangeWalletKyc:
       return false;
     case PendingTaskType.Deposit:
     case PendingTaskType.PeerPullCredit:
@@ -393,10 +396,13 @@ export class TaskSchedulerImpl implements TaskScheduler {
       try {
         res = await callOperationHandlerForTaskId(wex, taskId);
       } catch (e) {
-        logger.trace(`Shepherd error ${taskId} saving response ${e}`);
+        const errorDetail = getErrorDetailFromException(e);
+        logger.trace(
+          `Shepherd error ${taskId} saving response ${j2s(errorDetail)}`,
+        );
         res = {
           type: TaskRunResultType.Error,
-          errorDetail: getErrorDetailFromException(e),
+          errorDetail,
         };
       }
       if (info.cts.token.isCancelled) {
@@ -671,6 +677,8 @@ async function callOperationHandlerForTaskId(
       return await processPeerPullDebit(wex, pending.peerPullDebitId);
     case PendingTaskType.PeerPushCredit:
       return await processPeerPushCredit(wex, pending.peerPushCreditId);
+    case PendingTaskType.ExchangeWalletKyc:
+      return await processExchangeKyc(wex, pending.exchangeBaseUrl);
     case PendingTaskType.RewardPickup:
       throw Error("not supported anymore");
     default:
@@ -697,6 +705,7 @@ async function taskToRetryNotification(
 
   switch (parsedTaskId.tag) {
     case PendingTaskType.ExchangeUpdate:
+    case PendingTaskType.ExchangeWalletKyc:
       return makeExchangeRetryNotification(ws, tx, pendingTaskId, e);
     case PendingTaskType.PeerPullCredit:
     case PendingTaskType.PeerPullDebit:
@@ -853,8 +862,12 @@ async function makeExchangeRetryNotification(
 ): Promise<WalletNotification | undefined> {
   logger.info("making exchange retry notification");
   const parsedTaskId = parseTaskIdentifier(pendingTaskId);
-  if (parsedTaskId.tag !== PendingTaskType.ExchangeUpdate) {
-    throw Error("invalid task identifier");
+  switch (parsedTaskId.tag) {
+    case PendingTaskType.ExchangeUpdate:
+    case PendingTaskType.ExchangeWalletKyc:
+      break;
+    default:
+      throw Error("invalid task identifier");
   }
   const rec = await tx.exchanges.get(parsedTaskId.exchangeBaseUrl);
 
@@ -958,6 +971,7 @@ export async function getActiveTaskIds(
         "peerPushDebit",
         "peerPullDebit",
         "peerPushCredit",
+        "reserves",
       ],
     },
     async (tx) => {
@@ -1091,7 +1105,7 @@ export async function getActiveTaskIds(
         }
       }
 
-      // exchange update
+      // exchange update and KYC
 
       {
         const exchanges = await tx.exchanges.getAll();
@@ -1101,6 +1115,22 @@ export async function getActiveTaskIds(
             exchangeBaseUrl: rec.baseUrl,
           });
           res.taskIds.push(taskIdUpdate);
+
+          const reserveId = rec.currentMergeReserveRowId;
+          if (reserveId == null) {
+            continue;
+          }
+          const reserveRec = await tx.reserves.get(reserveId);
+          if (
+            reserveRec?.status != null &&
+            reserveRec.status != ReserveRecordStatus.Done
+          ) {
+            const taskIdKyc = constructTaskIdentifier({
+              tag: PendingTaskType.ExchangeWalletKyc,
+              exchangeBaseUrl: rec.baseUrl,
+            });
+            res.taskIds.push(taskIdKyc);
+          }
         }
       }
 
