@@ -57,6 +57,8 @@ import {
   FailTransactionRequest,
   ForgetKnownBankAccountsRequest,
   GetActiveTasksResponse,
+  GetBankingChoicesForPaytoRequest,
+  GetBankingChoicesForPaytoResponse,
   GetContractTermsDetailsRequest,
   GetCurrencySpecificationRequest,
   GetCurrencySpecificationResponse,
@@ -147,6 +149,7 @@ import {
   codecForForgetKnownBankAccounts,
   codecForGetAmountRequest,
   codecForGetBalanceDetailRequest,
+  codecForGetBankingChoicesForPaytoRequest,
   codecForGetContractTermsDetails,
   codecForGetCurrencyInfoRequest,
   codecForGetDepositWireTypesForCurrencyRequest,
@@ -181,12 +184,14 @@ import {
   codecForSetCoinSuspendedRequest,
   codecForSetWalletDeviceIdRequest,
   codecForSharePaymentRequest,
+  codecForStartExchangeWalletKycRequest,
   codecForStartRefundQueryRequest,
   codecForSuspendTransaction,
   codecForTestPayArgs,
   codecForTestingGetDenomStatsRequest,
   codecForTestingGetReserveHistoryRequest,
   codecForTestingSetTimetravelRequest,
+  codecForTestingWaitWalletKycRequest,
   codecForTransactionByIdRequest,
   codecForTransactionsRequest,
   codecForUpdateExchangeEntryRequest,
@@ -264,6 +269,9 @@ import {
   getExchangeResources,
   getExchangeTos,
   getExchangeWireDetailsInTx,
+  handleStartExchangeWalletKyc,
+  handleTestingWaitExchangeState,
+  handleTestingWaitExchangeWalletKyc,
   listExchanges,
   lookupExchangeByUri,
 } from "./exchanges.js";
@@ -335,8 +343,8 @@ import {
   failTransaction,
   getTransactionById,
   getTransactions,
-  getWithdrawalTransactionByUri,
   parseTransactionIdentifier,
+  rematerializeTransactions,
   restartAll as restartAllRunningTasks,
   resumeTransaction,
   retryAll,
@@ -428,6 +436,41 @@ async function fillDefaults(wex: WalletExecutionContext): Promise<void> {
   }
 }
 
+/**
+ * Incremented each time we want to re-materialize transactions.
+ */
+const MATERIALIZED_TRANSACTIONS_VERSION = 1;
+
+async function migrateMaterializedTransactions(
+  wex: WalletExecutionContext,
+): Promise<void> {
+  await wex.db.runAllStoresReadWriteTx({}, async (tx) => {
+    const ver = await tx.config.get("materializedTransactionsVersion");
+    if (ver) {
+      if (ver.key !== ConfigRecordKey.MaterializedTransactionsVersion) {
+        logger.error("invalid configuration (materializedTransactionsVersion)");
+        return;
+      }
+      if (ver.value == MATERIALIZED_TRANSACTIONS_VERSION) {
+        return;
+      }
+      if (ver.value > MATERIALIZED_TRANSACTIONS_VERSION) {
+        logger.error(
+          "database is newer than code (materializedTransactionsVersion)",
+        );
+        return;
+      }
+    }
+
+    await rematerializeTransactions(wex, tx);
+
+    await tx.config.put({
+      key: ConfigRecordKey.MaterializedTransactionsVersion,
+      value: MATERIALIZED_TRANSACTIONS_VERSION,
+    });
+  });
+}
+
 export async function getDenomInfo(
   wex: WalletExecutionContext,
   tx: WalletDbReadOnlyTransaction<["denominations"]>,
@@ -708,6 +751,9 @@ async function recoverStoredBackup(
   });
   logger.info(`backup found, now importing`);
   await importDb(wex.db.idbHandle(), bd);
+  await wex.db.runAllStoresReadWriteTx({}, async (tx) => {
+    await rematerializeTransactions(wex, tx);
+  });
   logger.info(`import done`);
 }
 
@@ -797,6 +843,9 @@ async function handleSetWalletRunConfig(
     logger.trace("filling defaults");
     await fillDefaults(wex);
   }
+
+  await migrateMaterializedTransactions(wex);
+
   const resp: InitResponse = {
     versionInfo: await handleGetVersion(wex),
   };
@@ -1015,6 +1064,50 @@ async function handleGetQrCodesForPayto(
   };
 }
 
+async function handleGetBankingChoicesForPayto(
+  wex: WalletExecutionContext,
+  req: GetBankingChoicesForPaytoRequest,
+): Promise<GetBankingChoicesForPaytoResponse> {
+  const parsedPayto = parsePaytoUri(req.paytoUri);
+  if (!parsedPayto) {
+    throw Error("invalid payto URI");
+  }
+  const amount = parsedPayto.params["amount"];
+  if (!amount) {
+    logger.warn("payto URI has no amount");
+    return {
+      choices: [],
+    };
+  }
+  const currency = Amounts.currencyOf(amount);
+  switch (currency) {
+    case "KUDOS":
+      return {
+        choices: [
+          {
+            label: "Demobank Website",
+            type: "link",
+            uri: `https://bank.demo.taler.net/webui/#/transfer/${encodeURIComponent(
+              req.paytoUri,
+            )}`,
+          },
+          {
+            label: "Demobank App",
+            type: "link",
+            uri: `https://bank.demo.taler.net/app/transfer/${encodeURIComponent(
+              req.paytoUri,
+            )}`,
+          },
+        ],
+      };
+      break;
+    default:
+      return {
+        choices: [],
+      };
+  }
+}
+
 async function handleConfirmPay(
   wex: WalletExecutionContext,
   req: ConfirmPayRequest,
@@ -1465,6 +1558,10 @@ interface HandlerWithValidator<Tag extends WalletApiOperation> {
 }
 
 const handlers: { [T in WalletApiOperation]: HandlerWithValidator<T> } = {
+  [WalletApiOperation.TestingWaitExchangeState]: {
+    codec: codecForAny(),
+    handler: handleTestingWaitExchangeState,
+  },
   [WalletApiOperation.AbortTransaction]: {
     codec: codecForAbortTransaction(),
     handler: handleAbortTransaction,
@@ -1526,10 +1623,6 @@ const handlers: { [T in WalletApiOperation]: HandlerWithValidator<T> } = {
     codec: codecForTransactionByIdRequest(),
     handler: getTransactionById,
   },
-  [WalletApiOperation.GetWithdrawalTransactionByUri]: {
-    codec: codecForGetWithdrawalDetailsForUri(),
-    handler: getWithdrawalTransactionByUri,
-  },
   [WalletApiOperation.AddExchange]: {
     codec: codecForAddExchangeRequest(),
     handler: handleAddExchange,
@@ -1634,10 +1727,7 @@ const handlers: { [T in WalletApiOperation]: HandlerWithValidator<T> } = {
   },
   [WalletApiOperation.ConfirmWithdrawal]: {
     codec: codecForConfirmWithdrawalRequestRequest(),
-    handler: async (wex, req) => {
-      await confirmWithdrawal(wex, req);
-      return {};
-    },
+    handler: confirmWithdrawal,
   },
   [WalletApiOperation.PrepareBankIntegratedWithdrawal]: {
     codec: codecForPrepareBankIntegratedWithdrawalRequest(),
@@ -1902,7 +1992,11 @@ const handlers: { [T in WalletApiOperation]: HandlerWithValidator<T> } = {
   [WalletApiOperation.ImportDb]: {
     codec: codecForImportDbRequest(),
     handler: async (wex, req) => {
+      // FIXME: This should atomically re-materialize transactions!
       await importDb(wex.db.idbHandle(), req.dump);
+      await wex.db.runAllStoresReadWriteTx({}, async (tx) => {
+        await rematerializeTransactions(wex, tx);
+      });
       return {};
     },
   },
@@ -2013,6 +2107,18 @@ const handlers: { [T in WalletApiOperation]: HandlerWithValidator<T> } = {
       throw Error("not implemented");
     },
   },
+  [WalletApiOperation.GetBankingChoicesForPayto]: {
+    codec: codecForGetBankingChoicesForPaytoRequest(),
+    handler: handleGetBankingChoicesForPayto,
+  },
+  [WalletApiOperation.StartExchangeWalletKyc]: {
+    codec: codecForStartExchangeWalletKycRequest(),
+    handler: handleStartExchangeWalletKyc,
+  },
+  [WalletApiOperation.TestingWaitExchangeWalletKyc]: {
+    codec: codecForTestingWaitWalletKycRequest(),
+    handler: handleTestingWaitExchangeWalletKyc,
+  },
 };
 
 /**
