@@ -5,7 +5,7 @@ set -e
 rm -rf dist lib tsconfig.tsbuildinfo .linaria-cache
 
 echo typecheck and bundle...
-node build-fast-with-linaria.mjs &
+node build.mjs &
 pnpm tsc --noEmit &
 wait -n
 wait -n
