@@ -23,7 +23,7 @@
 /**
  * Imports.
  */
-import { platform, setupPlatform } from "./platform/api.js";
+import { platform, setupPlatform } from "./platform/background.js";
 import chromeAPI from "./platform/chrome.js";
 import firefoxAPI from "./platform/firefox.js";
 import { wxMain } from "./wxBackend.js";
@@ -35,14 +35,15 @@ const isFirefox =
 // FIXME: create different entry point for any platform instead of
 // switching in runtime
 if (isFirefox) {
-  console.log("Wallet setup for Firefox API");
   setupPlatform(firefoxAPI);
 } else {
-  console.log("Wallet setup for Chrome API");
   setupPlatform(chromeAPI);
 }
 
 // setGlobalLogLevelFromString("trace")
-platform.notifyWhenAppIsReady(() => {
-  wxMain();
-});
+
+async function start() {
+  await platform.notifyWhenAppIsReady();
+  await wxMain();
+}
+start();
