@@ -17,7 +17,11 @@
 import {
   AmountJson,
   Amounts,
+  AmountString,
   parsePaytoUri,
+  PaytoUriIBAN,
+  PaytoUriTalerBank,
+  PaytoUriUnknown,
   segwitMinAmount,
   stringifyPaytoUri,
   TranslatedString,
@@ -26,10 +30,14 @@ import {
 import { useTranslationContext } from "@gnu-taler/web-util/browser";
 import { ComponentChildren, Fragment, h, VNode } from "preact";
 import { useEffect, useRef, useState } from "preact/hooks";
+import { Button } from "../mui/Button.js";
 import { CopiedIcon, CopyIcon } from "../svg/index.js";
 import { Amount } from "./Amount.js";
 import { ButtonBox, TooltipLeft, WarningBox } from "./styled/index.js";
-import { Button } from "../mui/Button.js";
+import { useAsyncAsHook } from "../hooks/useAsyncAsHook.js";
+import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
+import { useBackendContext } from "../context/backend.js";
+import { QR } from "./QR.js";
 
 export interface BankDetailsProps {
   subject: string;
@@ -59,71 +67,12 @@ export function BankDetailsByPaytoType({
   const payto = parsePaytoUri(selectedAccount.paytoUri);
 
   if (!payto) return <Fragment />;
+  // make sure the payto has the right params
   payto.params["amount"] = altCurrency
     ? selectedAccount.transferAmount!
     : Amounts.stringify(amount);
   payto.params["message"] = subject;
 
-  function Frame({
-    title,
-    children,
-  }: {
-    title: TranslatedString;
-    children: ComponentChildren;
-  }): VNode {
-    return (
-      <section
-        style={{
-          textAlign: "left",
-          border: "solid 1px black",
-          padding: 8,
-          borderRadius: 4,
-        }}
-      >
-        <div
-          style={{
-            display: "flex",
-            width: "100%",
-            justifyContent: "space-between",
-          }}
-        >
-          <p style={{ marginTop: 0 }}>{title}</p>
-          <div></div>
-        </div>
-
-        {children}
-
-        {accounts.length > 1 ? (
-          <Fragment>
-            {accounts.map((ac, acIdx) => {
-              const accountLabel = ac.bankLabel ?? `Account #${acIdx + 1}`;
-              return (
-                <Button
-                  key={acIdx}
-                  variant={acIdx === index ? "contained" : "outlined"}
-                  onClick={async () => {
-                    setIndex(acIdx);
-                  }}
-                >
-                  {accountLabel} (
-                  {ac.currencySpecification?.name ?? amount.currency})
-                </Button>
-              );
-            })}
-
-            {/* <Button variant={currency === altCurrency ? "contained" : "outlined"}
-            onClick={async () => {
-              setCurrency(altCurrency)
-            }}
-          >
-            <i18n.Translate>{altCurrency}</i18n.Translate>
-          </Button> */}
-          </Fragment>
-        ) : undefined}
-      </section>
-    );
-  }
-
   if (payto.isKnown && payto.targetType === "bitcoin") {
     const min = segwitMinAmount(amount.currency);
     const addrs = payto.segwitAddrs.map(
@@ -132,7 +81,13 @@ export function BankDetailsByPaytoType({
     addrs.unshift(`${payto.targetPath} ${Amounts.stringifyValue(amount)}`);
     const copyContent = addrs.join("\n");
     return (
-      <Frame title={i18n.str`Bitcoin transfer details`}>
+      <Frame
+        title={i18n.str`Bitcoin transfer details`}
+        accounts={accounts}
+        updateIndex={setIndex}
+        currentIndex={index}
+        defaultCurrency={amount.currency}
+      >
         <p>
           <i18n.Translate>
             The exchange need a transaction with 3 output, one output is the
@@ -176,6 +131,39 @@ export function BankDetailsByPaytoType({
     );
   }
 
+  return (
+    <Frame
+      title={i18n.str`Bank transfer details`}
+      accounts={accounts}
+      updateIndex={setIndex}
+      currentIndex={index}
+      defaultCurrency={amount.currency}
+    >
+      <IBANAccountInfoTable payto={payto} subject={subject} />
+    </Frame>
+  );
+}
+
+function IBANAccountInfoTable({
+  payto,
+  subject,
+}: {
+  subject: string;
+  payto: PaytoUriUnknown | PaytoUriIBAN | PaytoUriTalerBank;
+}) {
+  const { i18n } = useTranslationContext();
+  const api = useBackendContext();
+
+  const hook = useAsyncAsHook(
+    () =>
+      api.wallet.call(WalletApiOperation.GetQrCodesForPayto, {
+        paytoUri: stringifyPaytoUri(payto),
+      }),
+    [],
+  );
+
+  const qrCodes = !hook || hook.hasError ? [] : hook.response.codes;
+
   const accountPart = !payto.isKnown ? (
     <Fragment>
       <Row name={i18n.str`Account`} value={payto.targetPath} />
@@ -196,96 +184,105 @@ export function BankDetailsByPaytoType({
 
   const receiver =
     payto.params["receiver-name"] || payto.params["receiver"] || undefined;
+
   return (
-    <Frame title={i18n.str`Bank transfer details`}>
-      <table>
-        <tbody>
-          <tr>
-            <td colSpan={3}>
-              <i18n.Translate>Step 1:</i18n.Translate>
-              &nbsp;
-              <i18n.Translate>
-                Copy this code and paste it into the subject/purpose field in
-                your banking app or bank website
-              </i18n.Translate>
-            </td>
-          </tr>
-          <Row name={i18n.str`Subject`} value={subject} literal />
+    <table>
+      <tbody>
+        <tr>
+          <td colSpan={3}>
+            <i18n.Translate>Step 1:</i18n.Translate>
+            &nbsp;
+            <i18n.Translate>
+              Copy this code and paste it into the subject/purpose field in your
+              banking app or bank website
+            </i18n.Translate>
+          </td>
+        </tr>
+        <Row name={i18n.str`Subject`} value={subject} literal />
 
-          <tr>
-            <td colSpan={3}>
-              <i18n.Translate>Step 2:</i18n.Translate>
-              &nbsp;
-              <i18n.Translate>
-                If you don't already have it in your banking favourites list,
-                then copy and paste this IBAN and the name into the receiver
-                fields in your banking app or website
-              </i18n.Translate>
-            </td>
-          </tr>
-          {accountPart}
-          {receiver ? (
-            <Row name={i18n.str`Receiver name`} value={receiver} />
-          ) : undefined}
+        <tr>
+          <td colSpan={3}>
+            <i18n.Translate>Step 2:</i18n.Translate>
+            &nbsp;
+            <i18n.Translate>
+              If you don't already have it in your banking favourites list, then
+              copy and paste this IBAN and the name into the receiver fields in
+              your banking app or website
+            </i18n.Translate>
+          </td>
+        </tr>
+        {accountPart}
+        {receiver ? (
+          <Row name={i18n.str`Receiver name`} value={receiver} />
+        ) : undefined}
 
-          <tr>
-            <td colSpan={3}>
-              <i18n.Translate>Step 3:</i18n.Translate>
-              &nbsp;
-              <i18n.Translate>
-                Finish the wire transfer setting the amount in your banking app
-                or website, then this withdrawal will proceed automatically.
-              </i18n.Translate>
-            </td>
-          </tr>
-          <Row
-            name={i18n.str`Amount`}
-            value={
-              <Amount
-                value={altCurrency ? selectedAccount.transferAmount! : amount}
-                hideCurrency
-              />
-            }
-          />
+        <tr>
+          <td colSpan={3}>
+            <i18n.Translate>Step 3:</i18n.Translate>
+            &nbsp;
+            <i18n.Translate>
+              Finish the wire transfer setting the amount in your banking app or
+              website, then this withdrawal will proceed automatically.
+            </i18n.Translate>
+          </td>
+        </tr>
+        <Row
+          name={i18n.str`Amount`}
+          value={
+            <Amount
+              value={payto.params["amount"] as AmountString}
+              hideCurrency
+            />
+          }
+        />
 
-          <tr>
-            <td colSpan={3}>
-              <WarningBox style={{ margin: 0 }}>
-                <span>
-                  <i18n.Translate>
-                    Make sure ALL data is correct, including the subject;
-                    otherwise, the money will not arrive in this wallet. You can
-                    use the copy buttons (<CopyIcon />) to prevent typing errors
-                    or the "payto://" URI below to copy just one value.
-                  </i18n.Translate>
-                </span>
-              </WarningBox>
-            </td>
-          </tr>
+        <tr>
+          <td colSpan={3}>
+            <WarningBox style={{ margin: 0 }}>
+              <span>
+                <i18n.Translate>
+                  Make sure ALL data is correct, including the subject;
+                  otherwise, the money will not arrive in this wallet. You can
+                  use the copy buttons (<CopyIcon />) to prevent typing errors
+                  or the "payto://" URI below to copy just one value.
+                </i18n.Translate>
+              </span>
+            </WarningBox>
+          </td>
+        </tr>
 
-          <tr>
-            <td colSpan={2} width="100%" style={{ wordBreak: "break-all" }}>
-              <i18n.Translate>
-                Alternative if your bank already supports PayTo URI, you can use
-                this{" "}
-                <a
-                  target="_bank"
-                  rel="noreferrer"
-                  title="RFC 8905 for designating targets for payments"
-                  href="https://tools.ietf.org/html/rfc8905"
-                >
-                  PayTo URI
-                </a>{" "}
-                link instead
-              </i18n.Translate>
-            </td>
-            <td>
-              <CopyButton getContent={() => stringifyPaytoUri(payto)} />
-            </td>
-          </tr>
-        </tbody>
-      </table>
-    </Frame>
+        <tr>
+          <td colSpan={2} width="100%" style={{ wordBreak: "break-all" }}>
+            <i18n.Translate>
+              Alternative if your bank already supports PayTo URI, you can use
+              this{" "}
+              <a
+                target="_bank"
+                rel="noreferrer"
+                title="RFC 8905 for designating targets for payments"
+                href="https://tools.ietf.org/html/rfc8905"
+              >
+                PayTo URI
+              </a>{" "}
+              link instead
+            </i18n.Translate>
+          </td>
+          <td>
+            <CopyButton getContent={() => stringifyPaytoUri(payto)} />
+          </td>
+        </tr>
+
+        {qrCodes.map((qr, idx) => {
+          return (
+            <tr  key={idx}>
+              <td colSpan={3} width="100%">
+                <QR text={qr.qrContent} />
+              </td>
+            </tr>
+          );
+        })}
+      </tbody>
+    </table>
   );
 }
 
@@ -358,3 +355,71 @@ function Row({
     </tr>
   );
 }
+
+function Frame({
+  title,
+  children,
+  accounts,
+  defaultCurrency,
+  currentIndex,
+  updateIndex,
+}: {
+  title: TranslatedString;
+  children: ComponentChildren;
+  currentIndex: number;
+  updateIndex: (idx: number) => void;
+  defaultCurrency: string;
+  accounts: WithdrawalExchangeAccountDetails[];
+}): VNode {
+  return (
+    <section
+      style={{
+        textAlign: "left",
+        border: "solid 1px black",
+        padding: 8,
+        borderRadius: 4,
+      }}
+    >
+      <div
+        style={{
+          display: "flex",
+          width: "100%",
+          justifyContent: "space-between",
+        }}
+      >
+        <p style={{ marginTop: 0 }}>{title}</p>
+        <div></div>
+      </div>
+
+      {children}
+
+      {accounts.length > 1 ? (
+        <Fragment>
+          {accounts.map((ac, acIdx) => {
+            const accountLabel = ac.bankLabel ?? `Account #${acIdx + 1}`;
+            return (
+              <Button
+                key={acIdx}
+                variant={acIdx === currentIndex ? "contained" : "outlined"}
+                onClick={async () => {
+                  updateIndex(acIdx);
+                }}
+              >
+                {accountLabel} (
+                {ac.currencySpecification?.name ?? defaultCurrency})
+              </Button>
+            );
+          })}
+
+          {/* <Button variant={currency === altCurrency ? "contained" : "outlined"}
+          onClick={async () => {
+            setCurrency(altCurrency)
+          }}
+        >
+          <i18n.Translate>{altCurrency}</i18n.Translate>
+        </Button> */}
+        </Fragment>
+      ) : undefined}
+    </section>
+  );
+}
