@@ -80,39 +80,34 @@ export function PaymentButtons({
       case "age-acceptable": {
         BalanceMessage = i18n.str`Balance is not enough because you have ${Amounts.stringifyValue(
           payStatus.balanceDetails.balanceAgeAcceptable,
-        )} ${amount.currency}  to pay for contracts restricted for age above ${
-          payStatus.contractTerms.minimum_age
-        } years old`;
+        )} ${amount.currency} to pay for this contract which is restricted.`;
         break;
       }
       case "available": {
         BalanceMessage = i18n.str`Balance is not enough because you have ${Amounts.stringifyValue(
           payStatus.balanceDetails.balanceAvailable,
-        )} ${amount.currency}  available.`;
+        )} ${amount.currency} available.`;
         break;
       }
       case "merchant-acceptable": {
         BalanceMessage = i18n.str`Balance is not enough because merchant will just accept ${Amounts.stringifyValue(
           payStatus.balanceDetails.balanceReceiverAcceptable,
-        )} ${
-          amount.currency
-        } . To know more you can check which exchange and auditors the merchant trust.`;
+        )} ${amount.currency
+          } . To know more you can check which exchange and auditors the merchant trust.`;
         break;
       }
       case "merchant-depositable": {
         BalanceMessage = i18n.str`Balance is not enough because merchant will just accept ${Amounts.stringifyValue(
           payStatus.balanceDetails.balanceReceiverDepositable,
-        )} ${
-          amount.currency
-        } . To know more you can check which wire methods the merchant accepts.`;
+        )} ${amount.currency
+          } . To know more you can check which wire methods the merchant accepts.`;
         break;
       }
       case "material": {
         BalanceMessage = i18n.str`Balance is not enough because you have ${Amounts.stringifyValue(
           payStatus.balanceDetails.balanceMaterial,
-        )} ${
-          amount.currency
-        } to spend right know. There are some coins that need to be refreshed.`;
+        )} ${amount.currency
+          } to spend right know. There are some coins that need to be refreshed.`;
         break;
       }
       case "fee-gap": {
@@ -123,9 +118,8 @@ export function PaymentButtons({
               payStatus.balanceDetails.maxEffectiveSpendAmount,
             ).amount,
           ),
-        )} ${
-          amount.currency
-        } more balance in your wallet or ask your merchant to cover more of the fees.`;
+        )} ${amount.currency
+          } more balance in your wallet or ask your merchant to cover more of the fees.`;
         break;
       }
       default:
