@@ -180,7 +180,7 @@ function ShowBalanceChange({ events }: MoreInfoPRops): VNode {
       <dd>
         <a
           title={not.hintTransactionId}
-          href={Pages.balanceTransaction({ tid: not.hintTransactionId })}
+          href={`#${Pages.balanceTransaction({ tid: not.hintTransactionId })}`}
         >
           {not.hintTransactionId.substring(0, 10)}
         </a>
@@ -257,7 +257,7 @@ function ShowTransactionStateTransition({
       <dd>
         <a
           title={not.transactionId}
-          href={Pages.balanceTransaction({ tid: not.transactionId })}
+          href={`#${Pages.balanceTransaction({ tid: not.transactionId })}`}
         >
           {not.transactionId.substring(0, 10)}
         </a>
@@ -722,7 +722,7 @@ function refresh(
 let currentTab: chrome.tabs.Tab | undefined;
 // Allow running outside the extension for testing
 // tslint:disable-next-line:no-string-literal
-if (typeof chrome !== "undefined") {
+if (typeof chrome !== "undefined" && typeof chrome.tabs !== "undefined") {
   const p = chrome.tabs.getCurrent();
   // this may be called outside the render phase (in the background)
   // when this happen currentTab is not needed but also undefined
@@ -1069,7 +1069,9 @@ export function ActiveTasksTable(): VNode {
                   {task.transaction ? (
                     <a
                       title={task.transaction}
-                      href={Pages.balanceTransaction({ tid: task.transaction })}
+                      href={`#${Pages.balanceTransaction({
+                        tid: task.transaction,
+                      })}`}
                     >
                       {task.transaction.substring(0, 10)}
                     </a>
