@@ -19,7 +19,7 @@
  * @author Sebastian Javier Marchano (sebasjm)
  */
 
-import { AmountString, Amounts } from "@gnu-taler/taler-util";
+import { AmountString, Amounts, ScopeType, parsePayUri, parsePaytoUri } from "@gnu-taler/taler-util";
 import { expect } from "chai";
 import { createWalletApiMock } from "../../test-utils.js";
 import { useComponentState } from "./state.js";
@@ -28,42 +28,6 @@ import { Props } from "./index.js";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
 
 describe("Deposit CTA states", () => {
-  it("should tell the user that the URI is missing", async () => {
-    const { handler, TestingContext } = createWalletApiMock();
-
-    const props: Props = {
-      talerDepositUri: undefined,
-      amountStr: undefined,
-      cancel: async () => {
-        null;
-      },
-      onSuccess: async () => {
-        null;
-      },
-    };
-
-    const hookBehavior = await tests.hookBehaveLikeThis(
-      useComponentState,
-      props,
-      [
-        ({ status }) => {
-          expect(status).equals("loading");
-        },
-        ({ status, error }) => {
-          expect(status).equals("error");
-
-          if (!error) expect.fail();
-          // if (!error.hasError) expect.fail();
-          // if (error.operational) expect.fail();
-          expect(error.description).eq("ERROR_NO-URI-FOR-DEPOSIT");
-        },
-      ],
-      TestingContext,
-    );
-
-    expect(hookBehavior).deep.equal({ result: "ok" });
-    expect(handler.getCallingQueueState()).eq("empty");
-  });
 
   it("should be ready after loading", async () => {
     const { handler, TestingContext } = createWalletApiMock();
@@ -72,19 +36,22 @@ describe("Deposit CTA states", () => {
       WalletApiOperation.PrepareDeposit,
       undefined,
       {
-        effectiveDepositAmount: "EUR:1" as AmountString,
-        totalDepositCost: "EUR:1.2" as AmountString,
+        effectiveDepositAmount: "EUR:0" as AmountString,
+        totalDepositCost: "EUR:0" as AmountString,
         fees: {
           coin: "EUR:0" as AmountString,
-          refresh: "EUR:0.2" as AmountString,
+          refresh: "EUR:0" as AmountString,
           wire: "EUR:0" as AmountString,
         },
       },
     );
 
     const props = {
-      talerDepositUri: "payto://refund/asdasdas",
-      amountStr: "EUR:1" as AmountString,
+      account: parsePaytoUri("payto://refund/asdasdas")!,
+      scope: {
+        type: ScopeType.Global as const,
+        currency: "EUR",
+      },
       cancel: async () => {
         null;
       },
@@ -97,16 +64,26 @@ describe("Deposit CTA states", () => {
       useComponentState,
       props,
       [
-        ({ status }) => {
-          expect(status).equals("loading");
+        // ({ status }) => {
+        //   expect(status).equals("loading");
+        // },
+        (state) => {
+          if (state.status !== "ready") expect.fail();
+          if (state.error) expect.fail();
+          expect(state.confirm.onClick).not.undefined;
+          expect(state.amount.value).deep.eq(Amounts.parseOrThrow("EUR:0"));
+          expect(state.cost).deep.eq(Amounts.parseOrThrow("EUR:0"));
+          expect(state.fee).deep.eq(Amounts.parseOrThrow("EUR:0"));
+          expect(state.effective).deep.eq(Amounts.parseOrThrow("EUR:0"));
         },
         (state) => {
           if (state.status !== "ready") expect.fail();
           if (state.error) expect.fail();
           expect(state.confirm.onClick).not.undefined;
-          expect(state.cost).deep.eq(Amounts.parseOrThrow("EUR:1.2"));
-          expect(state.fee).deep.eq(Amounts.parseOrThrow("EUR:0.2"));
-          expect(state.effective).deep.eq(Amounts.parseOrThrow("EUR:1"));
+          expect(state.amount.value).deep.eq(Amounts.parseOrThrow("EUR:0"));
+          expect(state.cost).deep.eq(Amounts.parseOrThrow("EUR:0"));
+          expect(state.fee).deep.eq(Amounts.parseOrThrow("EUR:0"));
+          expect(state.effective).deep.eq(Amounts.parseOrThrow("EUR:0"));
         },
       ],
       TestingContext,
