@@ -15,7 +15,7 @@
  */
 
 /* eslint-disable react-hooks/rules-of-hooks */
-import { Amounts, TalerProtocolTimestamp } from "@gnu-taler/taler-util";
+import { AmountJson, Amounts, AmountString, TalerProtocolTimestamp } from "@gnu-taler/taler-util";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
 import { isFuture, parse } from "date-fns";
 import { useState } from "preact/hooks";
@@ -28,11 +28,10 @@ import { RecursiveState } from "../../utils/index.js";
 import { Props, State } from "./index.js";
 
 export function useComponentState({
-  amount: amountStr,
+  scope,
   onClose,
   onSuccess,
 }: Props): RecursiveState<State> {
-  const amount = Amounts.parseOrThrow(amountStr);
   const api = useBackendContext();
   const { pushAlertOnError } = useAlertContext();
 
@@ -72,12 +71,14 @@ export function useComponentState({
   const exchangeList = hook.response.exchanges;
 
   return () => {
+    const [amount, setAmount] = useState<AmountJson>(Amounts.zeroOfCurrency(scope.currency));
     const [subject, setSubject] = useState<string | undefined>();
     const [timestamp, setTimestamp] = useState<string | undefined>();
     const { pushAlertOnError } = useAlertContext();
 
+    const amountStr = Amounts.stringify(amount)
     const selectedExchange = useSelectedExchange({
-      currency: amount.currency,
+      currency: scope.currency,
       defaultExchange: undefined,
       list: exchangeList,
     });
@@ -97,39 +98,41 @@ export function useComponentState({
         },
       );
       return resp;
-    });
+    },[amountStr]);
 
-    if (!hook) {
-      return {
-        status: "loading",
-        error: undefined,
-      };
-    }
+    // if (!hook) {
+    //   return {
+    //     status: "loading",
+    //     error: undefined,
+    //   };
+    // }
 
-    if (hook.hasError) {
-      return {
-        status: "error",
-        retry: {
-          onClick: pushAlertOnError(async () => {
-            hook.retry();
-          }),
-        },
-        error: alertFromError(
-          i18n,
-          i18n.str`Could not load the invoice status`,
-          hook,
-        ),
-      };
-      // return {
-      //   status: "loading-uri",
-      //   error: hook,
-      // };
-    }
-
-    const { amountEffective, amountRaw } = hook.response;
-    const requestAmount = Amounts.parseOrThrow(amountRaw);
-    const toBeReceived = Amounts.parseOrThrow(amountEffective);
+    // if (hook.hasError) {
+    //   return {
+    //     status: "error",
+    //     retry: {
+    //       onClick: pushAlertOnError(async () => {
+    //         hook.retry();
+    //       }),
+    //     },
+    //     error: alertFromError(
+    //       i18n,
+    //       i18n.str`Could not load the invoice status`,
+    //       hook,
+    //     ),
+    //   };
+    //   // return {
+    //   //   status: "loading-uri",
+    //   //   error: hook,
+    //   // };
+    // }
 
+    // const { amountEffective, amountRaw } = hook.response;
+    // const requestAmount = Amounts.parseOrThrow(amountRaw);
+    // const toBeReceived = Amounts.parseOrThrow(amountEffective);
+    const requestAmount = !hook || hook.hasError ? Amounts.zeroOfCurrency(scope.currency) : Amounts.parseOrThrow(hook.response.amountRaw);
+    const toBeReceived = !hook || hook.hasError ? Amounts.zeroOfCurrency(scope.currency) : Amounts.parseOrThrow(hook.response.amountEffective);
+  
     let purse_expiration: TalerProtocolTimestamp | undefined = undefined;
     let timestampError: string | undefined = undefined;
 
@@ -174,6 +177,11 @@ export function useComponentState({
 
     return {
       status: "ready",
+      amount: {
+        value: amount,
+        onInput: pushAlertOnError(async (e) => setAmount(e)),
+        error: Amounts.isZero(amount) ? "Can't be zero" : undefined,
+      },
       subject: {
         error:
           subject === undefined
