@@ -57,9 +57,11 @@ export const NoEnoughBalanceAvailable = tests.createExample(BaseView, {
       balanceAvailable: "USD:9" as AmountString,
       balanceMaterial: "USD:9" as AmountString,
       balanceAgeAcceptable: "USD:9" as AmountString,
-      balanceMerchantAcceptable: "USD:9" as AmountString,
-      balanceMerchantDepositable: "USD:9" as AmountString,
-      feeGapEstimate: "USD:1" as AmountString,
+      balanceReceiverAcceptable: "USD:9" as AmountString,
+      balanceReceiverDepositable: "USD:9" as AmountString,
+      maxEffectiveSpendAmount: "USD:9.5" as AmountString,
+      balanceExchangeDepositable: "USD:9.5" as AmountString,
+      perExchange: {},
     },
     talerUri: "taler://pay/..",
 
@@ -97,9 +99,11 @@ export const NoEnoughBalanceMaterial = tests.createExample(BaseView, {
       balanceAvailable: "USD:10" as AmountString,
       balanceMaterial: "USD:9" as AmountString,
       balanceAgeAcceptable: "USD:9" as AmountString,
-      balanceMerchantAcceptable: "USD:9" as AmountString,
-      balanceMerchantDepositable: "USD:0" as AmountString,
-      feeGapEstimate: "USD:1" as AmountString,
+      balanceReceiverAcceptable: "USD:9" as AmountString,
+      balanceReceiverDepositable: "USD:0" as AmountString,
+      maxEffectiveSpendAmount: "USD:9.5" as AmountString,
+      balanceExchangeDepositable: "USD:9.5" as AmountString,
+      perExchange: {},
     },
     talerUri: "taler://pay/..",
 
@@ -137,9 +141,11 @@ export const NoEnoughBalanceAgeAcceptable = tests.createExample(BaseView, {
       balanceAvailable: "USD:10" as AmountString,
       balanceMaterial: "USD:10" as AmountString,
       balanceAgeAcceptable: "USD:9" as AmountString,
-      balanceMerchantAcceptable: "USD:9" as AmountString,
-      balanceMerchantDepositable: "USD:9" as AmountString,
-      feeGapEstimate: "USD:1" as AmountString,
+      balanceReceiverAcceptable: "USD:9" as AmountString,
+      balanceReceiverDepositable: "USD:9" as AmountString,
+      maxEffectiveSpendAmount: "USD:9.5" as AmountString,
+      balanceExchangeDepositable: "USD:9.5" as AmountString,
+      perExchange: {},
     },
     talerUri: "taler://pay/..",
 
@@ -178,9 +184,11 @@ export const NoEnoughBalanceMerchantAcceptable = tests.createExample(BaseView, {
       balanceAvailable: "USD:10" as AmountString,
       balanceMaterial: "USD:10" as AmountString,
       balanceAgeAcceptable: "USD:10" as AmountString,
-      balanceMerchantAcceptable: "USD:9" as AmountString,
-      balanceMerchantDepositable: "USD:9" as AmountString,
-      feeGapEstimate: "USD:1" as AmountString,
+      balanceReceiverAcceptable: "USD:9" as AmountString,
+      balanceReceiverDepositable: "USD:9" as AmountString,
+      maxEffectiveSpendAmount: "USD:9.5" as AmountString,
+      balanceExchangeDepositable: "USD:9.5" as AmountString,
+      perExchange: {},
     },
     talerUri: "taler://pay/..",
 
@@ -220,9 +228,11 @@ export const NoEnoughBalanceMerchantDepositable = tests.createExample(
         balanceAvailable: "USD:10" as AmountString,
         balanceMaterial: "USD:10" as AmountString,
         balanceAgeAcceptable: "USD:10" as AmountString,
-        balanceMerchantAcceptable: "USD:10" as AmountString,
-        balanceMerchantDepositable: "USD:9" as AmountString,
-        feeGapEstimate: "USD:1" as AmountString,
+        balanceReceiverAcceptable: "USD:10" as AmountString,
+        balanceReceiverDepositable: "USD:9" as AmountString,
+        maxEffectiveSpendAmount: "USD:9.5" as AmountString,
+        balanceExchangeDepositable: "USD:9.5" as AmountString,
+        perExchange: {},
       },
       talerUri: "taler://pay/..",
 
@@ -261,9 +271,11 @@ export const NoEnoughBalanceFeeGap = tests.createExample(BaseView, {
       balanceAvailable: "USD:10" as AmountString,
       balanceMaterial: "USD:10" as AmountString,
       balanceAgeAcceptable: "USD:10" as AmountString,
-      balanceMerchantAcceptable: "USD:10" as AmountString,
-      balanceMerchantDepositable: "USD:10" as AmountString,
-      feeGapEstimate: "USD:1" as AmountString,
+      balanceReceiverAcceptable: "USD:10" as AmountString,
+      balanceReceiverDepositable: "USD:10" as AmountString,
+      maxEffectiveSpendAmount: "USD:9.5" as AmountString,
+      balanceExchangeDepositable: "USD:9.5" as AmountString,
+      perExchange: {},
     },
     talerUri: "taler://pay/..",
 
