@@ -14,16 +14,13 @@
  GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
  */
 
-import { Amounts } from "@gnu-taler/taler-util";
+import { useTranslationContext } from "@gnu-taler/web-util/browser";
 import { Fragment, h, VNode } from "preact";
 import { Amount } from "../../components/Amount.js";
-import { LogoHeader } from "../../components/LogoHeader.js";
 import { Part } from "../../components/Part.js";
-import { ProductList } from "../../components/ProductList.js";
-import { Link, SubTitle, WalletAction } from "../../components/styled/index.js";
-import { useTranslationContext } from "@gnu-taler/web-util/browser";
 import { Button } from "../../mui/Button.js";
 import { State } from "./index.js";
+import { TermsOfService } from "../../components/TermsOfService/index.js";
 
 export function IgnoredView(state: State.Ignored): VNode {
   const { i18n } = useTranslationContext();
@@ -32,7 +29,7 @@ export function IgnoredView(state: State.Ignored): VNode {
     <Fragment>
       <section>
         <p>
-          <i18n.Translate>You&apos;ve ignored the tip.</i18n.Translate>
+          <i18n.Translate>You&apos;ve ignored the refund.</i18n.Translate>
         </p>
       </section>
     </Fragment>
