@@ -16,9 +16,9 @@
 
 import {
   AmountJson,
-  AmountString,
   CurrencySpecification,
   ExchangeListItem,
+  ScopeInfo
 } from "@gnu-taler/taler-util";
 import { Loading } from "../../components/Loading.js";
 import { State as SelectExchangeState } from "../../hooks/useSelectedExchange.js";
@@ -37,7 +37,7 @@ import { ErrorAlertView } from "../../components/CurrentAlerts.js";
 import { ErrorAlert } from "../../context/alert.js";
 import { ExchangeSelectionPage } from "../../wallet/ExchangeSelection/index.js";
 import { NoExchangesView } from "../../wallet/ExchangeSelection/views.js";
-import { FinalStateOperation, SelectAmountView, SuccessView } from "./views.js";
+import { FinalStateOperation, SuccessView } from "./views.js";
 
 export interface PropsFromURI {
   talerWithdrawUri: string | undefined;
@@ -47,10 +47,10 @@ export interface PropsFromURI {
 
 export interface PropsFromParams {
   talerExchangeWithdrawUri: string | undefined;
+  scope: ScopeInfo;
   amount: string | undefined;
   cancel: () => Promise<void>;
   onSuccess: (txid: string) => Promise<void>;
-  onAmountChanged: (amount: AmountString) => Promise<void>;
 }
 
 export type State =
@@ -58,7 +58,7 @@ export type State =
   | State.LoadingUriError
   | SelectExchangeState.NoExchangeFound
   | SelectExchangeState.Selecting
-  | State.SelectAmount
+  // | State.SelectAmount
   | State.AlreadyCompleted
   | State.Success;
 
@@ -72,14 +72,14 @@ export namespace State {
     error: ErrorAlert;
   }
 
-  export interface SelectAmount {
-    status: "select-amount";
-    error: undefined;
-    exchangeBaseUrl: string;
-    confirm: ButtonHandler;
-    amount: AmountFieldHandler;
-    currency: string;
-  }
+  // export interface SelectAmount {
+  //   status: "select-amount";
+  //   error: undefined;
+  //   exchangeBaseUrl: string;
+  //   confirm: ButtonHandler;
+  //   amount: AmountFieldHandler;
+  //   currency: string;
+  // }
   export interface AlreadyCompleted {
     status: "already-completed";
     operationState: "confirmed" | "aborted" | "selected";
@@ -126,7 +126,7 @@ export namespace State {
 const viewMapping: StateViewMap<State> = {
   loading: Loading,
   error: ErrorAlertView,
-  "select-amount": SelectAmountView,
+  // "select-amount": SelectAmountView,
   "no-exchange-found": NoExchangesView,
   "selecting-exchange": ExchangeSelectionPage,
   success: SuccessView,
