@@ -26,6 +26,7 @@ import {
   ExchangeListItem,
   ExchangeTosStatus,
   ScopeType,
+  TransactionIdStr,
 } from "@gnu-taler/taler-util";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
 import { expect } from "chai";
@@ -111,14 +112,18 @@ describe("Withdraw CTA states", () => {
       WalletApiOperation.PrepareBankIntegratedWithdrawal,
       undefined,
       {
-        transactionId: "123",
+        transactionId: "123" as TransactionIdStr,
         info: {
           status: "pending",
           operationId: "123",
           currency: "ARS",
           amount: "EUR:2" as AmountString,
           possibleExchanges: [],
-        }
+          editableAmount: false,
+          editableExchange: false,
+          maxAmount: "ARS:1",
+          wireFee: "ARS:0",
+        },
       },
     );
 
@@ -153,7 +158,7 @@ describe("Withdraw CTA states", () => {
       WalletApiOperation.PrepareBankIntegratedWithdrawal,
       undefined,
       {
-        transactionId: "123",
+        transactionId: "123" as TransactionIdStr,
         info: {
           status: "pending",
           operationId: "123",
@@ -161,7 +166,11 @@ describe("Withdraw CTA states", () => {
           amount: "ARS:2" as AmountString,
           possibleExchanges: exchanges,
           defaultExchangeBaseUrl: exchanges[0].exchangeBaseUrl,
-        }
+          editableAmount: false,
+          editableExchange: false,
+          maxAmount: "ARS:1",
+          wireFee: "ARS:0",
+        },
       },
     );
     handler.addWalletCallResponse(
@@ -175,7 +184,7 @@ describe("Withdraw CTA states", () => {
         scopeInfo: {
           currency: "ARS",
           type: ScopeType.Exchange,
-          url: "http://asd"
+          url: "http://asd",
         },
         withdrawalAccountsList: [],
         ageRestrictionOptions: [],
@@ -199,7 +208,7 @@ describe("Withdraw CTA states", () => {
           if (state.status !== "success") return;
 
           expect(state.toBeReceived).deep.equal(Amounts.parseOrThrow("ARS:2"));
-          expect(state.withdrawalFee).deep.equal(Amounts.parseOrThrow("ARS:0"));
+          expect(state.toBeSent).deep.equal(Amounts.parseOrThrow("ARS:2"));
           expect(state.amount.value).deep.equal(Amounts.parseOrThrow("ARS:2"));
 
           expect(state.doWithdrawal.onClick).not.undefined;
@@ -235,6 +244,10 @@ describe("Withdraw CTA states", () => {
         amount: "ARS:2" as AmountString,
         possibleExchanges: exchangeWithNewTos,
         defaultExchangeBaseUrl: exchangeWithNewTos[0].exchangeBaseUrl,
+        editableAmount: false,
+        editableExchange: false,
+        maxAmount: "ARS:1",
+        wireFee: "ARS:0",
       },
     );
     handler.addWalletCallResponse(
@@ -247,7 +260,7 @@ describe("Withdraw CTA states", () => {
         scopeInfo: {
           currency: "ARS",
           type: ScopeType.Exchange,
-          url: "http://asd"
+          url: "http://asd",
         },
         tosAccepted: false,
         withdrawalAccountsList: [],
@@ -266,6 +279,10 @@ describe("Withdraw CTA states", () => {
         amount: "ARS:2" as AmountString,
         possibleExchanges: exchanges,
         defaultExchangeBaseUrl: exchanges[0].exchangeBaseUrl,
+        editableAmount: false,
+        editableExchange: false,
+        maxAmount: "ARS:1",
+        wireFee: "ARS:0",
       },
     );
 
@@ -285,7 +302,7 @@ describe("Withdraw CTA states", () => {
           if (state.status !== "success") return;
 
           expect(state.toBeReceived).deep.equal(Amounts.parseOrThrow("ARS:2"));
-          expect(state.withdrawalFee).deep.equal(Amounts.parseOrThrow("ARS:0"));
+          expect(state.toBeSent).deep.equal(Amounts.parseOrThrow("ARS:2"));
           expect(state.amount.value).deep.equal(Amounts.parseOrThrow("ARS:2"));
 
           expect(state.doWithdrawal.onClick).not.undefined;
