@@ -35,6 +35,7 @@ import {
   Settings,
   defaultSettings,
 } from "./api.js";
+import { encodeCrockForURI } from "@gnu-taler/web-util/browser";
 
 const api: BackgroundPlatformAPI & ForegroundPlatformAPI = {
   isFirefox,
@@ -178,54 +179,54 @@ function openWalletURIFromPopup(uri: TalerUri): void {
     case TalerUriAction.WithdrawExchange:
     case TalerUriAction.Withdraw:
       url = chrome.runtime.getURL(
-        `static/wallet.html#/cta/withdraw?talerUri=${encodeURIComponent(
+        `static/wallet.html#/cta/withdraw?talerUri=${encodeCrockForURI(
           talerUri,
         )}`,
       );
       break;
     case TalerUriAction.Restore:
       url = chrome.runtime.getURL(
-        `static/wallet.html#/cta/recovery?talerUri=${encodeURIComponent(
+        `static/wallet.html#/cta/recovery?talerUri=${encodeCrockForURI(
           talerUri,
         )}`,
       );
       break;
     case TalerUriAction.Pay:
       url = chrome.runtime.getURL(
-        `static/wallet.html#/cta/pay?talerUri=${encodeURIComponent(talerUri)}`,
+        `static/wallet.html#/cta/pay?talerUri=${encodeCrockForURI(talerUri)}`,
       );
       break;
     case TalerUriAction.Refund:
       url = chrome.runtime.getURL(
-        `static/wallet.html#/cta/refund?talerUri=${encodeURIComponent(
+        `static/wallet.html#/cta/refund?talerUri=${encodeCrockForURI(
           talerUri,
         )}`,
       );
       break;
     case TalerUriAction.PayPull:
       url = chrome.runtime.getURL(
-        `static/wallet.html#/cta/invoice/pay?talerUri=${encodeURIComponent(
+        `static/wallet.html#/cta/invoice/pay?talerUri=${encodeCrockForURI(
           talerUri,
         )}`,
       );
       break;
     case TalerUriAction.PayPush:
       url = chrome.runtime.getURL(
-        `static/wallet.html#/cta/transfer/pickup?talerUri=${encodeURIComponent(
+        `static/wallet.html#/cta/transfer/pickup?talerUri=${encodeCrockForURI(
           talerUri,
         )}`,
       );
       break;
     case TalerUriAction.PayTemplate:
       url = chrome.runtime.getURL(
-        `static/wallet.html#/cta/pay/template?talerUri=${encodeURIComponent(
+        `static/wallet.html#/cta/pay/template?talerUri=${encodeCrockForURI(
           talerUri,
         )}`,
       );
       break;
     case TalerUriAction.AddExchange:
       url = chrome.runtime.getURL(
-        `static/wallet.html#/cta/add/exchange?talerUri=${encodeURIComponent(
+        `static/wallet.html#/cta/add/exchange?talerUri=${encodeCrockForURI(
           talerUri,
         )}`,
       );
