@@ -17,6 +17,7 @@
 import {
   Amounts,
   NotificationType,
+  ScopeInfo,
   WalletBalance,
 } from "@gnu-taler/taler-util";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
@@ -41,9 +42,9 @@ import { AddNewActionView } from "../wallet/AddNewActionView.js";
 import { NoBalanceHelp } from "./NoBalanceHelp.js";
 
 export interface Props {
-  goToWalletDeposit: (currency: string) => Promise<void>;
-  goToWalletHistory: (currency: string) => Promise<void>;
-  goToWalletManualWithdraw: () => Promise<void>;
+  goToWalletDeposit: (scope: ScopeInfo) => Promise<void>;
+  goToWalletHistory: (scope: ScopeInfo) => Promise<void>;
+  goToWalletManualWithdraw: (scope?: ScopeInfo) => Promise<void>;
 }
 
 export type State = State.Loading | State.Error | State.Action | State.Balances;
@@ -70,8 +71,8 @@ export namespace State {
     error: undefined;
     balances: WalletBalance[];
     addAction: ButtonHandler;
-    goToWalletDeposit: (currency: string) => Promise<void>;
-    goToWalletHistory: (currency: string) => Promise<void>;
+    goToWalletDeposit: (currency: ScopeInfo) => Promise<void>;
+    goToWalletHistory: (currency: ScopeInfo) => Promise<void>;
     goToWalletManualWithdraw: ButtonHandler;
   }
 }
@@ -105,8 +106,7 @@ function useComponentState({
   if (state.hasError) {
     return {
       status: "error",
-      error: alertFromError(          i18n,
-        i18n.str`Could not load the balance`, state),
+      error: alertFromError(i18n, i18n.str`Could not load the balance`, state),
     };
   }
   if (addingAction) {
@@ -126,7 +126,9 @@ function useComponentState({
       onClick: pushAlertOnError(async () => setAddingAction(true)),
     },
     goToWalletManualWithdraw: {
-      onClick: pushAlertOnError(goToWalletManualWithdraw),
+      onClick: pushAlertOnError(async () => {
+        goToWalletManualWithdraw(state.response.balances.length ? state.response.balances[0].scopeInfo : undefined);
+      }),
     },
     goToWalletDeposit,
     goToWalletHistory,
@@ -155,8 +157,8 @@ export function BalanceView(state: State.Balances): VNode {
   const currencyWithNonZeroAmount = state.balances
     .filter((b) => !Amounts.isZero(b.available))
     .map((b) => {
-      b.flags
-      return b.available.split(":")[0]
+      b.flags;
+      return b.scopeInfo;
     });
 
   if (state.balances.length === 0) {
@@ -172,7 +174,10 @@ export function BalanceView(state: State.Balances): VNode {
       <section>
         <BalanceTable
           balances={state.balances}
-          goToWalletHistory={state.goToWalletHistory}
+          goToWalletHistory={(e) => {
+            console.log("qwe", e);
+            state.goToWalletHistory(e);
+          }}
         />
       </section>
       <footer style={{ justifyContent: "space-between" }}>
@@ -184,7 +189,7 @@ export function BalanceView(state: State.Balances): VNode {
         </Button>
         {currencyWithNonZeroAmount.length > 0 && (
           <MultiActionButton
-            label={(s) => i18n.str`Send ${s}`}
+            label={(s) => i18n.str`Send ${s.currency}`}
             actions={currencyWithNonZeroAmount}
             onClick={(c) => state.goToWalletDeposit(c)}
           />
