@@ -34,10 +34,6 @@ export const WithdrawalAction = tests.createExample(TestedComponent, {
   url: "taler://withdraw/something",
 });
 
-export const TipAction = tests.createExample(TestedComponent, {
-  url: "taler://tip/something",
-});
-
 export const NotifyAction = tests.createExample(TestedComponent, {
   url: "taler://notify-reserve/something",
 });
