@@ -14,14 +14,14 @@
  GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
  */
 
-import { OperationFailWithBody, OperationOk, OperationResult, TalerExchangeApi } from "@gnu-taler/taler-util";
+import { OperationFailWithBody, OperationOk, TalerExchangeApi } from "@gnu-taler/taler-util";
 import { ErrorAlertView } from "../../components/CurrentAlerts.js";
 import { Loading } from "../../components/Loading.js";
 import { ErrorAlert } from "../../context/alert.js";
 import { TextFieldHandler } from "../../mui/handlers.js";
-import { compose, StateViewMap } from "../../utils/index.js";
+import { StateViewMap, compose } from "../../utils/index.js";
 import { useComponentState } from "./state.js";
-import { ConfirmView, VerifyView } from "./views.js";
+import { ConfirmAddExchangeView, VerifyView } from "./views.js";
 
 export interface Props {
   currency?: string;
@@ -37,6 +37,7 @@ export type State = State.Loading
 export type CheckExchangeErrors = {
   "invalid-version": string;
   "invalid-currency": string;
+  "not-found": void;
   "already-active": void;
   "invalid-protocol": void;
 }
@@ -80,7 +81,7 @@ export namespace State {
 const viewMapping: StateViewMap<State> = {
   loading: Loading,
   error: ErrorAlertView,
-  confirm: ConfirmView,
+  confirm: ConfirmAddExchangeView,
   verify: VerifyView,
 };
 
