@@ -22,14 +22,21 @@
 
 import {
   Amounts,
+  ScopeInfo,
   TalerUri,
   TalerUriAction,
   TranslatedString,
+  parsePaytoUri,
+  parseScopeInfoShort,
   parseTalerUri,
+  stringifyPaytoUri,
+  stringifyScopeInfoShort,
   stringifyTalerUri,
 } from "@gnu-taler/taler-util";
 import {
   TranslationProvider,
+  decodeCrockFromURI,
+  encodeCrockForURI,
   useTranslationContext,
 } from "@gnu-taler/web-util/browser";
 import { createHashHistory } from "history";
@@ -87,6 +94,7 @@ import { WalletActivity } from "../components/WalletActivity.js";
 import { EnabledBySettings } from "../components/EnabledBySettings.js";
 import { DevExperimentPage } from "../cta/DevExperiment/index.js";
 import { ConfirmAddExchangeView } from "./AddExchange/views.js";
+import { ManageAccountPage } from "./ManageAccount/index.js";
 
 export function Application(): VNode {
   const { i18n } = useTranslationContext();
@@ -96,7 +104,7 @@ export function Application(): VNode {
     redirectTo(Pages.balanceTransaction({ tid }));
   }
   function redirectToURL(str: string): void {
-    window.location.href = new URL(str).href
+    window.location.href = new URL(str).href;
   }
 
   return (
@@ -115,12 +123,15 @@ export function Application(): VNode {
           <Route
             path={Pages.qr}
             component={() => (
-              <WalletTemplate goToTransaction={redirectToTxInfo} goToURL={redirectToURL}>
+              <WalletTemplate
+                goToTransaction={redirectToTxInfo}
+                goToURL={redirectToURL}
+              >
                 <QrReaderPage
                   onDetected={(talerActionUrl: TalerUri) => {
                     redirectTo(
                       Pages.defaultCta({
-                        uri: stringifyTalerUri(talerActionUrl),
+                        uri: encodeCrockForURI(stringifyTalerUri(talerActionUrl)),
                       }),
                     );
                   }}
@@ -132,7 +143,10 @@ export function Application(): VNode {
           <Route
             path={Pages.settings}
             component={() => (
-              <WalletTemplate goToTransaction={redirectToTxInfo} goToURL={redirectToURL}>
+              <WalletTemplate
+                goToTransaction={redirectToTxInfo}
+                goToURL={redirectToURL}
+              >
                 <SettingsPage />
               </WalletTemplate>
             )}
@@ -159,17 +173,29 @@ export function Application(): VNode {
 
           <Route
             path={Pages.balanceHistory.pattern}
-            component={({ currency }: { currency?: string }) => (
-              <WalletTemplate path="balance" goToTransaction={redirectToTxInfo} goToURL={redirectToURL}>
+            component={({ scope }: { scope?: string }) => (
+              <WalletTemplate
+                path="balance"
+                goToTransaction={redirectToTxInfo}
+                goToURL={redirectToURL}
+              >
                 <HistoryPage
-                  currency={currency}
-                  goToWalletDeposit={(currency: string) =>
-                    redirectTo(Pages.sendCash({ amount: `${currency}:0` }))
+                  scope={
+                    !scope
+                      ? undefined
+                      : parseScopeInfoShort(decodeCrockFromURI(scope))
                   }
-                  goToWalletManualWithdraw={(currency?: string) =>
+                  goToWalletDeposit={(scope: ScopeInfo) =>
+                    redirectTo(
+                      Pages.sendCash({
+                        scope: encodeCrockForURI(stringifyScopeInfoShort(scope)),
+                      }),
+                    )
+                  }
+                  goToWalletManualWithdraw={(scope?: ScopeInfo) =>
                     redirectTo(
                       Pages.receiveCash({
-                        amount: !currency ? undefined : `${currency}:0`,
+                        scope: !scope ? undefined : encodeCrockForURI(stringifyScopeInfoShort(scope)),
                       }),
                     )
                   }
@@ -179,18 +205,30 @@ export function Application(): VNode {
           />
           <Route
             path={Pages.searchHistory.pattern}
-            component={({ currency }: { currency?: string }) => (
-              <WalletTemplate path="balance" goToTransaction={redirectToTxInfo} goToURL={redirectToURL}>
+            component={({ scope }: { scope?: string }) => (
+              <WalletTemplate
+                path="balance"
+                goToTransaction={redirectToTxInfo}
+                goToURL={redirectToURL}
+              >
                 <HistoryPage
-                  currency={currency}
-                  search
-                  goToWalletDeposit={(currency: string) =>
-                    redirectTo(Pages.sendCash({ amount: `${currency}:0` }))
+                  scope={
+                    !scope
+                      ? undefined
+                      : parseScopeInfoShort(decodeCrockFromURI(scope))
                   }
-                  goToWalletManualWithdraw={(currency?: string) =>
+                  search
+                  goToWalletDeposit={(scope: ScopeInfo) =>
+                    redirectTo(
+                      Pages.sendCash({
+                        scope: encodeCrockForURI(stringifyScopeInfoShort(scope)),
+                      }),
+                    )
+                  }
+                  goToWalletManualWithdraw={(scope?: ScopeInfo) =>
                     redirectTo(
                       Pages.receiveCash({
-                        amount: !currency ? undefined : `${currency}:0`,
+                        scope: !scope ? undefined : encodeCrockForURI(stringifyScopeInfoShort(scope)),
                       }),
                     )
                   }
@@ -200,37 +238,129 @@ export function Application(): VNode {
           />
           <Route
             path={Pages.sendCash.pattern}
-            component={({ amount }: { amount?: string }) => (
-              <WalletTemplate path="balance" goToURL={redirectToURL}>
-                <DestinationSelectionPage
-                  type="send"
-                  amount={amount}
-                  goToWalletBankDeposit={(amount: string) =>
-                    redirectTo(Pages.balanceDeposit({ amount }))
-                  }
-                  goToWalletWalletSend={(amount: string) =>
-                    redirectTo(Pages.ctaTransferCreate({ amount }))
-                  }
-                />
-              </WalletTemplate>
-            )}
+            component={({
+              scope,
+            }: {
+              scope?: string;
+            }) => {
+              if (!scope) return <Redirect to={Pages.balanceHistory({})} />;
+              const s = parseScopeInfoShort(decodeCrockFromURI(scope));
+              if (!s) return <Redirect to={Pages.balanceHistory({})} />;
+
+              return (
+                <WalletTemplate path="balance" goToURL={redirectToURL}>
+                  <DestinationSelectionPage
+                    type="send"
+                    scope={s}
+                    goToWalletKnownBankDeposit={(s, p) =>
+                      redirectTo(
+                        Pages.ctaDeposit({
+                          scope: encodeCrockForURI(stringifyScopeInfoShort(s)),
+                          account: encodeCrockForURI(stringifyPaytoUri(p)),
+                        }),
+                      )
+                    }
+                    goToWalletNewBankDeposit={(s) =>
+                      redirectTo(
+                        Pages.bankManange({
+                          scope: encodeCrockForURI(stringifyScopeInfoShort(s)),
+                        }),
+                      )
+                    }
+                    goToWalletWalletSend={(s) =>
+                      redirectTo(
+                        Pages.ctaTransferCreate({
+                          scope: encodeCrockForURI(stringifyScopeInfoShort(s)),
+                        }),
+                      )
+                    }
+                  />
+                </WalletTemplate>
+              );
+            }}
+          />
+          <Route
+            path={Pages.bankManange.pattern}
+            component={({
+              scope,
+            }: {
+              scope?: string;
+            }) => {
+              const s = !scope ? undefined : parseScopeInfoShort(decodeCrockFromURI(scope));
+              if (!s) return <div>missing scope</div>
+
+              return (
+                <WalletTemplate path="balance" goToURL={redirectToURL}>
+                  <ManageAccountPage
+                    scope={s}
+                    onAccountAdded={(account) => redirectTo(Pages.ctaDeposit({
+                      scope: encodeCrockForURI(stringifyScopeInfoShort(s)),
+                      account: encodeCrockForURI(stringifyPaytoUri(account)),
+                    }))}
+                    onCancel={() => {
+                      redirectTo(Pages.balanceHistory({
+                        scope: encodeCrockForURI(stringifyScopeInfoShort(s)),
+                      }))
+                    }}
+                  />
+                </WalletTemplate>
+              );
+            }}
+          />
+          <Route
+            path={Pages.receiveCashForPurchase.pattern}
+            component={({ id: _purchaseId }: { id?: string }) => {
+              return (
+                <WalletTemplate path="balance" goToURL={redirectToURL}>
+                  not yet implemented
+                </WalletTemplate>
+              );
+            }}
+          />
+          <Route
+            path={Pages.receiveCashForInvoice.pattern}
+            component={({ id: _invoiceId }: { id?: string }) => {
+              return (
+                <WalletTemplate path="balance" goToURL={redirectToURL}>
+                  not yet implemented
+                </WalletTemplate>
+              );
+            }}
           />
           <Route
             path={Pages.receiveCash.pattern}
-            component={({ amount }: { amount?: string }) => (
-              <WalletTemplate path="balance" goToURL={redirectToURL}>
-                <DestinationSelectionPage
-                  type="get"
-                  amount={amount}
-                  goToWalletManualWithdraw={(amount?: string) =>
-                    redirectTo(Pages.ctaWithdrawManual({ amount }))
-                  }
-                  goToWalletWalletInvoice={(amount?: string) =>
-                    redirectTo(Pages.ctaInvoiceCreate({ amount }))
-                  }
-                />
-              </WalletTemplate>
-            )}
+            component={({
+              scope,
+            }: {
+              scope?: string;
+            }) => {
+              const s = !scope
+                ? undefined
+                : parseScopeInfoShort(decodeCrockFromURI(scope));
+
+              return (
+                <WalletTemplate path="balance" goToURL={redirectToURL}>
+                  <DestinationSelectionPage
+                    type="get"
+                    scope={s}
+                    goToWalletManualWithdraw={(s,) =>
+                      redirectTo(
+                        Pages.ctaWithdrawManual({
+                          scope: encodeCrockForURI(stringifyScopeInfoShort(s)),
+                        }),
+                      )
+                    }
+                    goToWalletWalletInvoice={(s,) =>
+                      redirectTo(
+                        Pages.ctaInvoiceCreate({
+                          scope: encodeCrockForURI(stringifyScopeInfoShort(s)),
+                        }),
+                      )
+                    }
+                  />
+                </WalletTemplate>
+              );
+            }}
           />
 
           <Route
@@ -239,8 +369,12 @@ export function Application(): VNode {
               <WalletTemplate path="balance" goToURL={redirectToURL}>
                 <TransactionPage
                   tid={tid}
-                  goToWalletHistory={(currency?: string) =>
-                    redirectTo(Pages.balanceHistory({ currency }))
+                  goToWalletHistory={(scope: ScopeInfo) =>
+                    redirectTo(
+                      Pages.balanceHistory({
+                        scope: encodeCrockForURI(stringifyScopeInfoShort(scope)),
+                      }),
+                    )
                   }
                 />
               </WalletTemplate>
@@ -249,25 +383,45 @@ export function Application(): VNode {
 
           <Route
             path={Pages.balanceDeposit.pattern}
-            component={({ amount }: { amount: string }) => (
-              <WalletTemplate path="balance" goToURL={redirectToURL}>
+            component={({
+              scope,
+            }: {
+              scope: string;
+            }) => {
+              const s = parseScopeInfoShort(decodeCrockFromURI(scope))
+              if (!s) {
+                return <div>missing scope</div>
+              }
+              return <WalletTemplate path="balance" goToURL={redirectToURL}>
                 <DepositPage
-                  amount={amount}
-                  onCancel={(currency: string) => {
-                    redirectTo(Pages.balanceHistory({ currency }));
+                  scope={s}
+                  onCancel={(scope: ScopeInfo) => {
+                    redirectTo(
+                      Pages.balanceHistory({
+                        scope: encodeCrockForURI(stringifyScopeInfoShort(scope)),
+                      }),
+                    );
                   }}
-                  onSuccess={(currency: string) => {
-                    redirectTo(Pages.balanceHistory({ currency }));
+                  onSuccess={(scope: ScopeInfo) => {
+                    redirectTo(
+                      Pages.balanceHistory({
+                        scope: encodeCrockForURI(stringifyScopeInfoShort(scope)),
+                      }),
+                    );
                   }}
                 />
               </WalletTemplate>
-            )}
+            }}
           />
 
           <Route
             path={Pages.backup}
             component={() => (
-              <WalletTemplate path="backup" goToTransaction={redirectToTxInfo} goToURL={redirectToURL}>
+              <WalletTemplate
+                path="backup"
+                goToTransaction={redirectToTxInfo}
+                goToURL={redirectToURL}
+              >
                 <BackupPage
                   onAddProvider={() => redirectTo(Pages.backupProviderAdd)}
                 />
@@ -283,8 +437,9 @@ export function Application(): VNode {
                   onPayProvider={(uri: string) =>
                     redirectTo(`${Pages.ctaPay}?talerPayUri=${uri}`)
                   }
-                  onWithdraw={(amount: string) =>
-                    redirectTo(Pages.receiveCash({ amount }))
+                  onWithdraw={(_amount: string) =>
+                    // FIXME: use receiveCashForPurchase
+                    redirectTo(Pages.receiveCash({ scope: "FIXME missing" }))
                   }
                   onBack={() => redirectTo(Pages.backup)}
                 />
@@ -314,7 +469,11 @@ export function Application(): VNode {
           <Route
             path={Pages.dev}
             component={() => (
-              <WalletTemplate path="dev" goToTransaction={redirectToTxInfo} goToURL={redirectToURL}>
+              <WalletTemplate
+                path="dev"
+                goToTransaction={redirectToTxInfo}
+                goToURL={redirectToURL}
+              >
                 <DeveloperPage />
               </WalletTemplate>
             )}
@@ -326,7 +485,7 @@ export function Application(): VNode {
           <Route
             path={Pages.defaultCta.pattern}
             component={({ uri }: { uri: string }) => {
-              const path = getPathnameForTalerURI(uri);
+              const path = getPathnameForTalerURI(decodeCrockFromURI(uri));
               if (!path) {
                 return (
                   <CallToActionTemplate title={i18n.str`Taler URI handler`}>
@@ -343,14 +502,37 @@ export function Application(): VNode {
               return <Redirect to={path} />;
             }}
           />
+          {/* // FIXME: mem leak problems */}
+          <Route
+            path={Pages.defaultCtaSimple.pattern}
+            component={({ uri }: { uri: string }) => {
+              const path = getPathnameForTalerURI(decodeURIComponent(uri));
+              if (!path) {
+                return (
+                  <CallToActionTemplate title={i18n.str`Taler URI handler`}>
+                    <AlertView
+                      alert={{
+                        type: "warning",
+                        message: i18n.str`Could not found a handler for the Taler URI`,
+                        description: i18n.str`The uri read in the path parameter is not valid: "${uri}"`,
+                      }}
+                    />
+                  </CallToActionTemplate>
+                );
+              }
+              return <Redirect to={path} />;
+            }}
+          />
+
           <Route
             path={Pages.ctaPay}
             component={({ talerUri }: { talerUri: string }) => (
               <CallToActionTemplate title={i18n.str`Digital cash payment`}>
                 <PaymentPage
-                  talerPayUri={decodeURIComponent(talerUri)}
-                  goToWalletManualWithdraw={(amount?: string) =>
-                    redirectTo(Pages.receiveCash({ amount }))
+                  talerPayUri={decodeCrockFromURI(talerUri)}
+                  goToWalletManualWithdraw={(_amount?: string) =>
+                    // FIXME: use receiveCashForPruchase
+                    redirectTo(Pages.receiveCash({}))
                   }
                   cancel={() => redirectTo(Pages.balance)}
                   onSuccess={(tid: string) =>
@@ -365,9 +547,10 @@ export function Application(): VNode {
             component={({ talerUri }: { talerUri: string }) => (
               <CallToActionTemplate title={i18n.str`Digital cash payment`}>
                 <PaymentTemplatePage
-                  talerTemplateUri={decodeURIComponent(talerUri)}
-                  goToWalletManualWithdraw={(amount?: string) =>
-                    redirectTo(Pages.receiveCash({ amount }))
+                  talerTemplateUri={decodeCrockFromURI(talerUri)}
+                  goToWalletManualWithdraw={(_amount?: string) =>
+                    // FIXME: use receiveCashForPruchase
+                    redirectTo(Pages.receiveCash({}))
                   }
                   cancel={() => redirectTo(Pages.balance)}
                   onSuccess={(tid: string) =>
@@ -382,7 +565,7 @@ export function Application(): VNode {
             component={({ talerUri }: { talerUri: string }) => (
               <CallToActionTemplate title={i18n.str`Digital cash refund`}>
                 <RefundPage
-                  talerRefundUri={decodeURIComponent(talerUri)}
+                  talerRefundUri={decodeCrockFromURI(talerUri)}
                   cancel={() => redirectTo(Pages.balance)}
                   onSuccess={(tid: string) =>
                     redirectTo(Pages.balanceTransaction({ tid }))
@@ -396,7 +579,7 @@ export function Application(): VNode {
             component={({ talerUri }: { talerUri: string }) => (
               <CallToActionTemplate title={i18n.str`Digital cash withdrawal`}>
                 <WithdrawPageFromURI
-                  talerWithdrawUri={decodeURIComponent(talerUri)}
+                  talerWithdrawUri={decodeCrockFromURI(talerUri)}
                   cancel={() => redirectTo(Pages.balance)}
                   onSuccess={(tid: string) =>
                     redirectTo(Pages.balanceTransaction({ tid }))
@@ -408,55 +591,70 @@ export function Application(): VNode {
           <Route
             path={Pages.ctaWithdrawManual.pattern}
             component={({
+              scope,
               amount,
               talerUri,
             }: {
+              scope: string;
               amount: string;
               talerUri: string;
-            }) => (
-              <CallToActionTemplate title={i18n.str`Digital cash withdrawal`}>
-                <WithdrawPageFromParams
-                  onAmountChanged={async (newamount) => {
-                    const page = `${Pages.ctaWithdrawManual({ amount: newamount })}?talerUri=${encodeURIComponent(talerUri)}`;
-                    redirectTo(page);
-                  }}
-                  talerExchangeWithdrawUri={talerUri}
-                  amount={amount}
-                  cancel={() => redirectTo(Pages.balance)}
-                  onSuccess={(tid: string) =>
-                    redirectTo(Pages.balanceTransaction({ tid }))
-                  }
-                />
-              </CallToActionTemplate>
-            )}
+            }) => {
+              if (!scope) return <Redirect to={Pages.balanceHistory({})} />;
+              const s = parseScopeInfoShort(decodeCrockFromURI(scope));
+              if (!s) return <Redirect to={Pages.balanceHistory({})} />;
+
+              return (
+                <CallToActionTemplate title={i18n.str`Digital cash withdrawal`}>
+                  <WithdrawPageFromParams
+                    scope={s}
+                    talerExchangeWithdrawUri={talerUri}
+                    amount={amount}
+                    cancel={() => redirectTo(Pages.balance)}
+                    onSuccess={(tid: string) =>
+                      redirectTo(Pages.balanceTransaction({ tid }))
+                    }
+                  />
+                </CallToActionTemplate>
+              );
+            }}
           />
           <Route
-            path={Pages.ctaDeposit}
+            path={Pages.ctaDeposit.pattern}
             component={({
-              amount,
-              talerUri,
+              scope,
+              account,
             }: {
-              amount: string;
-              talerUri: string;
-            }) => (
-              <CallToActionTemplate title={i18n.str`Digital cash deposit`}>
+              scope: string,
+              account: string;
+            }) => {
+              const s = parseScopeInfoShort(decodeCrockFromURI(scope))
+              if (!s) {
+                return <div>missing scope</div>
+              }
+              const p = parsePaytoUri(decodeCrockFromURI(account))
+              if (!p) {
+                return <div>missing account</div>
+              }
+
+              return <CallToActionTemplate title={i18n.str`Digital cash deposit`} >
                 <DepositPageCTA
-                  amountStr={Amounts.stringify(Amounts.parseOrThrow(amount))}
-                  talerDepositUri={decodeURIComponent(talerUri)}
+                  scope={s}
+                  account={p}
                   cancel={() => redirectTo(Pages.balance)}
                   onSuccess={(tid: string) =>
                     redirectTo(Pages.balanceTransaction({ tid }))
                   }
                 />
               </CallToActionTemplate>
-            )}
+            }
+            }
           />
           <Route
             path={Pages.ctaInvoiceCreate.pattern}
-            component={({ amount }: { amount: string }) => (
+            component={({ scope }: { scope: string }) => (
               <CallToActionTemplate title={i18n.str`Digital cash invoice`}>
                 <InvoiceCreatePage
-                  amount={Amounts.stringify(Amounts.parseOrThrow(amount))}
+                  scope={parseScopeInfoShort(decodeCrockFromURI(scope))!}
                   onClose={() => redirectTo(Pages.balance)}
                   onSuccess={(tid: string) =>
                     redirectTo(Pages.balanceTransaction({ tid }))
@@ -467,10 +665,10 @@ export function Application(): VNode {
           />
           <Route
             path={Pages.ctaTransferCreate.pattern}
-            component={({ amount }: { amount: string }) => (
+            component={({ scope }: { scope: string }) => (
               <CallToActionTemplate title={i18n.str`Digital cash transfer`}>
                 <TransferCreatePage
-                  amount={Amounts.stringify(Amounts.parseOrThrow(amount))}
+                  scope={parseScopeInfoShort(decodeCrockFromURI(scope))!}
                   onClose={() => redirectTo(Pages.balance)}
                   onSuccess={(tid: string) =>
                     redirectTo(Pages.balanceTransaction({ tid }))
@@ -484,9 +682,10 @@ export function Application(): VNode {
             component={({ talerUri }: { talerUri: string }) => (
               <CallToActionTemplate title={i18n.str`Digital cash invoice`}>
                 <InvoicePayPage
-                  talerPayPullUri={decodeURIComponent(talerUri)}
-                  goToWalletManualWithdraw={(amount?: string) =>
-                    redirectTo(Pages.receiveCash({ amount }))
+                  talerPayPullUri={decodeCrockFromURI(talerUri)}
+                  goToWalletManualWithdraw={(_amount?: string) =>
+                    // FIXME: use receiveCashForInvoice
+                    redirectTo(Pages.receiveCash({}))
                   }
                   onClose={() => redirectTo(Pages.balance)}
                   onSuccess={(tid: string) =>
@@ -501,7 +700,7 @@ export function Application(): VNode {
             component={({ talerUri }: { talerUri: string }) => (
               <CallToActionTemplate title={i18n.str`Digital cash transfer`}>
                 <TransferPickupPage
-                  talerPayPushUri={decodeURIComponent(talerUri)}
+                  talerPayPushUri={decodeCrockFromURI(talerUri)}
                   onClose={() => redirectTo(Pages.balance)}
                   onSuccess={(tid: string) =>
                     redirectTo(Pages.balanceTransaction({ tid }))
@@ -515,7 +714,7 @@ export function Application(): VNode {
             component={({ talerRecoveryUri }: { talerRecoveryUri: string }) => (
               <CallToActionTemplate title={i18n.str`Digital cash recovery`}>
                 <RecoveryPage
-                  talerRecoveryUri={decodeURIComponent(talerRecoveryUri)}
+                  talerRecoveryUri={decodeCrockFromURI(talerRecoveryUri)}
                   onCancel={() => redirectTo(Pages.balance)}
                   onSuccess={() => redirectTo(Pages.backup)}
                 />
@@ -527,7 +726,7 @@ export function Application(): VNode {
             component={({ talerUri }: { talerUri: string }) => (
               <CallToActionTemplate title={i18n.str`Development experiment`}>
                 <DevExperimentPage
-                  talerExperimentUri={decodeURIComponent(talerUri)}
+                  talerExperimentUri={decodeCrockFromURI(talerUri)}
                   onCancel={() => redirectTo(Pages.balanceHistory({}))}
                   onSuccess={() => redirectTo(Pages.balanceHistory({}))}
                 />
@@ -537,23 +736,26 @@ export function Application(): VNode {
           <Route
             path={Pages.ctaAddExchange}
             component={({ talerUri }: { talerUri: string }) => {
-              const tUri = parseTalerUri(decodeURIComponent(talerUri))
-              const baseUrl = tUri?.type === TalerUriAction.AddExchange ? tUri.exchangeBaseUrl : undefined
+              const tUri = parseTalerUri(decodeCrockFromURI(talerUri));
+              const baseUrl =
+                tUri?.type === TalerUriAction.AddExchange
+                  ? tUri.exchangeBaseUrl
+                  : undefined;
               if (!baseUrl) {
-                redirectTo(Pages.balanceHistory({}))
-                return <div>
-                  invalid url {talerUri}
-                </div>
+                redirectTo(Pages.balanceHistory({}));
+                return <div>invalid url {talerUri}</div>;
               }
-              return <CallToActionTemplate title={i18n.str`Add exchange`}>
-                <ConfirmAddExchangeView
-                  url={baseUrl}
-                  status="confirm"
-                  error={undefined}
-                  onCancel={() => redirectTo(Pages.balanceHistory({}))}
-                  onConfirm={() => redirectTo(Pages.balanceHistory({}))}
-                />
-              </CallToActionTemplate>
+              return (
+                <CallToActionTemplate title={i18n.str`Add exchange`}>
+                  <ConfirmAddExchangeView
+                    url={baseUrl}
+                    status="confirm"
+                    error={undefined}
+                    onCancel={() => redirectTo(Pages.balanceHistory({}))}
+                    onConfirm={() => redirectTo(Pages.balanceHistory({}))}
+                  />
+                </CallToActionTemplate>
+              );
             }}
           />
           {/**
@@ -574,7 +776,7 @@ export function Application(): VNode {
           <WalletActivity />
         </EnabledBySettings>
       </IoCProviderForRuntime>
-    </TranslationProvider>
+    </TranslationProvider >
   );
 }
 
@@ -613,7 +815,7 @@ function CallToActionTemplate({
     <WalletAction>
       <LogoHeader />
       <section style={{ display: "flex", justifyContent: "right", margin: 0 }}>
-        <LinkPrimary href={Pages.balance}>
+        <LinkPrimary href={`#${Pages.balance}`}>
           <div
             style={{
               height: 24,
@@ -633,7 +835,7 @@ function CallToActionTemplate({
         {children}
       </AlertProvider>
       <section style={{ display: "flex", justifyContent: "right" }}>
-        <LinkPrimary href={Pages.balance}>
+        <LinkPrimary href={`#${Pages.balance}`}>
           <i18n.Translate>Return to wallet</i18n.Translate>
         </LinkPrimary>
       </section>
@@ -665,7 +867,8 @@ function WalletTemplate({
       <WalletNavBar path={path} />
       <PendingTransactions
         goToTransaction={goToTransaction}
-        goToURL={goToURL} />
+        goToURL={goToURL}
+      />
       <WalletBox>
         <AlertProvider>
           <CurrentAlerts />
