@@ -32,47 +32,35 @@ import { RecursiveState } from "../../utils/index.js";
 import { Props, State } from "./index.js";
 
 export function useComponentState({
-  amount: amountStr,
+  scope,
   onCancel,
   onSuccess,
 }: Props): RecursiveState<State> {
   const api = useBackendContext();
   const { i18n } = useTranslationContext();
   const { pushAlertOnError } = useAlertContext();
-  const parsed = amountStr === undefined ? undefined : Amounts.parse(amountStr);
-  const currency = parsed !== undefined ? parsed.currency : undefined;
+
+  const zero = Amounts.zeroOfCurrency(scope.currency);
 
   const hook = useAsyncAsHook(async () => {
     const { balances } = await api.wallet.call(
       WalletApiOperation.GetBalances,
-      {},
+      {
+      },
     );
+
     const { accounts } = await api.wallet.call(
       WalletApiOperation.ListKnownBankAccounts,
-      { currency },
+      { currency: scope.currency },
     );
 
     return { accounts, balances };
   });
 
-  const initialValue =
-    parsed !== undefined
-      ? parsed
-      : currency !== undefined
-        ? Amounts.zeroOfCurrency(currency)
-        : undefined;
-  // const [accountIdx, setAccountIdx] = useState<number>(0);
   const [selectedAccount, setSelectedAccount] = useState<PaytoUri>();
 
   const [addingAccount, setAddingAccount] = useState(false);
 
-  if (!currency) {
-    return {
-      status: "amount-or-currency-error",
-      error: undefined,
-    };
-  }
-
   if (!hook) {
     return {
       status: "loading",
@@ -102,9 +90,9 @@ export function useComponentState({
     return {
       status: "manage-account",
       error: undefined,
-      currency,
-      onAccountAdded: (p: string) => {
-        updateAccountFromList(p);
+      scope,
+      onAccountAdded: (p: PaytoUri) => {
+        updateAccountFromList(stringifyPaytoUri(p));
         setAddingAccount(false);
         hook.retry();
       },
@@ -115,17 +103,17 @@ export function useComponentState({
     };
   }
 
-  const bs = balances.filter((b) => b.available.startsWith(currency));
+  const bs = balances.filter((b) => b.scopeInfo === scope);
   const balance =
     bs.length > 0
       ? Amounts.parseOrThrow(bs[0].available)
-      : Amounts.zeroOfCurrency(currency);
+      : Amounts.zeroOfCurrency(scope.currency);
 
   if (Amounts.isZero(balance)) {
     return {
       status: "no-enough-balance",
       error: undefined,
-      currency,
+      currency: scope.currency,
     };
   }
 
@@ -133,7 +121,7 @@ export function useComponentState({
     return {
       status: "no-accounts",
       error: undefined,
-      currency,
+      currency: scope.currency,
       onAddAccount: {
         onClick: pushAlertOnError(async () => {
           setAddingAccount(true);
@@ -143,10 +131,9 @@ export function useComponentState({
   }
   const firstAccount = accounts[0].uri;
   const currentAccount = !selectedAccount ? firstAccount : selectedAccount;
-  const zero = Amounts.zeroOfCurrency(currency)
   return (): State => {
     const [instructed, setInstructed] = useState(
-      {amount: initialValue ?? zero, type: TransactionAmountMode.Raw},
+      { amount: zero, type: TransactionAmountMode.Raw },
     );
     const amountStr = Amounts.stringify(instructed.amount);
     const depositPaytoUri = stringifyPaytoUri(currentAccount);
@@ -188,12 +175,12 @@ export function useComponentState({
     const totalFee =
       fee !== undefined
         ? Amounts.sub(fee.effectiveAmount, fee.rawAmount).amount
-        : Amounts.zeroOfCurrency(currency);
+        : zero;
 
     const totalToDeposit = Amounts.parseOrThrow(fee.rawAmount);
     const totalEffective = Amounts.parseOrThrow(fee.effectiveAmount);
 
-    const isDirty = instructed.amount !== initialValue;
+    const isDirty = instructed.amount !== zero;
     const amountError = !isDirty
       ? undefined
       : Amounts.cmp(balance, totalEffective) === -1
@@ -206,7 +193,7 @@ export function useComponentState({
       amountError !== undefined; //amount field may be invalid
 
     async function doSend(): Promise<void> {
-      if (!currency) return;
+      // if (!currency) return;
 
       const depositPaytoUri = stringifyPaytoUri(currentAccount);
       const amountStr = Amounts.stringify(totalEffective);
@@ -214,13 +201,13 @@ export function useComponentState({
         amount: amountStr,
         depositPaytoUri,
       });
-      onSuccess(currency);
+      onSuccess(scope!);
     }
 
     return {
       status: "ready",
       error: undefined,
-      currency,
+      currency: scope.currency,
       amount: {
         value: totalEffective,
         onInput: pushAlertOnError(async (a) => setInstructed({
@@ -250,7 +237,7 @@ export function useComponentState({
       currentAccount,
       cancelHandler: {
         onClick: pushAlertOnError(async () => {
-          onCancel(currency);
+          onCancel(scope!);
         }),
       },
       depositHandler: {
