@@ -53,7 +53,10 @@ export const WithNoAccountForIBAN = tests.createExample(ReadyView, {
     onClick: nullFunction,
   },
   totalFee: Amounts.zeroOfCurrency("USD"),
-  totalToDeposit: Amounts.parseOrThrow("USD:10"),
+  totalToDeposit: {
+    onInput:nullFunction,
+    value: Amounts.parseOrThrow("USD:10"),
+  },
   // onCalculateFee: alwaysReturnFeeToOne,
 });
 
@@ -82,7 +85,10 @@ export const WithIBANAccountTypeSelected = tests.createExample(ReadyView, {
     onClick: nullFunction,
   },
   totalFee: Amounts.zeroOfCurrency("USD"),
-  totalToDeposit: Amounts.parseOrThrow("USD:10"),
+  totalToDeposit: {
+    onInput:nullFunction,
+    value: Amounts.parseOrThrow("USD:10"),
+  },
   // onCalculateFee: alwaysReturnFeeToOne,
 });
 
@@ -111,6 +117,9 @@ export const NewBitcoinAccountTypeSelected = tests.createExample(ReadyView, {
     onClick: nullFunction,
   },
   totalFee: Amounts.zeroOfCurrency("USD"),
-  totalToDeposit: Amounts.parseOrThrow("USD:10"),
+  totalToDeposit: {
+    onInput:nullFunction,
+    value: Amounts.parseOrThrow("USD:10"),
+  },
   // onCalculateFee: alwaysReturnFeeToOne,
 });
