@@ -26,7 +26,7 @@ import { Grid } from "../../mui/Grid.js";
 import { State } from "./index.js";
 
 export function AmountOrCurrencyErrorView(
-  p: State.AmountOrCurrencyError,
+  _p: State.AmountOrCurrencyError,
 ): VNode {
   const { i18n } = useTranslationContext();
 
@@ -145,7 +145,7 @@ export function ReadyView(state: State.Ready): VNode {
         </p>
         <Grid container spacing={2} columns={1}>
           <Grid item xs={1}>
-            <AmountField label={i18n.str`Amount`} handler={state.amount} />
+            <AmountField label={i18n.str`Brut amount`} handler={state.amount} />
           </Grid>
           <Grid item xs={1}>
             <AmountField
@@ -156,12 +156,7 @@ export function ReadyView(state: State.Ready): VNode {
             />
           </Grid>
           <Grid item xs={1}>
-            <AmountField
-              label={i18n.str`Total deposit`}
-              handler={{
-                value: state.totalToDeposit,
-              }}
-            />
+            <AmountField label={i18n.str`Net amount`} handler={state.totalToDeposit} />
           </Grid>
         </Grid>
       </section>
@@ -180,7 +175,7 @@ export function ReadyView(state: State.Ready): VNode {
         ) : (
           <Button variant="contained" onClick={state.depositHandler.onClick}>
             <i18n.Translate>
-              Deposit&nbsp;{Amounts.stringifyValue(state.totalToDeposit)}{" "}
+              Deposit&nbsp;{Amounts.stringifyValue(state.totalToDeposit.value)}{" "}
               {state.currency}
             </i18n.Translate>
           </Button>
