@@ -25,7 +25,10 @@ import {
   stringifyWithdrawExchange,
 } from "@gnu-taler/taler-util";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
-import { useTranslationContext } from "@gnu-taler/web-util/browser";
+import {
+  encodeCrockForURI,
+  useTranslationContext,
+} from "@gnu-taler/web-util/browser";
 import { format } from "date-fns";
 import { Fragment, VNode, h } from "preact";
 import { useEffect, useRef, useState } from "preact/hooks";
@@ -337,7 +340,13 @@ export function DeveloperPage(): VNode {
                 return (
                   <tr key={idx}>
                     <td>
-                      <a href={!uri ? undefined : Pages.defaultCta({ uri })}>
+                      <a
+                        href={
+                          !uri
+                            ? undefined
+                            : `#${Pages.defaultCta({ uri: encodeCrockForURI(uri) })}`
+                        }
+                      >
                         {e.scopeInfo
                           ? `${e.scopeInfo.currency} (${
                               e.scopeInfo.type === ScopeType.Global
@@ -461,7 +470,7 @@ export function DeveloperPage(): VNode {
       )}
       <div style={{ display: "flex", justifyContent: "space-between" }}>
         <div />
-        <LinkPrimary href={Pages.settingsExchangeAdd({})}>
+        <LinkPrimary href={`#${Pages.settingsExchangeAdd({})}`}>
           <i18n.Translate>Add an exchange</i18n.Translate>
         </LinkPrimary>
       </div>
@@ -667,6 +676,12 @@ function ShowAllCoins({
   );
 }
 
+/**
+ *
+ * @param str
+ * @deprecated FIXME: use a better base64 function
+ * @returns
+ */
 function toBase64(str: string): string {
   return btoa(
     encodeURIComponent(str).replace(/%([0-9A-F]{2})/g, function (match, p1) {
