@@ -70,6 +70,7 @@ const exampleData = {
       reservePub: "A05AJGMFNSK4Q62NXR2FKNDB1J4EXTYQTE7VA4M9GZQ4TR06YBNG",
       confirmed: false,
       exchangePaytoUris: ["payto://x-taler-bank/bank/account"],
+      reserveClosingDelay: { d_us: "forever" },
       type: WithdrawalType.ManualTransfer,
       reserveIsReady: false,
     },
@@ -168,7 +169,12 @@ export const SomeBalanceWithNoTransactions = tests.createExample(
     transactionsByDate: {
       "11/11/11": [],
     },
-    balances: [
+    scope: {
+      currency: "Ásd",
+      type: ScopeType.Auditor,
+      url: "",
+    },
+      balances: [
       {
         available: "TESTKUDOS:10" as AmountString,
         flags: [],
@@ -183,7 +189,7 @@ export const SomeBalanceWithNoTransactions = tests.createExample(
         },
       },
     ],
-    balanceIndex: 0,
+    
   },
 );
 
@@ -191,6 +197,11 @@ export const OneSimpleTransaction = tests.createExample(TestedComponent, {
   transactionsByDate: {
     "11/11/11": [exampleData.withdraw],
   },
+  scope: {
+    currency: "Ásd",
+    type: ScopeType.Auditor,
+    url: "",
+  },
   balances: [
     {
       flags: [],
@@ -206,7 +217,7 @@ export const OneSimpleTransaction = tests.createExample(TestedComponent, {
       },
     },
   ],
-  balanceIndex: 0,
+  
 });
 
 export const TwoTransactionsAndZeroBalance = tests.createExample(
@@ -215,7 +226,12 @@ export const TwoTransactionsAndZeroBalance = tests.createExample(
     transactionsByDate: {
       "11/11/11": [exampleData.withdraw, exampleData.deposit],
     },
-    balances: [
+    scope: {
+      currency: "Ásd",
+      type: ScopeType.Auditor,
+      url: "",
+    },
+      balances: [
       {
         flags: [],
         available: "USD:0" as AmountString,
@@ -230,7 +246,7 @@ export const TwoTransactionsAndZeroBalance = tests.createExample(
         },
       },
     ],
-    balanceIndex: 0,
+    
   },
 );
 
@@ -245,6 +261,11 @@ export const OneTransactionPending = tests.createExample(TestedComponent, {
       },
     ],
   },
+  scope: {
+    currency: "Ásd",
+    type: ScopeType.Auditor,
+    url: "",
+  },
   balances: [
     {
       flags: [],
@@ -260,7 +281,7 @@ export const OneTransactionPending = tests.createExample(TestedComponent, {
       },
     },
   ],
-  balanceIndex: 0,
+  
 });
 
 export const SomeTransactions = tests.createExample(TestedComponent, {
@@ -282,6 +303,11 @@ export const SomeTransactions = tests.createExample(TestedComponent, {
       exampleData.deposit,
     ],
   },
+  scope: {
+    currency: "Ásd",
+    type: ScopeType.Auditor,
+    url: "",
+  },
   balances: [
     {
       flags: [],
@@ -297,7 +323,7 @@ export const SomeTransactions = tests.createExample(TestedComponent, {
       },
     },
   ],
-  balanceIndex: 0,
+  
 });
 
 export const SomeTransactionsInDifferentStates = tests.createExample(
@@ -378,7 +404,12 @@ export const SomeTransactionsInDifferentStates = tests.createExample(
         exampleData.deposit,
       ],
     },
-    balances: [
+    scope: {
+      currency: "Ásd",
+      type: ScopeType.Auditor,
+      url: "",
+    },
+      balances: [
       {
         flags: [],
         available: "USD:10" as AmountString,
@@ -393,7 +424,7 @@ export const SomeTransactionsInDifferentStates = tests.createExample(
         },
       },
     ],
-    balanceIndex: 0,
+    
   },
 );
 
@@ -411,7 +442,12 @@ export const SomeTransactionsWithTwoCurrencies = tests.createExample(
         exampleData.deposit,
       ],
     },
-    balances: [
+    scope: {
+      currency: "Ásd",
+      type: ScopeType.Auditor,
+      url: "",
+    },
+      balances: [
       {
         flags: [],
         available: "USD:0" as AmountString,
@@ -439,7 +475,7 @@ export const SomeTransactionsWithTwoCurrencies = tests.createExample(
         },
       },
     ],
-    balanceIndex: 0,
+    
   },
 );
 
@@ -447,6 +483,11 @@ export const FiveOfficialCurrencies = tests.createExample(TestedComponent, {
   transactionsByDate: {
     "11/11/11": [exampleData.withdraw],
   },
+  scope: {
+    currency: "Ásd",
+    type: ScopeType.Auditor,
+    url: "",
+  },
   balances: [
     {
       flags: [],
@@ -514,7 +555,7 @@ export const FiveOfficialCurrencies = tests.createExample(TestedComponent, {
       },
     },
   ],
-  balanceIndex: 0,
+  
 });
 
 export const FiveOfficialCurrenciesWithHighValue = tests.createExample(
@@ -523,7 +564,12 @@ export const FiveOfficialCurrenciesWithHighValue = tests.createExample(
     transactionsByDate: {
       "11/11/11": [exampleData.withdraw],
     },
-    balances: [
+    scope: {
+      currency: "Ásd",
+      type: ScopeType.Auditor,
+      url: "",
+    },
+      balances: [
       {
         flags: [],
         available: "USD:881001321230000" as AmountString,
@@ -590,7 +636,7 @@ export const FiveOfficialCurrenciesWithHighValue = tests.createExample(
         },
       },
     ],
-    balanceIndex: 0,
+    
   },
 );
 
@@ -603,6 +649,11 @@ export const PeerToPeer = tests.createExample(TestedComponent, {
       exampleData.push_debit,
     ],
   },
+  scope: {
+    currency: "Ásd",
+    type: ScopeType.Auditor,
+    url: "",
+  },
   balances: [
     {
       flags: [],
@@ -618,5 +669,4 @@ export const PeerToPeer = tests.createExample(TestedComponent, {
       },
     },
   ],
-  balanceIndex: 0,
 });
