@@ -19,11 +19,11 @@
  * @author Sebastian Javier Marchano (sebasjm)
  */
 
-import { createExample } from "../test-utils.js";
+import * as tests from "@gnu-taler/web-util/testing";
 import { QrReaderPage } from "./QrReader.js";
 
 export default {
-  title: "wallet/qr",
+  title: "qr reader",
 };
 
-export const Reading = createExample(QrReaderPage, {});
+export const Reading = tests.createExample(QrReaderPage, {});
