@@ -41,7 +41,7 @@ import {
   TransactionType,
   TransactionWithdrawal,
   WithdrawalDetails,
-  WithdrawalType
+  WithdrawalType,
 } from "@gnu-taler/taler-util";
 import * as tests from "@gnu-taler/web-util/testing";
 import beer from "../../static-dev/beer.png";
@@ -61,6 +61,7 @@ export default {
 const commonTransaction: TransactionCommon = {
   error: undefined,
   amountRaw: "KUDOS:11" as AmountString,
+  scopes: [],
   amountEffective: "KUDOS:9.2" as AmountString,
   txState: {
     major: TransactionMajorState.Done,
@@ -86,6 +87,7 @@ const exampleData = {
       confirmed: false,
       reservePub: "A05AJGMFNSK4Q62NXR2FKNDB1J4EXTYQTE7VA4M9GZQ4TR06YBNG",
       exchangePaytoUris: ["payto://x-taler-bank/bank.demo.taler.net/Exchange"],
+      reserveClosingDelay: { d_us: "forever" },
       type: WithdrawalType.ManualTransfer,
     },
   } as TransactionWithdrawal,
@@ -281,12 +283,17 @@ export const WithdrawPendingManual = tests.createExample(
         type: WithdrawalType.ManualTransfer,
         exchangePaytoUris: ["payto://iban/ES8877998399652238"],
         reservePub: "A05AJGMFNSK4Q62NXR2FKNDB1J4EXTYQTE7VA4M9GZQ4TR06YBNG",
-        exchangeCreditAccountDetails: [{
-          paytoUri: "payto://IBAN/1231231231",
+        reserveClosingDelay: {
+          d_us: 111,
         },
-        {
-          paytoUri: "payto://IBAN/2342342342",
-        }],
+        exchangeCreditAccountDetails: [
+          {
+            paytoUri: "payto://IBAN/1231231231",
+          },
+          {
+            paytoUri: "payto://IBAN/2342342342",
+          },
+        ],
       } as WithdrawalDetails,
       txState: {
         major: TransactionMajorState.Pending,
