@@ -20,11 +20,14 @@ import {
   Amounts,
   AmountString,
   DenomLossEventType,
+  Duration,
   MerchantInfo,
   NotificationType,
   OrderShortInfo,
   parsePaytoUri,
   PaytoUri,
+  ScopeInfo,
+  ScopeType,
   stringifyPaytoUri,
   TalerErrorCode,
   TalerPreciseTimestamp,
@@ -78,7 +81,7 @@ import { assertUnreachable } from "../utils/index.js";
 
 interface Props {
   tid: string;
-  goToWalletHistory: (currency?: string) => Promise<void>;
+  goToWalletHistory: (scope: ScopeInfo) => Promise<void>;
 }
 
 export function TransactionPage({ tid, goToWalletHistory }: Props): VNode {
@@ -116,7 +119,13 @@ export function TransactionPage({ tid, goToWalletHistory }: Props): VNode {
     );
   }
 
-  const currency = Amounts.parse(state.response.amountRaw)?.currency;
+  const currency = Amounts.parse(state.response.amountEffective)!.currency;
+  const txScope = !state.response.scopes.length
+    ? {
+        type: ScopeType.Global as const,
+        currency,
+      }
+    : state.response.scopes[0];
 
   return (
     <TransactionView
@@ -125,44 +134,44 @@ export function TransactionPage({ tid, goToWalletHistory }: Props): VNode {
         await api.wallet.call(WalletApiOperation.FailTransaction, {
           transactionId,
         });
-        goToWalletHistory(currency);
+        // goToWalletHistory(txScope);
       }}
       onSuspend={async () => {
         await api.wallet.call(WalletApiOperation.SuspendTransaction, {
           transactionId,
         });
-        goToWalletHistory(currency);
+        // goToWalletHistory(txScope);
       }}
       onResume={async () => {
         await api.wallet.call(WalletApiOperation.ResumeTransaction, {
           transactionId,
         });
-        goToWalletHistory(currency);
+        // goToWalletHistory(txScope);
       }}
       onAbort={async () => {
         await api.wallet.call(WalletApiOperation.AbortTransaction, {
           transactionId,
         });
-        goToWalletHistory(currency);
+        // goToWalletHistory(txScope);
       }}
       onRetry={async () => {
         await api.wallet.call(WalletApiOperation.RetryTransaction, {
           transactionId,
         });
-        goToWalletHistory(currency);
+        // goToWalletHistory(txScope);
       }}
       onDelete={async () => {
         await api.wallet.call(WalletApiOperation.DeleteTransaction, {
           transactionId,
         });
-        goToWalletHistory(currency);
+        goToWalletHistory(txScope);
       }}
       onRefund={async (transactionId) => {
         await api.wallet.call(WalletApiOperation.StartRefundQuery, {
           transactionId,
         });
       }}
-      onBack={() => goToWalletHistory(currency)}
+      onBack={() => goToWalletHistory(txScope)}
     />
   );
 }
@@ -268,14 +277,6 @@ function TransactionTemplate({
                   ),
               }}
             />
-          ) : transaction.txState.minor ===
-            TransactionMinorState.AmlRequired ? (
-            <WarningBox>
-              <i18n.Translate>
-                The transaction has been blocked since the account required an
-                AML check.
-              </i18n.Translate>
-            </WarningBox>
           ) : (
             <WarningBox>
               <div style={{ justifyContent: "center", lineHeight: "25px" }}>
@@ -452,8 +453,7 @@ export function TransactionView({
     //     ? transaction.withdrawalDetails.exchangeCreditAccountDetails ?? []
     //     : [];
     const blockedByKycOrAml =
-      transaction.txState.minor === TransactionMinorState.KycRequired ||
-      transaction.txState.minor === TransactionMinorState.AmlRequired;
+      transaction.txState.minor === TransactionMinorState.KycRequired;
     return (
       <TransactionTemplate
         transaction={transaction}
@@ -520,6 +520,30 @@ export function TransactionView({
             />
           }
         />
+        {transaction.txState.major === TransactionMajorState.Aborted &&
+        transaction.withdrawalDetails.type === WithdrawalType.ManualTransfer ? (
+          <AlertView
+            alert={{
+              type: "info",
+              message: i18n.str`Withdrawal incomplete.`,
+              description: (
+                <i18n.Translate>
+                  If you have already sent money to the service provider account
+                  it will wire it back at{" "}
+                  <Time
+                    timestamp={AbsoluteTime.addDuration(
+                      AbsoluteTime.fromPreciseTimestamp(transaction.timestamp),
+                      Duration.fromTalerProtocolDuration(
+                        transaction.withdrawalDetails.reserveClosingDelay,
+                      ),
+                    )}
+                    format="dd MMMM yyyy, HH:mm"
+                  />
+                </i18n.Translate>
+              ),
+            }}
+          />
+        ) : undefined}
       </TransactionTemplate>
     );
   }
@@ -575,9 +599,9 @@ export function TransactionView({
                         <i18n.Translate>
                           {<Amount value={r.amountEffective} />}{" "}
                           <a
-                            href={Pages.balanceTransaction({
+                            href={`#${Pages.balanceTransaction({
                               tid: r.transactionId,
-                            })}
+                            })}`}
                           >
                             was refunded
                           </a>{" "}
@@ -714,9 +738,19 @@ export function TransactionView({
         ) : transaction.wireTransferProgress === 0 ? (
           <AlertView
             alert={{
-              type: "warning",
-              message: i18n.str`Wire transfer is not initiated.`,
-              description: i18n.str` `,
+              type: "info",
+              message: i18n.str`Wire transfer still pending.`,
+              description: (
+                <i18n.Translate>
+                  The service provider deadline to make the wire transfer is:{" "}
+                  <Time
+                    timestamp={AbsoluteTime.fromProtocolTimestamp(
+                      transaction.wireTransferDeadline,
+                    )}
+                    format="dd MMMM yyyy, HH:mm"
+                  />
+                </i18n.Translate>
+              ),
             }}
           />
         ) : transaction.wireTransferProgress === 100 ? (
@@ -743,7 +777,17 @@ export function TransactionView({
             alert={{
               type: "info",
               message: i18n.str`Wire transfer in progress.`,
-              description: i18n.str` `,
+              description: (
+                <i18n.Translate>
+                  The service provider deadline to make the wire transfer is:{" "}
+                  <Time
+                    timestamp={AbsoluteTime.fromProtocolTimestamp(
+                      transaction.wireTransferDeadline,
+                    )}
+                    format="dd MMMM yyyy, HH:mm"
+                  />
+                </i18n.Translate>
+              ),
             }}
           />
         )}
@@ -801,9 +845,9 @@ export function TransactionView({
         >
           {transaction.paymentInfo ? (
             <a
-              href={Pages.balanceTransaction({
+              href={`#${Pages.balanceTransaction({
                 tid: transaction.refundedTransactionId,
-              })}
+              })}`}
             >
               {transaction.paymentInfo.summary}
             </a>
@@ -1746,7 +1790,7 @@ function TrackingDepositDetails({
   );
 }
 
-function DepositDetails({ amount }: { amount: AmountWithFee }): VNode {
+export function DepositDetails({ amount }: { amount: AmountWithFee }): VNode {
   const { i18n } = useTranslationContext();
 
   return (
