@@ -19,30 +19,22 @@
  * @author Sebastian Javier Marchano (sebasjm)
  */
 
-import { createExample } from "../test-utils.js";
+import * as tests from "@gnu-taler/web-util/testing";
 import { View as TestedComponent } from "./Welcome.js";
 
 export default {
-  title: "wallet/welcome",
+  title: "welcome",
   component: TestedComponent,
 };
 
-export const Normal = createExample(TestedComponent, {
-  permissionToggle: { value: true, button: {} },
-  diagnostics: {
-    errors: [],
-    walletManifestVersion: "1.0",
-    walletManifestDisplayVersion: "1.0",
-    firefoxIdbProblem: false,
-    dbOutdated: false,
-  },
-});
-
-export const TimedoutDiagnostics = createExample(TestedComponent, {
-  timedOut: true,
+export const Normal = tests.createExample(TestedComponent, {
   permissionToggle: { value: true, button: {} },
 });
 
-export const RunningDiagnostics = createExample(TestedComponent, {
+export const TimedoutDiagnostics = tests.createExample(TestedComponent, {
+  permissionToggle: { value: true, button: {} },
+});
+
+export const RunningDiagnostics = tests.createExample(TestedComponent, {
   permissionToggle: { value: true, button: {} },
 });
