@@ -1,30 +1,40 @@
-import { useContext, useState } from "preact/compat";
+import { useContext } from "preact/compat";
 import { FieldUIOptions, FormContext } from "./FormProvider.js";
+import { TranslatedString } from "@gnu-taler/taler-util";
 
 export interface InputFieldHandler<Type> {
   value: Type;
   onChange: (s: Type) => void;
   state: FieldUIOptions;
-  isDirty: boolean;
+  error?: TranslatedString | undefined;
 }
 
+/**
+ * @deprecated removing this so we don't depend on context to create a form
+ * @param name 
+ * @returns 
+ */
 export function useField<T extends object, K extends keyof T>(
   name: K,
-): InputFieldHandler<T[K]> {
+): InputFieldHandler<T[K]> | undefined {
+  const ctx = useContext(FormContext);
+  if (!ctx) {
+    //no context, can't be used
+    return undefined;
+  }
   const {
     value: formValue,
     computeFormState,
     onUpdate: notifyUpdate,
     readOnly: readOnlyForm,
-  } = useContext(FormContext);
+  } = ctx
 
   type P = typeof name;
   type V = T[P];
   const formState = computeFormState ? computeFormState(formValue.current) : {};
 
   const fieldValue = readField(formValue.current, String(name)) as V;
-  // console.log("USE FIELD", String(name), formValue.current, fieldValue);
-  const [currentValue, setCurrentValue] = useState<any | undefined>(fieldValue);
+
   const fieldState =
     readField<Partial<FieldUIOptions>>(formState, String(name)) ?? {};
 
@@ -32,13 +42,12 @@ export function useField<T extends object, K extends keyof T>(
   const state = {
     disabled: readOnlyForm ? true : (fieldState.disabled ?? false),
     hidden: fieldState.hidden ?? false,
-    error: fieldState.error,
     help: fieldState.help,
     elements: "elements" in fieldState ? fieldState.elements ?? [] : [],
   };
 
   function onChange(value: V): void {
-    setCurrentValue(value);
+    // setCurrentValue(value);
     formValue.current = setValueDeeper(
       formValue.current,
       String(name).split("."),
@@ -52,7 +61,6 @@ export function useField<T extends object, K extends keyof T>(
   return {
     value: fieldValue,
     onChange,
-    isDirty: currentValue !== undefined,
     state,
   };
 }
@@ -67,18 +75,8 @@ export function useField<T extends object, K extends keyof T>(
 function readField<T>(
   object: any,
   name: string,
-  debug?: boolean,
 ): T | undefined {
   return name.split(".").reduce((prev, current) => {
-    if (debug) {
-      console.log(
-        "READ",
-        name,
-        prev,
-        current,
-        prev ? prev[current] : undefined,
-      );
-    }
     return prev ? prev[current] : undefined;
   }, object);
 }
