@@ -41,8 +41,10 @@ const logger = new Logger("browserHttpLib");
 /**
  * An implementation of the [[HttpRequestLibrary]] using the
  * browser's XMLHttpRequest.
+ * 
+ * @deprecated use BrowserFetchHttpLib
  */
-export class BrowserHttpLib implements HttpRequestLibrary {
+export class BrowserHttpLibDepreacted implements HttpRequestLibrary {
   private throttle = new RequestThrottler();
   private throttlingEnabled = true;
   private requireTls = false;
