@@ -99,6 +99,17 @@ export class BrowserFetchHttpLib implements HttpRequestLibrary {
       });
     }
 
+    /**
+     * default header assume everything is json
+     * in case of formData the content-type will be
+     * auto generated
+     */
+    if (requestBody instanceof FormData) {
+      delete requestHeadersMap["Content-Type"]
+    } else if (requestBody instanceof URLSearchParams) {
+      requestHeadersMap["Content-Type"] = "application/x-www-form-urlencoded"
+    }
+
     const controller = new AbortController();
     let timeoutId: ReturnType<typeof setTimeout> | undefined;
     if (requestTimeout.d_ms !== "forever") {
