@@ -75,7 +75,7 @@ export function findMatch<T extends ObjectOf<RouteDefinition>>(
   pageList: Array<keyof T>,
   path: string,
   params: Record<string, string[]>,
-): Location<T> | undefined {
+): Location<T> | LocationNotFound<T> {
   for (let idx = 0; idx < pageList.length; idx++) {
     const name = pageList[idx];
     const found = pagesMap[name].pattern.exec(path);
@@ -92,7 +92,8 @@ export function findMatch<T extends ObjectOf<RouteDefinition>>(
       return { name, parent: pagesMap, values, params };
     }
   }
-  return undefined;
+  // @ts-expect-error values is a map string which is equivalent to the RouteParamsType
+  return { name: undefined, parent: pagesMap, values: {}, params };
 }
 
 /**
@@ -109,13 +110,13 @@ type RouteParamsType<
  */
 type MapKeyValue<Type> = {
   [Key in keyof Type]: Key extends string
-  ? {
-    parent: Type;
-    name: Key;
-    values: RouteParamsType<Type, Key>;
-    params: Record<string, string[]>;
-  }
-  : never;
+    ? {
+        parent: Type;
+        name: Key;
+        values: RouteParamsType<Type, Key>;
+        params: Record<string, string[]>;
+      }
+    : never;
 };
 
 /**
@@ -124,3 +125,9 @@ type MapKeyValue<Type> = {
 type EnumerationOf<T> = T[keyof T];
 
 export type Location<T> = EnumerationOf<MapKeyValue<T>>;
+export type LocationNotFound<Type> = {
+  parent: Type;
+  name: undefined;
+  values: RouteParamsType<Type, keyof Type>;
+  params: Record<string, string[]>;
+};
